/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -----------------------------------------
  Betriebssystemabhaengige Teile fuer Linux
  -----------------------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980329 OE3DZW added vendor-id
//19980614 OE3DZW dfree in kbytes
//19980909 OE3DZW added cpu-ticks to cpuinfo(), added rdtsc()

#include "baycom.h"
#include <sys/ioctl.h>
#include <sys/vfs.h>
#include <fcntl.h>
#include <termios.h>

/*---------------------------------------------------------------------------*/

char* strupr(char *s)
{ if(s)
  { while(s[0])
    { s[0]=toupper(s[0]);
      s++;
    }
  }
 return s;
}

/*---------------------------------------------------------------------------*/

char* strlwr(char *s)
{ if(s)
  { while(s[0])
    { s[0]=tolower(s[0]);
      s++;
    }
  }
  return s;
}

/*---------------------------------------------------------------------------*/

int bioskey(int mode)
{ 
  return 0;
}

/*---------------------------------------------------------------------------*/

void cdhome(void)
{ 
  chdir(globalpath);
}

/*---------------------------------------------------------------------------*/

long filelength(int fd)
{ long pos=lseek(fd,0,SEEK_CUR);
  long len=lseek(fd,0,SEEK_END);
  lseek(fd,pos,SEEK_SET);
  return len;
}

/*---------------------------------------------------------------------------*/

int mkdir(char const *path)
{ 
  return mkdir(path,0777);
}

/*---------------------------------------------------------------------------*/

int random_max(int max)
/* from man page "rand" (Linux Programmers Manual":
       In Numerical Recipes in C: The Art of Scientific Computing
       (William  H.  Press, Brian P. Flannery, Saul A. Teukolsky,
       William T.  Vetterling;  New  York:  Cambridge  University
       Press,  1990 (1st ed, p. 207)), the following comments are
       made:
              "If you want to generate a random integer between 1
              and 10, you should always do it by

                     j=1+(int) (10.0*rand()/(RAND_MAX+1.0));

              and never by anything resembling

                     j=1+((int) (1000000.0*rand()) % 10);

              (which uses lower-order bits)."

       Random-number  generation is a complex topic.  The Numeri
       cal Recipes in C book (see reference  above)  provides  an
       excellent discussion of practical random-number generation
       issues in Chapter 7 (Random Numbers).



*/
{
  //return random()%max;
  return (int) ((1.0 * max)*rand()/(RAND_MAX+1.0));
}

/*---------------------------------------------------------------------------*/

int l2aufruf1(int,int,int,char *)
{ return 0;
}

/*---------------------------------------------------------------------------*/

static int find_pty(char *ptyname) //ptyname = /dev/tty??
{ char master[80]; // master = /dev/pty??
  int fdm,fds,num;
  // tries to open pty, this should fail, then tries to open tty, this
  // should succeed, then tries to open pty, this should succeed
  // there seems to be a difference between kernel 2.0.0 and 2.0.27
  for (num=160;num>0;num--) 
  { sprintf(ptyname, "/dev/tty%c%x", 'q' + (num >> 4), num & 0xf);
    if (1) // ((fds = open(ptyname, O_RDWR | O_NONBLOCK |O_EXCL)) < 0) 
    { sprintf(master, "/dev/pty%c%x", 'q' + (num >> 4), num & 0xf);
      if ((fdm = open(master, O_RDWR | O_NONBLOCK| O_EXCL)) >= 0) 
      { if ((fds = open(ptyname, O_RDWR | O_NONBLOCK |O_EXCL)) >= 0)
        { //trace(report,"find_pty","open %s",ptyname);
          close(fds);
          return fdm;
        }
        else
          close(fdm);
      } 
    }
    else
      close(fds);
  }
  trace(serious,"find_pty","no pty found");
  return EOF;
}

/*---------------------------------------------------------------------------*/

int oshell(char *name,shellinput_t input)
{ 
  int fd,ret;
  char s[200];
  char cl[80];
  char c;
  int wasdagewesen=0;
  int wasalife=0;
  struct termios termios;
  int childpid;
  char ptyname[100];
  time_t begintime=ad_time();

  fd=find_pty(ptyname);
  if(fd==EOF)
    return 1;
  s_set(fd,ptyname,"lab");

  signal(SIGCLD,SIG_IGN);
  if((childpid=fork())==0)
  { 
    int i;
    static struct itimerval rttimer;
    static struct itimerval old_rttimer;

    memset(&rttimer,0,sizeof(struct itimerval));
    setitimer (ITIMER_REAL, &rttimer, &old_rttimer);
    signal(SIGCLD,SIG_IGN);
    
    setpgrp();
    setsid();
    for(i=0;i<FD_SETSIZE;i++)
      close(i); 
    fd=open(ptyname,O_RDWR,0666);
    dup(0);
    dup(0);
    //chmod(ptyname, 0622);
    ioctl(0, TIOCSCTTY, (char *) 0);
    memset((char *) &termios, 0, sizeof(termios));
    termios.c_iflag = ICRNL | IXOFF;
    termios.c_oflag = OPOST | ONLCR;
    termios.c_cflag = CS8 | CREAD | CLOCAL;
    termios.c_lflag = ISIG | ICANON | ECHO;
    termios.c_cc[VINTR]  =  3;  // ^C
    termios.c_cc[VSUSP]  =  26; // ^Z
    termios.c_cc[VQUIT]  =  28; //
    termios.c_cc[VERASE] =   8; // ^H
    termios.c_cc[VKILL]  =  24; // ^X  
    termios.c_cc[VEOF]   =   4; // ^D
    cfsetispeed(&termios, B1200);
    cfsetospeed(&termios, B1200);
    tcsetattr(fd, TCSANOW, &termios);
    tcsetpgrp(fd,getpgrp());
    if(name[0]==0)
    { printf("Type `exit' or ctrl-d to return to BayCom-Mailbox...\n\n");
      execl("/bin/bash","-bash",NULL);
    }
    else
    { char cmd[200];
      if(filetime("bashrc.bcm"))
        sprintf(cmd,"source bashrc.bcm;%s",name);
      else
        strcpy(cmd,name);
      execl("/bin/bash","bash","-c",cmd,NULL);
    }  
    perror("execlp");  
    exit(0); // sollte nie aufgerufen werden
  }
  t->childpid=childpid;

  while(fd!=EOF)
  {
    while((ret=read(fd,&c,1))>0)
    {
      wasalife=1; 
      wasdagewesen=1;
      putv(c);
    }
    if(ret<0)
    { if(((ad_time()>(begintime+2)) || wasalife) && errno!=EAGAIN)
      { s_close(fd);
        break;
      }	
    }
    else
      wasalife=1;

    if(wasdagewesen)
    { putflush();
      wasdagewesen=0;
    }
    if(input==sh_ifmultitask || input==sh_forceinput)
    {
      while(getvalid() && fd!=EOF)
      { getline(s,198,1);
        if(s[0]!=4 && name[0]==0)
	      { sprintf(cl,"oshell \"%.70s\"",s);
	        cmdlog(cl);
	        lastcmd(cl);
	      }
        strcat(s,"\n");
        if(fd!=EOF)
        { if(write(fd,s,strlen(s))!=(signed int)strlen(s))
          { s_close(fd);
  	        fd=EOF;
      	  }
  	    }  
      }
    } 
    wdelay(133);
  }
  putflush();
  kill(-childpid,9);
  t->childpid=0;
  return 0;
}

/*---------------------------------------------------------------------------*/

void resetreadonly(char *name)
{ // dummy, only valid for DOS
}

/*---------------------------------------------------------------------------*/

unsigned long dfree(char *path,int total)
// returns available space in kbytes (max. 4 TBytes)

{
  struct statfs s;
  long kblock;
  statfs(path,&s);

  if(total)
    kblock=s.f_blocks/1024L;
  else
    kblock=s.f_bavail/1024L;  // f_bfree would be correct for "root"
 
  return s.f_bsize*kblock;
}

/*---------------------------------------------------------------------------*/

char *memfree(int swapfree)
{ FILE *f=s_fopen("/proc/meminfo","srt");
  char buf[100],b1[20],b2[20],b3[20],b4[20];
  char ram[20],buffer[20],cache[20]="0";
  static char memresult[20],swapresult[20];

  strcpy(swapresult,"-");
  strcpy(memresult,"-");

  if(f)
  { fgets(buf,100,f); // remove headerline
    fgets(buf,100,f); // read swap
    if(sscanf(buf,"%s %s %s %s %s %s %s",b1,b2,b3,ram,b4,buffer,cache)>=6)
      sprintf(memresult,"%lu",(atol(ram)+atoi(buffer)+atoi(cache))/1024);
    fgets(buf,100,f); // read memory
    if(sscanf(buf,"%s %s %s",b1,b2,b3)==3)
      sprintf(swapresult,"%lu",atol(b3)/1024);
    s_fclose(f);
  }
  if(swapfree)
    return swapresult;
  else
    return memresult;
}

/*---------------------------------------------------------------------------*/
/* Source: 
        Linux I/O port programming mini-HOWTO

        Author: Riku Saikkonen <Riku.Saikkonen@hut.fi>
        Last modified: Mar 30 1997
*/
   extern __inline__ unsigned long long int rdtsc()
   {
     unsigned long long int x;
     __asm__ volatile (".byte 0x0f, 0x31" : "=A" (x));
     return x;
   }

/*---------------------------------------------------------------------------*/

char *cpuinfo(void)
{ FILE *f=s_fopen("/proc/cpuinfo","srt");
  char buf[100],cpu[20]="??",bogo[20]="??";
  char vendor[30]="";
  static char cpuresult[200];
  char *pk=NULL;

  strcpy(cpuresult,"-");
  if(f)
  {
    while(fgets(buf,100,f))
    { if(stristr(buf,"cpu"))
      { pk=strstr(buf,": ");
        if(pk && cpu[0]=='?')
          sscanf(pk+2,"%s",cpu);
      }
      if(stristr(buf,"BogoMips"))
      { pk=strstr(buf,": ");
        if(pk)
          sscanf(pk+2,"%s",bogo);
      }
      if(stristr(buf,"vendor_id"))
      { pk=strstr(buf,": ");
        if(pk)
          sscanf(pk+2,"%s",vendor);
      }
    }
    s_fclose(f);
    if (vendor[0]) //only newer cpus show infos
    { if (cpu[0] >= '5') //if pentium
       sprintf(cpuresult,"CPU: %s(%s) %lld ticks BogoMips: %s",
        cpu,vendor,rdtsc(),bogo);
      else
        sprintf(cpuresult,"CPU: %s(%s) BogoMips: %s",cpu,vendor,bogo);
    }
    else
      sprintf(cpuresult,"CPU: %s  BogoMips: %s",cpu,bogo);
  }
  return cpuresult;
}

/*---------------------------------------------------------------------------*/

void testcputask(void)
{ //dummy
}

/*---------------------------------------------------------------------------*/

void task_idle(int active)
{ static long lasttic=0;
  if(active==0)
    pause();
  if(systic!=lasttic)
  { call_l2(active==2);
    lasttic=systic;
  }
}

/*---------------------------------------------------------------------------*/

void coldstart(void)
{
  stopreason="WATCHDOG";
  exit(1);
}

/*---------------------------------------------------------------------------*/

