/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -----------------
  Diverse Utilities
  -----------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980115 OE3DZW added ad_timezone()
//                added DOS-timezone instead of utcoffset

#include "baycom.h"


char     globalpath[80];
time_t   last_time=0;

/*---------------------------------------------------------------------------*/

#ifdef __DOS16__
void fixpath(char *argv0)
//*************************************************************************
//
//  Stellt den Pfad fest, aus dem das Programm gestartet worden ist
//  und legt ihn in globalpath ab
//
//*************************************************************************
{ int i=0;
  char psave[80];

  strcpy(psave,argv0);
  i=strlen(psave);
  while(i && (psave[i]!='/' && psave[i]!='\\')) i--;
  for(;i+1;i--)
    globalpath[i]=psave[i];
  killbackslash(globalpath);
}
#else
void fixpath(char *)
{ char buf[80];
  char *s=getenv("BCMHOME");
  if(s==NULL)
    s=getcwd(buf,80);
  strcpy(globalpath,s);  
  killbackslash(globalpath);
  trace(report,"working dir","'%s'",globalpath);  
  if(globalpath[0]==0 || globalpath[strlen(globalpath)-1]!='/')  
    strcat(globalpath,"/");
}
#endif

/*---------------------------------------------------------------------------*/

struct tm *ad_comtime(time_t time)
{
  return localtime(&time);
}

/*---------------------------------------------------------------------------*/

time_t ad_time(void)
//*************************************************************************
//
//  Gibt Zeit im ANSI-Format zurueck. 
//  Haeufige Aufrufe werden nur dann zum Systemcall time() 
//  durchgereicht, wenn die Sekunde tatsaechlich umgeschlagen hat.
//   
//  Hat sich die Zeit nach unten veraendert, so wird die Differenz
//  zur bisherigen Zeit gebildet und die tatsaechliche Zeit sukzessive
//  angenaehert.
// 
//*************************************************************************
{ 
  static long lasttic=0;
  static time_t time_correct=0;
  static time_t last_decrement=0;
  time_t newvalue;

  // mittels schnellen Ticker-Vergleich feststellen, ob sich die
  // Zeit veraendert hat.
  if(lasttic<(systic+(1000/MS_PER_TIC)-1) || lasttic>systic)
  {
    newvalue=time(NULL);
    // ist die Uhr zurueckgelaufen?
    if(newvalue<last_time)
    {
      // Differenz merken
      time_correct=last_time-newvalue;
      last_decrement=newvalue;
    }
    // wurde die Uhr vorgestellt aber es ist noch eine Differenz aktiv? 
    if(newvalue>(last_time+1))
    { if(time_correct > (newvalue-last_time)+1)
        time_correct-=(newvalue-last_time);
      else
        time_correct=0;
      last_decrement=newvalue;
    }
    // ist eine Differenz aktiv?
    if(time_correct>0  && newvalue > last_decrement+1)
    {
      // alle 2 Sekunden die Differenz um eine Sekunde verringern
      time_correct--;
      last_decrement=newvalue;
    }
    last_time=newvalue;
  }
  return last_time+time_correct;
}

/*---------------------------------------------------------------------------*/

time_t ad_mktime(struct tm *tt)
{
  return mktime(tt);
}

/*---------------------------------------------------------------------------*/

#ifdef __DOS16__
time_t isdst(void)   //returns daylight-saving in seconds
{
  #define DSTOFFSET (-3600)
  time_t ti=ad_time();
  signed long to=0;
  struct tm *tt;
  tt=ad_comtime(ti);
  if ((tt->tm_mon > 2) && (tt->tm_mon < 9))    // mon 0..11 mday 0..30
  { to=DSTOFFSET;                             // wday 0-6 so=0)
  }
  else
  { if ((tt->tm_mon==2) &&  // March
       ((tt->tm_mday > 23) && ( (31-tt->tm_mday) <= (7-tt->tm_wday) ) &&
       ((tt->tm_wday!=0) || (tt->tm_hour>=2))))
        to=DSTOFFSET;
    if ((tt->tm_mon==9) && //October
       ((tt->tm_mday < 22) || ( (30-tt->tm_mday) > (7-tt->tm_wday) ) ||
       ((tt->tm_wday==0) && (tt->tm_hour <=3))))
        to=DSTOFFSET;
  }
  return to;
}
#endif

/*---------------------------------------------------------------------------*/

time_t ad_timezone(void)
{
#ifdef __FLAT__
  time_t t=ad_time();
  struct tm * tt;
  tt=ad_comtime(t); //set timezone-variable
  return timezone-tt->tm_isdst*3600;
#else
  if (m.pcisutc)
    return 0;
  else
    return (isdst()+m.stimeoffset*3600);
#endif
}

/*---------------------------------------------------------------------------*/

unsigned ad_minutes(void)
{ time_t ti=ad_time();
  struct tm *tt;

  tt=ad_comtime(ti);
  return 60*tt->tm_hour+tt->tm_min;
}

/*---------------------------------------------------------------------------*/

#if defined(__LINUX__) || defined(_WIN32)
int  mausistda(void)                  { return 0; }
int  mausinit(int zeilen,int spalten) { return 0; }
void maussetz(int xpos,int ypos)      {           }
void mausein(void)                    {           }
void mausaus(void)                    {           }
int  mauszeile(void)                  { return 0; }
int  mausspalte(void)                 { return 0; }
int  mausknopf(void)                  { return 0; }
int  mausrechts(void)                 { return 0; }
#endif
