/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Layer1-Driver


  --------------------
  AX.25 Call Utilities
  --------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights reserved

 ***************************************************************/

#include "baycom.h"

#define cbuchst(c) ((c>=('A'*2))&&(c<=('Z'*2)))
#define czahl(c)   ((c>=('0'*2))&&(c<=('9'*2)))
#define cbz(x)     (cbuchst(x)||czahl(x))

/*---------------------------------------------------------------------------*/

int callcmp(byte *call1, byte *call2)
//*************************************************************************
//
//  vergleicht 2 Rufzeichen incl SSID - optimiert auf Tempo
//
//*************************************************************************
{ int i;
  for(i=5;i>=0;--i)
  { if(call1[i]!=call2[i])
      return 0;
  }
  return (!((call1[6]^call2[6])&0x1e));
}

/*---------------------------------------------------------------------------*/

int callwildcmp(byte *call1, byte *call2)
//*************************************************************************
//
//  vergleicht 2 Rufzeichen, Joker erlaubt
//  Joker duerfen nur im zweiten Rufzeichen enthalten sein
//
//*************************************************************************
{ int i;
  for(i=5;i>=0;--i)
  { if((call1[i]!=call2[i]) && call2[i]!=('?'<<1))
      return 0;
  }
  return ((((call1[6]^call2[6])&0x1e)==0)||((call2[6]&0x7e)==0x60));
}

/*---------------------------------------------------------------------------*/

void callasc(byte *rufz,char *buf)
//*************************************************************************
//
//  Rufzeichen im AX.25-Format in String umwandeln
//
//*************************************************************************
{ int i;
  for(i=0;i<6;i++)
    if(rufz[i]>0x40) *(buf++)=rufz[i]>>1;
  if(rufz[6]&0x1e || rufz[0]==0x40)
    sprintf((char *)buf,"-%d",(rufz[6]&0x1e)>>1);
  else
    buf[0]=0;
}

/*---------------------------------------------------------------------------*/

int callcmp1(byte *call1, byte *call2)
//*************************************************************************
//
//  vergleicht 2 Rufzeichen ohne SSID - optimiert auf Tempo
//
//*************************************************************************
{ int i;
  for(i=5;i>=0;--i)
  { if(call1[i]!=call2[i])
      return FALSE;
  }
  return TRUE;
}

/*---------------------------------------------------------------------------*/

int callord(byte *call1, byte *call2,int wildcard)
//*************************************************************************
//
//  vergleicht 2 Rufzeichen mit/ohne SSID    grer/kleiner
//
//*************************************************************************
{ int i;
  if(call1[0]==0) return HIGHER;  /* fr binres Suchen erforderlich */
  for(i=0;i<6;i++)
  { if(call1[i]!=call2[i])
    { if(call1[i]<call2[i])
        return LOWER;
      if(call1[i]>call2[i])
        return HIGHER;
    }
  }
  if(wildcard && ((call1[6]&0x7e)==0x60))
    return SAME;
  else if((call1[6]&0x1e)<(call2[6]&0x1e))
    return LOWER;
  else if((call1[6]&0x1e)>(call2[6]&0x1e))
    return HIGHER;
  else return SAME;
}

/*---------------------------------------------------------------------------*/

void callcopy(byte *call1,byte *call2)
//*************************************************************************
//
//  kopiert 1 Rufzeichen und lscht H-Bit und Endebit
//
//*************************************************************************
{ int i;
  for(i=0;i<7;i++)
  { call2[i]=call1[i];
  }
  call2[6]=call2[6]&0x7e;     /* etwaiges C- oder H-Bit loeschen */
}

/*---------------------------------------------------------------------------*/

int ischannel(byte *call)
//*************************************************************************
//
//  hier wird festgestellt, ob sich in einem Rufzeichen eine
//  gueltige Kanalnummer
//  der Art  A B C
//  oder     0 1 2 3  ... etc.
//  oder     P0 P1 P2 ... etc.
//  befindet. Wenn ja, wird die Kanalnummer zurckgegeben, wenn nicht
//  dann wird -1 zurckgegeben
//
//*************************************************************************
{ int index=(call[0]==('P'*2));              // P am anfang eliminieren
  int a;

  if(cbuchst(call[0])&&(call[1]==0x40))      // Einstelliger Buchstabe?
    return (call[0]>>1)-'A';                 // wert des Kanals
  if(!czahl(call[index]))                    // Zahl am Anfang?
    return -1;                               // nein
  a=(call[index]>>1)-'0';                    // wert des Kanals
  if(call[index+1]==0x40)
    return a;
  if(czahl(call[index+1])&&(call[index+2]==0x40))
    return a*10+((call[index+1]>>1)-'0');           // 2stelliger Wert
  else
    return -1;                                      // unpassend
}

/*---------------------------------------------------------------------------*/

#if (defined TERMCOMPILE) || (defined BOXCOMPILE)

int callok(byte *call)
{ int i,j=0;
  for(i=0;i<6;i++)
    if(czahl(call[i])) j++;
                        
  return(cbuchst(call[3]) &&
	 (cbuchst(call[4])||(call[4]==0x40)) &&
	 (cbuchst(call[5])||(call[5]==0x40)) &&
	 cbz(call[0]) &&
	 cbz(call[1]) &&
	 cbz(call[2]) &&
	 (j>0) && (j<3));                     
}


void onlycall(byte *rufz,char *buf)
//*************************************************************************
//
//  Rufzeichen im AX.25-Format in String umwandeln ohne SSID
//
//*************************************************************************
{ int i;
  for(i=0;i<6;i++)
    if(rufz[i] && rufz[i]!=0x40) *(buf++)=(rufz[i])/2;
  *buf=0;
}

void putcall(byte *rufz,int stern)
//*************************************************************************
//
//  Rufzeichen im AX.25-Format auf dem Bildschirm ausgeben
//           stern==0 nur Rufzeichen
//           stern==1 mit H-Bit-Anzeige
//           stern==2 mit Tabulator
//
//*************************************************************************
{
  int i;
  int screen=0;

  for(i=0;i<6;i++)
  { if(rufz[i] && rufz[i]!=0x40)
    { putv((rufz[i])/2);
      screen++;
    }
  }
  if((rufz[6]&0x60)==0)
  { putf("-0");
    screen+=2;
  }
  if(rufz[6]&0x1e || rufz[0]==0x40)
  { putf("-%d",(rufz[6]&0x1e)/2);
    screen+=2;
    if((rufz[6]&0x1e)>=20)
      screen++;
  }
  if((stern==1) && rufz[6]&0x80)
    putv('*');
  if(stern==2)
  { while((screen++)<9)
      putv(' ');
  }
}

void putclist(byte *af)
{ int i=0;
  do
  { putcall(af+i,0);
    putv(' ');
    i+=7;
  }
  while(!(af[i-1]&1) && i<63);
}
#endif
#if (defined __FLAT__) || (defined TERMCOMPILE) || (defined BOXCOMPILE)

int readcall(char *bf,byte *axcall)
{ int k,i,j;

  j=i=blkill(bf);
  if((toupper(bf[i])=='V'))
  { i++;
    if(toupper(bf[i])=='I')
    { i++;
      if(toupper(bf[i])=='A')
        i++;
    }
  }
  if(bf[i]!=' ') i=j;

  j=0;
  i+=blkill(bf+i);
  if(bf[i]==0) return 0;

  for(k=0;k<6;k++)axcall[k]=0x40;             /* Call vorbesetzen */
  axcall[6]=0x60;
  while((bf[i]!='-')&&(bf[i]!=',')&&(bf[i]>' ')&&(j<6))
  { axcall[j]=toupper(bf[i])*2;
    i++;
    j++;
  }
  if(bf[i]=='-')
  { i++;
    if(bf[i]=='0') axcall[6]=0;
    else if(0x62==(axcall[6]=bf[i++]*2))
    { if(bf[i]<'6' && bf[i]>='0')
        axcall[6]+=bf[i++]*2-0x4e;
    }
  }
  while(bf[i]&&(bf[i]!=' ')&&(bf[i]!=','))i++;
  return i+blkill(bf+i);
}


int readclist(char *bf,byte *axlist,int maxanzahl)
{ int x=0,z=0,i=0;

  while(maxanzahl-- && ((z=readcall(bf+x,axlist+i))>0))
  { x+=z;
    i+=7;
  }
  if(i) axlist[i-1]|=1;
  return x;
}

#endif
