/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -----------------
  Diverse Utilities
  -----------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980118 OE3DZW only 2 digits for year in date (02 instead of 102 for 2002)
//                seconds in timestr() fn, new parameter sec
//19980505 OE3DZW added rmcrlf
//19990110 OE3DZW y2k: parse_time,zeitstring
//19990207 OE3DZW added new datestr-type
//19990615 DH3MB  Moved getmd2sum, getmd5sum, mdsum2str to mbutil.cpp

#include "baycom.h"

/*---------------------------------------------------------------------------*/

unsigned strcrc(char *s)
//*************************************************************************
//
//  Berechnet ber den gegebenen String eine CRC-Prfsumme
//  Wird fr Hashing verwendet
//
//*************************************************************************
{ short int word=0;
  int b;
  byte byt;

  while((byt = *(s++))!=0)
  { for(b=0;b<8;b++)
    { if(((word<0)+(byt<128))==1)
	word<<=1;
      else
	word=(word<<1)^0xa097;
      byt<<=1;
    }
  }
  return (short unsigned)word;
}

/*---------------------------------------------------------------------------*/

char *stristr(char *s1,char *s2)
//*************************************************************************
//
//  Sucht nach einem Teilstring in einem String.
//  Die Funktion ist analog zur Bibliotheksfunktion strstr,
//  unterscheidet aber nicht zwischen Gro- und Kleinschreibung
//
//*************************************************************************
{ int i;
  int l=strlen(s2);

  while(s1[0])
  { i=0;
    while((((s1[i]^s2[i])&0x5f)==0) && s2[i])
      i++;
    if(i==l)
      return s1;
    s1++;
  }
  return NULL;
}

/*---------------------------------------------------------------------------*/

//*************************************************************************
//
//  strpos(Substring,String)
//
//  Gibt die Position von Substring in String zurck
//  Wird Substring nicht gefunden, oder ist einer der beiden Strings leer,
//  so wird -1 zurckgegeben.
//
//*************************************************************************
int strpos(char *s1,char *s2)
{ int i;
  int l=strlen(s2);
  int pos=0;

  while(s1[0])
  { i=0;
    while((((s1[i]^s2[i])&0x5f)==0) && s2[i])
      i++;
    if(i==l)
      return pos;
    s1++;
    pos++;
  }
  return (-1);
}

/*---------------------------------------------------------------------------*/

char *nexttoken(char *buf,char *token,int maxlen)
//*************************************************************************
//
//  Extrahiert den nchsten alphanumerischen Teilstring aus einem
//  String. Dabei wird entweder blank/tab/@ als Ende betrachtet, oder
//  der String kann mit "Anfhrungszeichen" eingeklammert sein
//
//*************************************************************************
{ int i=0;

  while((*buf)==' ')   buf++;
  if(buf[0]=='"')
  { buf++;
    while(((*buf)!='"') && ((*buf)>=' ') && (i<maxlen))
      token[i++]=*(buf++);
    while(*buf && (*buf)!='"') buf++;
    if((*buf)=='"') buf++;
  }
  else
  { while(((*buf)!='@') && ((*buf)> ' ') && (i<maxlen))
      token[i++]=*(buf++);
    while(*buf && ((*buf)!='@') && ((*buf)!=' ')) buf++;
  }
  while((*buf)==' ')   buf++;
  token[i]=0;
  return buf;
}

/*---------------------------------------------------------------------------*/

char *get_searchitem(char *instr,char *str,int maxlen)
//*************************************************************************
//
//  Extrahiert in einer Befehlszeile einen Suchbegriff in
//  "Anfhrungszeichen". Dieser wird herauskopiert und gegen Blanks
//  ersetzt. Blanks am Zeilenende werden entfernt.
//
//*************************************************************************
{ char *qm=strchr(instr,'"');

  if(qm)
  { char *end=strchr(qm+1,'"');
    if(end)
    { while(qm!=end)
      { qm[0]=' ';
        qm++;
        if(maxlen && qm!=end)
        { str[0]=toupper(qm[0]);
          str++;
          maxlen--;
        }
      }
      str[0]=0;
      end[0]=' ';
      while(instr[0] && instr[strlen(instr)-1]==' ')
        instr[strlen(instr)-1]=0;
      return str;
    }
  }
  return NULL;
}

/*---------------------------------------------------------------------------*/

char *datestr(time_t time,int year)
//*************************************************************************
//
//  Legt das Datum (UNIX-Format) in einem String ab
//
//  0 .. without year
//  1 .. with 2 digit year (for compatiblity)
//  2 .. 4 digit-year (where possible)
//  3 .. 4 digit-year, compressed format, eg 20011003 for Oct 3rd, 2001
//  4 .. 2 digit-year, compressed format used for WP-mails
//
//*************************************************************************
{ static char datum[12];
  struct tm *tt=ad_comtime(time);
  if(tt)
  { int year2=tt->tm_year;
    if (year2 > 99)
       year2-=100;

    if (year==1) 
      sprintf(datum,"%02d.%02d.%02d",tt->tm_mday,tt->tm_mon+1,year2);
    else if (year==2)
      sprintf(datum,"%02d.%02d.%04d",tt->tm_mday,tt->tm_mon+1,tt->tm_year+1900);
    else if (year==3)
      sprintf(datum,"%04d%02d%02d",tt->tm_year+1900,tt->tm_mon+1,tt->tm_mday);
    else if (year==4)
      sprintf(datum,"%02d%02d%02d",year2,tt->tm_mon+1,tt->tm_mday);
    else
      sprintf(datum,"%02d.%02d",tt->tm_mday,tt->tm_mon+1);
  }
  else
    datum[0]=0;
  //strftime(datum,sizeof(datum),year?"%d.%m.%y":"%d.%m",localtime(&time));
  return datum;
}

/*---------------------------------------------------------------------------*/

char *timestr(time_t time,int sec)
//*************************************************************************
//
//  Legt die Uhrzeit (UNIX-Format) in einem String ab
//
//*************************************************************************
{ static char zeit[9];
  struct tm *tt=ad_comtime(time);
  if(tt)
  { if (sec)
      sprintf(zeit,"%02d:%02d:%02d",tt->tm_hour,tt->tm_min,tt->tm_sec);
    else
      sprintf(zeit,"%02d:%02d",tt->tm_hour,tt->tm_min);
    //strftime(zeit,sizeof(zeit),"%H:%M",localtime(&time));
  }
  else
    zeit[0]=0;
  return zeit;
}

/*---------------------------------------------------------------------------*/

//*************************************************************************
//
//  Replaces a character by another in a string
//
//  DG4MMI: replaced strchr() by strrchr(), so replacing with a 0-byte
//          is now possible
//
//*************************************************************************
void subst(char *s,char i,char j)
{ char *p;

 while((p=strrchr(s,i))!=NULL)
    p[0]=j;
}

/*---------------------------------------------------------------------------*/

char *zeitstring(time_t zeit,int sec)
{ 
  // sec = 0 ... with 2digi year, without seconds
  // sec = 1 ... with 2digi year, with seconds (compatiblity)
  // sec = 2 ... with 4digi year, with seconds (prefered)

  static char zeitdatum[28];
  struct tm *tt=ad_comtime(zeit);

  if(tt)
  { if (sec == 2)
      sprintf(zeitdatum,"%02d.%02d.%04d %02d:%02d:%02d",
           tt->tm_mday,tt->tm_mon+1,tt->tm_year+1900,
           tt->tm_hour,tt->tm_min,tt->tm_sec);
        else
    {
      sprintf(zeitdatum,"%02d.%02d.%02d %02d:%02d",
           tt->tm_mday,tt->tm_mon+1,tt->tm_year%100,tt->tm_hour,tt->tm_min);
      if(sec)
        sprintf(zeitdatum+strlen(zeitdatum),":%02d",tt->tm_sec);    
    }    
  }
  else
    zeitdatum[0]=0;

  return zeitdatum;
}

/*---------------------------------------------------------------------------*/

#ifdef __FLAT__

char *rfc_date(time_t t)
{ static char buffer[64];
  static char *wd[]={"Sun","Mon","Tue","Wed","Thu","Fri","Sat"};
  static char *mn[]={"Jan","Feb","Mar","Apr","May","Jun",
                     "Jul","Aug","Sep","Oct","Nov","Dec"};

  // Mon, 7 Jul 1997 20:13:48 +0200 (MET DST)
  struct tm* lt=localtime(&t);

#if defined __UNIX__ || defined _WIN32
  sprintf(buffer,"%s, %d %s %d %02d:%02d:%02d",
                 wd[lt->tm_wday],lt->tm_mday,mn[lt->tm_mon],lt->tm_year+1900,
                 lt->tm_hour,lt->tm_min,lt->tm_sec);
#endif

  return buffer;
}

#endif

/*---------------------------------------------------------------------------*/

#ifdef __FLAT__

char *full_date(time_t t)
{ static char buffer[64];
  static char *wd[]={"Mon","Tue","Wed","Thu","Fri","Sat","Sun"};
  static char *mn[]={"Jan","Feb","Mar","Apr","May","Jun",
                     "Jul","Aug","Sep","Oct","Nov","Dec"};

  // Mon, 7 Jul 1997 20:13:48 +0200 (MET DST)
  struct tm* lt=localtime(&t);

  sprintf(buffer,"%s, %d %s %d %02d:%02d:%02d %+03ld%02d (%s)",
      wd[lt->tm_wday],lt->tm_mday,mn[lt->tm_mon],lt->tm_year+1900,
                 lt->tm_hour,lt->tm_min,lt->tm_sec,timezone/3600,
                 abs(timezone%60),tzname[0]);

  return buffer;
}

#endif

/*---------------------------------------------------------------------------*/

int newdate(time_t d1,time_t d2)
//*************************************************************************
//
//  Vergleicht das Datum 2er Tage im UNIX-Format
//
//*************************************************************************
{ char datum[12];

  if(d1 && d2)
  { strcpy(datum,datestr(d1,1));
    return strcmp(datum,datestr(d2,1));
  }
  else
    return 1;
}

/*---------------------------------------------------------------------------*/

char *zeitspanne(time_t zeit,delta_t mode)
//*************************************************************************
//
//  Gibt einen Zeitunterschied als String zurueck.
//
//  Werte fuer mode:
//  zs_runtime  : Zeit (in sec) wird als [dd.].hh.mm  ausgegeben
//  zs_seconds  : Zeit (in sec) wird als [t]s [t]m [t]h [t]d ausgegeben
//  zs_cputime  : Zeit (in 55msec) wird wie <seconds> ausgegeben
//  zs_cpuexact : Zeit (in 55msec) wird dezimal sss.ms ausgegeben
//
//*************************************************************************
{ unsigned long hours=zeit/3600;
  unsigned long days=hours/24;
  static char s[16];

  s[0]=0;

  if(mode>zs_runtime)
  { if(mode>=zs_cputime)
    { zeit*=MS_PER_TIC;        // from now on take milliseconds

      if(mode==zs_cpuexact)
      { sprintf(s,"%u.%02u",(unsigned)(zeit/1000),((unsigned)(zeit%1000)/10));
        return s;
      }
      if(zeit<10000)
      {	sprintf(s,"%u.%us",(unsigned)(zeit/1000),((unsigned)(zeit%1000))/100);
	return s;
      }
      zeit/=1000UL;                     // too much for decimal display
    }
    if(zeit<240L)                        // below 4min show seconds
      sprintf(s,"%lds",zeit);
    else if(zeit<(4*60L*60))            // below 4h show minutes
      sprintf(s,"%ldm",zeit/60);
    else if(zeit<(4*24L*60L*60))
      sprintf(s,"%ldh",zeit/(60*60));   // below 4days in show hours
    else if(zeit<(365*24L*60L*60))
      sprintf(s,"%ldd",zeit/(60L*60*24));  //from year on show in years
    else if(zeit<(10*365*24L*60L*60))
      sprintf(s,"%ldy",zeit/(60L*60*24*365));
  }
  else
  { if(days>999UL)
      sprintf(s,"-:--");
    else
    { zeit-=hours*3600UL;
      hours-=days*24UL;

      if(days)
	sprintf(s,"%lu.",days);
      sprintf(s+strlen(s),"%02lu:%02lu",hours,zeit/60UL);
    }
  }
  return s;
}

/*---------------------------------------------------------------------------*/

time_t parse_time(char *datum)
//*************************************************************************
//
//  Wandelt einen String, der ein Datum enthaelt in eine ANSI-Zeit um.
//  Dabei werden unbekannte Teile mit dem aktuellen Datum ersetzt.
//
//*************************************************************************
{ struct tm tt;
  char *olddatum=datum;
  time_t ti=ad_time();

  memcpy(&tt,ad_comtime(ti),sizeof(tt));
  tt.tm_hour=0;
  tt.tm_min=0;
  tt.tm_sec=0;

  if(datum[0])
  { tt.tm_mday=atoi(datum);
    while(isdigit(*datum)) datum++;
    while(*datum && !isdigit(*datum)) datum++;
    if(datum[0])
    { tt.tm_mon=atoi(datum)-1;
      while(isdigit(*datum)) datum++;
      while(*datum && !isdigit(*datum)) datum++;
      if(datum[0])
      { tt.tm_year=atoi(datum);
	if(tt.tm_year<90)  
	  tt.tm_year+=100;
        if(tt.tm_year>1900) //y2k, accept 4 digit date
          tt.tm_year-=1900;
	while(isdigit(*datum)) datum++;
      }
    }
    if((tt.tm_mday<1)||(tt.tm_mday>31)||(tt.tm_mon<0)||(tt.tm_mon>11))
    { putf("Incorrect date.\n");
      return 0L;
    }
  }
  while((olddatum!=datum) && olddatum[0])
  { olddatum[0]=' ';
    olddatum++;
  }
  return ad_mktime(&tt);
}

/*---------------------------------------------------------------------------*/

int filecopy(char *src,char *dest)
//*************************************************************************
//
//  Kopiert ein File
//
//  Returnwert  0 ok
//             -1 Fehler
//
//*************************************************************************
{ handle inf,outf;
  char *buf;
  unsigned z;
  int retwert=0;

  inf=s_open(src,"srb");
  if(inf!=EOF)
  { outf=s_open(dest,"swb");
    if(outf!=EOF)
    { buf=(char *)t_malloc(8192,"copy");
      do
      { z=_read(inf,buf,8192);
	if(z>0)
	  _write(outf,buf,z);
	waitfor(e_ticsfull);
      } while(z==8192);
      t_free(buf);
      s_close(outf);
    }
    else
      retwert=(-1);
    s_close(inf);
  }
  else
    retwert=(-1);
  return retwert;
}

/*---------------------------------------------------------------------------*/

int xunlink(char *name)
//*************************************************************************
//
//  Loescht ein File unter Beruecksichtigung der Mehrfachzugriffe
//  Ggf. wird gewartet bis das File frei ist.
//
//*************************************************************************
{ while(sema_access(name))
    wdelay(107);
  resetreadonly(name);
  return unlink(name);
}

/*---------------------------------------------------------------------------*/

int xrename(char *src,char *dest)
//*************************************************************************
//
//  Verschiebt ein File unter Beruecksichtigung der Mehrfachzugriffe
//  Ggf. wird gewartet bis das File frei ist.
//  Wenn Verschieben nicht moeglich ist wird kopiert und danach geloescht.
//
//*************************************************************************
{ while(sema_access(src))
    wdelay(124);
  while(sema_access(dest))
    wdelay(125);
  resetreadonly(src);
  if(rename(src,dest)!=0)
  { filecopy(src,dest);
    return unlink(src);
  }
  return 0;
}

/*---------------------------------------------------------------------------*/

long filesize(char *name)
//*************************************************************************
//
//  Gibt die Groesse eines Files zurueck
//
//*************************************************************************
{ struct stat st;

  if(stat(name,&st)==0)
    return st.st_size;
  else
    return 0L;
}

/*---------------------------------------------------------------------------*/

void rmcrlf(char *line)
//*************************************************************************
//
//  Removes cr or linefeed from a line
//
//*************************************************************************
{
  char *a;

  if ((a=strchr(line,'\n'))!=0)
    *a=0;

  if ((a=strchr(line,'\r'))!=0)
    *a=0;

}

/*---------------------------------------------------------------------------*/

int isdir(char *name)
//*************************************************************************
//
//  Stellt fest, ob der uebergebene Name ein Verzeichnis ist
//
//*************************************************************************
{ struct stat st;
  if(strcmp(name+strlen(name)-2,"..")==0) // workaround WIN32 Bug
    return 1;
  if(stat(name,&st)==0)
    return (st.st_mode&S_IFDIR)!=0;
  else
    return 0L;
}

/*---------------------------------------------------------------------------*/

time_t filetime(char *name)
//*************************************************************************
//
//  Stellt die Zeit der letzten Modifikation eines Files fest
//  (wird auch zum Feststellen der Existenz eines Files verwendet)
//
//*************************************************************************
{
  struct stat st;

  if(stat(name,&st)==0)
    return st.st_mtime;
  else
    return 0L;
}

/*---------------------------------------------------------------------------*/

char *defext(char *name,char *ext)
//*************************************************************************
//
//  Fuegt eine default extension zu einem Filename, sofern eine solche
//  noch nicht existiert.
//
//*************************************************************************
{ static char nam2[FNAMELEN-1];

  strlwr(name);
  strlwr(ext);
  if(strchr(name,'.'))
    return name;
  strcpy(nam2,name);
  strcat(nam2,".");
  strcat(nam2,ext);
  return nam2;
}

/*---------------------------------------------------------------------------*/

void killbackslash(char *s)
//*************************************************************************
//
//  Macht aus allen Backslashes einen Slash /
//  (Wird verwendet zur Umwandlung von DOS-Filepfaden nach UNIX)
//
//*************************************************************************
{ while(*s)
  { if(s[0]=='\\') s[0]='/';
    s++;
  }
}

/*---------------------------------------------------------------------------*/

char jokcmp(char *string, char *jokex, char negotiation)
//*************************************************************************
//
//  Eine einfache Match-Routine (NICHT regex!), die die Verwendung der
//  Jokerzeichen '?' und '*' (auch mitten im String) erlaubt.
//  Stimmt "negotiation" mit dem ersten Zeichen von "jokex" ueberein,
//  so wird der Ausdruck negiert.
//  (DG4MMI)
//
//*************************************************************************
{ unsigned char i1=0, i2=0, neg=0;

  if(negotiation==jokex[0])
  { neg=1;
    jokex++;
  }  

  for(;i2<strlen(jokex);i2++)
  {
    switch(jokex[i2])
    {
    case '?':
      if(i1==strlen(string)) // '?' stated, but no char left in string
        return 0+neg;
      i1++;
      break;

    case '*':
      if(i2==(strlen(jokex)-1)) // '*' at right bound of jokex
        return 1-neg;
      while(jokex[i2]=='*') i2++;
      while((string[i1]!=jokex[i2]) && (jokex[i2]!='?'))
      { i1++;
        if(i1==strlen(string))
          return 0+neg;
      }
      i1++;
      break;

    default:
      if(string[i1]!=jokex[i2])
        return 0+neg;
      i1++;
      break;
    }
  }

  if(i1==strlen(string))
    return 1-neg;
  else
    return 0+neg;
}

/*---------------------------------------------------------------------------*/

#ifdef L2COMPILE //these functions are never used .. obsolete ?

static void filepath(char *name,char *filename)
{ if((name[0]=='\\') || (name[0]=='/') || (name[1]==':'))
    strcpy(filename,name);
  else
  { strcpy(filename,globalpath);
    strcat(filename,name);
  }
}

static FILE *sysopen(char *name,char *mode)
{ char filename[40];
  filepath(name,filename);
  return fopen(filename,mode);
}

static char *syspath(char *name)
{ static char filename[60];
  filepath(name,filename);
  return filename;
}

#endif
