/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -------------------------------------
  Umwandlung von DOS-Version nach Linux
  -------------------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

#define BOXCOMPILE
#include "baycom.h"

#undef fopen
#undef fclose
#define ADJNUM 5
#define HEADERLEN 60

/*---------------------------------------------------------------------------*/

typedef struct hadr_t
{ char   adr[HADRESSLEN+1];        // 40
  char   adjacent  [ADJNUM][7];    // 35
  short int    rel_mails [ADJNUM]; // 10
  time_t delay     [ADJNUM];       // 20
  time_t lastupdate[ADJNUM];       // 20
  short int    hops      [ADJNUM]; // 10
  long   bulletins;                // 4
  long   usermails;                // 4
  time_t lasthtime;                // 4
  char   lastheader[HEADERLEN+1];  // 61
  char   lastbid[BIDLEN+1];        // 13
  char   lastboard[9];             // 9
  char   lastuser[7];              // 7
  char   reserved[17];             // 51
  short unsigned nextsamehash;     // 2
} hadr_t;

/*---------------------------------------------------------------------------*/

typedef struct l_hadr_t
{
  long   bulletins;                // 4
  long   usermails;                // 4
  time_t delay     [ADJNUM];       // 20
  time_t lastupdate[ADJNUM];       // 20
  time_t lasthtime;                // 4
  short int    rel_mails [ADJNUM]; // 10
  short int    hops      [ADJNUM]; // 10
  short unsigned nextsamehash;     // 2
  char   adr[HADRESSLEN+1];        // 40
  char   adjacent  [ADJNUM][7];    // 35
  char   lastheader[HEADERLEN+1];  // 61
  char   lastbid[BIDLEN+1];        // 13
  char   lastboard[9];             // 9
  char   lastuser[7];              // 7
  char   reserved[17];             // 51
} l_hadr_t;


/*---------------------------------------------------------------------------*/

void main(void)
{
  FILE *fi;
  FILE *fo;

  hadr_t         hd;
  l_hadr_t       hl;
  user_t         ud;
  user_linux_t   ul;
  int i;
  long pos=0;

  fi=fopen(HADRNAME,"rb");
  fo=fopen(HADRNAME_L,"wb");
  if(!fi)
  { printf("bcm2linu: can't open %s\n",HADRNAME);
    exit(1);
  }
  if(!fo)
  { printf("bcm2linu: can't open %s\n",HADRNAME_L);
    exit(1);
  }
  setvbuf(fi,NULL,_IOFBF,16384);
  setvbuf(fo,NULL,_IOFBF,16384);
  printf("bcm2linu: processing h-addresses.\n");

  while(fread(&hd,sizeof(hadr_t),1,fi)==1)
  { memset(&hl,0,sizeof(hadr_t));
    hl.bulletins=hd.bulletins;
    hl.usermails=hd.usermails;
    hl.lasthtime=hd.lasthtime;
    for(i=0;i<ADJNUM;i++)
    { hl.rel_mails[i]=hd.rel_mails[i];
      hl.hops[i]=hd.hops[i];
      hl.delay[i]=hd.delay[i];
      hl.lastupdate[i]=hd.lastupdate[i];
      strcpy(hl.adjacent[i],hd.adjacent[i]);
    }
    strcpy(hl.adr,hd.adr);
    strcpy(hl.lastheader,hd.lastheader);
    strcpy(hl.lastuser,hd.lastuser);
    strcpy(hl.lastbid,hd.lastbid);
    strcpy(hl.lastboard,hd.lastboard);
    hl.nextsamehash=hd.nextsamehash;
    fwrite(&hl,sizeof(l_hadr_t),1,fo);
    if((pos%31)==0)
      printf("bcm2linu: hadr %ld\r",pos);
    pos++;
  }
  fclose(fi);
  fclose(fo);

  pos=0;

  fi=fopen(USERNAME,"rb");
  fo=fopen(USERNAME_L,"wb");
  if(!fi)
  { printf("bcm2linu: can't open %s\n",USERNAME);
    exit(1);
  }
  if(!fo)
  { printf("bcm2linu: can't open %s\n",USERNAME_L);
    exit(1);
  }
  setvbuf(fi,NULL,_IOFBF,16384);
  setvbuf(fo,NULL,_IOFBF,16384);
  printf("bcm2linu: processing userdata.\n");

  while(fread(&ud,sizeof(user_t),1,fi)==1)
  { memset(&ul,0,sizeof(user_linux_t));

    ul.lastboxlogin=ud.lastboxlogin;
    ul.lastdirnews=ud.lastdirnews;
    ul.lastquit=ud.lastquit;
    ul.mybbstime=ud.mybbstime;
    ul.lastload=ud.lastload;
    ul.daybytes=ud.daybytes;
    ul.mailsent=ud.mailsent;
    ul.mailgot=ud.mailgot;
    ul.mailread=ud.mailread;
    ul.logins=ud.logins;
    for(i=0;i<8;i++)
      ul.opt[i]=ud.opt[i];
    ul.helplevel=ud.helplevel;
    ul.zeilen=ud.zeilen;
    ul.lf=ud.lf;
    ul.status=ud.status;
    ul.filepos=ud.filepos;
    ul.nextsamehash=ud.nextsamehash;
    ul.fdelay=ud.fdelay;
    ul.fhold=ud.fhold;
    ul.readlock=ud.readlock;
    ul.pwline=ud.pwline;
    ul.echo=ud.echo;
    ul.rlimit=ud.rlimit;
    ul.nopurge=ud.nopurge;
    ul.nameok=ud.nameok;
    ul.mybbsok=ud.mybbsok;
    ul.umlaut=ud.umlaut;
    strcpy(ul.call,ud.call);
    strcpy(ul.name,ud.name);
    strcpy(ul.mybbs,ud.mybbs);
    strcpy(ul.prompt,ud.prompt);
    strcpy(ul.sprache,ud.sprache);
    strcpy(ul.password,ud.password);
    strcpy(ul.notvisible,ud.notvisible);
    strcpy(ul.firstcmd,ud.firstcmd);
    strcpy(ul.ttypw,ud.ttypw);
    strcpy(ul.uplink,ud.uplink);
    fwrite(&ul,sizeof(user_linux_t),1,fo);
    if((pos%31)==0)
      printf("bcm2linu: user %ld\r",pos);
    pos++;
  }
  fclose(fi);
  fclose(fo);
  printf("bcm2linu: ready.         \n");

}
