/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -------------------------------------
  AX.25-Funktionen zur Infoverarbeitung
  -------------------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

#include "baycom.h"

/*---------------------------------------------------------------------------*/

char *doszeit(unsigned minuten)
{ static char zeitstr[10];
  sprintf(zeitstr," %02u:%02u ",minuten/60,minuten%60);
  return zeitstr;
}

/*---------------------------------------------------------------------------*/

void iot(byte zeichen)
//*************************************************************************
//
//  Ein Zeichen in den Empfangspuffer des aktuellen Ports ausgeben
//
//*************************************************************************
{ frame_t *td=p->rxinfout;
  if(!td || (td->len==MAXLEN))
  {
    disable();
    if(bfree && bfree->next)
    { alloc(td);
      enable();
      td->next=p->rxinfout;
      p->rxinfout=td;
      td->len=1;    // jump over PID byte
      td->index=0;
      td->channel=0;
    }
    else
    { enable();
    }
  }
  td->inhalt[(td->len)++]=zeichen;
}

/*---------------------------------------------------------------------------*/

void iot_frame(frame_t *info)
{
  info->next=p->rxinfout;
  p->rxinfout=info;

}

/*---------------------------------------------------------------------------*/

void iotf(char *format, ...)
//*************************************************************************
//
//  String in Empfangspuffer ausgeben
//
//*************************************************************************
{ va_list argpoint;
  char cbuf[60];
  char *cb=cbuf;

  va_start(argpoint,format);
  vsprintf(cbuf,format,argpoint);
  va_end(argpoint);
  while(*cb)
    iot(*(cb++));
}

/*---------------------------------------------------------------------------*/

void iotcall(byte *rufz,int stern)
//*************************************************************************
//
//  Rufzeichen im AX.25-Format auf dem Bildschirm ausgeben
//
//*************************************************************************
{ char asc[10];

  callasc(rufz,asc);
  iotf("%s",asc);
  if(stern && rufz[6]&0x80)
    iot('*');
}

/*---------------------------------------------------------------------------*/

void monitor(frame_t *packl)
//*************************************************************************
//
//  Ein Frame im Monitor ausgeben
//
//*************************************************************************
{ byte command=0,response=0,poll=0,ctrl;
  byte *desti;
  int  plen=packl->len;
  byte *inh=packl->inhalt;
  int index=0;
  int pid,i=0;
  int damafl=0;
  port_t *psave=p;
  p=pfeld[0];

  while((inh[i]&1)==0)i++;
  if(i==1)
  {
    iot(0xfd);
    iot(packl->channel&0x80?'T':'R');
    if(channels>1)
      iotf("%d",packl->channel&0x7f);
    if(packl->txdtime!=255)
      iotf(":%02d",(packl->txdtime*13)/10);
    iotf(doszeit(biosminuten));

    iotf("%d>",(inh[0]<<6)+(inh[1]>>2));
    iot((inh[2]>>2) + 0x20);
    iot((((inh[2]&0x03)<<4) | ((inh[3]&0xf0)>>4)) + 0x20);
    iot((((inh[3]&0x0f)<<2) | ((inh[4]&0xc0)>>4)) + 0x20);
    iot((inh[4]&0x3f) + 0x20);
    iot((inh[5]>>2) + 0x20);
    iot((((inh[5]&0x03)<<4) | ((inh[6]&0xf0)>>4)) + 0x20);
    if(inh[6]&15)iotf("-%d",inh[6]&15);
    index=7;
    command=!!(inh[1]&2);
    response=!command;
  }
  else
  {
    if((i%7)!=6 || i>69) goto mend;
    damafl=(inh[13]&0x20)==0;

    mhput(packl);
    if(chainlen(p->rxinfout)>w.maxmon) goto mend;
    if(!((w.val.monitor>>(packl->channel&127))&1)) goto mend;

    if(w.val.mselect)
    { int jawoll=0;
      i=0;
      do
      { switch(w.val.mselect%10)
        { case 1:
	    if(((callwildcmp(inh+7,w.val.mcalls+i)    &&
                 callwildcmp(inh  ,w.val.mcalls+i+7)) ||
                (callwildcmp(inh+7,w.val.mcalls+i+7)  &&
                 callwildcmp(inh  ,w.val.mcalls+i) )) && !(i%14)) jawoll++;
            break;
          case 2:
            if(callwildcmp(inh,w.val.mcalls+i)) jawoll++;
            break;
          case 3:
	    if(callwildcmp(inh+7,w.val.mcalls+i)) jawoll++;
            break;
        }
        i+=7;
      }
      while((w.val.mcalls[i-1]&1)==0);
      if(w.val.mselect>=10) jawoll=!jawoll;
      if(!jawoll) goto mend;
    }

    command=inh[6]&0x80;
    response=inh[13]&0x80;
    desti=inh;
    index=index+7;
    iot(0xfd);
    iot(packl->channel&0x80?'T':'R');
    if(channels>1)
      iotf("%d",packl->channel&0x7f);
    if(packl->txdtime!=255)
      iotf(":%02d",(packl->txdtime*13)/10);
    iotf(doszeit(biosminuten));
    iotcall(inh+7,0);	                	 // Sourcecall ausgeben

    while(!(inh[6+index]&0x01))                         // Digicalls ausgeben
    { index=index+7;
      iot('/');
      iotcall(inh+index,1);
    }
    iot('>');                                // Destinationcall ausgeben
    iotcall(desti,0);
    index=index+7;
  }
  iot('>');
  poll=inh[index] & 0x10;
  ctrl=inh[index++] & 0xef;
  if(!(ctrl&1))
   iot('I'),iot(((ctrl>>1)&7)|'0'),iot(((ctrl>>5)&7)|'0');
  else if(ctrl==SABM) iotf("SABM");
  else if(ctrl==DISC) iotf("DISC");
  else if(ctrl==UA)   iotf("UA");
  else if(ctrl==UI)   iotf("UI");
  else if(ctrl==DM)   iotf("DM");
  else if(ctrl==FRMR) iotf("FRMR");
  else if((ctrl&3)==1)
  { if((ctrl&0x0f)==RR)        iotf("RR");
    else if ((ctrl&0x0f)==REJ) iotf("REJ");
    else if ((ctrl&0x0f)==RNR) iotf("RNR");
    iot((ctrl>>5)|'0');
  }
  else iotf("%02X",ctrl);  		   // Kontrollfeld ausgeben
  if(command ^ response)
  { iot(',');
    if(command && poll)   iot('P');
    if(command && !poll)  iot('C');
    if(response && poll)  iot('F');
    if(response && !poll) iot('R');
  }
  else if(poll) iotf(",S");
  if(damafl)   iotf(",DAMA");         // "DAMA"
  if(index<plen)
  { byte i;
    iotf(",%02X:\xfe",pid=inh[index++]);
    if(w.val.headeron)i=13;else i=32;

    if((pid==0xcf) && (ctrl!=UI))
    { if((index+19)<plen)
      { iot(i);
	iotcall(inh+index,0);
	iot('>');
	iotcall(inh+index+7,0);
	index+=14;
	for(i=0;i<6;i++)
	  iotf(" %02X",inh[index++]);
	iot(' ');
	i=13;
      }
    }
    else if((pid==0xcc)||(pid==0xcd))
    { index+=40;
      iotf("<TCP/IP>");
    }
    else if((plen-index)>40)
    { int binfl=0,j;
      for(j=index;j<(index+40);j++)
	if((inh[j]>'z')||(inh[j]<' '))binfl++;
      if(binfl>10)
      { iotf("%c<BIN %d Bytes>",i,plen-index);
	index=plen;
	i=13;
      }
    }
    while(index<plen)
    { if((i!=inh[index]) && (i<0xfd))
	iot(i);
      else if((i!=13) && (i<0xfd))
	iot(i);
      i=inh[index++];
    }
    if(i!=13) iot(i);
  }
  iot(0xff);
  mend:
  enable();
  p=psave;      // wieder auf alten Port zurckschalten
}

/*---------------------------------------------------------------------------*/

void linkmsg(linkmsg_t message)
{ int i=7;
  char *s;

  if(glob_messages)
  { frame_t *info;

    if(bfree)
    {
      alloc(info);
    }
    else
      return;

    info->len=0;
    info->index=0;
    info->channel=1;
    s=(char *)info->inhalt;

    if(message!=msg_disconnected)
      p->logintime=biosminuten;

    sprintf(s," %c%s",(int)message,doszeit(biosminuten));
    callasc(p->adrfield,s+strlen(s));

    while((p->adrfield[i+6]&1)==0)
    { i+=7;
      strcat(s," ");
      callasc(p->adrfield+i,s+strlen(s));
    }
    if(w.val.blink==2) w.val.blink=3;
    strcat(s,"\r");

    info->len=strlen(s);
    iot_frame(info);
  }
}
