/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  ------------------------------------
  AX.25 Data Link Layer main functions
  ------------------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19990215 OE3DZW added sample init.l2

#include "baycom.h"
#include "paramsg.h"

#undef fopen
#undef fclose

#undef L2INITNAME
#define L2INITNAME       "init.l2"


l2_para_t *wl=&w;
static unsigned linenumber=0;
static int assignkanal= -1;
static int validl1=0;
static int errors=0;
unsigned damatimer[KANAELE];   // wenn DAMA-Master erkannt wurde
byte     damafree[KANAELE];    // Es darf ein Schwung Frames gesendet werden
int  tfaellig;                 // Flag fuer Layer2-Timer zu bedienen
int  rekfl;
byte rxdaflag;

/*---------------------------------------------------------------------------*/

char *beftab[]=
    { 
      "CONNECT","MONITOR","MYCALL","LINKS","MODE",
      "MCALLS","CNOT","CNUSER","L2CTEXT",
      "DCALL", "BADRESS", "ASSIGN", "MHLEN",
      "TXDELAY","TAIL","DWAIT","BEACON",
      "MAXFRAME","XMITOK",
      "BTEXT","DEVICE","PEER","PORT","TXPORT",
      "BUFFER","MBUFFER","RBUFFER","TPORTS",
      "PACLEN","ITIME","RETRY","FRACK","RESPTIME",
      "LINKTIME","IPOLL","DISCTIME","MSELECT",
      NULL
    };

static enum befnum
    { unsinn,
      uc,moni,my,li,mode,
      mcalls,cnot,cnuser,l2ct,
      dc, ba, assign, xmhlen,
      txdelay,txtail,dwait,btime,
      maxframe,xmitok,
      _btext,device,peer,port,txport,
      dezwerte
    }cmd=unsinn;

static unsigned *vartab[55];

/*---------------------------------------------------------------------------*/

void l2init(void)
{ port_t *allocblk;
  int pnummer;
  allocblk=(port_t *)xalloc((ports+2)*sizeof(port_t));
  for(pnummer=0;pnummer<=ports;pnummer++)
  { p=pfeld[pnummer]=allocblk+pnummer;
    callcopy(w.val.mycall,p->adrfield+7);
    callcopy(w.unproto,p->adrfield);
    p->adrfield[13]|=1;
    p->pid=0xf0;
    p->lstate=disconnected;
    p->adrfield[13]|=1;
    p->number=pnummer;
    p->mode.l2ctext=1;
    lreset(0);
  }
  pfeld[MSGPORT]=allocblk+ports;
  dobeacon(TRUE);
}

/*---------------------------------------------------------------------------*/

void initax25(int einschalt)
{ 
  if(einschalt)
  { 
    ports=w.maxports;
    meminit();
    l2init();
  }
  l1init(einschalt);
}

/*---------------------------------------------------------------------------*/

void varset(void)
{ int x=0;
  vartab[x++]=&wl->maxbuf;
  vartab[x++]=&wl->maxmon;
  vartab[x++]=&wl->maxrx;
  vartab[x++]=&wl->maxports;
  vartab[x++]=&wl->val.paclen;
  vartab[x++]=&wl->val.itime;
  vartab[x++]=&wl->val.retry;
  vartab[x++]=&wl->val.frack;
  vartab[x++]=&wl->val.resptime;
  vartab[x++]=&wl->val.linktime;
  vartab[x++]=&wl->val.ipoll;
  vartab[x++]=&wl->val.disctime;
  vartab[x++]=&wl->val.mselect;
}

/*---------------------------------------------------------------------------*/

void beftest(char *befbuf)
{ int i;
  char *s;
  char *befbuf1=befbuf;
  befbuf+=blkill(befbuf);
  if(!(*befbuf)) return;     /* kein Kommando eingegeben */
  cmd=(befnum)readcmd(beftab,&befbuf,0);
  for(i=strlen(befbuf)-1;i;i--)
  { if(befbuf[i]==' ')
      befbuf[i]=0;
    else
      break;
  }
  switch(cmd)
  {
    case unsinn:
    { error:
      s=E_ERROR;
      printf(E_MSG,linenumber,s,befbuf1);
      errors++;
    } break;
    case moni:
    { long unsigned mm=0;
      while(*befbuf)
      { if((*befbuf>'9')||(*befbuf<'0')) goto error;
        mm|=1<<(nextdez(&befbuf));
        wl->val.monitor=mm;
      }
    } break;
    case dc:
    { if(readcall(befbuf,wl->val.dcall)<3) goto error;
    } break;
    case uc:
    { readcall(befbuf,wl->unproto);
    } break;
    case my:
    { readclist(befbuf,wl->val.mycall,4);
    } break;
    case mcalls:
    { readclist(befbuf,wl->val.mcalls,10);
    } break;
    case cnot:
    { readclist(befbuf,wl->val.cnot,10);
    } break;
    case l2ct:
    { sprintf(wl->val.l2ctext,"%.79s\r",befbuf);
    } break;
    case xmhlen:
    { int l=nextdez(&befbuf);
      int i=0;
      while(l)
      { i++;
        l/=2;
      }
      wl->mhlen=(1<<i)-1;
    } break;
    case assign:
    { static int firstassign=1;
      if(firstassign)
      { 
        firstassign=0;
      }
      if(assignkanal>=KANAELE)
      { printf(E_MAXCHAN);
        errors++;
      }
      else
      { strupr(befbuf);
        validl1=1;
        if(strcmp(befbuf,"KISS")==0)
        {
          wl->mode[++assignkanal].chmode=c_kiss;
        }
        else if(strcmp(befbuf,"AXIP")==0)
        {
          wl->mode[++assignkanal].chmode=c_axip;
        }
        else if(strcmp(befbuf,"LOOP")==0)
        { ++assignkanal;
          wl->internkanal=assignkanal;
        }
        else
          goto error;
        channels=assignkanal;  
      }
    } break;

    case _btext:
    { if(validl1)
      { wl->mode[assignkanal].btext[0]=0xf0;
        strcpy(wl->mode[assignkanal].btext,befbuf);
      }
    } break;

    case peer:
    case device:
      if(validl1) 
        strcpy(wl->mode[assignkanal].device,befbuf);
      break;
    
    case port:
      if(validl1)
      { wl->mode[assignkanal].port=atoi(befbuf);
        wl->mode[assignkanal].txport=atoi(befbuf);
      }
      break;
    
    case txport:
      if(validl1)
        wl->mode[assignkanal].txport=atoi(befbuf);
      break;
    
    case ba:
    { if(validl1)
        readclist(befbuf,wl->mode[assignkanal].badress,8);
    } break;
    
    case mode:
    { if(validl1)
      { l1_mode_t *m=&wl->mode[assignkanal];
        m->hbaud=nextdez(&befbuf);
        m->t=m->r=m->z=m->d=m->c=0;
        while(befbuf[0])
        { switch(toupper(befbuf[0]))
          { case 'T': m->t=1; break;
            case 'R': m->r=1; break;
            case 'Z': m->z=1; break;
            case 'D': m->d=1; break;
            case 'C': m->c=1; break;
          }
          befbuf++;
        }
      }
    } break;
    
    case li:
    { byte fromcall[7];
      byte tocall[7];
      int j;
      if(*befbuf)
      { befbuf+=readcall(befbuf,tocall);
        if(*befbuf)
        { for(j=0;j<2;j++)
          { for(i=0;i<MAXLINKS;i++)
            { if((callcmp(tocall,wl->linkcall[i])&& !j) || (!wl->linkcall[i][0]&&j))
              { befbuf+=readcall(befbuf,fromcall);
                if((j=ischannel(fromcall))<0)
                { for(j=0;j<MAXLINKS;j++)
                  { if(callcmp(fromcall,wl->linkcall[j]))
                    { wl->linkkanal[i]=j+128;
                      goto found;
                    }
                  }
                  goto error;
                }
                else
                  wl->linkkanal[i]=j;

                found:
                wl->linkoption[i]=*befbuf;
                callcopy(tocall,wl->linkcall[i]);
                goto endlinks;
              }
            }
          }
        }
        endlinks:;
      }
    } break;

    case txdelay:
    case txtail:
    case dwait:
    case btime:
    case maxframe:
    case xmitok:
      { if(validl1)
        { sscanf(befbuf,"%d",&i);
          if(assignkanal<0)
          { printf("ASSIGN statement missing\n");
            goto error;
          }
          switch(cmd)
          {
            case txdelay:   wl->mode[assignkanal].txdelay=i;  break;
            case txtail:    wl->mode[assignkanal].txtail=i;   break;
            case dwait:     wl->mode[assignkanal].dwait=i;    break;
            case btime:     wl->mode[assignkanal].btime=i;    break;
            case maxframe:  wl->mode[assignkanal].maxframe=i; break;
            case xmitok:    wl->mode[assignkanal].xmitok=i;   break;
          }
        }
      } break;
    default:
    { 
      if(cmd>=dezwerte)
      { if(befbuf[0]=='$')
          sscanf(befbuf+1,"%x",&i);
        else
          sscanf(befbuf,"%d",&i);
        *vartab[cmd-dezwerte]=i;
      }
    }
  }
}

/*---------------------------------------------------------------------------*/

void initparameter(void)
{ int i;
  // default l2 parameters
  memset(wl,0,sizeof(l2_para_t));
  wl->val.headeron=1;
  wl->val.mcalls[6]=1;
  wl->val.cnot[6]=1;
  wl->val.paclen=256;
  wl->val.itime=2;
  wl->val.ipoll=255;
  wl->val.dama=36;
  wl->val.retry=40;
  wl->val.frack=30;
  wl->val.resptime=1;
  wl->val.linktime=3;
  wl->val.qtime=30;
  wl->val.disctime=0;
  wl->val.monitor=0xffff;
  wl->val.blink=1;
  wl->val.mycall[6]=1;
  for(i=0;i<KANAELE;i++)
  { l1_mode_t *m=&wl->mode[i];
    m->txdelay=20;
    m->txtail=2;
    m->dwait=30;
    m->hbaud=9600;
    m->c=1;
    m->tchannel=i;
    m->maxframe=7;
    m->xmitok=1;
    m->chmode=c_unassigned;
    strcpy(m->device,"/dev/null");
  }
  wl->mhlen=MHLEN;
  wl->sccint[0]=7;
  wl->comkanal=wl->fskkanal=wl->internkanal=(-1);
  wl->maxports=201;
  wl->maxbuf=300;
  wl->maxmon=20;
  wl->maxrx=20;
  wl->ipxkanal=(-1);
  wl->unproto[6]=1;
  wl->kennung=0xaa55;
}

/*---------------------------------------------------------------------------*/
static char l2f_init[]=
"; Sample Paramaterfile for AX.25\n"
";\n"
"dcall    df0ar-8    ; Digicall\n"
"monitor  4\n"
"mselect  0\n"
"mcalls   -0\n"
";\n"
";assign   kiss\n"
";mode     19200cd\n" 
";device   /dev/ttyS0 ; under Win32: device COM2\n"
";\n"
";assign   axip\n"
";peer  digi.local.net\n"
";port     4719\n"
";\n"
"assign axip\n"
"peer localhost\n"
"port 4866; udp port, no raw axip\n"
"txport 4867\n"
";------\n";

/*---------------------------------------------------------------------------*/

void parainit(void)
{ FILE *parmf;
  int i=0;
  char inbuf[142];
  parmf=fopen(L2INITNAME,"rt");
  if (!parmf)
  { parmf=fopen(L2INITNAME,"w+t");
    if (parmf)
    { fwrite(l2f_init,sizeof(l2f_init)-1,1,parmf);
      rewind(parmf);
    }
  }
  if(parmf)
  { 
    while(fgets(inbuf,141,parmf))
    { i=0;
      while((inbuf[i]!='\n')&&(inbuf[i]!=';')&&(inbuf[i]!='\r')&&inbuf[i])
      { if(inbuf[i]==9) inbuf[i]=' ';                     /* Tab weg */
        i++;
      }
      linenumber++;
      if(i)
      { inbuf[i]=0;
        beftest(inbuf);
      }
    }
    fclose(parmf);
  }
  else
  { perror("init.l2");
    errors=100;
  }
}

/*---------------------------------------------------------------------------*/

void call_l2(int off)
{ static int sequence=0;
  
  if(sequence==0)
  { 
    sequence=1;
    paralloc();
    initparameter();
    varset();
    parainit();
    initax25(1);
  }
  if(off && sequence==1)
  { 
    sequence=2;
    initax25(0);
  }
  tfaellig+=2;
  if(sequence==1)
  { 
    layer1();
    layer2();
  }
}
