/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -----------------------------------------
  Automatische Berechnung von Forwardpfaden
  -----------------------------------------


  Copyright (C)       Florian Radlherr
		      Taubenbergstr. 32
		      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980618 OE3DZW -1 -> MAXBEREICH (neg value to unsigned variable)
//19980618 OE3DZW fixed defines for uclose,hadr_tryopen,loadhadr
//19981005 jj     search in infobase only manual, fwdlist-debugging
//19990818 OE3DZW Bugfix, now returns static strings 

#include "baycom.h"
#ifdef _AUTOFWD

extern void   uclose(void); 
extern void   hadr_tryopen(void); 
extern int    loadhadr(char *hadrcall,hadr_t *u,int anlegen); 


//definitionen fr checksuche
#define LEN 128
#define BUFLEN 8192
#define BUFLINES 32
#define BUFSIZE  (BUFLINES*LEN)

long hoursek(long h)
{ long t=h*3600;
  return t;
}

long minsek(long h)
{ long t=h*60;
  return t;
}

/*---------------------------------------------------------------------------*/
// Berechnet Sekunden einer Tageszeit
time_t parse_day(char *time)
{ int a=0;
  char s[3];
  time_t t;
  while (strncmp(time+a,":",1)!=0 && a<strlen(time)) a++;
  if (strlen(time)>4 && strlen(time)>a+1 && a>1)
  { strncpy(s,time+a-2,2);                               // Stunden
    s[2]=0;
    t=hoursek(atoi(s));
    strncpy(s,time+a+1,2);                               // Minuten
    s[2]=0;
    t+=minsek(atoi(s));
    if (strncmp(time+a+3,":",1)!=0 && strlen(time)>a+4)  // Sekunden
    { strncpy(s,time+a+4,2);
      s[2]=0;
      t+=atoi(s);
    }
  }
  return t;
}
/*---------------------------------------------------------------------------*/
// Zhlt Headereintrge in einer Mail und gibt sie evtl. aus

int headcount(char *filename,int ausgabe,char stime[15],char rtime[15],char hline[256])
{ int c=0;

  if(filename[0])
    if (filetime(filename))
    { int a;
      int y=0;
      int z=0;
      char line[256];
      FILE *g=s_fopen(filename,"lrt");
      int fileok=1;

      while(fileok)
      { waitfor(e_ticsfull);
	a=fgetc(g);
	line[z]=a;
	z++;
	if(a==EOF || z>255) fileok=0;
	if(a==10)
	{ line[z]=0;
	  z=0;
	  if(strncmp(line,"R:",2)==0 || y<5)
	  { y++;
	    if(y>4)
	    { if(ausgabe) putf(line);      // ?
	      strcpy(hline,line);
	      c++;
	      // Zeit aufteilen
	      if(y==5)
	      { rtime[0]=0;
		strncat(rtime,line+6,2);
		strcat(rtime,".");
		strncat(rtime,line+4,2);
		strcat(rtime,".");
		strncat(rtime,line+2,2);
		strcat(rtime," ");
		strncat(rtime,line+9,2);
		strcat(rtime,":");
		strncat(rtime,line+11,2);
	      }
	      if(y>5)
	      { stime[0]=0;
		strncat(stime,line+6,2);
		strcat(stime,".");
		strncat(stime,line+4,2);
		strcat(stime,".");
		strncat(stime,line+2,2);
		strcat(stime," ");
		strncat(stime,line+9,2);
		strcat(stime,":");
		strncat(stime,line+11,2);
	      }
	    }
	  }
	  else
	    fileok=0;
	}
      }
      s_fclose(g);
    }

  return c;
}

/*---------------------------------------------------------------------------*/
// Sucht Mailboxpfad nach HADR (viel schneller als check)

char *autohadr(char *call,int ausgabe)
{ hadr_t ha;
  char *adjacent=0;
  bitfeld  options      = b->optplus;
  fwdpara_t *ff;
  char     oldcall[CALLEN+1];

  if(m.hadrstore)
  { strupr(call);
    hadr_tryopen();
    if(loadhadr(call,&ha,0))
    { uclose();
      time_t mindelay=ad_time();
      time_t mtime=ad_time()+ad_timezone()/3600;  // UTC
      int i=0;
      int j=0;
      int hops=1000;
      int adj=0;
      time_t last=0;
      int ausgabe_a=ausgabe && (options & o_a);
      int treffer=0;
      int isok=0;
      int maxdays=m.autofwdtime;
      if (maxdays==0) maxdays=365;

      //while(mbcallok(ha.adjacent[adj+1]) && adj+1<ADJNUM) adj++; // Standardberechnung
      if(ausgabe_a)
      { putf("Forward data from HADR-entry\n\n");
	putf("H-Addr     : %s\n",ha.adr);
	putf("R-Header   : %s\n",ha.lastheader);
	putf("Last BID   : %s\n",ha.lastbid);
	putf("Last Board : %s\n",ha.lastboard);
	putf("Last User  : %s\n",ha.lastuser);
	putf("Header Date: %s %s\n",datestr(ha.lasthtime,1),
				  timestr(ha.lasthtime,1));
	putf("Bulletins  : %ld\n",ha.bulletins);
	putf("Usermails  : %ld\n",ha.usermails);
	putf("Neighbour Count Update +Delay Hops\n");
	for(i=0;i<ADJNUM;i++)
	{ if(ha.adjacent[i][0])
	  { if(ha.delay[i] < mindelay)
	      mindelay=ha.delay[i];
	  }
	}
      }

      for(i=0;i<ADJNUM;i++)
      { // Ausgabe HADR-Eintrag
	if(ha.adjacent[i][0])
	{ if(ausgabe_a)
	  { putf("%-9s%4d%8s",ha.adjacent[i],ha.rel_mails[i],
		 zeitspanne(mtime-ha.lastupdate[i],zs_seconds));
	    putf("%7s%5d",zeitspanne(ha.delay[i]-mindelay,zs_seconds),
		 ha.hops[i]);
	  }
	  if(isforwardpartner(ha.adjacent[i])!=NOTFOUND)
	  { // sucht neusten Eintrag mit folgenden Bedingungen
	    //  - hop-count kleiner/gleich als aktueller
	    //  - Eintrag wurde innerhalb von <maxdays> Tagen aktualisiert
	    //  - Ist hop-count gleich, zhlt neuerer Eintrag
	    isok=0;
	    if(ha.hops[i]<=hops && ha.lastupdate[i]>(mtime-DAY*maxdays))
	    { isok=1;
	      if(ha.hops[i]==hops && ha.lastupdate[i]<last) isok=0;
	    }
	    // - FWD-Partner und keine Einschrnkung in FWD.BCM
	    if(isok)
	    { strcpy(oldcall,b->logincall);
	      strcpy(b->logincall,ha.adjacent[i]);
	      ff=fpara();
	      strcpy(b->logincall,oldcall);
	      if(ff->options & o_c)
	      { isok=fwdcallok(call,ff->fwdcallformat);
		if(isok && b->herkunft[0]!=0) isok=fwdcallok(b->herkunft,ff->fwdcallformat);
	      }
	    }
	    // Eintrag bernehmen
	    if(isok)
	    { hops=ha.hops[i];
	      adj=i;
	      treffer++;
	      if(ausgabe_a)
	      { putf(" ");
		for(j=0;j<treffer;j++) putf("<");
	      }
	    }
	  }
	  else
	    if(ausgabe_a) putf(" fwd terminated");
	  if(ausgabe_a) putf("\n");
	}
      }

      if(treffer)
      { adjacent=ha.adjacent[adj];
	call=ha.lastbid;
      }
    }
    uclose();
  }

  if(ausgabe)
  { if(adjacent)
     putf("\nAutofwd to %s\n",adjacent);   // Standardausgabe
    else
     putf("\nAutofwd found no path.\n");
  }

  return adjacent;
}

/*---------------------------------------------------------------------------*/
// Liest CHECK-Zeile aus und vergleicht BID mit Call, Rckgabe Frombox-call

char *checkline_afwd(char *dirline,int boardnum,char *call,int ausgabe)
{ char bid[BIDLEN+1];
  char frombox[CALLEN+1];
  char filename[7];
  char *fcall="      ";
  long dirtime;
  char herkunft[CALLEN+1];
  char ziel[ZIELLEN+1];
  char at[HADRESSLEN+1];
  char betreff[BETREFFLEN+1];
  int mailtype;
  int life;
  int lifetime;
  int i=125;
  int lines;
  long bytes;
  bitfeld  options      = b->optplus;
  dirline[7]=0;
  dirtime=filename2time(dirline);
  strcpy(filename,dirline);
  char stime[15];
  char rtime[15];
  char hline[256];

  while(dirline[i]==' ')
    dirline[i--]=0;

  dirline[14]=0;
  blankweg(dirline+8);
  strcpy(herkunft,dirline+8);

  dirline[23]=0;
  blankweg(dirline+15);
  strcpy(ziel,dirline+15);

  mailtype=mbcallok(ziel);

  dirline[30]=0;
  blankweg(dirline+24);
  strcpy(frombox,dirline+24);

  dirline[37]=0;
  blankweg(dirline+31);
  strcpy(at,dirline+31);

  dirline[50]=0;
  blankweg(dirline+38);
  strcpy(bid,dirline+38);

  life=atoi(dirline+51);
  if(life==999)
    lifetime=999;
  else
    lifetime=life-(int)((ad_time()-dirtime)/(1440*60L));
  if(lifetime<(-99))
    lifetime=(-99);

  lines=(unsigned)ascdez(dirline+55,2);

  bytes=ascdez(dirline+57,3);

  dirline[126]=0;
  if (dirline[60] == '|')
    strcpy(betreff,dirline+78);
  else
    strcpy(betreff,dirline+61);

  if(strstr(bid,call)==0 || isforwardpartner(frombox)==NOTFOUND)
    return 0;
  else
  { if(ausgabe)
    { if((options & o_c)) putf("Autofwd to %s from database\n",frombox);  	 // Standardausgabe

      fileio file;
      char filepath[FNAMELEN+1];
      strcpy(filepath,ziel);
      nameconvert(filepath);
      int pathlen=0;

      char *xs;                 	         // Mainboard suchen
      int j;
      int i;
      for (i=0;i<treelen;i++)
      { xs=tree[i].name;
	j=strlen(xs)-1;
	while(j && (xs[j-1]!='/')) j--;
	if(strncmp(ziel,xs+j,6)==0)
	  strcpy(filepath,xs);
      }

      char fp[255];                  		 // Dateipfad festlegen
      if(mailtype)
       sprintf(fp,"%s/%s/%s",m.userpath,filepath,filename);
      else
       sprintf(fp,"%s/%s/%s",m.infopath,filepath,filename);

      if((options & o_b) && b->sysop)            // Datei Autobin-Download fr Sysops
	file.send_autobin(fp, 0);

      if(options & o_h)                  	 // Header-Pfad ausgeben und zhlen
      { putf("\n");
	pathlen=headcount(fp,1,stime,rtime,hline);
      }
      else
	pathlen=headcount(fp,0,stime,rtime,hline);

      if(options & o_d)               		 // Quelldateidaten ausgeben
      {	char *header=strstr(hline,"@:");
	header=header+2;
	header[strlen(header)-1]=0;

	putf("\nForward data from CHECK-entry\n\n");
	putf("Box     : %s\n",header);
	putf("Call    : %s\n",herkunft);
	putf("Board   : %s",ziel);
	if(at[0])
	  putf("@%s",at);
	putf(" (%i)\n",boardnum);
	putf("Title   : %s\n",betreff);
	putf("BID     : %s\n",bid);
	putf("Filename: %s\n",fp);
	putf("FileTime: %s %s\n",datestr(dirtime,1),timestr(dirtime,0));
	putf("Lifetime: %i\n",life);
	putf("Lines   : %i\n",lines);
	putf("Bytes   : %i\n",bytes);
      }
      if(options & o_t)               		 // Zeitmessung ausgeben
      {	putf("\nPathlen : %i Boxes\n",pathlen);
	putf("Tx Time : %s\n",stime);
	putf("Rx Time : %s\n",rtime);
	time_t st=parse_day(stime+9);
	time_t rt=parse_day(rtime+9);
	stime[8]=0;
	rtime[8]=0;
	st+=parse_time(stime);
	rt+=parse_time(rtime);
	// putf("Tx Time2: %s %s\n",datestr(st,1),timestr(st,0));
	// putf("Rx Time2: %s %s\n",datestr(rt,1),timestr(rt,0));
	putf("Delay   : %s\n",zeitspanne(rt-st,zs_runtime));
      }
    }
    strcpy(fcall,frombox);
    return fcall;
  }
}

/*---------------------------------------------------------------------------*/
// Sucht CHECK-Eintrag mit passender Mailbox

char *autofwd(char *call,int ausgabe)
{ char *fwdcall=0;
  scanoptions(call);
  bitfeld  options      = b->optplus;
  call+=blkill(call);
  blankweg(call);
  strupr(call);

  // erst hier hadr-eintrag scannen
  if(!(options & o_c) || options & o_a)
    fwdcall=autohadr(call,ausgabe);

  if(options & o_b ||
     options & o_c ||
     options & o_d ||
     options & o_h ||
     options & o_t)
  { int lfd=1;
    handle fh=EOF;
    handle posf=EOF;
    char *buf=(char *)t_malloc(BUFSIZE,"chek");
    int buflen=0;
    short unsigned numbuf[BUFLINES*2];
    long flen;

    fh=s_open(CHECKNAME,"lrb");
    if(!b->semalock && !sema_access(CHECKNUMTMP))
    { if(fh==EOF)
	goto nocheckfile;
      posf=s_open(CHECKNUMNAME,"lrb");
      if(posf==EOF || filelength(posf)!=(filelength(fh)>>6))
	goto nocheckfile;
      if(fh!=EOF)
      { lseek(fh,0L,SEEK_END);
	flen=ltell(fh);
	b->beginn=1;
	b->ende=MAXBEREICH;
	if(b->beginn<=(flen>>7))
	{ flen|=(BUFSIZE-1);
	  flen+=1;

	  while(lfd<=b->ende)
	  { if(buflen==0)
	    { flen-=BUFSIZE;
	      lseek(fh,flen,SEEK_SET);
	      lseek(posf,flen>>6,SEEK_SET);
	      buflen=_read(fh,buf,BUFSIZE);
	      _read(posf,numbuf,BUFLINES*2);
	    }
	    if(buflen>=LEN)
	    { buflen-=LEN;
	      if(buf[buflen+7]!=' ')
	      { buf[buflen+126]=0;
		fwdcall=checkline_afwd(buf+buflen,numbuf[buflen>>7],call,ausgabe);
		if(fwdcall)
		goto fileend;
		lfd++;
	      }
	    }
	      if(buflen==0 && flen==0)
	      goto fileend;
	      waitfor(e_ticsfull);
	  }
	}
	else
	  putf(ms(m_checkentries),ltell(fh)>>7);
      }
      else
	trace(serious,"mbcheck","fopen "CHECKNAME" errno=%d",errno);
    }
    else
    { nocheckfile:
      putf(ms(m_checknoaccess));
    }
    fileend:
    if(fh!=EOF)
      s_close(fh);
    if(posf!=EOF)
      s_close(posf);
    t_free(buf);
    if(!fwdcall)
      putf("Autofwd found no path in database.\n");
  }
  return fwdcall;
}

/*---------------------------------------------------------------------------*/
// Holt sich von rechts eine anzahl Zeichen nach lbuf, bis zum angegebenen
// Zeichen search und lscht den Teil im alten string buf

char *strlstr(char *buf,char *search)
{ static char lbuf[50]; //dzw
  int i=strlen(buf);

  while(strncmp(buf+i,search,1)!=0 && i>0) i--;
  strcpy(lbuf,buf+i);
  buf[i]=0;
  return lbuf;
}

/*---------------------------------------------------------------------------*/
// Kuerzt einen Adressheader unter Beruecksichtigung der eigenen Adresse

char *shortadr(char *header)
{ char boxadress[50];
  char shortheader[50];
  char op[50];
  static char hp[50]; //dzw
  int i=0;
  strcpy(boxadress,m.boxadress);
  strcpy(shortheader,header);

  do
  { i++;
    strcpy(op,strlstr(boxadress,"."));
    strcpy(hp,strlstr(shortheader,"."));
  }
  while(strcmp(op,hp)==0 && i<10);
  return hp;
}

/*---------------------------------------------------------------------------*/
// Erstellt eine afwd.bcm im format der fwd.bcm, komplett nach autorouter
#define AFWDTMP1 "temp/afwd1.tmp"
#define AFWDTMP2 "temp/afwd2.tmp"
#define AFWDNAME "afwd.bcm"

void fwdlist(char *search)
{ hadr_t hads;
  unsigned count=0,found=0;
  char *fwdcall;
  char boxcall[CALLEN+1];
  char *header;
  char shortheader[50];
  int i=0;
  char fwdcalls[162];
  int fwdcallnum=0;
  fwdcalls[0]=0;
  time_t upd=0;

  // Alle boxen durchsuchen
  trace(report,"fwdlist","scanning HADR");
  if(m.hadrstore)
  { FILE *f=s_fopen(HADRNAME,"lrb");
    FILE *g=s_fopen(AFWDTMP1,"lwt");
    if(f && g)
    { setvbuf(f,NULL,_IOFBF,4096);    // buffer for increasing performance
      fseek(f,sizeof(hadr_t),SEEK_SET);
      while(fread(&hads,sizeof(hadr_t),1,f))
      { if(hads.adr[0]==0) continue;
	count++;
	boxcall[0]=0;
	// Alter des Pfades
	upd=0;
	for(i=0;i<ADJNUM;i++)
	{ if(hads.adjacent[i][0] && hads.lastupdate[i]>upd)
	  upd=hads.lastupdate[i];
	}
	// Forwardweg suchen
	header=strstr(hads.adr,".");
	if(header)
	{ i=strlen(hads.adr)-strlen(header);
	  strcpy(shortheader,shortadr(hads.adr));
	}
	else
	{ i=strlen(hads.adr);
	  strcpy(shortheader,boxcall);
	}
	if(i>CALLEN) i=CALLEN;
	strncpy(boxcall,hads.adr,i);
	boxcall[i]=0;
	if(mbcallok(boxcall))
	{ fwdcall=autofwd(boxcall,0);
	  if(fwdcall)
	  { found++;
	    // putf("%i) %s%s -> %s to %s\n",count,boxcall,header,shortheader,fwdcall);
	    // Routerfile schreiben
	    if(strncmp(fwdcalls,fwdcall,strlen(fwdcall))!=0)
	    { strcat(fwdcalls,fwdcall);
	      strcat(fwdcalls,"  ");
	      fwdcallnum++;
	    } 
	    fprintf(g,"%s %s%s %s %u\n",fwdcall,boxcall,header,shortheader,upd);
	  }
	  // else
	  //  putf("%i) %s%s -> no path found\n",count,boxcall,header);
	}

	waitfor(e_ticsfull);
	if(testabbruch()) break;
      }
      s_fclose(f);
      s_fclose(g);
      trace(report,"fwdlist","%u searched, %u found, %i partners",count,found,fwdcallnum);

      /*
      // Suche neuste Adressen heraus
      int ok=0;
      char linein[256];
      char linetmp[256];
      char dummy[10];

      trace(report,"fwdlist","searching newest entries");
      FILE *g=s_fopen(AFWDTMP2,"lwt");
      s_fclose(g);

      FILE *f=s_fopen(AFWDTMP1,"srt");
      if(f)
      { while(fgets(linein,255,f))
	{ i++;
	  if(linein[strlen(linein)-1]=='\n') linein[strlen(linein)-1]=0;
	  if(linein[strlen(linein)-1]=='\r') linein[strlen(linein)-1]=0;
	  sscanf(linein,"%s%s%s%u",dummy,boxcall,shortheader,upd);

	  // Suche, ob schon vorhanden
	  ok=1;
	  FILE *g=s_fopen(AFWDTMP2,"lrt");
	  if(g)
	  { while(fgets(linetmp,255,g))
	    { if(linetmp[strlen(linetmp)-1]=='\n') linetmp[strlen(linetmp)-1]=0;
	      if(linetmp[strlen(linetmp)-1]=='\r') linetmp[strlen(linetmp)-1]=0;
	      if(strncmp(linetmp,shortheader,strlen(shortheader))==0) ok=0;
	      waitfor(e_ticsfull);
	    }
	    s_fclose(g);
	  }

	  // Eintrag wenn nicht vorhanden
	  if(ok)
	  { putf("ok %s\n",linein);
	    FILE *g=s_fopen(AFWDTMP2,"lat");
	    if(g)
	    { fprintf(g,"%s\n",linein);
	      s_fclose(g);
	    }
	  }
	  else
	    putf("no %s\n",linein);

	  waitfor(e_ticsfull);
	  if(testabbruch()) break;
	}
	s_fclose(f);
      }

      // Aufteilen in einzelne Forwardpartner
      trace(report,"fwdlist","splitting partners");

      // Zusammenfgen der AFWD.BCM
      trace(report,"fwdlist","creating FWD-File");
    */
    }
  }
}

/*---------------------------------------------------------------------------*/
#endif
