/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -------------------
  Memory-Ueberwachung
  -------------------


  Copyright (c)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights reserved

 ***************************************************************/

#include "baycom.h"

/*---------------------------------------------------------------------------*/

static void    *alloctab[MAXBLKS];
static char    allocname[MAXBLKS][5];
static char    alloctask[MAXBLKS];
static size_t  allocsize[MAXBLKS];
void          *nextfree=NULL;
 
/*---------------------------------------------------------------------------*/

void *t_malloc(size_t size,char *name)
{ int i;
  for(i=0;i<MAXBLKS;i++)
  { if(alloctab[i]==NULL)
    { alloctab[i]=malloc(size);
      if(alloctab[i]==NULL)
        trace(fatal,"malloc","%s fail %ld Bytes",name,size);
      if(strlen(name)>4)
	name[4]=0;
      strcpy(allocname[i],name);
      if(name[0]=='*')
        alloctask[i]=(unsigned char)255;
      else
        alloctask[i]=gettaskid();
      allocsize[i]=size;
      return alloctab[i];
    }
  }
  trace(fatal,"malloc","%s blk full",name);
  return NULL;
}

/*---------------------------------------------------------------------------*/

int t_settaskid(void *blk,int tid)
{ int i;

  if(blk)
  { for(i=0;i<MAXBLKS;i++)
    { if(alloctab[i]==blk)
      { alloctask[i]=tid;
	return 1;
      }
    }
    trace(serious,"t_settaskid","%lp no alloc",blk);
  }
  else
    trace(serious,"t_settaskid","NULL pointer");
  return 0;
}

/*---------------------------------------------------------------------------*/

void t_free(void *blk)
{ int i;

  if(blk)
  { for(i=0;i<MAXBLKS;i++)
    { if(alloctab[i]==blk)
      { free(blk);
	alloctab[i]=NULL;
	alloctask[i]=(unsigned char)255;
	return;
      }
    }
    trace(serious,"t_free","%lp no alloc",blk);
  }
  else
    trace(serious,"t_free","NULL dealloc");
}

/*---------------------------------------------------------------------------*/

void t_checkfree(void)
{
  if(nextfree)
  { free(nextfree);
    nextfree=0;
  }
}

/*---------------------------------------------------------------------------*/

void t_allfree(int task)
{ int i;
  
  for(i=0;i<MAXBLKS;i++)
  { if(alloctask[i]==task)
    { 
      if(alloctab[i]!=t) 
      // eigenen TCB noch nicht loeschen
      {
        if(alloctab[i]!=NULL)
          free(alloctab[i]);
        else
        { trace(serious,"t_allfree","NULL %d %s \"%s\"",
          	      task,gettaskname(task),allocname[i]);
        }
      }
      else
      { if(nextfree==NULL)
          nextfree=alloctab[i];
        else
	  trace(serious,"t_allfree","nextfree busy");
      }
      alloctab[i]=NULL;
      alloctask[i]=(unsigned char)255;
    }
  }
}

/*---------------------------------------------------------------------------*/

void *t_realloc(void *blk,size_t newsize)
{ int i;

  for(i=0;i<MAXBLKS;i++)
  { if(alloctab[i]==blk)
    { alloctab[i]=realloc(blk,newsize);
      allocsize[i]=newsize;
      if(alloctab[i]==NULL)
      { trace(fatal,"realloc","%s fail %u Bytes",allocname[i],newsize);
	return NULL;
      }
      return(alloctab[i]);
    }
  }
  trace(fatal,"realloc","%lp no alloc",blk);
  return NULL;
}

/*---------------------------------------------------------------------------*/

void t_alloclist(void)
{ int i;

  putf("Blk Size Address  Task Name\n");
  for(i=0;i<MAXBLKS;i++)
  { if(alloctab[i]!=NULL)
      putf("%2d%6u %08lX%5d %s\n",i,allocsize[i],(long)alloctab[i],
	   alloctask[i],allocname[i]);
  }
}

/*---------------------------------------------------------------------------*/

void t_allocinit(void)
{ int i;
  for(i=0;i<MAXBLKS;i++)
  { alloctab[i]=NULL;
    allocname[i][0]=0;
    alloctask[i]=(unsigned char)255;
  }
}
