/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -----------------------------------------------
  Einstellung der persoenlichen Benutzerparameter
  -----------------------------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights reserved

 ***************************************************************/

//19980115 OE3DZW utcoffset -> ad_timezone
//19980118 OE3DZW sec in timestr
//19980215 OE3DZW text for userfwd with no path "passive" instead of "disabled" 
//19980324 DG4MMI support for md-pw
//19980329 OE3DZW changes to md-pw
//19980330 OE3DZW pwtype is reset to BayCom when pw is erased/set to off
//19980404 hrx    support for guests
//19980408 OE3DZW "notvisible" was too long -> DO NOT TOUCH!!
//19980423 OE3DZW sfpwtype instead of fwdpwtype
//                added newcall, get_newcall
//19980426 OE3DZW ttypw may be 8 chars long (was 7 only)
//19980427 OE3DZW pwline only when pw set, info when pw is set
//19980430 OE3DZW fixed readlock, could only be set to 0 or 1
//19980505 OE3DZW alter sfpwtype/loginpwtype will only show valid types
//19980506 OE3DZW fixes to fwdhold
//19980615 OE3DZW commented out newcall
//19980614 hrx    added paclen to display_parameter, mbalter befnum/beftab
//                and cases
//19980618 OE3DZW temp variable for fdelay short unsigned (warn. dos)
//19980619 OE3DZW typecast for short unsigned, removed check for < 0
//19980830 OE3DZW alter default only when password setting has not been
//                switched off by user (alter pw off)             
//19980830 OE3DZW name is only put to upper or lower case when all 
//                digits are in upper or lower case and there is no
//                non-character in between.
//                lastcmd - at top of pw-function
//19980914 OE3DZW guest excluded
//19980925 OE3DZW will not allow "." in userfwd-path
//19981015 OE3DZW md-pw-string was unterminated, fixed
//19981015 OE3DZW userpw off -> "disabled"
//19981015 OE3DZW ufwd can be set to "off" or "passive"
//19981016 OE3DZW changed text for ufwd (removed dot at end)
//19990220 DK2UI  elseif instead of if (alter pw)
//19990227 DK2UI  alter pw, fix
//19990227 OE3ZDW removed multiple mybbs-infos, fixed ufwd=0 
//19990329 OE3DZW if userpw contains "DUMMY" http/pop3-logins are allowed
//19990412 OE3DZW mybbs update only if setting different/>1week
//19990721 hpk    added support for minpaclen and maxpaclen
//19990812 DH3MB  Removed trailing blank from "Password removed."-pwlog-entry
//19990823 DH3MB  Added option "BINMODE"
//19991110 DB1RAS user can see and change paclen with ALTER (not only Sysop)
//                if readlock in init.bcm not 0 -> user can see A REA

#include "baycom.h"

static user_t us;    // do not waste stack. The following functions
                     // no not need to be re-entrant

/*---------------------------------------------------------------------------*/

char *get_ttypw(char *call,char *pw)
{ strupr(call);
  if(mbcallok(call) && loaduser(call,&us,1))
    strcpy(pw,us.ttypw);
  else
    pw[0]=0;
  return pw;
}

/*---------------------------------------------------------------------------*/

int get_nopurge(char *call)
{ strupr(call);
  if(mbcallok(call) && loaduser(call,&us,1))
    return us.nopurge;
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

int get_fdelay(char *call)
{ strupr(call);
  if(mbcallok(call) && loaduser(call,&us,1))
    return (b->forwarding==fwd_none)*us.fdelay+
           (us.fhold*60)*(!b->sysop)*(!b->usermail);
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

char *get_ufwd(char *call)
{ strupr(call);
  if(mbcallok(call) && loaduser(call,&us,1))
    return us.ufwd;
  else
    return "";
}

/*---------------------------------------------------------------------------*/

int get_readlock(char *call)
{ strupr(call);
  if(mbcallok(call) && loaduser(call,&us,1))
    return us.readlock;
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

void inc_mailgot(char *call)
{ strupr(call);
  if(mbcallok(call) && loaduser(call,&us,1))
  { us.mailgot++;
    saveuser(&us);
  }
}

/*---------------------------------------------------------------------------*/

void inc_mailsent(char *call)
{ strupr(call);
  if(mbcallok(call) && loaduser(call,&us,1))
  { us.mailsent++;
    saveuser(&us);
  }
}

/*---------------------------------------------------------------------------*/

int get_mybbs(char *call,char *bbs,int setmybbs)
{ strupr(call);
  if(mbcallok(call) && loaduser(call,&us,1))
  { if(bbs[0]==0)
      strcpy(bbs,us.mybbs);
    else if(setmybbs && (us.mybbs[0]==0 || (!us.mybbsok && setmybbs==2)))
    { if(strlen(bbs)<=MYBBSLEN)
        strcpy(us.mybbs,bbs);
      else
        strcpy(us.mybbs,atcall(bbs));
      us.mybbsok=0;
      us.mybbstime=ad_time();
      saveuser(&us);
    }
    return us.mybbsok;
  }
  return 0;
}

/*---------------------------------------------------------------------------*/

#ifdef __FLAT__
int get_smtp_security(char *call)
{ strupr(call);
  if(mbcallok(call) && loaduser(call,&us,1))
    return (us.unsecure_smtp & 1); 
  else
    return 0; // 0 => only secure smtp
}
#endif

/*---------------------------------------------------------------------------*/

void set_name(char *call,char *name)
{ strupr(call);
  if(name[0] && mbcallok(call) && loaduser(call,&us,1))
  { if(us.name[0]==0 || us.nameok==0)
    { strcpy(us.name,name);
      saveuser(&us);
    }
  }
}

/*---------------------------------------------------------------------------*/

char *get_name(char *call,int par)
{ static char name[NAMELEN+5];
  name[0]=0;
  strupr(call);
  if(mbcallok(call) && loaduser(call,&us,1) && us.name[0])
  { if (par)
      sprintf(name,"(%s) ",us.name);
    else
      sprintf(name,"%s ",us.name);                            
  }
  return name;
}

/*---------------------------------------------------------------------------*/

int get_usr_local(char *call, time_t &lastlogin)
/*
returns 1 if a user is known and local, 0 else
lastlogin: last login of user when found (else 0)

*/
{
  strupr(call);
  lastlogin=0;
  if(mbcallok(call) && loaduser(call,&us,1) && us.mybbsok)
  { lastlogin = us.lastboxlogin;
    if (strcmp(m.boxname,atcall(us.mybbs))==0)
      return 1;
  }
  return 0;
}

/*---------------------------------------------------------------------------*/

int get_newcall(char *call,char *newcall)
{ int i=0;
  char name[]="get_newcall";
  if (strlen(call) > CALLEN)
    return 0;
  strupr(call); 
  strcpy(newcall,call); 
  while (i<20 && mbcallok(newcall) && loaduser(newcall,&us,1) && 
     us.newcall[0])
  { i++;
    us.newcall[CALLEN]=0;
    strcpy(newcall,us.newcall); 
    if (strcmp(newcall,call)==0)
    { trace(serious,name,"loop %s",call);
      return 0;
    }
  }
  if (i==20 || i==0)
  { strcpy(newcall,call);
    return 0;
  }
  else
  { //trace(report,name,"upd %s -> %s",call,newcall);
    return 1;
  }
}
 
/*---------------------------------------------------------------------------*/

char *get_httppw(char *call)
{ static char name[NAMELEN+5]; 
  name[0]=0;
  strupr(call);
  if(mbcallok(call) && loaduser(call,&us,1) && us.name[0] 
    && ( !us.password[0] || (us.password[0]==1 &&
    us.password[1] == 0) || strstr(us.password,"DUMMY"))) 
      //do not allow login when password is set!
      //but accept login when password is contains "DUMMY"!
      sprintf(name,"%s",us.name); 
  return name;
}

/*---------------------------------------------------------------------------*/

void set_mybbs(char *call,char *bbs,long btime,char *bid,
     char *origin, long hops,char *w_name)
{
   hops++;

  static char name[]="set_mybbs";
  time_t actual=ad_time()+(ad_timezone());
  int isnew=0;

  strupr(call);
  if(!mbcallok(call))
  { trace(replog,name,"call %s",call);
    return;
  }
  strupr(bbs);
  if(mbhadrok(bbs)!=1)
  { trace(report,name,"hadr %s@%s",call,bbs);
    return;
  }
  if(bid)
  { strupr(bid);
    if((strlen(bid)>12)||(strlen(bid)<10))
    { trace(serious,name,"bid %s",bid);
      return;
    }
  }

  if(loaduser(call,&us,1))
  { if(us.mybbstime>actual)
      us.mybbstime=actual;
    if(btime>actual)
      btime=actual;

    if(!btime || (btime>us.mybbstime) || !us.mybbsok)
    { isnew=1;
      // fern-mybbs nur aendern wenn
      // - mybbs auf anderem als eigenen Boxcall steht
      // - oder kein Passwort gesetzt ist
      // - der eigene Mybbs-Eintrag aelter als 1 Woche ist
      //
      char bbs_call[CALLEN+1];
      char usbbs_call[CALLEN+1];
      strncpy(bbs_call,atcall(bbs),CALLEN);
      bbs_call[CALLEN]=0;
      strncpy(usbbs_call,atcall(us.mybbs),CALLEN);
      usbbs_call[CALLEN]=0;
   
      //only send remote update if local setting is different and
      //older than one week (no update within 1 week)
      if (strcmp(bbs_call,usbbs_call)==0 &&
         (actual-us.mybbstime)<(3600L*24*7))
      { isnew=0;
      }

      if(btime &&
         strcmp(usbbs_call,m.boxname)==0 &&
         strlen(us.password)>3 &&
         (actual-us.mybbstime)>(3600L*24*7))
      { bbs=m.boxadress;
        btime=actual;
      }
      if(btime)
        us.mybbstime=btime;
      else
        us.mybbstime=actual;
      if(strlen(bbs)<=MYBBSLEN)
        strcpy(us.mybbs,bbs);
      else
        strcpy(us.mybbs,atcall(bbs));
      us.mybbsok=1;
      if(strcmp(atcall(bbs),m.boxname)!=0 && us.ufwd[0]>1)
        us.ufwd[0]=0; 
                                   // wenn das MYBBS nicht die eigene
                                   // Box ist, dann kein aktiver Ufwd
                                   // User may set parameter to disabled/passive

      saveuser(&us);
      if(bid && isnew) 
      { wpdata_t *wp=(wpdata_t *) t_malloc(sizeof(wpdata_t),"wpa");
        if (w_name) //check for name received in WP/WPROT message
           strcpy(wp->name,w_name);
        else if (strcmp(b->logincall,origin)==0 && us.name[0] && us.nameok)
        { strncpy(wp->name,us.name,NAMELEN-1); //use local name
          wp->name[NAMELEN-1]=0; 
        }
        else
           strcpy(wp->name,"?"); //name is unknown

        strcpy(wp->call,call);
        strcpy(wp->bid,bid);
        wp->mybbstime=us.mybbstime;
        strcpy(wp->logincall,b->logincall);
        strcpy(wp->origin,origin);
        wp->hops = hops;
        strcpy(wp->bbs,bbs);
        strcpy(wp->zip,"?"); //not supported yet
        strcpy(wp->qth,"?"); //not supported yet
        addwp_m(wp);
        t_free(wp);
      }
      // Nachrichten nicht nachsenden, da dies von "postfwd" ohnehin 
      // gemacht wird und sonst evtl die Nachrichten das M-File
      // ueberholen koennten.
    }
  }
}

/*---------------------------------------------------------------------------*/

static int near mybbs(char *adr,int weitergeben,time_t tm)
{ char *bid=NULL;
  char bbs[HADRESSLEN+1];

  safe_strcpy(bbs,adr);
  if(weitergeben && strchr(bbs,'-'))
    strchr(bbs,'-')[0]=0;
  if(!weitergeben || mbhadrok(bbs)==1)
  { if(weitergeben)
    { expand_hadr(bbs,m.hadrstore);
      if (strcmp(u->call,bbs)==0) //bbs=call only if hadr given..
        return 0;
      bid=newbid(0);
    }
    set_mybbs(u->call,bbs,tm,bid,b->logincall,0,NULL);
    loaduser(u->call,u,0);
    putf(ms(m_mybbsset),bbs);
    if(bid)
      putf(ms(m_mybbsforward));
    showpath(bbs,0);
    return 1;
  }
  return 0;
}

/*---------------------------------------------------------------------------*/

char *optstr(bitfeld option)
{ static char str[33];
  int i,j=0;

  for(i=0;i<32;i++)
  { if(option&1)
      str[j++]=i+'A';
    option>>=1;
  }
  str[j]=0;
  return str;
}

/*---------------------------------------------------------------------------*/

void display_parameter(char *call,int full)
{ user_t us;
  char *z=ms(m_neverlogin);

  strupr(call);

  if(loaduser(call,&us,0))
  { if(us.notvisible[0]==0)
      us.notvisible[1]=0;    // Vorsichtsmanahme
    putf(ms(m_a_headline),us.call);
    if (us.newcall[0] || b->sysop)
      putf("New callsign..(A NE).%s\n",us.newcall[0]?us.newcall:"none");  
    putf(ms(m_a_mybbs),us.mybbsok?'@':'?',us.mybbs);
    if(us.mybbstime)
      putf(" (%s %s utc)",datestr(us.mybbstime,2),timestr(us.mybbstime,1));
    putf("\n");
    if(full)
    { putf(ms(m_a_reject),us.notvisible+1);
      putf(ms(m_a_prompt),us.prompt);
      putf(ms(m_a_firstcmd),us.firstcmd);
      putf(ms(m_a_name),us.name);
      putf(ms(m_a_speech),us.sprache);
      putf(ms(m_a_lines),us.zeilen);
      putf(ms(m_a_helplevel),us.helplevel);
      putf(ms(m_a_linefeed),us.lf);
      putf(ms(m_a_idir),optstr(us.opt[o_id]));
      putf(ms(m_a_udir),optstr(us.opt[o_ud]));
      putf(ms(m_a_ilist),optstr(us.opt[o_il]));
      putf(ms(m_a_ulist),optstr(us.opt[o_ul]));
      putf(ms(m_a_iread),optstr(us.opt[o_ir]));
      putf(ms(m_a_uread),optstr(us.opt[o_ur]));
      putf(ms(m_a_check),optstr(us.opt[o_ch]));
      //putf(ms(m_a_grep),optstr(us.opt[o_gr]));
      putf("FWD-Delay..(A FD).%d min\n",us.fdelay);
      putf("User-Forward..(A UF).");
      // userforward is either disabled, passive or passive and active
      if (us.ufwd[0]==0)
        putf("passive\n");
      else if (us.ufwd[0]==1)
        putf("off\n");
      else
        putf("%s\n",us.ufwd);
      if (us.dirformat)
        putf("Dirformat.....(A DI).%d\n",us.dirformat);
#ifdef __FLAT__
      if (us.unsecure_smtp)
        putf("Unsecure SMTP.(A UN).%d\n",(us.unsecure_smtp & 1));
#endif
#ifdef _PACLEN
        if (us.paclen)
          putf("Paclen........(A PA).%i\n",us.paclen); 
#endif
      if(b->sysop)
      { 
        putf("Status........(A ST).%-20d",us.status);
        putf("No-Purge...(A NO).%d\n",us.nopurge);
        putf("Passwordlen...(A PW).%-20d",
             us.password[0]==1?-1:(int)strlen(us.password));
        putf("TTYPWlen..(A TTY).%d\n",strlen(us.ttypw));
        if (us.password[0] > 1)
        {
#ifdef FEATURE_MDPW
          putf("Login-pwtype..(A LO).%-20s",pwtypestr(us.loginpwtype)); 
          putf("S&F-pwtype.(A SF).%s\n",pwtypestr(us.sfpwtype)); 
#endif 
          putf("PW-Line......(A PWL).%d\n",us.pwline);
        }
        putf("FHold.........(A FH).%d\n",us.fhold);
        //putf("RLimit........(A RL).%d\n",us.rlimit);
      }
      if (b->sysop || m.readlock)
        putf("Readlock.....(A REA).%-20d\n",us.readlock);
    }
    else
      putf(ms(m_a_name2),us.name);
    if(us.lastboxlogin)
    { if(us.lastdirnews)
        putf(ms(m_a_lastdirnew),zeitstring(us.lastdirnews,2));
      if(us.lastquit)
        putf(ms(m_a_lastquit),zeitstring(us.lastquit,2));
      z=zeitstring(us.lastboxlogin,2);
    }
    putf(ms(m_a_lastlogin),z);

    if(us.uplink[0] && us.lastboxlogin)
      putf("  via %s",us.uplink);
    putv('\n');

    putf(ms(m_a_loginmails),
         us.logins,us.mailgot,us.mailread,us.mailsent);
  }
  else
    putf(ms(m_a_unknown),call);
}

/*---------------------------------------------------------------------------*/

static void near changeoption(char *buf,int optgroup)
{ b->optminus=b->optplus=0;

  if(buf[0])
  { if((strlen(buf)==1) || strchr(buf,'-') || strchr(buf,'+'))
    { if(isalpha(buf[0]))
      { if(buf[1]=='-')
          b->optminus|=(1L<<(buf[0]-'A'));
        else
          b->optplus|=(1L<<(buf[0]-'A'));
      }
      else if(buf[0]=='-')
        b->optminus|=(1L<<(buf[1]-'A'));
      else if(buf[0]=='+')
        b->optplus|=(1L<<(buf[1]-'A'));

      b->optminus=(~b->optminus);
      u->opt[optgroup]=(u->opt[optgroup] & b->optminus) | b->optplus;
    }
    else
    { u->opt[optgroup]=0;
      while(buf[0])
      { if(isalpha(buf[0]))
          u->opt[optgroup]|=(1L<<(buf[0]-'A'));
        buf+=1;
      }
    }
  }
  putf(ms(m_optionsfor),optstr(u->opt[optgroup]));
  switch(optgroup)
  { case o_id: putf("Info-Dir");  break;
    case o_ud: putf("User-Dir");  break;
    case o_il: putf("Info-List"); break;
    case o_ul: putf("User-List"); break;
    case o_ir: putf("Info-Read"); break;
    case o_ur: putf("User-Read"); break;
    case o_ch: putf("Check");     break;
    case o_gr: putf("GREP");      break;
  }
  putf(ms(m_optionsset));
}

/*---------------------------------------------------------------------------*/

int mbalter(char *bef,char *selektor,char *call)
{ static char *beftab[]=
    { "COMMAND","SPEECH","FORWARD","HELPLEVEL","LINES","DEFAULT",
      "NAME","PROMPT","REJECT","TTYPW","PW","LF","UMLAUT",
      "IDIR","UDIR","ILIST","ULIST","IREAD","UREAD","CHECK","GREP",
      "STATUS","QUOTA","NOPURGE","PWLINE","READLOCK","ECHO","MYBBS",
      "DELETE","FDELAY","FHOLD","RLIMIT","UFWD","BINMODE",
#ifdef _PACLEN
      "PACLEN", 
#endif
#ifdef __FLAT__
      "UNSECURESMTP",
#endif 
      "LOGINPWTYPE","SFPWTYPE","NEWCALL","DIRFORMAT",
      NULL
    };

  enum befnum
    { unsinn,
      command,speech,forward,helplevel,lines,adefault,
      name,prompt,reject,ttypw,pw,lf,umlaut,
      idir,udir,ilist,ulist,iread,uread,check,grep,
      status,quota,nopurge,pwline,readlock,echo,mybbs_,
      delete_,fdelay,fhold,rlimit,ufwd,binmode,
#ifdef _PACLEN
      paclen, 
#endif
#ifdef __FLAT__
      unsecuresmtp,
#endif 
      loginpwtype,sfpwtype,newcall,dirformat
    }cmd=unsinn;

 static char n[]="mbalter"; 

 static char msnopwset[]="Error: no password set.\n";
 static char mssysoponly[]="Sysop only.\n";
 static char mspwdisabled[]="Password disabled.\n";
 static char mspwerased[]="Password removed.";
 static char msaltermd[]=
#ifdef FEATURE_MDPW
     "Syntax: ALTER LOGINPWTYPE BAYCOM|MD2|MD5|INACTIVE\n";
#else
     "Syntax: ALTER LOGINPWTYPE BAYCOM|INACTIVE\n";
#endif

#ifdef _GUEST
    if (strcmp(b->logincall,m.guestcall)==0)   
    { putf(ms(m_commandnotavail),"");
      return 0;
    }  
#endif

  int optgroup=0;
  int del_user_cache=0;

  if(!loaduser(call,u,1))
    return 0;

  selektor+=blkill(selektor);
  if((bef==NULL) && selektor[0]==0)
  { display_parameter(call,1);
    return 1;
  }

  if(bef)
    cmd=(befnum)readcmd(beftab,&bef,0);
  else
    cmd=(befnum)readcmd(beftab,&selektor,0);

#ifdef _EXDLOGIN	//db1ras
  if (b->guestlevel && !b->sysop)
    {
      switch(cmd)
      {
	case forward:
	case mybbs_:
	case newcall:
	case pw:
	case delete_:
	case adefault:
	case name:
	case ufwd:
	case loginpwtype:
	case sfpwtype:
	  putf("Als Gast nicht moeglich, Hilfe mit HELP GAST.\n");
	  goto abfrageende;
      }
    }
#endif

  switch(cmd)
  { case unsinn:
      if(mbcallok(selektor)==1)
        display_parameter(selektor,1);
      else
        putf(ms(m_alterunknown));
      break;

    case forward:
    case mybbs_:
    { char *tm=NULL;
      time_t tmx=0;
      scanoptions(selektor);
      selektor+=blkill(selektor);
      strupr(selektor);
      tm=blankweg(selektor);
      if(tm)
        tmx=atol(tm);
      if(selektor[0]==0)
      { if(u->mybbs[0])
          putf(ms(m_dispmybbs),u->mybbs);
        else
          putf(ms(m_inputaddress));
      }
      else
      { if(!mybbs(selektor,!(b->optplus & o_l),tmx))
          putf(ms(m_invalidaddress));
      }
    } break;

    case command:
      strupr(selektor);
      if(strlen(selektor)>FIRSTCMDLEN)
        putf(ms(m_cmdtoolong));
      else if(selektor[0]=='Q')
        putf(ms(m_commandquit));
      else
      { strcpy(u->firstcmd,selektor);
        putf(ms(m_cmdstored),selektor);
      }
      break;

    case prompt:
      if(strlen(selektor)>PROMPTLEN)
        putf(ms(m_prompttoolong));
      else
      { if(selektor[0])
          strcpy(u->prompt,selektor);
        else
          strcpy(u->prompt,m.prompt);
        putf(ms(m_promptstored));
      }
      break;

    case newcall:
      selektor+=blkill(selektor);
      strupr(selektor);
      if (!selektor[0] ||
          strcmp(selektor,u->call)==0)
      { memset(u->newcall,0,CALLEN+1);
        putf("New call removed.\n");        
        break;
      }                
      if (strlen(selektor)>CALLEN ||   
          !mbcallok(selektor))
      { putf("New call invalid.\n");        
        break;
      }
      strcpy(u->newcall,selektor);
      putf("New call %s stored.\n",u->newcall);
      trace(replog,n,"%s newcall %s",u->call,u->newcall);
      break;

    case pw:
      lastcmd("-");
      if(stricmp(selektor,"disable")==0)
      { u->password[0]=1;
        u->password[1]=0;
        u->loginpwtype=0;
        u->sfpwtype=0;
        u->pwline=0;
        putf(mspwdisabled);
      }
      else if((m.userpw && u->password[0]!=1) || b->sysop)
      { if((strlen(selektor)+strlen(u->password))>39)
          putf(ms(m_pwdtoolong));
        else
        { if(selektor[0]==0 || stricmp(selektor,"off")==0)
          { u->password[0]=0;
            u->loginpwtype=0;
            u->sfpwtype=0;
            u->pwline=0;
            putf(ms(m_pwderased));
            pwlog(b->logincall,b->uplink,mspwerased);
          }
          else
          { char pwlstr[60];
            int oldlen=strlen(u->password);
            strcat(u->password,selektor);
            int newlen=strlen(u->password);
            putf("Using %s-PW for login, %s-PW for S&F. ",
              pwtypestr(u->loginpwtype),pwtypestr(u->sfpwtype));
            putf(ms(m_pwdlength),newlen);
            sprintf(pwlstr,"pw appended (len %d -> %d)",oldlen,newlen);
            pwlog(b->logincall,b->uplink,pwlstr);
          }
        }
      }
      else
        putf(mssysoponly);
      if(u->password[0]==1)
        putf(mspwdisabled);
      break;

    case pwline:
    { if (u->password[0]<2)
      { putf(msnopwset);
        break;
      }
      if(selektor[0])
      { u->pwline=!!atoi(selektor);        
      }
      putf("PWLINE %d\n",u->pwline);
    } break;

    case echo:
    { if(selektor[0])
        u->echo=!!atoi(selektor);
      if (u->echo)
        putf("ECHO 1 (ON).\n");
      else
        putf("ECHO 0 (OFF).\n");
    } break;

#ifdef __FLAT__
    case unsecuresmtp:
    { if(selektor[0])
        u->unsecure_smtp=!!atoi(selektor);
      if (u->unsecure_smtp)
        putf("Unsecure SMTP 1 (No POP before SMTP needed).\n");
      else
        putf("Unsecure SMTP 0 (Needs POP before SMTP).\n");
    } break;
#endif

    case fdelay:  // usermails and bulletins will be delayed by xx min
    { if(selektor[0])
      { short unsigned a=(short unsigned)atoi(selektor); 
        if (a <= 60)
          u->fdelay=a;
        else
        { putf("Syntax: FDELAY 0..60\n");
          break;
        }
      }
      putf("FDELAY %d min\n",u->fdelay);
    } break;

    case fhold:   // bulletins will not be forwarded automatically
    { if(selektor[0] && b->sysop)
        u->fhold=!!atoi(selektor);
      putf("FHOLD %d\n",u->fhold);
    } break;

    case rlimit:
    { if(selektor[0] && b->sysop)
        u->rlimit=!!atoi(selektor);
      putf("RLIMIT %d\n",u->rlimit);
    } break;

    case readlock:
    { if(selektor[0])
      {  int num=atoi(selektor);
         if (num < 0 || num >2)
         { putf("Syntax: ALTER READLOCK 0..2\n");
           break;
         }
         if (b->sysop || m.readlock) 
          u->readlock=num;
         else
          putf(mssysoponly);
      }
      putf("READLOCK %d\n",u->readlock);
    } break;

    case delete_:
    { if(b->sysop && strcmp(u->call,b->logincall)!=0)
      { putf("%s deleted.\n",u->call);
        u->call[0]=0;
        us_deletecache();
        del_user_cache=1;
      }
    } break;

    case umlaut:
      /*
      strupr(selektor);
      if(strstr("AEOEUE",selektor))
      { u->umlaut=um_aeoeue;
        putf("Umlaute werden als ae oe ue dargestellt.\n");
      }
      else if(strstr("DIN",selektor))
      { u->umlaut=um_din;
        putf("DIN-Umlaute aktiviert.\n");
      }
      else if(strstr("IBM",selektor))
      { u->umlaut=um_ibm;
        putf("IBM-Umlaute aktiviert.\n");
      }
      else
      { u->umlaut=um_none;
        putf("Umlautwandlung abgeschaltet.\n");
      }
      */
      break;

    case helplevel:
    { int h=atoi(selektor);
      if((h<0) || (h>2))
        putf(ms(m_helpvalues));
      else
      { u->helplevel=h;
        putf(ms(m_helpstored),h);
      }
    } break;

    case dirformat:
    { int h=atoi(selektor);
      if((h<0) || (h>1))
        putf("Syntax: alter dirformat 0|1\n");
      else
      { u->dirformat=h;
        putf("dirformat %d saved.\n",u->dirformat);
      }
    } break;

    case lf:
    { int h=atoi(selektor);
      if((h<0) || (h>5))
        putf(ms(m_lfvalues));
      else
      { u->lf=h;
        if(!u->lf)
          putf(ms(m_nolinefeeds));
        else if(u->lf==1)
          putf(ms(m_onelinefeed),h);
        else
          putf(ms(m_linefeeds),h);
      }
    } break;

    case lines:
    { int lines=atoi(selektor);
      if(lines==0)
      { u->zeilen=0;
        putf(ms(m_nolines));
      }
      else if(lines>3)
      { u->zeilen=lines;
        putf(ms(m_linesset),lines);
      }
      else
        putf(ms(m_linesvalues));
    } break;

#ifdef _PACLEN
    case paclen:                                                     
//  { if (b->sysop)
    { int h=atoi(selektor);
//    if((h<256 && h>39) || h==0)
      if(h<m.maxpaclen+1 && h>m.minpaclen-1)
        { if(selektor[0]) u->paclen=h;                                
          putf("Paclen %i\n",u->paclen);                              
        }                                                             
        else                                                          
//        putf("Valid range for Paclen is 40-255. Disable with 0.\n");
          putf("Possible range: %i - %i.\n",m.minpaclen,m.maxpaclen);
//    }
//    else
//      putf(mssysoponly);
    } break;                                                        
#endif

    case adefault:
      if (u->password[0] == 1 && !b->sysop)
        putf(mssysoponly);
      else
      { putf(ms(m_default));
        if(janein(ms(m_defaultquery))==JA)
        { putf(ms(m_defaultmessage));
          defaultuser(call,u);
        }
      }
      break;

    case name:
      if(strlen(selektor)>NAMELEN)
        putf(ms(m_nametoolong));
      else
      { int i=0;
        unsigned int gross=0;
        for (i=0;selektor[i];i++)
        {if (isupper(selektor[i]))
            gross++;         
        }        
        if (islower(selektor[0]) ||
            gross==strlen(selektor) )
        //if(islower(selektor[0]) || isupper(selektor[1]))
        { strlwr(selektor);
          selektor[0]=toupper(selektor[0]);
        }
        strcpy(u->name,selektor);
        u->nameok=1;
        putf(ms(m_nameset),selektor);
      }
      break;

    case reject:
      strupr(selektor);
      if(strlen(selektor)>(REJECTLEN-3)) //DO NOT TOUCH!! 
        putf(ms(m_toomanyreject));
      else
      { strcpy(u->notvisible+1,selektor);
        u->notvisible[0]=' ';
        strcat(u->notvisible," ");
        putf(ms(m_rejectedboards),selektor);
      }
      break;

    case speech:
      { char *spkenn;
        strupr(selektor);

        spkenn=msg_landsuch(selektor);
        if(spkenn[0]!='?')
        { strcpy(u->sprache,spkenn);
          b->msg_loadnum=0;
          putf(ms(m_languageok),spkenn);
        }
        else
        { putf(ms(m_possiblespeech));
          msg_listspeech();
        }
      }
      break;

    case grep:
      optgroup++;
    case check:
      optgroup++;
    case uread:
      optgroup++;
    case iread:
      optgroup++;
    case ulist:
      optgroup++;
    case ilist:
      optgroup++;
    case udir:
      optgroup++;
    case idir:
      strupr(selektor);
      changeoption(selektor,optgroup);
      break;

    case status:
    { if(b->sysop)
      { if(selektor[0])
          u->status=atoi(selektor);
        putf(ms(m_userstateset),call,u->status);
      }
      else
        putf(mssysoponly);
    } break;

    case nopurge:
    { if(b->sysop || m.nopurge&2)
      { if(selektor[0])
          u->nopurge=!!atoi(selektor);
      }
      if (u->nopurge)
        putf("NOPURGE 1 (purge disabled)\n");
      else
        putf("NOPURGE 0 (purge enabled)\n");
    } break;

    case quota:
    { if(selektor[0] && b->sysop)
      { u->daybytes=1024L*atoi(selektor);
        b->rxbytes=b->txbytes=0;
        putf(ms(m_quotaset),u->daybytes/1024);
      }
      else
      { long rambytes=0;
        if(strcmp(b->logincall,call)==0)
          rambytes=b->rxbytes+b->txbytes;
        putf(ms(m_quotaoutput),(u->daybytes+rambytes)/1024L);
        if(m.userquota)
          putf(ms(m_maxquota),m.userquota);
      }
    } break;

    case ttypw:
    { if(b->sysop || u->ttypw[0])
      { if(strlen(selektor)<9)
        { strcpy(u->ttypw,selektor);
#ifdef _TTYPW_OPT
          if(strlen(u->ttypw)>=1)
	  putf("TTY Password added (len: %i).\n",strlen(u->ttypw));
          else
	  putf("TTY Password removed.\n");
	}
#else
          putf("ok.\n");
        }
#endif
      }
      else
        putf(mssysoponly);
      lastcmd("-");
    } break;

    case ufwd:
    { //check for key words
      if(stricmp(selektor,"PASSIV")==0||
         stricmp(selektor,"PASSIVE")==0)
        u->ufwd[0]=0;
      else if(stricmp(selektor,"OFF")==0)
      { u->ufwd[0]=1;
        u->ufwd[1]=0;
      }
      //check for correct length/syntax of path
      else if (strlen(selektor)<5 ||
               strlen(selektor)> (sizeof(u->ufwd)-1) ||   
               strstr(selektor,"."))
        putf("Syntax: ALTER UFWD <path>|PASSIVE|OFF\n");
      //check for correct mybbs
      else if (strcmp(atcall(u->mybbs),m.boxname)!=0)
        putf("MYBBS is not set to %s\n",m.boxadress);
      else
        strcpy(u->ufwd,selektor);

      //messages
      if(u->ufwd[0]==0)
        putf("User-FWD: PASSIVE.\n");
      else if(u->ufwd[0]==1)
        putf("User-FWD: OFF.\n");
      else
        putf("User-FWD connect path set to \"%s\"\n",u->ufwd);
    } break;

    case loginpwtype: 
    { 
      if (u->password[0] < 2 )
      { putf(msnopwset);
        break;
      }

      char type=pwtypenum(selektor);

      switch(type)
      { case 100:
          putf("AX25-login-password inactive.\n");
          u->loginpwtype=100;
          break;

        case 101:
          putf(msaltermd);
          break;

        default:
          u->loginpwtype=type;
          putf("Password-type for AX25-logins set to %s.\n", pwtypestr(type));
      }

    } break;

    case sfpwtype: 
    { if (u->password[0] < 2)
      { putf(msnopwset);
        break;
      }

      char type=pwtypenum(selektor);

      switch(type)
      { case 100:
          putf("S&F-password inactive.\n");
          u->sfpwtype=100;
          break;

        case 101:
          putf(msaltermd);
          break;

        default:
          u->sfpwtype=type;
          putf("Password-type for S&F set to %s.\n", pwtypestr(type));          
      }

    } break;

    case binmode: // DH3MB
      if(*selektor)
      {

        if(stricmp(selektor, "AUTOBIN")==0)
          u->binmode=BINMODE_AUTOBIN;
#ifdef FEATURE_YAPP
        else if(stricmp(selektor, "YAPP")==0)
          u->binmode=BINMODE_YAPP;
#endif
#ifdef FEATURE_DIDADIT
        else if(stricmp(selektor, "DIDADIT")==0)
          u->binmode=BINMODE_DIDADIT;
#endif

        else
        { putf("Syntax: ALTER BINMODE AUTOBIN"
#ifdef FEATURE_YAPP
               "|YAPP"
#endif
#ifdef FEATURE_DIDADIT
               "|DIDADIT"
#endif
               "\n");
          break;
        }
      }

      putf("Binary parts of mails will be sent using the ");

      switch(u->binmode)
      {
#ifdef FEATURE_DIDADIT
      case BINMODE_DIDADIT:
        putf("DIDADIT");
        break;
#endif

#ifdef FEATURE_YAPP
      case BINMODE_YAPP:
        putf("YAPP");
        break;
#endif

      default:
        putf("AutoBIN");

      }

      putf(" protocol.\n");
      break;

  }
#ifdef _EXDLOGIN	//db1ras
  abfrageende:
#endif
  saveuser(u);
  if (del_user_cache)
    us_deletecache();
  return 1;
}

/*---------------------------------------------------------------------------*/
