/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  ---------------------------------
  Linux Kernel AX25 Support for BCM  
  ---------------------------------

  Copyright (C)       Deti Fliegl, deti@gmx.de, DG9MHZ

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980810 DH3MB  fixed bug of single cr's
//19980824 CZ4WAY added multiple ax-interfaces 
// CZ4WAY (CB) = Jan Wasserbauer <jan.wasserbauer@spsselib.hiedu.cz>
//19980831 OE3DZW merged both versions, used indent to get a common code
//                addrlen is SIGNED int!
//19980914 OE3DZW added unsigned for glibc (from 1.40c, dh3mb)
//19980917 OE3DZW made Jans interface optional (if _JAN is set)
//19980924 OE3DZW fixes by Jan
//19990205 DG9MHZ included Jans improvements again; fixes

#include "baycom.h"

#ifdef _AX25K

#include <sys/socket.h>
#include <linux/ax25.h>
#include <sys/ioctl.h>

#define TXLEN 512
#define RXLEN 512

/*---------------------------------------------------------------------------*/

typedef struct sock_t
{
  char rxbuf[RXLEN];
  char txbuf[TXLEN];
  char mycall[16];
  int handle;
  int rxc;
  int txc;
  int rxlen;
  int wasput;
  int wasda;
  char dasda;
  time_t lastflush;
}
sock_t;

#define MAXSOCK 256
#define MAXMYCALLS 4
#define MAXLSOCK 20

static sock_t so[MAXSOCK];
static int ax25k_ok = 0;
static int ax25k_default = 0;

static char ax25k_default_port[80];
static int listen_sock[MAXLSOCK];
static int listen_sock_num = 0;

/*---------------------------------------------------------------------------*/

static int find_free_sock(void)
{
  for (int i = 1; i < MAXSOCK; i++)
    if (so[i].handle == EOF)
    {
      memset(&so[i], 0, sizeof(sock_t));
      //so[i].handle=EOF;
      return i;
    }
  return EOF;
}

/*---------------------------------------------------------------------------*/

static char *ax2asc(ax25_address * a)
{
  static char buf[11];
  char c, *s;
  int n;

  for (n = 0, s = buf; n < 6; n++)
  {
    c = (a->ax25_call[n] >> 1) & 0x7F;
    if (c != ' ')
      *s++ = c;
  }
  *s++ = '-';
  if ((n = ((a->ax25_call[6] >> 1) & 0x0F)) > 9)
  {
    *s++ = '1';
    n -= 10;
  }
  *s++ = n + '0';
  *s++ = '\0';
  return buf;
}

/*---------------------------------------------------------------------------*/

static int convert_call_entry(const char *name, char *buf)
{
  int ct = 0;
  int ssid = 0;
  const char *p = name;
  char c;

  while (ct < 6)
  {
    c = toupper(*p);
    if (c == '-' || c == '\0')
      break;
    if (!isalnum(c))
    {
      trace(serious, "convert_call_entry", "invalid symbol in callsign %s", name);
      return EOF;
    }
    buf[ct] = c << 1;
    p++;
    ct++;
  }
  while (ct < 6)
  {
    buf[ct] = ' ' << 1;
    ct++;
  }
  if (*p != '\0')
  {
    p++;

    if (sscanf(p, "%d", &ssid) != 1 || ssid < 0 || ssid > 15)
    {
      trace(serious, "convert_call_entry", "SSID must follow '-' and be numeric in the range 0-15 - %s", name);
      return -1;
    }
  }
  buf[6] = ((ssid + '0') << 1) & 0x1E;
  return 0;
}

/*---------------------------------------------------------------------------*/

static int convert_call_arglist(char *call, struct full_sockaddr_ax25 *sax)
{
  char *bp;
  char *addrp;
  char *seps = " ";
  int n = 0;

  addrp = sax->fsa_ax25.sax25_call.ax25_call;
  if ((bp = strtok(call, seps)) != NULL)
    do
    {
      /* Process the token */
      if (convert_call_entry(bp, addrp) == -1)
	return -1;
      n++;
      if (n == 1)
	addrp = sax->fsa_digipeater[0].ax25_call;	/* First digipeater address */
      else
	addrp += sizeof(ax25_address);
    }
    while (n < AX25_MAX_DIGIS && ((bp = strtok(NULL, seps)) != NULL));

  /* Tidy up */
  sax->fsa_ax25.sax25_ndigis = n - 1;
  sax->fsa_ax25.sax25_family = AF_AX25;
  return sizeof(struct full_sockaddr_ax25);
}

/*---------------------------------------------------------------------------*/

static int linkcall(struct full_sockaddr_ax25 *addr, char *s)
{
  sprintf(s, "%s ", ax2asc(&addr->fsa_ax25.sax25_call));
#ifdef _DEBUG
  printf("ndigis: %d -> %s\n", addr->fsa_ax25.sax25_ndigis, s);
  for (int i = 0; i < addr->fsa_ax25.sax25_ndigis; i++)
  {
    printf("digi: %d ", i);
    for (int j = 0; j < 6; j++)
      printf("%02X ", addr->ax25.fsa_digipeater[i].ax25_call[j]);
    printf("\n");
  }
#endif

  if (addr->fsa_ax25.sax25_ndigis < AX25_MAX_DIGIS)
  {
    if (addr->fsa_ax25.sax25_ndigis > 0)
      strcat(s, ax2asc(&addr->fsa_digipeater[addr->fsa_ax25.sax25_ndigis - 1]));
  }
  else
  {
    //trace(serious,"linkcall","ndigis: %d",addr->fsa_ax25.sax25_ndigis);
    return 0;
  }
  return 1;
}

/*---------------------------------------------------------------------------*/

static int holsock(sock_t * s)
{
  if (t->port == EOF)
    suicide();

  if (s->rxc >= s->rxlen)
  {
    fd_set rfds;
    struct timeval tv;

    tv.tv_sec = 0;
    tv.tv_usec = 0;
    FD_ZERO(&rfds);
    FD_SET(s->handle, &rfds);

    if (!select(s->handle + 1, &rfds, NULL, NULL, &tv))
      return EOF;

    if ((s->rxlen = read(s->handle, s->rxbuf, RXLEN)) < 1)
    {
      if ((s->rxlen == EOF || errno == ENOTCONN || errno == EBADF) && errno != EAGAIN)
      {
	suicide();
      }
      return EOF;
    }
    s->rxc = 0;
  }
  return s->rxbuf[s->rxc++];
}

/*---------------------------------------------------------------------------*/

int ax25k_putfree()
{
  return 1;
}

/*---------------------------------------------------------------------------*/

void ax25k_putv(int c)
{
  if (t->port == EOF)
    suicide();
  sock_t *s = &so[t->port];
  s->txbuf[s->txc++] = c;
  s->wasput = 1;
  if (s->txc >= TXLEN)
    ax25k_flush();
}

/*---------------------------------------------------------------------------*/

void ax25k_flush()
{
  int ok = 0;
  int sum = 0;

#ifdef _PACLEN
  b->pacbytes = 0;
#endif

  if (t->port == EOF)
    suicide();

  sock_t *s = &so[t->port];

  if (s->txc)
  {
    while (1)
    {
      if (t->port == EOF)
	suicide();
      fd_set wfds;
      struct timeval tv;

      tv.tv_sec = 0;
      tv.tv_usec = 0;
      FD_ZERO(&wfds);
      FD_SET(s->handle, &wfds);

      if (select(s->handle + 1, NULL, &wfds, NULL, &tv))
      {
	ok = write(s->handle, s->txbuf + sum, s->txc - sum);
	if (ok < 0)
	{
	  if (errno != EAGAIN && errno != EINTR)
	  {
	    suicide();
	  }
	}
	else
	  sum += ok;
	if (sum == s->txc)
	  break;
      }
      wdelay(4);
    }
    s->txc = 0;
  }
}

/*---------------------------------------------------------------------------*/

void ax25k_putbeacon(char *addr, char *beacontext)
{
  char *r = "ax25k_putbeacon";
  int sock, addrlen = sizeof(struct full_sockaddr_ax25);
  struct full_sockaddr_ax25 axbind, axconnect;
  char *port;
  char *path;
  char *u;

  if ((u = strchr(addr, ':')) != NULL)
  {
    port = strtok(u + 1, " ");
    path = u + strlen(port) + 2;
    for (; *path == ' '; path++) ;
  }
  else
  {
    port = ax25k_default_port;
    path = addr;
  }

  axconnect.fsa_ax25.sax25_family = axbind.fsa_ax25.sax25_family = AF_AX25;
  axbind.fsa_ax25.sax25_ndigis = 1;

  if (convert_call_entry(port, axbind.fsa_digipeater[0].ax25_call) == -1)
  {
    trace(serious, r, "invalid AX.25 interface callsign - %s", addr);
    return;
  }
  if (convert_call_entry(m.mycall[0], axbind.fsa_ax25.sax25_call.ax25_call) == -1)
  {
    trace(serious, r, "invalid callsign - %s", m.mycall[0]);
    return;
  }
  if (convert_call_arglist(path, &axconnect) == -1)
  {
    trace(serious, r, "invalid destination callsign or digipeater");
    return;
  }
  if ((sock = socket(AF_AX25, SOCK_DGRAM, 0)) < 0)
  {
    trace(serious, r, "cannot open AX.25 socket, %s", strerror(errno));
    return;
  }

  if (bind(sock, (struct sockaddr *) &axbind, addrlen) != 0)
  {
    trace(serious, r, "cannot bind AX.25 socket, %s", strerror(errno));
    close(sock);
    return;
  }

  if (connect(sock, (struct sockaddr *) &axconnect, addrlen) != 0)
  {
    switch (errno)
    {
    case ECONNREFUSED:
      trace(report, r, "*** Connection refused - aborting");
      break;
    case ENETUNREACH:
      trace(report, r, "*** No known route - aborting");
      break;
    case EINTR:
      trace(report, r, "*** Connection timed out - aborting");
      break;
    default:
      trace(report, r, "cannot connect to %s, %s", addr, strerror(errno));
      break;
    }
    close(sock);
    return;
  }
  write(sock, beacontext, strlen(beacontext));
  close(sock);
}

/*---------------------------------------------------------------------------*/

int ax25k_getvalid_tnc()
{
  int a;

  if (t->port == EOF)
    suicide();

  sock_t *s = &so[t->port];

  if (s->wasda)
  {
    if (s->dasda == 13)
      return CR;
    else
      return 1;
  }
  //a = holsock (s->handle);
  a = holsock(s);
  if (a != EOF)
  {
    s->wasda = 1;
    s->dasda = (char) a;
  }
  if (s->wasda)
  {
    if (s->dasda == 13)
      return CR;
    else
      return 1;
  }
  return 0;
}

/*---------------------------------------------------------------------------*/

int ax25k_getv()
{
  int a;

  if (t->port == EOF)
    suicide();

  sock_t *s = &so[t->port];
  if (s->wasda)
  {
    s->wasda = 0;
    return s->dasda;
  }
  while ((a = holsock(s)) == EOF)
    wdelay(47);
  return a;
}

/*---------------------------------------------------------------------------*/

void ax25k_getclear()
{
  sock_t *s = &so[t->port];
  while (holsock(s) != EOF) ;	// RX-Puffer ausleeren

}

/*---------------------------------------------------------------------------*/

void ax25k_getmycall(char *s)
{
  strcpy(s, so[t->port].mycall);
}

/*---------------------------------------------------------------------------*/

void ax25k_inputline(char *s, int maxlen, char cut)
{
  int i = 0;
  int semi = 0;

  if (maxlen < 0)
  {
    if (cut)
      semi = 1;
    maxlen = (-maxlen);
  }

  if (so[t->port].wasput && !ax25k_getvalid_tnc())
    ax25k_flush();

  while (1)
  {
    if (!cut && i == maxlen)
      break;

    do
      s[i] = ax25k_getv();
    while (s[i] == LF);

    if (semi && s[i] == ';')
      break;

    if (s[i] == CR)
    {
      if (!cut)
	s[i++] = LF;
      break;
    }

    if ((s[i] == 8) || (s[i] == 127))
    {
      if (i)
	i--;
      continue;
    }

    if (i < maxlen)
      i++;
  }

  s[i] = 0;
}

/*---------------------------------------------------------------------------*/

int ax25k_connect(char *mycall, char *conncall)
{
  char *r = "ax25k_connect";
  int addrlen = sizeof(struct full_sockaddr_ax25);
  struct full_sockaddr_ax25 axbind, axconnect;
  int s;
  char *port;
  char *path;
  char *u;

  if ((u = strchr(conncall, ':')) != NULL)
  {
    port = strtok(u + 1, " ");
    path = u + strlen(port) + 2;
    for (; *path == ' '; path++) ;
  }
  else
  {
    port = ax25k_default_port;
    path = conncall;
  }

  axconnect.fsa_ax25.sax25_family = axbind.fsa_ax25.sax25_family = AF_AX25;
  axbind.fsa_ax25.sax25_ndigis = 1;

  if (convert_call_entry(port, axbind.fsa_digipeater[0].ax25_call) == -1)
  {
    trace(serious, r, "invalid AX.25 port callsign - %s", port);
    return 0;
  }
  if (convert_call_entry(mycall, axbind.fsa_ax25.sax25_call.ax25_call) == -1)
  {
    trace(serious, r, "invalid callsign - %s", mycall);
    return 0;
  }
  if (convert_call_arglist(path, &axconnect) == -1)
  {
    trace(serious, r, "invalid destination callsign or digipeater %s", path);
    return 0;
  }
  if ((s = socket(AF_AX25, SOCK_SEQPACKET, 0)) < 0)
  {
    trace(serious, r, "cannot open AX.25 socket, %s", strerror(errno));
    return 0;
  }
  if (fcntl(s, F_SETFL, fcntl(s, F_GETFL) | O_NONBLOCK) < 0)
  {
    trace(serious, r, "fcntl: %s", strerror(errno));
    close(s);
    return 0;
  }
  if (bind(s, (struct sockaddr *) &axbind, addrlen) != 0)
  {
    trace(serious, r, "cannot bind AX.25 socket, %s", strerror(errno));
    close(s);
    return 0;
  }
  if (connect(s, (struct sockaddr *) &axconnect, addrlen) != 0)
  {
    if (errno != EINPROGRESS)
    {
      trace(report, r, "conn to %s, %s", conncall, strerror(errno));
      close(s);
      return 0;
    }
  }

  fd_set wfds, cfds;
  struct timeval tv;

  tv.tv_sec = 0;
  tv.tv_usec = 0;
  FD_ZERO(&wfds);
  FD_ZERO(&cfds);

  int ret = 0;
  do
  {
    FD_SET(s, &wfds);
    FD_SET(s, &cfds);
    wdelay(243);
  }
  while (!(ret = select(s + 1, NULL, &wfds, &cfds, &tv)));

  if (FD_ISSET(s, &cfds))
  {
    trace(report, r, "fail %s %s", path, strerror(errno));
    close(s);
    return 0;
  }

  int socknum = find_free_sock();
  if (socknum == -1)
  {
    trace(serious, r, "socket array full");
    close(s);
    return 0;
  }
  so[socknum].handle = s;
  strcpy(so[socknum].mycall, mycall);
  t->port = socknum;
  t->processkind |= P_AX25K;
  trace(report, r, "ok %s", path);
  return 1;
}

/*---------------------------------------------------------------------------*/

void ax25k_disconnect(int wait)
{
  if (t->port == EOF)
    return;
  close(so[t->port].handle);
  so[t->port].handle = EOF;
  t->port = EOF;
}

/*---------------------------------------------------------------------------*/

void ax25k_checklogin()
{
  int i;
  struct full_sockaddr_ax25 newconn;
#ifdef __GLIBC__
  unsigned
#endif
  int addrlen;
  int ret;
  char *r = "ax25k_checklogin";
  if (!ax25k_ok)
    return;

  fd_set rfds;
  struct timeval tv;

  tv.tv_sec = 0;
  tv.tv_usec = 0;
  FD_ZERO(&rfds);
  int maxsock = 0;

  for (i = 0; i < listen_sock_num; i++)
  {
    if (listen_sock[i] > maxsock)
      maxsock = listen_sock[i];
    FD_SET(listen_sock[i], &rfds);
  }

  if (select(maxsock + 1, &rfds, NULL, NULL, &tv) <= 0)
    return;

  for (i = 0; i < listen_sock_num; i++)
  {
    if (!FD_ISSET(listen_sock[i], &rfds))
      continue;

    // from ax25d: 
    /* 
     * Setting up a non-blocking accept() so is does not hang up
     *  - I am not sure at this time why I didn't/don't assign
     *  the socket non-blocking to start with.
     */
    // in bcm sockets are assigned non-blocking in ax25k_init
    // -> FIONBIO not not needed
    /*      int j = TRUE;
       ioctl(listen_sock[i], FIONBIO, &j); */

    addrlen = sizeof(struct full_sockaddr_ax25);
    ret = accept(listen_sock[i], (struct sockaddr *) &newconn, &addrlen);

/*      j = FALSE;
   ioctl(listen_sock[i], FIONBIO, &j); */

    if (ret >= 0)
    {
      char buf[80];
      //printf("login: %d, %d addrlen: %d\n", i, ret, addrlen);
      if (!linkcall(&newconn, buf))
      {
	close(ret);
	break;
      }
      buf[30] = 0;
      if (fcntl(ret, F_SETFL, fcntl(ret, F_GETFL) | O_NONBLOCK) < 0)
      {
	trace(serious, r, "fcntl: %s", strerror(errno));
	close(ret);
	break;
      }
      int socknum = find_free_sock();
      if (socknum == -1)
      {
	trace(serious, r, "socket array full");
	close(ret);
	break;
      }

      so[socknum].handle = ret;
      strcpy(so[socknum].mycall, m.mycall[i % m.mycalls]);
      fork(P_MAIL | P_AX25K, socknum, boxusertask, buf);
    }
    else
    {
      trace(report, r, "accept: %d %s", errno, strerror(errno));
    }
  }
}

/*---------------------------------------------------------------------------*/

int ax25k_isdefault()
{
  return ax25k_default;
}

/*---------------------------------------------------------------------------*/

int ax25k_init()
{
  static int sockinit = 0;
  char *r = "ax25k_init";
  char port[80];
  int i;
  int addrlen = sizeof(struct full_sockaddr_ax25);
  struct full_sockaddr_ax25 axbind;

  if (!sockinit)
  {
    sockinit++;
    for (i = 0; i < MAXSOCK; i++)
      so[i].handle = EOF;
  }

  if (!strcasecmp(m.ax25k_if, "off"))
  {
    trace(report, r, "Kernel AX25 disabled");
    return 0;
  }

  axbind.fsa_ax25.sax25_family = AF_AX25;
  axbind.fsa_ax25.sax25_ndigis = 1;

  int j = 0;
  int k = 0;

  // remove trailing spaces - would be interpreted as "empty" interfaces
  i = strlen(m.ax25k_if);
  while (i && m.ax25k_if[i - 1] == ' ')
    i--;
  m.ax25k_if[i] = 0;

  for (i = 0; i <= strlen(m.ax25k_if); i++)
  {
    if (m.ax25k_if[i] != ' ' && m.ax25k_if[i] != 0)
    {
      port[j] = m.ax25k_if[i];
      j++;
    }
    else
    {
      port[j] = 0;
      trace(report, r, "Initializing Interface %s", port);

      if (m.ax25k_if[i] == ' ' && toupper(m.ax25k_if[i + 1]) == 'D'
	  && (m.ax25k_if[i + 2] == ' ' || m.ax25k_if[i + 2] == 0))
      {
	ax25k_default = 1;
	strcpy(ax25k_default_port, port);
	i += 2;
      }

      if (convert_call_entry(port, axbind.fsa_digipeater[0].ax25_call) == -1)
      {
	trace(serious, r, "invalid AX.25 interface callsign - %s", m.ax25k_if);
	return 0;
      }

      for (j = 0; j < m.mycalls; j++)
      {
	if (convert_call_entry(m.mycall[j], axbind.fsa_ax25.sax25_call.ax25_call) == -1)
	{
	  trace(serious, r, "invalid callsign - %s", m.mycall[j]);
	  return 0;
	}

	if ((listen_sock[k] = socket(AF_AX25, SOCK_SEQPACKET, 0)) < 0)
	{
	  trace(serious, r, "cannot open AX.25 socket, %s", strerror(errno));
	  return 0;
	}
	if (fcntl(listen_sock[k], F_SETFL, fcntl(listen_sock[k], F_GETFL) | O_NONBLOCK) < 0)
	{
	  trace(serious, r, "fcntl: %s", strerror(errno));
	  close(listen_sock[k]);
	  return 0;
	}
	if (bind(listen_sock[k], (struct sockaddr *) &axbind, addrlen) != 0)
	{
	  trace(serious, r, "cannot bind AX.25 socket, %s", strerror(errno));
	  close(listen_sock[k]);
	  return 0;
	}
	if (listen(listen_sock[k], 5) != 0)
	{
	  trace(serious, r, "cannot listen to AX.25 socket, %s", strerror(errno));
	  close(listen_sock[k]);
	  return 0;
	}
	k++;
      }
      j = 0;
    }
  }

  ax25k_ok = 1;
  listen_sock_num = k;

  trace(report, r, "Kernel AX25 enabled");
  if (ax25k_default)
    trace(report, r, "Using Default Interface %s", ax25k_default_port);
  return 1;
}

/*---------------------------------------------------------------------------*/

void ax25k_shutdown()
{
  int i;

  if (!ax25k_ok)
    return;

  for (i = 0; i < listen_sock_num; i++)
    close(listen_sock[i]);

  for (i = 0; i < MAXSOCK; i++)
  {
    if (so[i].handle != EOF)
    {
      //trace (serious, "ax25k_shutdown", "open socket %d %d", i, so[i].handle);
      close(so[i].handle);
    }
  }
}

/*---------------------------------------------------------------------------*/

#endif
