/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  ---------------------------
  Verwaltung von Bulletin-IDs
  ---------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980208 OE3DZW changed output format for bid -i
//19980208 OE3DZW fixed above format
//19980223 OE3DZW fixed fsetopt 
//19990219 OE3DZW fixed check for unique bid
//19991111 OE3DZW 11:11h ;-) removed check for length of BID

#include "baycom.h"

/*---------------------------------------------------------------------------*/

#define HLEN 4
#define BID_FILENAMELEN 16

#define nextpos(fptr) (filelength(fileno(fptr))/sizeof(BID))

/*---------------------------------------------------------------------------*/

// Needed to get the temp-BIDs
extern task_t *tfeld[TASKS];

/*---------------------------------------------------------------------------*/

typedef FILE *fp;
static int is_null=0;

/*---------------------------------------------------------------------------*/

typedef struct
{ char name[BIDLEN+1];
  char filename[BID_FILENAMELEN+1];
  long nextsamehash;
} BID;

/*---------------------------------------------------------------------------*/

typedef struct
{ char name[BIDLEN+1];
  short unsigned nextsamehash;
  char filename[BID_FILENAMELEN+1];
} BID_OLD;

/*---------------------------------------------------------------------------*/

typedef struct
{ char reserved[BIDLEN+BID_FILENAMELEN];
  short unsigned lastbid;                 // letzte vergebene BID-Nummer
  long filesize;                          // Gre des BID-File
} KOPF;

/*---------------------------------------------------------------------------*/

long bid_len(void)
{ return filesize(BIDNAME)/sizeof(BID);
}

/*---------------------------------------------------------------------------*/

static void near bidfile_newformat(void)
{ FILE *fi;
  FILE *fo;
  BID b;
  BID_OLD bo;

  if(filetime(BIDNAME) < filetime(BIDNAME_OLD))
  { trace(replog,"bid","converting format");
    fi=s_fopen(BIDHASHNAME_OLD,"srb");
    fo=s_fopen(BHTMP,"swb");
    if (fo)
      s_fsetopt(fo,1);
    if(fi && fo)
    { struct { short unsigned l;short unsigned h; } b={0,0};
      while(fread(&b.l,2,1,fi))
        fwrite(&b,4,1,fo);
    }
    if(fo)s_fclose(fo);
    if(fi)s_fclose(fi);
    fi=s_fopen(BIDNAME_OLD,"srb");
    fo=s_fopen(BTMP,"swb");
    if (fo)
       s_fsetopt(fo,1);
    if(fi && fo)
    { while(fread(&bo,sizeof(BID_OLD),1,fi))
      { strcpy(b.name,bo.name);
        strcpy(b.filename,bo.filename);
        b.nextsamehash=bo.nextsamehash;
        fwrite(&b,sizeof(BID),1,fo);
      }
    }
    if(fi)s_fclose(fi);
    if(fo)s_fclose(fo);
    xunlink(BIDNAME);
    xunlink(BIDHASHNAME);
    xrename(BTMP,BIDNAME);
    xrename(BHTMP,BIDHASHNAME);
  }
}

/*---------------------------------------------------------------------------*/

void tmpbidrein(char *bid,int number)
//*************************************************************************
//
//  Locks a BID temporarily.
//  This prevents, that we accept a BID, which we are currently
//  receiving on another channel
//
//  (DG4MMI)
//
//*************************************************************************
{ strcpy(b->tmpbid[number],bid);
}

/*---------------------------------------------------------------------------*/

int tmpbidvorhanden(char *bid)
//*************************************************************************
//
//  Checks, if we receive a specific BID at the moment
//
//  (DG4MMI)
//
//*************************************************************************
{ int i,j;
  task_t *t;

  for(i=0;i<TASKS;i++)
  { t=tfeld[i];
    if(t)
    { tvar_t *bb=(tvar_t *)t->b;

      if(bb)
        for(j=0;j<5;j++)
          if(strcmp(bb->tmpbid[j],bid)==0)
            return 1;
    }
  }

  return 0;
}

/*---------------------------------------------------------------------------*/

void tmpbidraus(int number)
//*************************************************************************
//
//  Deletes a temporary BID
//
//  (DG4MMI)
//
//*************************************************************************
{ b->tmpbid[number][0]=0;
}

/*---------------------------------------------------------------------------*/

static void near listtmpbid(char *bid)
//*************************************************************************
//
//  Lists all temporary BIDs (i.e. the BIDs, which we are receiving at the
//  moment)
//
//  (DG4MMI)
//
//*************************************************************************
{ int i,j, firstfound;
  task_t *t;

  putf("Incoming BIDs:\n");

  for(i=0;i<TASKS;i++)
  { t=tfeld[i];
    if(t)
    { tvar_t *bb=(tvar_t *)t->b;

      if(bb)
      { firstfound=1;
        for(j=0;j<5;j++)
        { if(bb->tmpbid[j][0] && strstr(bb->tmpbid[j], bid))
          { if(firstfound)
              putf("\n%-6s: ", bb->logincall);
            else
              putf("        ");
            putf("$%-12s", bb->tmpbid[j]);
            if(firstfound)
            { putf(" (%c %-6s > %-8s @ %.22s)",bb->mailtype, 
                 bb->herkunft,bb->ziel,bb->at);
              firstfound=0;
            }
            putf("\n");
          }
        }
      }
    }
  }
}

/*---------------------------------------------------------------------------*/

static void near tryopen(int first,fp &bidf,fp &bidhf)
//*************************************************************************
//
//  ffnet Bid-File und Hash-File
//  ggf. werden die Files neu angelegt, wenn sie nicht geffnet
//  werden knnen
//
//*************************************************************************
{ KOPF kopf={"",0,0};             // Speicher fr BID-File Kopf

  bidf=s_fopen(BIDNAME,"sr+b");
  if(!bidf)
  { if(first)
    { bidf=s_fopen(BIDNAME,"sw+b");
      if (bidf)
        s_fsetopt(bidf,1);
      fwrite(&kopf,sizeof(KOPF),1,bidf);
      xunlink(BIDHASHNAME);
    }
    else
      trace(tr_abbruch,"bid","fopen errno=%d",errno);
  }
  bidhf=s_fopen(BIDHASHNAME,"sr+b");
  if(!bidhf)
  { if(first)
    { bidhf=s_fopen(BIDHASHNAME,"sw+b");
      if (bidhf)
        s_fsetopt(bidhf,1);
    }
    if(!bidhf)
      trace(tr_abbruch,"bidhash","fopen errno=%d",errno);
  }
  first=0;
}

/*---------------------------------------------------------------------------*/
#ifndef _BCMNET_FWD
char *newbid(int reset)
//*************************************************************************
//
//  Erzeugt ein neues Bulletin-ID (aus Datum, Call und laufender Nummer)
//
//*************************************************************************
{ static char bid[BIDLEN+1];
  struct tm *d;
  time_t t;
  static KOPF k={"",0,0};
  FILE *bidf;           // offenes BID-File
  FILE *bidhf;          // offenes BID-Hashfile
  char callbuf[7]; // DG4MMI

  tryopen(0,bidf,bidhf);

  rewind(bidf);
  fread(&k,sizeof(k),1,bidf);
  if(reset)
    k.lastbid=0;
  else
    k.lastbid=(k.lastbid+1)%(36U*36*36);
  rewind(bidf);
  fwrite(&k,sizeof(k),1,bidf);
  s_fclose(bidf);
  s_fclose(bidhf);

  // DG4MMI: Fill up the boxname to 6 characters
  strcpy(callbuf,m.boxname);
  while(strlen(callbuf)<6)
    strcat(callbuf, "_");

  t=ad_time();
  d=ad_comtime(t);

//format:
// <day><month><year><call><num>  <num>.. 0..9,A..Z
// DMYCCCCCC123
// 012345678901

  sprintf(bid,"%c%1X%1d%s%c%c%c",deznib(d->tm_mday),d->tm_mon+1,d->tm_year%10,
	  callbuf,
	  deznib((k.lastbid/(36*36))),
	  deznib((k.lastbid/36)%36),
	  deznib((k.lastbid)   %36));
  while(bidvorhanden(bid))
  { if(bid[9]<'Z')
      bid[9]++;
    else trace(fatal,"newbid","unique BID impossible %s",bid);
  }
  return bid;
}
#else
char *newbid (int reset)
//*************************************************************************
//
//  Erzeugt eine neue CB-BCMNET Bulletin-ID
//
//*************************************************************************
{ static char bid[BIDLEN+1];
  struct tm *d;
//  static int lastday = 0;
  static KOPF k = { "", 0, 0 };
  FILE *bidf;      // offenes BID-File
  FILE *bidhf;    // offenes BID-Hashfile
  char callbuf[7]; // DH3MB
  short unsigned lbid;

  d = ad_comtime (ad_time());
//  if ( ! lastday) lastday = d->tm_mday;
  tryopen (0, bidf, bidhf);
  rewind (bidf);
  fread (&k, sizeof (k), 1, bidf);
  s_fclose (bidf);
  s_fclose (bidhf);
//  if (lastday != d->tm_mday)
//  {
//    k.lastbid = 0;
//    lastday = d->tm_mday;
//  }
  if(reset)
    k.lastbid=0;
  else
  {
    if (k.lastbid >= (36U * 36U)) k.lastbid = 0;
  }
  // DH3MB: Fill up the boxname to 6 characters
  strcpy (callbuf, m.boxname);
  while (strlen (callbuf) < 6) strcat (callbuf, "_");
  do {
    lbid = ++k.lastbid;
// the BID/MID for the CB-BCMNET is built as following
// <day><year><month><call>_<num>
// DYMCCCCCC_23
    sprintf (bid, "%c%c%1X%s_%c%c",
                   deznib (d->tm_mday),
                     deznib (d->tm_year - 90),  //this is a 2026er bug ;-)
                       d->tm_mon + 1,
                          callbuf,
                              deznib ((lbid / 36) % 36),
                                deznib ((lbid) % 36));
  } while (bidvorhanden (bid));
  tryopen (0, bidf, bidhf);
  rewind (bidf);
  fwrite (&k, sizeof (k), 1, bidf);
  s_fclose (bidf);
  s_fclose (bidhf);
  return bid;
}
#endif
/*---------------------------------------------------------------------------*/

static void near updatehash(FILE *bidhf,long index,BID *bid)
//*************************************************************************
//
//  Trgt einen neuen Eintrag in die Hashtabelle ein
//  und verkettet ihn mit evtl bereits bestehenden Eintrgen
//  Dabei wird der neue Eintrag stets zu dem Eintrag, auf den
//  der Hashzeiger zeigt. Ggf. alte Eintrge werden vom neuen
//  Eintrag aus angesprochen.
//
//*************************************************************************
{ long unsigned hsh=strcrc(bid->name);           // CRC als Hashfunktion

  fseek(bidhf,hsh*HLEN,SEEK_SET);        // bestehenden Hashzeiger lesen
  fread(&bid->nextsamehash,HLEN,1,bidhf);
  fseek(bidhf,hsh*HLEN,SEEK_SET);        // neuen Zeiger schreiben
  if(fwrite(&index,HLEN,1,bidhf)!=1)
    trace(serious,"bidupd","fwrite $%s",bid->name);
}

/*---------------------------------------------------------------------------*/

char *bidvorhanden(char *bidname)
// nur fuer kompatibilitaet
{ return bidnewfilename(bidname,NULL);
}

/*---------------------------------------------------------------------------*/

char *bidnewfilename(char *bidname,char *newfilename)
//*************************************************************************
//
//  schaut nach, ob bidname schon im Bid-File enthalten ist
//  Zurckgegeben wird ein damit verbundener Filename, falls das
//  BID gefunden worden ist, oder NULL wenn das BID nicht vorhanden ist
//
//*************************************************************************
{ long unsigned hsh;
  long unsigned idx;
  static BID bidtmp;
  FILE *bidf=NULL;           // offenes BID-File
  FILE *bidhf=NULL;          // offenes BID-Hashfile

  if(bidname[0]==0)
    return NULL;
  strupr(bidname);
  hsh=strcrc(bidname);

  tryopen(0,bidf,bidhf);
  if(bidname[0])
  { fseek(bidhf,hsh*HLEN,SEEK_SET);
    if(fread(&idx,HLEN,1,bidhf)!=1)
      trace(serious,"bidvorhanden","hash %s",bidname);
    if(idx)
    { int num=0;
      while(idx)
      { fseek(bidf,idx*sizeof(BID),SEEK_SET);
	if(fread(&bidtmp,sizeof(BID),1,bidf)!=1)
	{
	  if(!sema_test("purgereorg"))
            fork(P_BACK|P_MAIL,0,mbreorg,"i");
          else
            reorg_noetig++;
          trace(fatal,"bidvorhanden","fread %s",bidname);
        }
	if((num++)>100)
	{ trace(serious,"bidvorhanden","loop");
	  s_fclose(bidf);
          s_fclose(bidhf);
	  if(!sema_test("purgereorg"))
            fork(P_BACK|P_MAIL,0,mbreorg,"i");
          else
            reorg_noetig++;
	  return "";
	}
	if(strcmp(bidname,bidtmp.name)==0)
	{ if(newfilename)
	  {
            if(strlen(newfilename)>BID_FILENAMELEN)
            { trace(serious,"bidnewfilename","file %s",newfilename);
              newfilename[BID_FILENAMELEN]=0;
            }
            strcpy(bidtmp.filename,newfilename);
	    fseek(bidf,idx*sizeof(BID),SEEK_SET);
	    fwrite(&bidtmp,sizeof(BID),1,bidf);
	  }
	  s_fclose(bidf);
          s_fclose(bidhf);
	  return bidtmp.filename;
	}
	idx=bidtmp.nextsamehash;
      }
    }
  }
  s_fclose(bidf);
  s_fclose(bidhf);
  return NULL;
}

/*---------------------------------------------------------------------------*/

unsigned bidrein(char *bidname,char *filename)
//*************************************************************************
//
//  legt ein BID im bullid-File ab
//
//  Achtung: es muss an der Stelle sichergestellt sein, dass nicht
//           schon ein BID mit gleichem Namen existiert!
//
//*************************************************************************
{ BID newbid;
  FILE *bidf=NULL;           // offenes BID-File
  FILE *bidhf=NULL;          // offenes BID-Hashfile

  strupr(bidname);
  if(bidname[0])
  { tryopen(0,bidf,bidhf);
    memset(&newbid,0,sizeof(BID));           // neuen Eintrag vorbesetzen
    if(strlen(bidname)>BIDLEN)
    { trace(serious,"bidrein","bid %s",bidname);
      bidname[BIDLEN]=0;
    }
    if(strlen(filename)>BID_FILENAMELEN)
    { trace(serious,"bidrein","file %s",filename);
      filename[BID_FILENAMELEN]=0;
    }
    strcpy(newbid.name,bidname);             // String hineinschreiben
    strcpy(newbid.filename,filename);
    fseek(bidf,0L,SEEK_END);
    updatehash(bidhf,ftell(bidf)/sizeof(BID),&newbid);
    if(fwrite(&newbid,sizeof(BID),1,bidf)!=1)
      trace(serious,"bidrein","fwrite $%s",newbid.name);
    if((ftell(bidf)/sizeof(BID))>((unsigned long)m.maxbids+10000))
    { if(!sema_test("purgereorg"))
        fork(P_BACK|P_MAIL,0,mbreorg,"i");
    }
    s_fclose(bidf);
    s_fclose(bidhf);
    return 1;
  }
  return 0;
}

/*---------------------------------------------------------------------------*/

static void near listhash(char *bidname)
//*************************************************************************
//
//  gibt hashcode zu <bidname> aus
//
//*************************************************************************
{ long unsigned hsh=strcrc(bidname);
  long unsigned idx;
  static BID bidtmp;
  FILE *bidf=NULL;           // offenes BID-File
  FILE *bidhf=NULL;          // offenes BID-Hashfile

  putf("Hashcode=%04X\n",hsh);
  if(tmpbidvorhanden(bidname))
    putf("BID in TMP buffer.\n");

  tryopen(0,bidf,bidhf);
  if(bidname[0])
  { fseek(bidhf,hsh*HLEN,SEEK_SET);
    fread(&idx,HLEN,1,bidhf);
    while(idx)
    { fseek(bidf,idx*sizeof(BID),SEEK_SET);
      fread(&bidtmp,sizeof(BID),1,bidf);
      putf("%-6lu%17s:%-12s --> %lu\n",idx,bidtmp.filename,bidtmp.name,
	   bidtmp.nextsamehash);
      idx=bidtmp.nextsamehash;
    }
  }
  s_fclose(bidf);
  s_fclose(bidhf);
}

/*---------------------------------------------------------------------------*/

void listbids(char *sel)
//*************************************************************************
//
//  Showas extracts of the BID-table
//
//  Options:
//     -c     Only counts the found entries
//     -k     Only lists entries with collisions
//     -h     gibt vollstndige Hashverkettung aus
//     -s     Input a BID manually
//     -i     Lists the BIDs, which are currently received (DG4MMI)
//
//*************************************************************************
{ BID bid;
  KOPF k;

  long count=0;
  FILE *bidf;

  scanoptions(sel);

  sel+=blkill(sel);
  strupr(sel);

  if(b->optplus & o_i) // DG4MMI
  { listtmpbid(sel);
    return;
  }

  if(b->optplus & o_h)
    listhash(sel);
  else if(b->optplus & o_s)
  { if(!bidvorhanden(sel))
      bidrein(sel,"DUMMY");
  }
  else
  { bidf=s_fopen(BIDNAME,"srb");
    if(bidf!=NULL)
    { fread(&k,sizeof(BID),1,bidf);          // delete first entry
      putf("Filesize=%lu lastbid=%u\n",nextpos(bidf),k.lastbid);
      while(fread(&bid,sizeof(BID),1,bidf))
      { if((!sel[0] || strstr(bid.filename,sel) || strstr(bid.name,sel)) &&
	   (!(b->optplus & o_k) || bid.nextsamehash))
	{ if(!(b->optplus & o_c))
	  { putf("%-6lu",(ftell(bidf)/sizeof(BID))-1);
	    putf("%17s:%-12s",bid.filename,bid.name);
	    if(bid.nextsamehash)
	    { long save=ftell(bidf);
	      putf(" --> %ld",bid.nextsamehash);
	      fseek(bidf,sizeof(BID)*(long)bid.nextsamehash,SEEK_SET);
	      fread(&bid,sizeof(BID),1,bidf);
	      putf(" (%s) %ld",bid.name,bid.nextsamehash);
	      fseek(bidf,save,SEEK_SET);
	    }
	    putf("\n");
	  }
	  count++;
	}
	if(testabbruch())
	  break;
	waitfor(e_ticsfull);
      }
      s_fclose(bidf);
    }
    putf("%ld found.\n",count);
  }
}

/*---------------------------------------------------------------------------*/

void bidinit(int begin)
//*************************************************************************
//
//  Zugriff BID-File vorbereiten
//
//*************************************************************************
{ FILE *bidf;           // offenes BID-File
  FILE *bidhf;          // offenes BID-Hashfile

  if(sizeof(BID)!=sizeof(KOPF) || is_null)
    trace(tr_abbruch,"BID","size %d %d",sizeof(BID),sizeof(KOPF));

  if(begin) bidfile_newformat();
  tryopen(1,bidf,bidhf);

  if(!bidf)
  { trace(tr_abbruch,"bidinit","fopen bidf errno=%d",errno);
    return;
  }
  if(bidhf)
    hashinit(fileno(bidhf),HASH32);
  else
  { trace(tr_abbruch,"bidinit","fopen bidhf errno=%d",errno);
    return;
  }
  s_fclose(bidf);
  s_fclose(bidhf);
}

/*---------------------------------------------------------------------------*/

void reorgbid(void)
//*************************************************************************
//
//  Reorganises BID-file and generates the hashing file new.
//
//*************************************************************************
{ FILE *balt,*bneu,*bhneu;
  int error;
  KOPF kopf={"",0,0};
  BID bid;
  long currpos,maxpos;
  unsigned e_nlen=0,e_flen=0;
  int len,i;

  if(sema_lock("sendlock")==0)
    return;

  trace(replog,"reorg","bidlist");
  lastcmd("BID-Reorg");
  unlink(BTMP);                             // remove old temporary files
  unlink(BHTMP);
  balt=s_fopen(BIDNAME,"lrb");            // the current BID file
  if(dfree(".",0) < (filelength(fileno(balt))/1024UL+HOMEDISKFULL))
  { s_fclose(balt);
    trace(serious,"reorgbid","disk full");
    return;
  }
  bneu=s_fopen(BTMP,"lw+b");         // open temporary write file
  if (bneu)
    s_fsetopt(bneu,1);
  bhneu=s_fopen(BHTMP,"lw+b");
  if (bhneu)
    s_fsetopt(bhneu,1);
  if((balt!=NULL) && (bneu!=NULL) && (bhneu!=NULL))
  { setvbuf(balt,NULL,_IOFBF,8192);     // buffer for increasing performance
    setvbuf(bneu,NULL,_IOFBF,8192);

    hashinit(fileno(bhneu),HASH32);     // fill new hash file with zero value

    maxpos=nextpos(balt);
    if(maxpos>m.maxbids)           // if more than maxbids
      currpos=maxpos-m.maxbids;    // the beginning of the file
    else
      currpos=1;

    fread(&kopf,sizeof(KOPF),1,balt);            // read old header
    fwrite(&kopf,sizeof(KOPF),1,bneu);           // write first header
    fseek(balt,currpos*sizeof(BID),SEEK_SET);

    while(currpos!=maxpos)
    { waitfor(e_ticsfull);                // do not stand still
      error=0;

      if(!fread(&bid,sizeof(BID),1,balt))
	break;
      currpos++;
      len=strlen(bid.name);
      if(bid.name[BIDLEN] || (len>BIDLEN))
      { error=1;
	e_nlen++;
      }
      else
      { for(i=0;i<len;i++)
        { if((bid.name[i]<'!')||(bid.name[i]>128))
	    error=1;
        }
      }
      len=strlen(bid.filename);
      if(bid.filename[BID_FILENAMELEN] || (len>BID_FILENAMELEN) || (len<3))
      { error=1;
	e_flen++;
      }
      else
      { for(i=0;i<len;i++)
        { if((bid.filename[i]<'!')||(bid.filename[i]>127))
	    error=1;
        }
      }
      if(!error)
      { fseek(bneu,0L,SEEK_END);
        updatehash(bhneu,ftell(bneu)/sizeof(BID),&bid);
	fwrite(&bid,sizeof(BID),1,bneu);
      }
      else
      { if((e_nlen+e_flen)<20)
          trace(report,"reorgbid","er:%1.20s:%1.20s",bid.name,bid.filename);
      }
      if((currpos%10000)==0)
      { char s[40];
	sprintf(s,"bid %lu/%lu:%s",currpos,maxpos,bid.name);
	trace(report,"reorg",s);  // damits dem Zuschauer nicht fad wird
	lastcmd(s);               // ... und auch im PS auftaucht
      }
    }
    currpos=nextpos(bneu);
    if(currpos) currpos--;

    trace(replog,"reorgbid","%lu bids, %u/%u errors",currpos,e_nlen,e_flen);
    s_fclose(balt);
    s_fclose(bneu);
    s_fclose(bhneu);
    xunlink(BBAK);
    xunlink(BHBAK);
    xrename(BIDNAME,BBAK);
    xrename(BIDHASHNAME,BHBAK);
    xrename(BTMP,BIDNAME);
    xrename(BHTMP,BIDHASHNAME);
    sema_unlock("sendlock");
  }
}
/*---------------------------------------------------------------------------*/

