/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -------------------------
  Ausgabe des CHECK-Befehls
  -------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980118 OE3DZW removed sec in timestr
//19980223 OE3DZW fixed fsetopt
//19980308 OE3DZW check for mails > 100000 bytes will show size in kbytes
//19980406 hrx    added clcount(), mbcheckcount()
//19980411 hrx    changed type of mbcheckcount() to long
//19980615 hrx    removed checknumname/checknumtmp meta definitions, moved
//                to mail.h
//19990711 DH3MB  Renamed nexthash to nextsamehash in struct check_typ

#include "baycom.h"

/*---------------------------------------------------------------------------*/

#define LEN 128
#define BUFLEN 8192
#define BUFLINES 32
#define BUFSIZE  (BUFLINES*LEN)

/*---------------------------------------------------------------------------*/

typedef struct
{ char name[9];
  unsigned nummer;
  unsigned nextsamehash;
} check_typ;

/*---------------------------------------------------------------------------*/

static int checkmerge(handle lfh,handle cfh,char *outf)
{ char *lbuf=(char *)t_malloc(BUFLEN,"ch_l");
  char *cbuf=(char *)t_malloc(BUFLEN,"ch_c");
  unsigned llen,clen;
  unsigned lpos,cpos;
  FILE *of;
  char *nextl,*nextc;

  if(lbuf==NULL || cbuf==NULL)
    return 0;

  of=s_fopen(outf,"swb");
  if(of==NULL)
  { trace(serious,"checkmerge","fopen %s errno=%d",outf,errno);
    return 0;
  }
  s_fsetopt(of,1);
  setvbuf(of,NULL,_IOFBF,BUFLEN);
  lpos=llen=cpos=clen=0;
  do
  { if(lpos>=llen && lfh!=EOF)
    { llen=_read(lfh,lbuf,BUFLEN);
      waitfor(e_ticsfull);
      lpos=0;
    }
    if(lpos<llen)
      nextl=lbuf+lpos;
    else
      nextl=NULL;

    if(cpos>=clen && cfh!=EOF)
    { clen=_read(cfh,cbuf,BUFLEN);
      waitfor(e_ticsfull);
      cpos=0;
    }
    if(cpos<clen)
      nextc=cbuf+cpos;
    else
      nextc=NULL;

    if((nextc && !nextl)||(nextc && nextl && strncmp(nextc,nextl,7)<0))
    { fwrite(nextc,LEN,1,of);
      cpos+=LEN;
    }
    else
    if((!nextc && nextl)||(nextc && nextl && strncmp(nextc,nextl,7)>=0))
    { fwrite(nextl,LEN,1,of);
      lpos+=LEN;
    }
  }
  while(nextl || nextc);
  s_fclose(of);
  t_free(lbuf);
  t_free(cbuf);
  return 1;
}

/*---------------------------------------------------------------------------*/

#ifdef HRX_OPT
static int clcount(long unsigned lfd,char *dirline,int boardnum,time_t abwann)  
{ long time;                                                                    
                                                                                
  time=dirline_data(dirline);                                                   
  if((time>=abwann) || (abwann && (lfd==1)))                                    
   return 1;                                                                    
  else                                                                          
   return 0;                                                                    
}                                                                               
                                                                                
/*---------------------------------------------------------------------------*/ 
                                                                                
long mbcheckcount(void) 
{ time_t abwann=0;                                                              
  b->job=j_search;                                                              
  b->optgroup=o_ch;                                                             
  b->usermail=0;                                                                
  if(b->lastdirnews==0)                                                         
   abwann=ad_time();                                                            
  else                                                                          
   abwann=b->lastdirnews;                                                       
                                                                                
  int lfd=1;                                                                    
  handle fh=EOF;                                                                
  handle posf=EOF;                                                              
  char *buf=(char *)t_malloc(BUFSIZE,"chek");                                   
  int buflen=0;                                                                 
  short unsigned numbuf[BUFLINES*2];                                            
  long flen;                                                                    
                                                                                
  fh=s_open(CHECKNAME,"lrb");                                                   
  if(!b->semalock && !sema_access(CHECKNUMTMP))                                 
  { if(fh==EOF)                                                                 
      goto nocheckfile;                                                         
    posf=s_open(CHECKNUMNAME,"lrb");                                            
    if(posf==EOF || filelength(posf)!=(filelength(fh)>>6))                      
      goto nocheckfile;                                                         
    if(fh!=EOF)                                                                 
    { lseek(fh,0L,SEEK_END);                                                    
      flen=ltell(fh);                                                           
      b->beginn=1;                                                              
      b->ende=MAXBEREICH;                                                               
      if(b->beginn<=(flen>>7))                                                  
      { flen|=(BUFSIZE-1);                                                      
        flen+=1;                                                                
                                                                                
        while(lfd<=b->ende)                                                     
        { if(buflen==0)                                                         
          { flen-=BUFSIZE;                                                      
            lseek(fh,flen,SEEK_SET);                                            
            lseek(posf,flen>>6,SEEK_SET);                                       
            buflen=_read(fh,buf,BUFSIZE);                                       
            _read(posf,numbuf,BUFLINES*2);                                      
          }                                                                     
          if(buflen>=LEN)                                                       
          { buflen-=LEN;                                                        
            if(buf[buflen+7]!=' ')                                              
            { buf[buflen+126]=0;                                                
              if(!clcount(lfd,buf+buflen,numbuf[buflen>>7],abwann))             
                  goto checkcountok;                                            
              lfd++;                                                            
            }                                                                   
          }                                                                     
          waitfor(e_ticsfull);                                                  
        }                                                                       
      }                                                                         
      else                                                                      
        putf(ms(m_checkentries),ltell(fh)>>7);                                  
                                                                                
    }                                                                           
    else                                                                        
      trace(serious,"mbcheck","fopen "CHECKNAME" errno=%d",errno);              
  }                                                                             
  else                                                                          
  { nocheckfile:                                                                
    putf(ms(m_checknoaccess));                                                  
  }                                                                             
  checkcountok:                                                                 
  if(fh!=EOF)                                                                   
    s_close(fh);                                                                
  if(posf!=EOF)                                                                 
    s_close(posf);                                                              
  t_free(buf);                                                                  
  return lfd-1;                                                                 
}                                                                               
#endif

/*---------------------------------------------------------------------------*/

void makecheck(void)
{ handle lfh,cfh;

  int  tmpnum=0;
  char tmpname1[20];
  char tmpname2[20];
  char tmpname3[20]="";
  int  level=0;
  int  i,j;
  int  tmpfiles=2;

  if(sema_lock("sendlock")==0)
    return;

  trace(replog,"reorg","create check list");
  if(dfree(".",0) < (filesize(CHECKNAME)*3L/1024L+HOMEDISKFULL))
  { trace(serious,"makecheck","disk full");
    return;
  }

  i=0;
  while(i<treelen)
  { lastcmd(tree[i].name);
    trace(report,"makecheck","%s",tree[i].name);
    lfh=preparedir(tree[i++].name,0);

    if(lfh!=EOF)
    { cfh=EOF;
      while(cfh==EOF && i<treelen)
        cfh=preparedir(tree[i++].name,0);
      sprintf(tmpname3,"c%d_%d.$$$",level,tmpnum++);
      if(!checkmerge(lfh,cfh,tmpname3)) goto error;
      s_close(lfh);
      if(cfh!=EOF) s_close(cfh);
    }
    waitfor(e_ticsfull);
  }
  if(tmpname3[0])
  {
    while(tmpfiles>1)
    { i=0;
      j=0;
      tmpfiles=0;
      trace(report,"makecheck",tmpname3);
      lastcmd(tmpname3);
      do
      { sprintf(tmpname1,"c%d_%d.$$$",level,i++);
        sprintf(tmpname2,"c%d_%d.$$$",level,i++);
        lfh=s_open(tmpname1,"srb");
        cfh=s_open(tmpname2,"srb");
        if(lfh!=EOF || cfh!=EOF)
        { sprintf(tmpname3,"c%d_%d.$$$",level+1,j++);
          if(!checkmerge(lfh,cfh,tmpname3)) goto error;
          if(lfh!=EOF) s_close(lfh);
          if(cfh!=EOF) s_close(cfh);
          xunlink(tmpname1);
          xunlink(tmpname2);
          tmpfiles++;
        }
        else
          break;
        waitfor(e_ticsfull);
      } while(1);
      level++;
    }
    xunlink(CHECKNAME);
    xrename(tmpname3,CHECKNAME);
  }
  else
  { lfh=s_open(CHECKNAME,"swb");  // create empty file
    if(lfh!=EOF) s_close(lfh);
  }

  error:
  sema_unlock("sendlock");
  make_checknum("reorg");
}

/*---------------------------------------------------------------------------*/

static int checkline(long unsigned lfd,char *dirline,
                     int boardnum,time_t abwann)
{ char line[140];
  long time;
  int length=73;

  time=dirline_data(dirline);
  if((time>=abwann) || (abwann && (lfd==1)))
  {
#ifdef _MAILFILTER	//db1ras
   if ((b->sysop) || (strcmp(b->herkunft,b->logincall)==0) || (b->at[0]!='~'))
   {
#endif
    sprintf(line,"%-7s> %s",b->herkunft,b->ziel);
    if (strlen(b->ziel) == 8 && boardnum>9999)
      line[strlen(line)-1]=0;
    while(strlen(line)<(16+(boardnum<10000)+
       (boardnum<1000)+(boardnum<100)+(boardnum<10)))
      strcat(line,".");
    sprintf(line+strlen(line),"%u ",boardnum);
    if(b->opt & o_d)
      sprintf(line+strlen(line),"%s ",datestr(time,
        !!(b->opt & o_y)*(1+2*u->dirformat)));
    if(b->opt & o_t)
    { sprintf(line+strlen(line),"%s ",timestr(time,0));
    }
    if(b->opt & o_f)
      sprintf(line+strlen(line),"%s@",b->frombox);
    if(b->opt & o_m)
      sprintf(line+strlen(line),"%-6s ",b->at);
    if(b->opt & o_b)
    { if (b->bytes < 100000L)
        sprintf(line+strlen(line),"%5lu ",b->bytes);
      else
        sprintf(line+strlen(line),"%4luk ",b->bytes/1024L);
    }
    if(b->opt & o_z)
      sprintf(line+strlen(line),"%4u ",b->lines);
    //added -s - output of mailtype
    if(b->opt & o_s)
      sprintf(line+strlen(line),"%c ",b->mailtype);
    if(b->opt & o_i)
      sprintf(line+strlen(line),"$%-12s ",b->bid);
    if(b->opt & o_l)
      sprintf(line+strlen(line),"%3d ",b->lifetime);
    line[71]=0;
   
    if(lfd>99999L)
      length--;
    b->betreff[length-strlen(line)]=0;
    if(b->opt & o_w)
      strcat(line,b->betreff);

    //only if bulletin or option s is set
    if ((b->mailtype=='B')||(b->opt&o_s))
    { if(b->http)
      { int i=0;
        strlwr(b->ziel);
        strlwr(dirline);
        putf("%5lu ",lfd);
        html_putf("<a href=\"/read/%s/%s\">",b->ziel,dirline);
        while(line[i]!=' ')
        putv(line[i++]);
        html_putf("</a>");
        putf("%s\n",line+i);
      }
      else
        putf("%5lu %s\n",lfd,line);      
    }
#ifdef _MAILFILTER	//db1ras
   }
#endif
    return 1;
  }
  return 0;
}

/*---------------------------------------------------------------------------*/

static int searchboardlen(char *name)
{ handle fh;
  int retwert=(-1);

  fh=preparedir(name,0);
  if(fh!=EOF)
  { retwert=(unsigned)(filelength(fh)>>7);
    s_close(fh);
  }
  return retwert;
}

/*---------------------------------------------------------------------------*/

static short unsigned numofboard(int *hash,char *line,check_typ *p,int *listlen)
{ char name[9];
  int code,entry;

  nexttoken(line+15,name,8);
  code=strcrc(name)&255;
  entry=hash[code];
  while(entry && (strcmp(name,p[entry].name)!=0))
    entry=p[entry].nextsamehash;

  if(entry==0)
  { entry=((*listlen)++);
    if(entry>=(treelen+50))
    { trace(serious,"numofboard","num dirs %d %s",entry,name);
      return 0;
    }
    if(hash[code])
      p[entry].nextsamehash=hash[code];
    hash[code]=entry;
    p[entry].nummer=searchboardlen(name);
    strcpy(p[entry].name,name);
  }
  else
  { if((p[entry].nummer==0) && !reorg_noetig)
    { trace(serious,"numofboard","%s",name);
      reorg_noetig++;
      fork(P_BACK|P_MAIL,0,mbreorg,"c");
    }
  }
  if(p[entry].nummer>0)          // negativ: Board unsichtbar, 0: Fehler
    return p[entry].nummer--;
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

static int boardreject(char *line)
{ char board[12];
  int i=8;
  int not=(u->notvisible[1]!='-');

  if(u->notvisible[1])
  { strncpy(board+1,line+15,8);
    board[0]=' ';
    board[9]=0;
    while(board[i]==' ')
      board[i--]=0;
    strcat(board," ");

    if(strstr(u->notvisible,board))
      return not;
  }
  return !not;
}

/*---------------------------------------------------------------------------*/

void make_checknum(char *from)
{ char line[128];
  short unsigned boardnr=0;
  handle fh;
  unsigned filepos;
  int listlen=1;
  int sysop=b->sysop;
  b->sysop=1;        //for F-Board etc.

  if(sema_access(CHECKNUMTMP))
    return;
  if(sema_lock("sendlock")==0)
    return;

  FILE *posf=s_fopen(CHECKNUMTMP,"lwb");
  if(!posf) return;
  s_fsetopt(posf,1);

  trace(replog,from,"create check numbers");
  lastcmd("checknum");

  check_typ *cp=(check_typ *)t_malloc(sizeof(check_typ)*(treelen+52),"c_bo");
  int *hash=(int *)t_malloc(256*sizeof(int),"c_ha");

  memset(cp,0,sizeof(check_typ)*(treelen+52));
  // DH3MB: Changed value from 2 to 52   ^^
  memset(hash,0,256*sizeof(int));                  // Hash lschen

  fh=s_open(CHECKNAME,"lrb");
  if(!b->semalock)
  { if(fh==EOF && !sema_test("purgereorg"))
    { makecheck();
      fh=s_open(CHECKNAME,"lrb");
    }
    if(fh!=EOF)
    { if(lseek(fh,0L,SEEK_END)!=0)
      {
        lseek(fh,-(LEN),SEEK_END);
        filepos=unsigned(ltell(fh)>>7);
        do
        { _read(fh,line,LEN);
	  if(line[7]!=' ')
	  { boardnr=numofboard(hash,line,cp,&listlen);
            fseek(posf,((long)filepos)<<1,SEEK_SET);
            fwrite(&boardnr,2,1,posf);
	  }
	  if(filepos==0)
	    break;
	  lseek(fh,-(2L*LEN),SEEK_CUR);
          filepos--;
	  waitfor(e_ticsfull);
        } while(1);
      }
      s_close(fh);
    }
  }
  t_free(cp);
  t_free(hash);
  s_fclose(posf);
  xunlink(CHECKNUMNAME);
  xrename(CHECKNUMTMP,CHECKNUMNAME);
  sema_unlock("sendlock");
  b->sysop=sysop;
}

/*---------------------------------------------------------------------------*/

void purgecheck(char *)
{ handle fi=s_open(CHECKNAME,"srb");
  handle fo=s_open(CHECKTMPNAME,"swb");
  char line[LEN];

  if(sema_lock("sendlock")==0)
    return;
  if(dfree(".",0) < (filelength(fi)/1024L+HOMEDISKFULL ))
    trace(serious,"purgecheck","disk full");
  else if(fi!=EOF && fo!=EOF)
  { while(_read(fi,line,LEN)==LEN)
    { if(line[7]!=' ')
        _write(fo,line,LEN);
      waitfor(e_ticsfull);
    }
    s_close(fi);
    s_close(fo);
    xunlink(CHECKNAME);
    xrename(CHECKTMPNAME,CHECKNAME);
  }
  sema_unlock("sendlock");
  make_checknum("mbpurge");
}

/*---------------------------------------------------------------------------*/

void mbcheck(char *selektor,time_t abwann)
{ long unsigned lfd=1;
  int doprint=0;
  handle fh=EOF;
  handle posf=EOF;
  char *buf=(char *)t_malloc(BUFSIZE,"chek");
  int buflen=0;
  short unsigned numbuf[BUFLINES*2];
  long flen;
  char era;

  if((selektor[0]=='<') && (selektor[1]==' '))
    selektor++;
  selektor+=blkill(selektor);

  fh=s_open(CHECKNAME,"lrb");
  if(!b->semalock && !sema_access(CHECKNUMTMP))
  { if(fh==EOF)
    { fork(P_BACK|P_MAIL,0,mbreorg,"c");
      goto nocheckfile;
    }
    posf=s_open(CHECKNUMNAME,"lrb");
    if(posf==EOF || filelength(posf)!=(filelength(fh)>>6))
    { fork(P_BACK|P_MAIL,0,mbreorg,"n");
      goto nocheckfile;
    }
    if(fh!=EOF)
    { lseek(fh,0L,SEEK_END);
      flen=ltell(fh);
      if((b->beginn==0) || (b->beginn==b->ende))
	b->beginn=1;
      if(b->beginn<=(flen>>7))
      { flen|=(BUFSIZE-1);
        flen+=1;

	if(b->opt & o_j)
	  putf(ms(m_checktitle));

        while((lfd<=b->ende) && !testabbruch())
	{ if(buflen==0)
          { flen-=BUFSIZE;
            lseek(fh,flen,SEEK_SET);
            lseek(posf,flen>>6,SEEK_SET);
            buflen=_read(fh,buf,BUFSIZE);
            _read(posf,numbuf,BUFLINES*2);
          }
          if(buflen>=LEN)
          { buflen-=LEN;
	    if(buf[buflen+7]!=' ')
	    { buf[buflen+126]=0;
              era=buf[buflen+14];
              if(selektor[0])
	        //doprint=!!stristr(buf+buflen,selektor);// OE3DZW
              { CRegEx reg;
                char a[130];
                sprintf(a,"%.127s\n",buf+buflen);
                doprint = reg.regex_match(selektor,a);   
              }
	      else
	        doprint=1;
	      doprint= doprint && ((era=='>')||(b->opt & (o_v|o_r)));
              doprint= doprint && ((era=='L')||(era=='X')|| !(b->opt & o_r));
              if(selektor[0]==0) doprint= doprint && !boardreject(buf+buflen);
              if(doprint && lfd>=b->beginn && 
                (buf[buflen+16]!=' '||b->sysop)) 
              { if(!checkline(lfd,buf+buflen,numbuf[buflen>>7],abwann))
		  break;
              }
	      lfd++;
	    }
          }
	  if(buflen==0 && flen==0)
	    break;
	  waitfor(e_ticsfull);
	}
      }
      else
	putf(ms(m_checkentries),ltell(fh)>>7);

    }
    else
      trace(serious,"mbcheck","fopen "CHECKNAME" errno=%d",errno);
  }
  else
  { nocheckfile:
    putf(ms(m_checknoaccess));
  }
  if(fh!=EOF)
    s_close(fh);
  if(posf!=EOF)
    s_close(posf);
  t_free(buf);
}
