/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -----------------------------
  Zeitgesteuerte Job-Triggerung
  -----------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980212 OE3DZW delay before start of cron-daemon removed
//19980614 OE3DZW define as limit for disable

#include "baycom.h"

/*---------------------------------------------------------------------------*/

static char cron_init[]=
"*    * * * * minute\n"
"0,30 * * * * halfhour\n"
"1,31 * * * * beacon\n"
"3,33 * * * * forward\n"
"5    2 * * * postfwd\n"
"5    3 * * * purge\n"
"35   3 * * * save\n"
"5    4 * * 0 reorg\n"
"0,15,30,45 * * * * quarter\n";

/*---------------------------------------------------------------------------*/

static int near cron_match(char *s,int t)
//*************************************************************************
//
//  test if given pattern matches the current time
//
//*************************************************************************
{ char *c=s;

  if(s[0]=='*') return 1;               // match everytime

  while(isdigit(c[0])) c++;             // remember next non-decimal

  if(c[0]=='-')                         // check for range
    return (atoi(s)<=t && atoi(c+1)>=t) || cron_match(c+1,t);

  if(c[0]==',')                         // check for enumeration
    return atoi(s)==t || cron_match(c+1,t);

  if(c!=s)                              // begin of string is decimal
    return atoi(s)==t;

  return 0;
}

/*---------------------------------------------------------------------------*/

static void near run_crontab(struct tm *tt)
{ FILE *f=s_fopen(CRONTABNAME,"srt");
  char s[200];
  char mi[51],h[51],d[51],m[51],wd[51],script[51];
  int  line=0;

  if(!f) // if crontab does not exist, create it
  { f=s_fopen(CRONTABNAME,"sw+t");
    if(!f) return;                  // file is not accessible (why ever...)
    fwrite(cron_init,sizeof(cron_init)-1,1,f);
    trace(replog,"cron","crontab.bcm generated");
    rewind(f);
  }
  
  while(fgets(s,sizeof(s),f))
  {
    line++;                    // remember line for error message
    // recognise comment lines
    if((s[0]!='*' && !isdigit(s[0])) || strlen(s)<10)
      continue;
    if(sscanf(s,"%50s %50s %50s %50s %50s %50s\n",mi,h,d,m,wd,script)==6)
    {
      if(cron_match(mi,tt->tm_min)  && cron_match(h,tt->tm_hour) &&
         cron_match(m,tt->tm_mon+1) && cron_match(d,tt->tm_mday) &&
         cron_match(wd,tt->tm_wday) )
      { //oe3dzw debug
        //trace(report,"run_crontab","Starting %s",script);
        if(stricmp(script,"beacon")==0)
          fork(P_BACK|P_MAIL,0,sendmailbake,"Beacon");
        else if(stricmp(script,"purge")==0)
          fork(P_BACK|P_MAIL,0,mbpurge,"Purge");
        else if(stricmp(script,"forward")==0)
        { fork(P_BACK,0,startfwd,"Forward");
        }
        else if(stricmp(script,"postfwd")==0)
          fork(P_BACK|P_MAIL,0,postfwd,"Postfwd");
        else if(stristr(script,"reorg")==script)
        { char *cmd=script+5;
          while(cmd[0]==' ') cmd++;
          fork(P_BACK|P_MAIL,0,mbreorg,cmd);
        }
        else runbatch(script);
      }
    }
    else
      trace(serious,"crontab","syntax line %d",line);
  }
  s_fclose(f);
}

/*---------------------------------------------------------------------------*/

static void near cron_hour(void)
{ if(dfree(".",0)<HOMEDISKFULL && !m.disable)
  { trace(serious,"bcm home","disk full");
    m.disable=1;
  }
  if(dfree(m.userpath,0)<USERDISKFULL)
  { trace(serious,"user path","disk full");
    m.disable=1;
  }
  if(dfree(m.infopath,0)<INFODISKFULL)
  { trace(serious,"info path","disk full");
    m.disable=1;
  }
  if(mbhadrok(m.boxadress)!=1)
  { trace(serious,"cron","no valid H-boxaddress %s",m.boxadress);
    m.disable=1;
  }
  //oe3dzw Test auf .# //regional address 
  if ((strstr(m.boxadress,".DEU") || strstr(m.boxadress,".AUT"))
     && !strstr(m.boxadress,".#"))
  { trace(serious,"cron","no '.#'in boxaddress %s",m.boxadress);
    m.disable=1;
  }
  //oe3dzw Test auf kein EURO
  if (strstr(m.boxadress,".EURO"))
  { trace(serious,"cron",".EURO in boxaddress %s",m.boxadress);
    m.disable=1;
  } 
  if(ad_time()<842639000L)
  { trace(serious,"cron","date setup incorrect");
    m.disable=1;
  }
  if(!filetime("checknum.bcm") && filetime(CHECKNAME))
    fork(P_BACK|P_MAIL,0,mbreorg,"n");
}

/*---------------------------------------------------------------------------*/

void mbcron(char *)
{ int oldmin=0;
  int oldhour=25;
#ifdef __DOS16__
  struct time tdos;
  long olddst=-1;
  long thisdst;
#endif
  time_t ti;
  struct tm *tt;
  struct tm tmbuf;
  wdelay(685);
  testcputask();
  runbatch("startup");
  //oe3dzw, debug
  trace(report,"mbcron","started");
  while(!runterfahren)
  { wdelay(2514);

    ti=ad_time();
    tt=ad_comtime(ti);
    memcpy(&tmbuf,tt,sizeof(tmbuf));
    tt=&tmbuf;

    if(tt->tm_hour!=oldhour)
    { if(tt->tm_hour==0 && oldhour!=25) // new day
       newbid(1);
#ifdef __DOS16__
      if ((oldhour!=25) && !m.pcisutc &&
        ((thisdst=isdst())!=olddst))
      { gettime(&tdos);
        tdos.ti_hour=(olddst-thisdst)/3600+tdos.ti_hour;
        settime(&tdos);
        trace(replog,"mbcron","new daylightsaving: %lds",thisdst);
        olddst=thisdst;
      }
      else
        olddst=isdst();
#endif
      oldhour=tt->tm_hour;
      cron_hour();
    }

    if(tt->tm_min!=oldmin)
    {
      oldmin=tt->tm_min;

      run_crontab(tt);

      if((oldmin%10)==7)
	testcputask();

      msg_dealloc(0);                  // purge unused messages
    }
    test_killtime();                   // callback to scheduler
  }
}

/*---------------------------------------------------------------------------*/
