/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  ------------------------
  DIDADIT-Protocol for BCM
  ------------------------


  Copyright (C)       Johann Hanne, DH3MB, jonny@baycom.org
                      Falkenweg 6
                      D-85122 Hitzhofen

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19990401 DH3MB  Begin of implementation (SPEC V0.5.3)
//19990406 DH3MB  Switched to SPEC V0.5.4
//19990416 DH3MB  Switched to SPEC V0.5.5
//19990520 DH3MB  Switched to SPEC V0.5.7

#include "baycom.h"

#ifdef FEATURE_DIDADIT

/*---------------------------------------------------------------------------*/

fileio_dida::fileio_dida(void) : fileio()
{ bl=(struct block_info *)t_malloc(sizeof(struct block_info), "dida");
  bl->maxlen=0;
  f->requires=REQ_MD5HASH;
}

/*---------------------------------------------------------------------------*/

fileio_dida::~fileio_dida(void) // TODO? : fileio::~fileio()
{ if(bl->maxlen>0)
    free_blockmem();

  t_free(bl);
}

/*---------------------------------------------------------------------------*/

void fileio_dida::tx_stuffed(char byte)
{ switch(byte)
  {
  case FEND:
    bputv(FESC);
    bputv(TFEND);
    break;

  case FESC:
    bputv(FESC);
    bputv(TFESC);
    break;

  default:
    bputv(byte);
    break;
  }
}

/*---------------------------------------------------------------------------*/

signed short int fileio_dida::rx_stuffed(void)
{ char byte=getv();

  switch(byte)
  {
  case FEND:
    return(-1);

  case FESC:
    byte=getv();
    switch(byte)
    {
    case TFEND:
      return(FEND);

    case TFESC:
      return(FESC);

    default:
      return(-2);
    }

  default:
    return(byte);
  }
}

/*---------------------------------------------------------------------------*/

void fileio_dida::tx_block(unsigned short int type, char tx_md5hash,
                       char *data, unsigned short int length)
{ char i;
  crcthp crc;

  bputv(FEND);

  tx_stuffed(type % 256);
  tx_stuffed(type / 256);

  crc.update(type % 256);
  crc.update(type / 256);

  if(tx_md5hash)
    for(i=0;i<16;i++)
    { tx_stuffed(f->md5hash[i]);
      crc.update(f->md5hash[i]);
    }

  while(length--)
  { tx_stuffed(*data);
    crc.update(*(data++));
  }

  tx_stuffed(crc.result % 256);
  tx_stuffed(crc.result / 256);

  bputv(FEND);
}

/*---------------------------------------------------------------------------*/

void fileio_dida::rx_block()
//*************************************************************************
//
//  Empfngt ein DIDADIT-Frame; es wird dabei solange empfangen, bis
//  ein korrektes Frame angekommen ist; alle ungltigen Frames werden
//  einfach verworfen (egal, ob es sich um einen CRC-Fehler handelt,
//  das Frame zu lang war, etc.).
//
//  An das letzte empfange Byte in data wird ein 0-Byte angehngt, was
//  fr die Auswertung von Text-Blcken ganz ntzlich ist.
//
//*************************************************************************
{ char rxstate=R_RX_STATE_TYPE;
  char typelen=0; // Number of type-bytes already received
  char md5len=0; // Number of md5hash-bytes already received
  unsigned short int datalen=0; // Number of data-bytes already received
  signed short int rx;
  crcthp crc;
  char crcbuf[2]; // Die CRC wird um 2 Bytes verzgert mitgerechnet, um
                  // die empfangene CRC selbst nicht miteinzuberechnen

  for(;;)
  { while((rx=rx_stuffed())!=-1)
    { if(rx==-2)
      { // Received an invalid ESCape-sequence
        rxstate=R_RX_STATE_INVALID;
      }

      // Wenn die Typ-ID (zwei Bytes) vollstndig empfangen wurde, kann mit
      // der Berechnung der CRC begonnen werden
      if(typelen==2)
        crc.update(crcbuf[0]);

      crcbuf[0]=crcbuf[1];
      crcbuf[1]=rx;

      switch(rxstate)
      {
      case R_RX_STATE_TYPE:
        if(typelen==0) // Received first type-byte
          *((char *)&bl->type)=rx;
        else // Received second type-byte
        { *(((char *)&bl->type)+1)=rx;
          // Frame-types which include a md5-hash:
          if(//rcvd_md5 && // TODO!!!
             (bl->type==R_BLOCK_ERR ||
              bl->type==R_BLOCK_DATA ||
              bl->type==R_BLOCK_REQ ||
              bl->type==R_BLOCK_FIN ||
              bl->type==R_BLOCK_FINACK ||
              bl->type==R_BLOCK_ABORT))
            rxstate=R_RX_STATE_MD5HASH;
          else
            rxstate=R_RX_STATE_DATA;
        }
        typelen++;
        break;

      case R_RX_STATE_MD5HASH:
///TODO!!!        rcvd_md5[md5len]=rx;
        if(++md5len==16)
          rxstate=R_RX_STATE_DATA;
        break;

      case R_RX_STATE_DATA:
        bl->data[datalen++]=rx;
        if(datalen>(bl->maxlen+2))
          rxstate=R_RX_STATE_INVALID;
        break;

      case R_RX_STATE_INVALID:
        // Frame is already known to be invalid, do nothing
        break;
      }
    }

    // Frame not complete?
    if(!(rxstate==R_RX_STATE_DATA && datalen>=2))
      continue;

    // Check the CRC (stored in the last two bytes of the frame)
    datalen-=2;
    if(*(unsigned short int*)(bl->data+datalen)==crc.result)
    { bl->data[datalen]=0;
      break;
    }
  }

  bl->len=datalen;
}

/*---------------------------------------------------------------------------*/

void fileio_dida::alloc_blockmem(unsigned short int size)
//*************************************************************************
//
//  Allocates memory for receiving a DIDADIT block
//
//  rx_block checks, if the allocated memory for the block is sufficient
//  if not, the received block is silently ignored
//
//*************************************************************************
{ if(bl->maxlen>0)
    free_blockmem();

  bl->data=(char *)t_malloc(size+2, "dida");
  bl->maxlen=size;
}

/*---------------------------------------------------------------------------*/

void fileio_dida::free_blockmem()
//*************************************************************************
//
//  Frees memory allocated with alloc_blockmem()
//
//*************************************************************************
{ t_free(bl->data);
  bl->maxlen=0; // TODO: Beim Block-Empfang checken!
}

/*---------------------------------------------------------------------------*/

char *fileio_dida::parse_textblock(char *block, char **id, char **value)
//*************************************************************************
//
//  Wird zum parsen eines DIDADIT-Text-Blocks (INFO, START)
//  verwendet, das Informationen in der Form "ID=Wert\r" enthlt
//  Es werden das "=" und das "\r" (=CR) durch ein 0-Byte ersetzt
//  *id wird auf den Anfang der ID gesetzt, *value auf den Anfang
//  des Werts
//  Zurckgegeben wird ein Pointer auf das nchste Feld oder NULL,
//  wenn kein Feld gefunden wurde
//
//  Wichtig: Der gesamte Block mu mit einem 0-Byte abgeschlossen sein,
//  auerdem wird der bergebene Block verndert!
//
//*************************************************************************
{ char *equal, *end;

  if((equal=strchr(block, '='))!=0 && (end=strchr(block, '\r'))!=0)
  { *equal=0;
    *end=0;

    *id=block;
    *value=equal+1;
    return(end+1);
  }
  else
    return(0);
}

/*---------------------------------------------------------------------------*/

void fileio_dida::parse_INFO(char *md5str, unsigned short int *rxed_blocksize)
//*************************************************************************
//
//
//
//*************************************************************************
{ char *block=bl->data, *id, *value;

  *f->fullname=0;
  *f->name=0;
  *f->path=0;
  f->size=0L;
  *md5str=0;
  f->unixtime=0;
  *rxed_blocksize=0;
  *partner_ver=0;

  while(block=parse_textblock(block, &id, &value))
  { if(strcasecmp(id, "FILENAME")==0)
    { strncpy(f->name, value, 99);
      f->name[99]=0;
    }
    else if(strcasecmp(id, "PATH")==0)
    { strncpy(f->path, value, 99);
      f->name[99]=0;
    }
    else if(strcasecmp(id, "SIZE")==0)
      f->size=atol(value);
    else if(strcasecmp(id, "MD5")==0)
    { strncpy(md5str, value, 32);
      md5str[32]=0;
    }
    else if(strcasecmp(id, "TIME")==0)
      f->unixtime=atol(value);
    else if(strcasecmp(id, "BLOCKSIZE")==0)
      *rxed_blocksize=atoi(value);
    else if(strcasecmp(id, "VERSION")==0)
    { strncpy(partner_ver, value, 9);
      partner_ver[9]=0;
    }
  }
}

/*---------------------------------------------------------------------------*/
 
void fileio_dida::parse_START(unsigned long int *resume_offset, char *part_md5str,
                          unsigned short int *rxed_blocksize)
//*************************************************************************
//
//
//
//*************************************************************************
{ char *block=bl->data, *id, *value;

  *resume_offset=0L;
  *part_md5str=0;
  *rxed_blocksize=0;
  *partner_ver=0;

  // TODO: check length of received fields

  while(block=parse_textblock(block, &id, &value))
  { if(strcasecmp(id, "OFFSET")==0)
      *resume_offset=atol(value);
    else if(strcasecmp(id, "PARTMD5")==0)
    { strncpy(part_md5str, value, 32);
      part_md5str[32]=0;
    }
    else if(strcasecmp(id, "BLOCKSIZE")==0)
      *rxed_blocksize=atoi(value);
    else if(strcasecmp(id, "VERSION")==0)
    { strncpy(partner_ver, value, 9);
      partner_ver[9]=0;
    }
  }
}

/*---------------------------------------------------------------------------*/

void fileio_dida::tx_INFO()
{ sprintf(b->line,
          "FILENAME=%s\r"
          "PATH=%s\r"
          "SIZE=%ld\r"
          "MD5=%s\r"
          "TIME=%ld\r"
          "BLOCKSIZE=%d\r"
          "VERSION=" DIDADIT_VERSION "\r",
          f->name, f->path, f->size, f->md5str,
          f->unixtime, DEFAULT_BLOCKSIZE);

  tx_block(R_BLOCK_INFO, 0, b->line, strlen(b->line));
}

/*---------------------------------------------------------------------------*/

void fileio_dida::tx_START(unsigned long int offset, char *part_md5str,
                       unsigned short int blocksize)
{ *b->line=0;

//  if(offset) TODO
    sprintf(b->line, "OFFSET=%ld\r", offset);
//                     "PART_MD5=%s\r",
//                     offset, part_md5str);

  if(blocksize)
    sprintf(b->line+strlen(b->line), "BLOCKSIZE=%d\r", blocksize);

  sprintf(b->line+strlen(b->line), "VERSION=" DIDADIT_VERSION "\r");

  tx_block(R_BLOCK_START, 0, b->line, strlen(b->line));
}

/*---------------------------------------------------------------------------*/

void fileio_dida::tx_ERR(char *errorinfo)
{ tx_block(R_BLOCK_ERR, 1, b->line, strlen(b->line));
}

/*---------------------------------------------------------------------------*/

void fileio_dida::tx_DATA(unsigned long int offset,
                      unsigned short int length, char *data)
{ char *datptr=(char *)t_malloc(length+6, "dida");

  memcpy(datptr, &offset, 4);
  memcpy(datptr+4, &length, 2);
  memcpy(datptr+6, data, length);

  tx_block(R_BLOCK_DATA, 1, datptr, length+6);
  t_free(datptr);
}

/*---------------------------------------------------------------------------*/

void fileio_dida::tx_REQ(unsigned long int offset, unsigned long int length)
{ memcpy(b->line, &offset, 4);
  memcpy(b->line+4, &length, 4);

  tx_block(R_BLOCK_REQ, 1, b->line, 8);
}

/*---------------------------------------------------------------------------*/

void fileio_dida::tx_FIN(char *data)
{ tx_block(R_BLOCK_FIN, 1, 0, 0);
}

/*---------------------------------------------------------------------------*/

void fileio_dida::tx_FINACK(void)
{ tx_block(R_BLOCK_FINACK, 1, 0, 0);
}

/*---------------------------------------------------------------------------*/

void fileio_dida::tx_TYPEREJ(unsigned short int type)
{ tx_block(R_BLOCK_TYPEREJ, 0, (char *)&type, 2);
}

/*---------------------------------------------------------------------------*/

void fileio_dida::tx_ECHOREQ(void)
{ tx_block(R_BLOCK_ECHOREQ, 0, 0, 0);
}

/*---------------------------------------------------------------------------*/

void fileio_dida::tx_ECHOREP(void)
{ tx_block(R_BLOCK_ECHOREP, 0, 0, 0);
}

/*---------------------------------------------------------------------------*/

void fileio_dida::tx_ABORT(void)
{ tx_block(R_BLOCK_ABORT, 1, 0, 0);
}

/*---------------------------------------------------------------------------*/

void fileio_dida::_rx(void)
{ char md5str[33];
  unsigned short int rxed_blocksize;
  char *data_start;
  unsigned long int *rxed_offset;
  unsigned short int *rxed_blocklen;

//  putf("Now waiting for DIDADIT-file %s\n", f->fullname);

  putf("\n#OK#\n");

  alloc_blockmem(1000); // TODO: 1000 bytes max.?

  // We now expect an INFO-frame
  rx_block();

  if(bl->type!=R_BLOCK_INFO)
  { tx_ABORT();
    putf("*** Expected INFO-Frame, got 0x%04X.\n", bl->type);
    return;
  }

  parse_INFO(md5str, &rxed_blocksize);

printf("FILENAME: %s\n"
       "PATH: %s\n"
       "SIZE: %ld\n"
       "MD5: %s\n"
       "TIME: %ld\n"
       "BLOCKSIZE: %d\n"
       "VERSION: %s\n",
       f->name, f->path, f->size, md5str, f->unixtime, rxed_blocksize,
       partner_ver);

  // TODO: Resume

  blocksize=rxed_blocksize;
  if(blocksize>MAX_BLOCKSIZE)
    blocksize=MAX_BLOCKSIZE;

  tx_START(0L, "", blocksize);

  // TODO: Vor dem berschreiben Warnung ausgeben...

  f->handle=s_open(f->name, "lwb");

alloc_blockmem(9000);
//  alloc_blockmem(blocksize+6); // +4 bytes for the offset
                               // +2 bytes for the blocklength

  f->pos=0L;

  while(f->pos<f->size)
  { rx_block();
    rxed_offset=(unsigned long int *)bl->data;
    rxed_blocklen=(unsigned short int *)(bl->data+4);
    data_start=bl->data+6;

printf("rxed_offset: %ld\n"
       "rxed_blocklen: %d\n"
       "total_blocklen: %d\n---\n",
       *rxed_offset,* rxed_blocklen, bl->len);


  }

  s_close(f->handle);


}

/*---------------------------------------------------------------------------*/

void fileio_dida::_tx(void)
{ char part_md5str[33];
//  char part_md5hash[16];
  unsigned long int resume_offset;
  unsigned short int rxed_blocksize;
  char *buf;
  unsigned long int rnum;

  if(!f->unixtime)
  { putf("File not found.\n");
    return;
  }

  putf("\n#DIDADIT#\n");

  tx_INFO();

  getline(b->line, 100, 1); // #OK#

  if(strstr(b->line, "#OK#")!=b->line)
    return;

  alloc_blockmem(1000); // TODO: 1000 bytes max.?

  // We now expect an ERR- or a START-frame
  rx_block();

  switch(bl->type)
  {
  case R_BLOCK_ERR:
    putf("*** Transfer aborted by ERR-Frame\n");
    return;

  case R_BLOCK_START:
    break;

  default:
    tx_ABORT();
    putf("*** Expected ERR- or START-Frame, got 0x%04X.\n", bl->type);
    return;
  }

  parse_START(&resume_offset, part_md5str, &rxed_blocksize);
  if(rxed_blocksize && rxed_blocksize<DEFAULT_BLOCKSIZE)
    blocksize=rxed_blocksize;
  else
    blocksize=DEFAULT_BLOCKSIZE;

  buf=(char *)t_malloc(blocksize, "dida");
  f->handle=s_open(f->fullname, "lrb");
  f->pos=0L;
  while((rnum=__read(f->handle, buf, blocksize))>0)
  { tx_DATA(f->pos, rnum, buf);
    f->pos+=rnum;
    waitfor(e_ticsfull);
    // TODO: berprfen, ob etwas empfangen wurde
  }
  s_close(f->handle);
  t_free(buf);

  putflush();
  wdelay(1000);
  tx_FIN("???");

  // Expect FINACK- or REQ-block
  alloc_blockmem(1000); // TODO: Max. 1000 Bytes
  rx_block();

//  switch(bl->type)
//  {
//    TODO...
//  }

  putflush();
}

/*---------------------------------------------------------------------------*/

#endif
