/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  --------------------------------------
  DIDADIT-Protocol for BCM (Definitions)
  --------------------------------------


  Copyright (C)       Johann Hanne, DH3MB, jonny@baycom.org
                      Falkenweg 6
                      D-85122 Hitzhofen

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

/*---------------------------------------------------------------------------*/

#define DIDADIT_VERSION "0.5.6"

#define DEFAULT_BLOCKSIZE 4096
#define MAX_BLOCKSIZE 8192

/*---------------------------------------------------------------------------*/

// KISS-Konstanten
#define FEND	0xC0	// FrameEND
#define FESC	0xDB	// FrameESCape
#define TFEND	0xDC	// Transponiertes FrameEND
#define TFESC	0xDD	// Transponiertes FrameESCape

/*---------------------------------------------------------------------------*/

#define R_BLOCK_INFO	1
#define R_BLOCK_START	2
#define R_BLOCK_ERR	3
#define R_BLOCK_DATA	4
#define R_BLOCK_FIN	5
#define R_BLOCK_REQ	6
#define R_BLOCK_FINACK	7
#define R_BLOCK_TYPEREJ	8
#define R_BLOCK_ECHOREQ	9
#define R_BLOCK_ECHOREP	10
#define R_BLOCK_ABORT	11

/*---------------------------------------------------------------------------*/

#define R_RX_STATE_TYPE		1
#define R_RX_STATE_MD5HASH	2
#define R_RX_STATE_DATA		3
#define R_RX_STATE_INVALID	100

/*---------------------------------------------------------------------------*/

struct block_info {
  unsigned short int type;	// Block type
  unsigned short int maxlen;	// Number of allocated bytes for the block
  char *data;			// Pointer to block data
  unsigned short int len;	// Length of received block
};

/*---------------------------------------------------------------------------*/

class fileio_dida : public fileio
{

private:
  void tx_stuffed(char);
  signed short int rx_stuffed(void);
  void tx_block(unsigned short int, char, char *, unsigned short int);
  void rx_block(void);
  void alloc_blockmem(unsigned short int);
  void free_blockmem(void);
  char *parse_textblock(char *, char **, char **);
  void parse_INFO(char *, unsigned short int *rxed_blocksize);
  void parse_START(unsigned long int *, char *, unsigned short int *);
  void tx_INFO(void);
  void tx_START(unsigned long int, char *, unsigned short int);
  void tx_ERR(char *);
  void tx_DATA(unsigned long int, unsigned short int, char *);
  void tx_FIN(char *);
  void tx_REQ(unsigned long int, unsigned long int);
  void tx_FINACK(void);
  void tx_TYPEREJ(unsigned short int);
  void tx_ECHOREQ(void);
  void tx_ECHOREP(void);
  void tx_ABORT(void);
  char partner_ver[10];
  unsigned short int blocksize;
  struct block_info *bl;

protected:
  void _rx(void);
  void _tx(void);

public:
  fileio_dida(void);
  ~fileio_dida(void);

};

/*---------------------------------------------------------------------------*/
