/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -----------------------------------------
  Ausgaben und Zubehoer zum DIR/LIST-Befehl
  -----------------------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980118 OE3DZW removed sec from dir/list - timestr
//19980218 OE3DZW renamed flag5 to msgty
//19980308 OE3DZW dir-option "-r" -> only held msg
//19980312 OE3DZW above option replaced by "-p", above did not work correctly
//19980407 OE3DZW bytes korrekt
//19980423 OE3DZW new "holdusers" shows all usermails held
//19980430 OE3DZW fixed holdusers, only showed results on first call
//19980526 OE3DZW added debug-logoutput, to find "d b"  bug
//19980824 CZ4WAY added headerfile to mailbeacon
//         CZ4WAY (CB) = Jan Wasserbauer <jan.wasserbauer@spsselib.hiedu.cz>
//19980831 OE3DZW added usermail=0 to dir news etc. (else wrong options
//                could eventually be read
//19980906 DG0CDC added macro support to headerfile of mailbeacon
//19980903 OE3DZW fixed s_fclose not open bug (related to beacon update)
//19980914 OE3DZW added header for dir/list 
//19980915 OE3DZW fixed header - was missing for usermails
//                fixed search algorithm - will find correct boards
//19980924 OE3DZW d n shows correct boardinfo
//19980924 OE3DZW added oldestbeacon
//19980924 OE3DZW shows only local mail in mailbeacon
//19980925 OE3DZW fixed mailbeacon, will only show mail to local users 
//                not logged in since mail received
//19981001 OE3DZW becon could become longer 256 Bytes
//19990824 DH3MB  Changed format of "DIR BOARDS *"

#include "baycom.h"

/*---------------------------------------------------------------------------*/

#define BEACON_LEN 3000

/*---------------------------------------------------------------------------*/

int  finddir (char *path,int one_letter_boards)
//***********************************************************************
//
//  Sucht einen Bulletin- oder Benutzernamen
//
//  Rckgabewerte:
//     0 Board nicht gefunden
//    -1 Usermail, keine Nachricht vorhanden
//    -2 Usermail, Nachricht vorhanden
//
//***********************************************************************
{
  int ret = (-2), j;
  char cmp[7];

  b->boardname[0] = 0;
  b->boardpath[0] = 0;
  b->boardfullname[0] = 0;

  strupr (path);
  j = strlen (path) - 1;
  while (j && (path[j - 1] != '/'))
    j--;

  // cmp contains the first six characters of a board name, which eventually
  // may have been cut by BBS-systems which only allow six characters for
  // board names
  strncpy (cmp, path + j, 6);
  cmp[6] = 0;

  if ((strlen (cmp) == 1) && !one_letter_boards && !b->forwarding)
    return 0;

  if (mbcallok (path))
    {
      sprintf (b->boardpath, "%s/%s", m.userpath, path);
      strlwr (b->boardpath);
      if (!filetime (b->boardpath))
	ret = (-1);
      strcpy (b->boardname, path);
      strcpy (b->boardfullname, path);
      b->boardlife_max = m.userlife;
      b->boardlife_min = 0;
      b->usermail = 1;
    }
  else
    {
      int i, found = 0;

      b->usermail = 0;
      strcpy (b->boardpath, m.infopath);
      strcat (b->boardpath, "/");
      for (i = 0; i < treelen; i++)
	{
	  if (strncmp (cmp,tree[i].name+tree[i].flatoffset, 6) == 0)
	    {
	      strcat (b->boardpath, tree[i].name);
	      strcpy (b->boardfullname,tree[i].name);
	      strcpy (b->boardname,tree[i].name+tree[i].flatoffset);
	      b->boardlife_max = tree[i].lifetime_max;
	      b->boardlife_min = tree[i].lifetime_min;
	      found = 1;
	      ret = i + 1;
	      break;
	    }
	}
      if (!found)
	return 0;
    }

  return ret;
}

/*---------------------------------------------------------------------------*/

char *
make_mask (char *mask, char *path)
{
  if (mbcallok (path))
    strcpy (mask, m.userpath);
  else
    strcpy (mask, m.infopath);
  strcat (mask, "/");
  strcat (mask, path);
  strcat (mask, "/*.*");
  return mask;
}

/*---------------------------------------------------------------------------*/

handle preparedir(char *selektor, int printerror)
{ handle fh = EOF;
  static char ts[25];

  waitfor (e_ticsfull);

  b->index = finddir (selektor,b->sysop);


  if (b->index != 0)
  {
    make_mask (b->mask, b->boardfullname);
    strcpy (b->listpath, b->mask);
    if(!strstr (b->listpath, "*.*"))
      trace (fatal, "preparedir", "mask %s", b->listpath);
    strcpy (strstr (b->listpath, "*.*"), DIRLISTNAME);
    strlwr (b->listpath);

    fh = s_open (b->listpath, "lrb");
    if (fh != EOF)
    {
      if (filelength (fh) < 128)
      {
        s_close(fh);
        trace(serious,"prepdir","len0 %s",b->listpath);
        // not really serious - why not?? (DH3MB)
        xunlink (b->listpath);
        fh = EOF;
      }
      else
      {
        _read (fh, ts, 24);
        lseek (fh, 0L, SEEK_SET);
        if ((ts[7] != '<') || (ts[23] != '~'))
        {
          s_close (fh);
          trace (serious, "prepdir", "kaputt %s", b->listpath);
          xunlink (b->listpath);
          fh = EOF;
        }
      }
    }

    if ((fh == EOF) && !b->semalock)
    { createdirlist(0);
      strlwr(b->listpath);
      fh=s_open(b->listpath, "lrb");
    }

  }
  else if (printerror && (b->index == 0))
    putf (ms (m_noboxfound), selektor);

  waitfor(e_ticsfull);

  return fh;
}

/*---------------------------------------------------------------------------*/

time_t 
dirline_data (char *dirline)
{
  int i = 125;
  long dirtime;
  int life;

  dirline[7] = 0;
  dirtime = filename2time (dirline);
  while (dirline[i] == ' ')
    dirline[i--] = 0;
  if (dirline[14] != '>')
    b->geloescht = dirline[14];
  else
    b->geloescht = 0;
  dirline[14] = 0;
  blankweg (dirline + 8);
  strcpy (b->herkunft, dirline + 8);
  dirline[23] = 0;
  blankweg (dirline + 15);
  strcpy (b->ziel, dirline + 15);
  b->usermail = mbcallok (b->ziel);
  if (b->usermail)
    b->gelesen = (dirline[22] == 'R');
  else
    b->gelesen = 0;
  dirline[30] = 0;
  blankweg (dirline + 24);
  strcpy (b->frombox, dirline + 24);
  dirline[37] = 0;
  blankweg (dirline + 31);
  strcpy (b->at, dirline + 31);
  dirline[50] = 0;
  blankweg (dirline + 38);
  strcpy (b->bid, dirline + 38);
  life = atoi (dirline + 51);
  if (life == 999)
    b->lifetime = 999;
  else
    b->lifetime = life - (int) ((ad_time () - dirtime) / (1440 * 60L));
  if (b->lifetime < (-99))
    b->lifetime = (-99);
  b->lines = (unsigned) ascdez (dirline + 55, 2);
  b->bytes = inonlin (ascdez (dirline + 57, 3));
  dirline[126] = 0;
//OE3DZW Anfangsposition von Betreff veraendert
  if (dirline[60] == '|')
    {
      b->mailtype = dirline[61];
      b->fwdhold = dirline[62];
      b->fwdprop = dirline[63];
      b->replied = dirline[64];
      b->msgty = dirline[65];
      b->flag6 = dirline[66];
      b->flag7 = dirline[67];
      b->flag8 = dirline[68];
      b->flag9 = dirline[69];
      b->flaga = dirline[70];
      b->flagb = dirline[71];
      b->flagc = dirline[72];
      b->flagd = dirline[73];
      b->flage = dirline[74];
      b->flagf = dirline[75];
      b->flag0 = dirline[76];
      strcpy (b->betreff, dirline + 78);
    }
//OE3DZW: alte Position war 61, falls noch altes dir/check vorhanden ist
  else
    {
      strcpy (b->betreff, dirline + 61);
      b->fwdhold = b->fwdprop = b->replied =
	b->msgty = b->flag6 = b->flag7 = b->flag8 = b->flag9 =
	b->flaga = b->flagb = b->flagc = b->flagd = b->flage =
	b->flagf = b->flag0 = '!';
      if (b->usermail)
	b->mailtype = 'P';
      else
	b->mailtype = 'B';
    }
  return dirtime;
}

/*---------------------------------------------------------------------------*/

static void near 
dir_line (int lfd, char *dirline)
{
  char line[140];
  long time;
  char flag = ')';

  if (strlen (dirline) == 7)
    time = filename2time (dirline);
  else
    time = dirline_data (dirline);

#ifdef _MAILFILTER	//db1ras
 if ((b->sysop) || (strcmp(b->herkunft,b->logincall)==0) || (b->at[0]!='~'))
 {
#endif
  if (b->gelesen)
    flag = 'R';
  if (b->replied == '1')
    flag = 'r';
  if (b->geloescht)
    flag = b->geloescht;
  if (b->opt & o_k)
    sprintf (line, "%c ", flag);
  else
    strcpy (line, " ");
  if (b->opt & o_a)
    sprintf (line + strlen (line), "%-6s ", b->herkunft);
  if (b->opt & o_e)
    sprintf (line + strlen (line), "> %-8s ", b->ziel);
  if (b->opt & o_d)
    sprintf (line + strlen (line), "%s ", datestr (time, 
      !!(b->opt & o_y)*(1+2*u->dirformat)));
  if (b->opt & o_t)
    {
      sprintf (line + strlen (line), "%s ", timestr (time, 0));
    }
  if (b->opt & o_z)
    sprintf (line + strlen (line), "%4u ", b->lines);
  if (b->opt & o_b)
    {
      if (b->bytes < 1000000L)
	sprintf (line + strlen (line), "%6lu ", b->bytes);
      else
	sprintf (line + strlen (line), "%5luk ", b->bytes / 1024L);
    }
  if (b->opt & o_i)
    sprintf (line + strlen (line), "$%-12s ", b->bid);
  if (b->opt & o_l)
    sprintf (line + strlen (line), "#%-3d ", b->lifetime);
  if (b->opt & o_f)
    {
      strcat (line, b->frombox);
      if (!(b->opt & o_m))
	strcat (line, "@ ");
    }
  if (b->opt & o_m)
    sprintf (line + strlen (line), "@%s ", b->at);
  if (b->opt & o_x)
    strcat (line, " ");
  line[73] = 0;
  b->betreff[75 - strlen (line)] = 0;
  if (b->opt & o_p)
    sprintf (line + strlen (line), "%c%c%c%c%c ",
	     b->mailtype, b->fwdhold, b->fwdprop, b->replied, b->msgty);
  if (b->opt & o_w)
    strcat (line, b->betreff);
  if (b->http)
    {
      int i = 1;
      strlwr (b->ziel);
      strlwr (dirline);
      putf ("%4d", lfd);
      putv (line[0]);
      if (line[0] != ' ')
	putv (line[i++]);
      html_putf ("<a href=\"/read/%s/%s\">", b->ziel, dirline);
      while (line[i] != ' ')
	putv (line[i++]);
      html_putf ("</a>");
      putf ("%s\n", line + i);
    }
  else
    putf ("%4d%s\n", lfd, line);
#ifdef _MAILFILTER	//db1ras
 }
#endif
  waitfor (e_ticsfull);
}

/*---------------------------------------------------------------------------*/

static char near 
dir_users (char *bake)
{
  int n = 0, i = 0, cnt = 0;
  char *userlist;
  handle fh;
  char buf[32];
  time_t tm = (ad_time() - (long) m.oldestbeacon * 24L * 3600L);
  time_t usrlastlogin;
  int prozent = 0;
  char buffer[50];
  char *msg;
  char *btmp=0;

  if (bake == NULL)
    putf (ms (m_dirofusers));
  else
    {
      FILE *f;
      f = s_fopen (BEACONHEADNAME, "srt");
      if (f)
	{
	  i = fgetc (f);
	  while (i != EOF && n < 150)
	    {
	      // DG0CDC: added support for macros
	      if (prozent)
		{
		  msg = chartomakro (i, buffer);
		  if (n + strlen (msg) < BEACON_LEN - 100)
		    while (*msg)
		      {
			bake[n] = *msg;
			msg++;
			n++;
		      }
		  prozent = 0;
		}
	      else if (i == '%')
		prozent = 1;
	      else
		{
		  bake[n] = i;
		  n++;
		}
	      i = fgetc (f);
	    }
	  if (bake[n - 1] != 0x0A)
	    {
	      bake[n] = 0x0A;
	      n++;
	    }
	  s_fclose (f);
	}
      bake[n] = 0;
      btmp = bake + n;
    }
  i = 0;
  n = 0;

  sprintf (b->mask, "%s/*.*", m.userpath);
  userlist = dirsort (m.userpath, 1);

  if (userlist)
    {
      for (i = 0; userlist[i * 8]; i++)
	{
	  sprintf (b->listpath, "%s/%s/" DIRLISTNAME, m.userpath, userlist + i * 8);
	  strlwr (b->listpath);
	  fh = s_open (b->listpath, "srb");

	  if (fh != EOF)
	    {
	      unsigned len = (unsigned) (filelength (fh) >> 7);

	      while (len)
		{
		  len--;
		  waitfor (e_ticsfull);
		  lseek (fh, 128L * len, SEEK_SET);
		  _read (fh, buf, 32);
		  if (((buf[14] == '>') || (b->optplus & o_v)) &&
		      (!bake || (toupper (buf[22]) != 'R')))
		    {
		      if (bake)
			{
			  buf[7] = 0;
                          time_t mailtime=filename2time(buf);
			  if (mailtime > tm &&
                               get_usr_local(userlist+i*8,usrlastlogin) &&
                               mailtime > usrlastlogin)
			    {
			      // end of frame ?
			      if (strlen (bake) > 220 && n == 0)
				{	// insert zero delimiter

                                  if (btmp)
				    strupr (btmp);
				  bake += (strlen (bake) + 1);
				  bake[0] = 0;
				  btmp = bake;
				  n = 0;
				}
                              // + 7 Bytes
			      strcat (bake, userlist + i * 8);
			      cnt++;
                              //make sure that beacon is not longer
                              //than 256 bytes 
			      if (++n > 10 || strlen(bake) > 243)
				{ // + 2 Bytes
				  strcat (bake, "\n");
				  n = 0;
				}
			      else
				strcat (bake, " ");
			    }
			}
		      else
			{
			  putf ("%-7s", userlist + i * 8);
			  if (++n > 10)
			    {
			      putv ('\n');
			      n = 0;
			    }
			}
		      break;
		    }
		}
	      s_close (fh);
	    }
	}
      t_free (userlist);
    }
  if (n)
    {
      if (bake)
	strcat (bake, "\n");
      else
	putv ('\n');
    }
  if (bake)
    {	
      // append double zero to terminate beacon string
      i = strlen (bake);
      bake[i] = 0;
      bake[i + 1] = 0;
      if (btmp)
        strupr (btmp);
    }
  return cnt;
}

/*---------------------------------------------------------------------------*/

void 
sendmailbake (char *)
{
  char s[100];
  FILE *f;
  int i = 0;
  char *buffer = (char *) t_malloc (BEACON_LEN, "beac");
  char *pb = buffer;
  int frames = 0;
  int users = 0;

  if (sema_lock ("beacon") == 0)
    return;

  if (!m.disable && !sema_test ("purgereorg"))
    {
      users = dir_users (buffer);

      if (users != 0)
	{
	  f = s_fopen (BEACONNAME, "srt");
	  if (f)
	    {
	      while (fgets (s, 70, f))
		{
		  if (s[0])
		    {
		      s[strlen (s) - 1] = 0;
		      pb = buffer;
		      while (pb[0])
			{	//trace(report,s,"%1.30s",pb);
			  //strupr(pb);

			  putbeacon_tnc (s, pb);
			  pb += (strlen (pb) + 1);
			  frames++;
			}
		      i++;
		    }
		}
	      s_fclose (f);
	    }
	}
      trace (report, "mailbeacon", "len=%lu num=%d frames=%d users=%d",
	   ((unsigned long) pb - (unsigned long) buffer), i, frames, users);
    }
  t_free (buffer);
  sema_unlock ("beacon");
}

/*---------------------------------------------------------------------------*/

static time_t near 
seek_time (handle fh, time_t zeit)
//*************************************************************************
//
//
//*************************************************************************
{
  char schwelle[8];

  strcpy (schwelle, time2filename (zeit));
  return seek_fname (fh, schwelle, schwelle);
}

/*---------------------------------------------------------------------------*/

static void near 
dir_boards (char *selektor)
{
  int i, tab = 0;
  handle fh;
  //char name[] = "dir_boards";

  strupr (selektor);

  if (selektor[0] == 0)   
    { 
      putf (ms (m_dirofbulletins));
      //trace (report, name, "size %d", treelen);
      for (i = 0; i < treelen; i++)
	{if ( b->sysop || strstr(tree[i].name,"TMP")!=tree[i].name)
	 {char *slash = strchr (tree[i].name, '/');
	  if (slash)
	    {
	      if (tab < 9)
		{
		  putf (" /");
		  tab = 11;
		}
	      else if ((tab + strlen (slash)) > 79)
		{
		  putf ("\n%9c/", ' ');
		  tab = 11;
		}
	      putf (" ");
	      html_putf ("<a href=\"/cmd?cmd=d+%s+-50\">", slash + 1);
	      putf ("%s", slash + 1);
	      html_putf ("</a>");
	      tab += strlen (slash);
	    }
	  else
	    { 
	      if ((strlen (tree[i].name) != 1) || b->sysop)
		{
		  if (tab)
		    putv ('\n');
		  html_putf ("<a href=\"/cmd?cmd=d+%s+-50\">", tree[i].name);
		  putf ("%-8s", tree[i].name);
		  html_putf ("</a>");
		  tab = 8;
		}
	    }
	  if (testabbruch ())
	    break;
         }
	}
      if (tab)
	putv ('\n');
    }
    else
    {
      putf (ms (m_boardlifemails));
      //trace (report, name, "size %d,%s", treelen, selektor);
      for (i = 0; i < treelen; i++)
	{
	  if ((selektor[0] == '*') || strstr (tree[i].name, selektor))
	    {
	      if ((strlen (tree[i].name) != 1) || b->sysop)
		{		
                  board_info(tree[i].name);
		  putf ("%-18s %3d-%3d", tree[i].name,
			tree[i].lifetime_min,
			tree[i].lifetime_max);

		  fh = preparedir (tree[i].name, 0);
		  if (fh != EOF)
		    {
		      long tim = seek_time (fh, ad_time ());
		      if (tim > 0)
			{
 			  putf ("%6ld %4s", lseek (fh, 0, SEEK_CUR) / 128,
                                 zeitspanne(ad_time()-tim,zs_seconds));

			}
		      s_close (fh);
		      waitfor (e_reschedule);
		    }
		  putf ("\n");
		}
	    }
	  if (testabbruch ())
	    break;
	}
    }
}

/*---------------------------------------------------------------------------*/

static int near 
do_dir (char *search, char *dirline, int *oldmail, int *notfound, int readlock)
{
  if (search && search[0] && (stristr (dirline, search) == 0))
    {
      (*notfound)++;
      return 0;			// Suchkriterium nicht erfllt

    }

  if (readlock)
    {
      char fromcall[10];
      strncpy (fromcall, dirline + 8, 6);
      fromcall[6] = 0;
      blankweg (fromcall);
      if (strcmp (fromcall, b->logincall) != 0 && !b->sysop)
	{
	  if (readlock == 2 || toupper (dirline[22]) != 'R')
	    return 0;
	}
    }

  if ((b->opt & o_g) && (toupper (dirline[22]) == 'R') && b->usermail && !b->bereich)
    {
      (*oldmail)++;
      return 0;			// G aktiv und Nachricht schon gelesen

    }

  if (!(b->opt & o_v) && (dirline[14] != '>'))
    return 0;			// V nicht aktiv aber Nachricht gelscht

  if ((b->opt & o_p) && (dirline[62] == '!') && (dirline[77] == '-'))
    return 0;			// Only held messages   //check for new format 


  return 1;
}

/*---------------------------------------------------------------------------*/
void board_info(char *board)
// outputs some info on a certain bulletin board
{  
  char line[89];
  char *p;
  char *tmpb=board;
  FILE *f=s_fopen(BOARDINFO,"srt");
  if (!f)
    return;
  if (strchr(tmpb,'/'))
    tmpb=strchr(tmpb,'/')+1;
  while (!feof(f))
  {
    if(fgets(line,88,f)) 
    { p=line;
      while (p[0]==' ')
        p++;
      if (stristr(p,tmpb)==p              //Board is found
         && (strlen(p) > strlen(tmpb)+2)  //String is long enougth
         && p[strlen(tmpb)]==' ')         //It is not a fragment  (eg T TAUSCH)
      { p=strchr(p,' ')+1;
        while (p[0]==' ')
          p++; 
        putf("%s",p);
      }
    } 
  }
  s_fclose(f);
}
/*---------------------------------------------------------------------------*/

static int near 
putdirkopf (void)
{
  char mybbs[HADRESSLEN + 1] = "";
  int ok = 0;
  int is_user=mbcallok(b->boardname);

  if (is_user)
    {
      ok = get_mybbs (b->boardname, mybbs, 0);
      if ((b->opt & o_n) && (strcmp (atcall (mybbs), m.boxname) == 0))
	return 0;
    }  
  if (!(b->opt & o_h))
    {
      putf (ms (m_directoryof), b->boardfullname);
      if (mybbs[0])
	putf (" %c%s", ok ? '@' : '?', mybbs);
      putf (":\n");
      if (!is_user)
      { board_info(b->boardfullname);
      }
      if (b->opt & o_j)
	{
	  putf (ms (m_list_nr));
	  if (b->opt & o_k)
	    putf (" ");
	  if (b->opt & o_a)
	    putf (ms (m_list_call));
	  if (b->opt & o_e)
	    putf (ms (m_list_board));
	  if (b->opt & o_d)
	    {
	      putf (ms (m_list_date));
	      if (b->opt & o_y)
		putf ("   ");
	    }
	  if (b->opt & o_t)
	    putf (ms (m_list_time));
	  if (b->opt & o_z)
	    putf (ms (m_list_lines));
	  if (b->opt & o_b)
	    putf (ms (m_list_bytes));
	  if (b->opt & o_i)
	    putf (ms (m_list_bid));
	  if (b->opt & o_l)
	    putf (ms (m_list_life));
	  if (b->opt & o_x)
	    putf (" ");
	  if ((b->opt & o_f) || (b->opt & o_m))
	    putf (ms (m_list_forw));
	  else
	    putf (ms (m_list_subject));
	  leerzeile ();
	}
    }
  return 1;
}

/*---------------------------------------------------------------------------*/

static void near 
dir_einboard (char *selektor, int printerror, char *search, char *lastkopf)
{
  unsigned kopf = 0, pos = 0;
  time_t filetime;
  char *line = b->line;
  int oldmail = 0, notfound = 0;
  handle fh;
  int nixneues = 0;
  int readlock = 0;
// dzw
  char callmybbs[HADRESSLEN + CALLEN + 4];
  char mybbs[HADRESSLEN + 1] = "";

  fh = preparedir (selektor, printerror);
  formoptions ();
  if (fh != EOF)
    {
      if (b->abwann)
	{
	  filetime = seek_time (fh, b->abwann);
	  if (b->index > 0)
	    tree[b->index - 1].newestmail = filetime;
	  if (printerror && leof (fh))
	    nixneues = 1;
	}
      else if (b->beginn > 0)
	{
	  if (b->beginn > (filelength (fh) >> 7))
	    nixneues = 1;
	  else
	    lseek (fh, ((long) (b->beginn - 1)) << 7, SEEK_SET);
	}
      else
	{
	  if (!seek_lastentry (fh, b->ende))
	    nixneues = 1;
	  b->ende = MAXBEREICH;
	}

      if (nixneues)
	{
	  if (seek_lastentry (fh, 1))
	    putf (ms (m_lastmessage));
	  else
	    pos = b->ende;
	}

      if (mbcallok (b->boardname) && strcmp (b->boardname, b->logincall) != 0)
	{
	  readlock = get_readlock (b->boardname);
	  if (readlock != 1 && u->rlimit)
	    readlock = u->rlimit;
	}

      while (_read (fh, line, 128) && (pos < b->ende) && !testabbruch ())
	{
	  waitfor (e_ticsfull);
	  line[126] = 0;
	  pos = (unsigned) (ltell (fh) >> 7);
	  if (do_dir (search, line, &oldmail, &notfound, readlock))
	    {
#ifdef _MAILFILTER	//db1ras
	     if ((b->sysop) || (line[31]!='~') ||
		 (strncmp(line+8,b->logincall,strlen(b->logincall))==0))
	     {
#endif
	      if (!kopf)
		{
		  if (!printerror && !mbcallok (b->boardname))
		    {
		      char *slash = strchr (b->boardfullname, '/');
		      if (slash)
			slash[0] = 0;
		      if (strcmp (b->boardfullname, lastkopf) == 0)
			kopf = 1;
		      else
			strcpy (lastkopf, b->boardfullname);
		    }
		  if (!kopf)
		    kopf = putdirkopf ();
		}
	      if (kopf)
		dir_line (pos, line);
#ifdef _MAILFILTER	//db1ras
	     }
#endif
	    }
	}
      s_close (fh);
    }
  else
    {
      if (b->index > 0)
	tree[b->index - 1].newestmail = BOARDLEER;
    }
  if (!kopf && printerror && b->boardname[0])
    {
      // oe3dzw: Ausgabe der Mybbs, falls keine (ungelesenen) Nachrichten
      // gefunden wurden.
      if (!mbcallok (b->boardname))
	strcpy (callmybbs, b->boardname);
      else if (!get_mybbs (b->boardname, mybbs, 0) == 0)
	sprintf (callmybbs, "%s @ %s", b->boardname, mybbs);
      else if (mybbs[0])
	sprintf (callmybbs, "%s ? %s", b->boardname, mybbs);
      else
	sprintf (callmybbs, "%s @ ???", b->boardname);
      if (oldmail)
	putf (ms (m_nounreaded), callmybbs);
      else if (notfound)
	putf (ms (m_nomessagefound));
      else
	putf (ms (m_forcallnomessage), callmybbs);
    }
}

/*---------------------------------------------------------------------------*/

static int near 
boardreject (char *boardpath)
{
  char board[30];
  char *slash;
  int not = (u->notvisible[1] != '-');

  strcpy (board + 1, boardpath);
  board[0] = ' ';
  strcat (board, " ");
  slash = strchr (board, '/');

  if (slash)
    {
      slash[0] = ' ';
      if (strstr (u->notvisible, slash))
	return not;
      slash[1] = 0;
    }
  if (strstr (u->notvisible, board))
    return not;
  else
    return !not;
}

/*---------------------------------------------------------------------------*/

static void near 
dmusers (char *search)
{
  char *userlist;
  char lastkopf[30] = "";
  int i = 0;

  b->usermail = 1;
  formoptions ();

  putf (ms (m_dirofusermails));
  if ((b->opt & o_q) && (!search || !search[0]))
    b->abwann = u->lastboxlogin;
  else
    b->abwann = 0;
  userlist = (char *) t_malloc (8U * MAXUSERMAILS, "user");
  if (userlist)
    {
      i = 0;
      struct dirent *di;
      DIR *d = opendir (m.userpath);

      while ((di = readdir (d)) != NULL)
	{
	  if (di->d_name[0] != '.')
	    {
	      strncpy (userlist + (i * 8), di->d_name, 6);
	      userlist[i * 8 + 6] = 0;
	      i++;
	      if (i >= MAXUSERMAILS)
		trace (fatal, "dmusers", "too many usermails");
	    }
	  waitfor (e_ticsfull);
	}
      closedir (d);

      while (i)
	{
	  i--;
	  b->beginn = 1;
	  b->ende = MAXBEREICH;
	  dir_einboard (userlist + (i * 8), 0, search, lastkopf);
	  if (testabbruch ())
	    break;
	}
      t_free (userlist);
    }
}

/*---------------------------------------------------------------------------*/

static int 
dir_outstanding (char *call)
//*************************************************************************
//
//  Gibt alle (bzw. eine Auswahl) Mails aus, die zu einem bestimmten
//  FWD-Partner zum Forward anstehen
//
//*************************************************************************
{
  char fwdfname[20];
  char s[256];
  int len;
  FILE *f, *mailf;
  int count = 0, imail;
  char *path[2] =
  {"u_", "i_"};
  int nokopf = 1;
  int retwert = 1;

  for (imail = 0; imail < 2; imail++)
    {
      sprintf (fwdfname, FWDPATH "/%s%s.bcm", path[imail], call);
      strlwr (fwdfname);
      b->opt = o_k | o_a | o_e | o_d | o_y | o_b | o_f | o_m | o_w;
      b->gelesen = 0;
      f = s_fopen (fwdfname, "lrt");
      if (f)
	{
	  while (fgets (s, 255, f))
	    {
	      if (testabbruch ())
		{
		  retwert = 0;
		  break;
		}
	      waitfor (e_ticsfull);
	      if (s[0] > ' ')
		{
		  len = strlen (s);	// remove newline

		  if (s[0])
		    s[len - 1] = 0;
		  count++;
		  blankweg (s);

		  if ((b->beginn == 0 || count >= b->beginn) && count <= b->ende &&
		      setfullpath (s))
		    {
		      strlwr (b->mailpath);
		      mailf = s_fopen (b->mailpath, "srt");
		      if (mailf == NULL)
			{
			  count--;
			  waitfor (e_reschedule);
			  continue;
			}
		      fgets (s, 255, mailf);
		      mbsend_parse (s, 0);
		      fgets (s, 255, mailf);	// fwd trigger

		      if (s[0] == '*')
			b->geloescht = s[1];
		      else
			b->geloescht = 0;
		      fgets (s, 255, mailf);	// read: line

		      if (fgets (s, 255, mailf))	// subject 

			{
			  if (s[0])
			    s[strlen (s) - 1] = 0;
			  s[78] = 0;
			  strcpy (b->betreff, s);
			  strcpy (b->at, atcall (b->at));
			  if (nokopf)
			    {
			      nokopf = 0;
			      putf ("\nForward queue of %s:\n", call);
			    }
			  s_fclose (mailf);
			  dir_line (count, b->mailfname);
			}
		      else
			s_fclose (mailf);
		      waitfor (e_reschedule);
		    }
		}
	    }
	  s_fclose (f);
	}
    }
  return retwert;
}

/*---------------------------------------------------------------------------*/

static void 
dir_outstanding_all (void)
//*************************************************************************
//
//  Gibt die komplette Liste aller ausstehenden Forwardfiles aus
//
//*************************************************************************
{
  struct dirent *di;
  DIR *d = opendir (FWDPATH);
  char calls[MAXFWD][7];
  char loccall[30];
  char *c;
  int found, i;

  memset (calls, 0, sizeof (calls));

  while ((di = readdir (d)) != NULL)
    {
      if (di->d_name[1] == '_')
	{
	  char ui = tolower (di->d_name[0]);

	  if (ui == 'u' || ui == 'i')
	    {
	      strcpy (loccall, di->d_name + 2);
	      c = strchr (loccall, '.');
	      if (c)
		c[0] = 0;
	      strupr (loccall);
	      if (strlen (loccall) <= 6 && isforwardpartner (loccall) >= 0)
		{
		  found = 0;
		  for (i = 0; i < MAXFWD; i++)
		    {
		      if (strcmp (loccall, calls[i]) == 0)
			found = 1;
		    }
		  for (i = 0; i < MAXFWD; i++)
		    {
		      if (found == 0 && calls[i][0] == 0)
			{
			  strcpy (calls[i], loccall);
			  break;
			}
		    }
		}
	    }
	}
      waitfor (e_ticsfull);
    }
  closedir (d);
  for (i = 0; i < MAXFWD; i++)
    {
      if (calls[i][0] != 0)
	{
	  if (dir_outstanding (calls[i]) == 0)
	    break;
	}
    }
}

/*---------------------------------------------------------------------------*/

static void near 
dir_messages (char *search)
{
  int i;
  char lastkopf[50] = "";
  if (b->abwann)
    putf (ms (m_dirafter), zeitstring (b->abwann, 2));
  if (search && search[0])
    putf (ms (m_searchfor), search);
  for (i = 0; i < treelen; i++)
    {
      if (!tree[i].newestmail || (tree[i].newestmail > b->abwann))
	{
	  b->beginn = 1;
	  b->ende = MAXBEREICH;
	  if (!boardreject (tree[i].name))
	    dir_einboard (tree[i].name, 0, search, lastkopf);
	}
      if (testabbruch ())
	break;
    }
  if (lastkopf[0] == 0)
    putf (ms (m_nomessagesfound));
}

/*---------------------------------------------------------------------------*/

void 
mbdir (char *selektor, int listcmd)
{
  char sub;
  char boardname[30];
  char searchitem[31] = "";
  char *search;

  static char *beftab[] =
  {"USERS", "BOARDS", "MESSAGES", "AFTER", "NEWS", "PATH",
   "SENT", "OUTSTANDING", "HOLDUSERS", "HEADERS", NULL
  };

  enum befnum
    {
      unsinn,
      users, boards, messages, after, news, path, 
      sent, outstanding, holdusers, headers
    }
  cmd = unsinn;

  scanoptions (selektor);
  get_searchitem (selektor, searchitem, 30);
  selektor += blkill (selektor);
  strupr (selektor);

  if (!listcmd)
    cmd = (befnum) readcmd (beftab, &selektor, 0);

  if (searchitem[0])
    search = searchitem;
  else
    search = selektor;

  switch (cmd)
    {
    case unsinn:
      {
	findbereich (selektor);

	if (searchitem[0] == 0)
	  {
	    search = strchr (selektor, '<');
	    if (search)
	      {
		search[0] = 0;
		search++;
		blankweg (selektor);
	      }
	    else
	      search = blankweg (selektor);
	  }
	if (search)
	  {
	    search += blkill (search);
	    if (search[0])
	      putf (ms (m_searchfor), search);
	  }
	selektor += blkill (selektor);
	if (selektor[0] == 0)
	  strcpy (boardname, b->prompt);
	else
	{ strncpy (boardname, selektor, 20);
          boardname[20]=0;
        }

        if (finddir (boardname,b->sysop)==0)
          nameconvert(boardname);

	if (finddir (boardname,b->sysop) != 0)
	  strcpy (b->prompt, b->boardfullname);
	formoptions ();

	if ((b->opt & o_q) && !b->bereich && (!search || !search[0]))
	  b->abwann = u->lastquit;
	else
	  b->abwann = 0;

	if (b->opt & o_s)
	  {
	    char lastkopf[10] = "";
	    int i;
	    unsigned beg = b->beginn;
	    unsigned end = b->ende;

	    for (i = 0; i < treelen; i++)
	      {
		if (strstr (tree[i].name, boardname))
		  {
		    b->beginn = beg;
		    b->ende = end;
		    dir_einboard (tree[i].name, 0, search, lastkopf);
		  }
		if (testabbruch ())
		  break;
	      }
	  }
	else
	  dir_einboard (boardname, 1, search, "");
      }
      break;

    case outstanding:
      findbereich (selektor);
      selektor += blkill (selektor);
      blankweg (selektor);
      strupr (selektor);
      if (selektor[0] == 0)
	dir_outstanding_all ();
      else if (isforwardpartner (selektor) >= 0)
	dir_outstanding (selektor);
      else
	putf ("Syntax: DIR OUTSTANDING <adjacent bbs call>\n");
      break;


    case holdusers:
      b->optplus = o_n + o_p;
      b->optminus = ~b->optplus | o_v | o_q;
      dmusers (selektor);
      break;

    case headers:
        if(selektor[0])
          putlog(BOARDINFO, selektor);
        else
        { fileio_text fio;

          fio.usefile(BOARDINFO);
          fio.tx();
        }  
      break;

    case users:
      sub = selektor[0];
      if (sub)
	{
	  while (selektor[0] > ' ')
	    selektor++;
	  selektor += blkill (selektor);
	  if (searchitem[0] == 0)
	    search = selektor;
	  switch (toupper (sub))
	    {
	    case 'A':
	      listuser (search, 0);
	      break;
	    case 'L':
	      listuser (search, 1);
	      break;
	    case 'M':
	      dmusers (search);
	      break;
	    }
	}
      else
	dir_users (NULL);
      break;

    case boards:
      dir_boards (search);
      break;

    case after:
      {
	b->abwann = parse_time (selektor);
	selektor += blkill (selektor);
	b->usermail = 0;
	dir_messages (selektor);
      }
      break;

    case sent:
      if (!search[0])
	strcpy (search, b->logincall);
      // intentionally NO break;
    case messages:
      b->abwann = 0;
      b->usermail = 0;
      dir_messages (search);
      break;

    case news:
      if (b->lastdirnews == 0)
	b->lastdirnews = ad_time ();
      loaduser (b->logincall, u, 1);
      u->lastdirnews = ad_time ();
      saveuser (u);
      b->abwann = b->lastdirnews;
      b->usermail = 0;
      dir_messages (search);
      break;

    case path:
      putf (ms (m_dirpath));
      printfwdlist (search);
      break;
    }
}
