	/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  ---------------------------------------------------
  Filetransfer base class + text- and AutoBIN-methods
  ---------------------------------------------------


  Copyright (C)       Florian Radlherr
		      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980315 hrx    void fileio::send_autobin -> on rbin/rprg: if
//                requested file's length is 0 bytes we don't send
//                out the file.
//19980430 OE3DZW tmp-files in tmp-path, not in bcm-path
//19990206 OE3DZW added check for missing filename
//19990618 DH3MB  Fixed send_autobin(): 0-byte-files were not closed
//                Cleaned up recv_autobin()
//                Adapted for new CRC classes
//19990703 DH3MB  Changed "recv" to "rx" and "send" to "tx"
//19990804 DH3MB  Introduced base classes for file transfer stuff
//19990807 DH3MB  Extended AutoBIN-statistics
//                More informative error messages on text- and autobin-rx
//                Changed type of file size variables to off_t
//19991011 F1ULQ  Corrected bug in rprg , filename in header was shit 
//19991025 OE3DZW shorter status
//19991026 OE3DZW removed F1ULQ's change, users complained about it...
#include "baycom.h"

/*---------------------------------------------------------------------------*/
fileio::fileio(void)
{ f=(file_info_t *)t_malloc(sizeof(file_info_t), "fio");

  *(f->fullname)=0;
  *(f->name)=0;
  *(f->path)=0;
  f->size=0;
  f->offset=0;
}
/*---------------------------------------------------------------------------*/

fileio::~fileio(void)
{ t_free(f);
}

/*---------------------------------------------------------------------------*/

void fileio::usefile(char *filename)
{ if(!filename[0])
    return;



#ifdef __LINUX__
  if(filename[0]=='/' || filename[0]=='\\')
  { strncpy(f->fullname, filename, 198);
    f->fullname[198]=0;
  }
  else
  { getcwd(f->fullname, 197-strlen(filename));
    strcat(f->fullname, "/");
    strcat(f->fullname, filename);
  }
#else
  strncpy(f->fullname,filename,198);
  f->fullname[198]=0;
#endif
  killbackslash(f->fullname);

  if (strchr(f->fullname,'/'))
  { strncpy(f->name, strrchr(f->fullname, '/')+1, 99);
    f->name[99]=0;
  }
  else  
  { strncpy(f->name,f->fullname,99);
    f->name[99]=0;
  }

  strncpy(f->path, f->fullname, 99);
  f->path[99]=0;
  if(strrchr(f->path, '/'))
    *strrchr(f->path, '/')=0;
  if((f->unixtime=filetime(filename))!=0)
  { f->exists=1;
    f->pos=0L;
    f->size=filesize(filename)-f->offset;
    time_unix2dos(f->unixtime, f->dostime);
#ifdef FEATURE_MD5
    if(f->requires && REQ_MD5HASH)
    { MD5 md5;
      md5.readfile(filename, f->offset);
      md5.getdigest(f->md5hash);
      md5.gethexdigest(f->md5str);
    }
#endif
    if(f->requires && REQ_CRCTHP)
    { crcthp crc;
      crc.readfile(filename, f->offset);
      f->crc=crc.result;
    }
  }
  else
    f->exists=0;

  f->eraseflag=0;
}

/*---------------------------------------------------------------------------*/

void fileio::set_offset(off_t offset)
//*************************************************************************
//
//  Specifies the position, where to start with the file transmission
//  That's nothing to do with resume - the file will be sent as
//  if the preciding part of the file did not exist!
//
//  This method *MUST* be called before usefile()!
//
//*************************************************************************
{ f->offset=offset;
}

/*---------------------------------------------------------------------------*/

void fileio::doerase(void)
{ f->eraseflag=1;
}

/*---------------------------------------------------------------------------*/

void fileio::tx(void)
{ if(*f->fullname==0)
  { putf("Filename missing.\n");
    return;
  }

  if(!f->exists)
  { putf("File not found.\n");
    return;
  }

  _tx();
}

/*---------------------------------------------------------------------------*/

void fileio::rx(void)
{ if(*f->fullname==0)
  { putf("Filename missing.\n");
    return;
  }

 _rx();
}

/*---------------------------------------------------------------------------*/

fileio_text::fileio_text(void) : fileio()
{ tail=0;
  f->requires=0;
}

/*---------------------------------------------------------------------------*/

void fileio_text::settail(signed short int t)
{ tail=t;
}

/*---------------------------------------------------------------------------*/

void fileio_text::_tx(void)
//*************************************************************************
//
//  Sends a text file
//
//*************************************************************************
{ int a, last=0;

  f->ptr=s_fopen(f->fullname, "lrt");
  if(f->ptr)
  { if(f->eraseflag)
      s_fsetopt(f->ptr, 2);

    if(tail<0 && (f->size)>(-tail))
    { fseek(f->ptr, tail, SEEK_END);
      while((a=fgetc(f->ptr))!='\n' && a!=EOF); // go to begin of next line
    }

    while((a=fgetc(f->ptr))!=EOF)
    { last=a;
      putv(a);
      waitfor(e_ticsfull);

      if(testabbruch())
        break;

      if(tail>0 && (f->pos++)>tail && a=='\n')
        break;
    }

    if(last!='\n')
      putv('\n');

    s_fclose(f->ptr);
  }
  else
    putf("Can't open %s\n", f->fullname);
}

/*---------------------------------------------------------------------------*/

void fileio_text::_rx(void)
//*************************************************************************
//
//  Receives a text file
//
//*************************************************************************
{ char *line=b->line;
  char *eof;
  char tmpname[20];

  if(f->exists)
    putf("File exists, abort with CTRL-Z.\n");
  putf("Now waiting for text file %s, terminate with CTRL-Z.\n", f->fullname);
  putflush();

  sprintf(tmpname, TEMPPATH "/%s", time2filename(0));

  f->ptr=s_fopen(tmpname, "lwt");
  if(f->ptr)
  { s_fsetopt(f->ptr, 1);

    do
    { getline(line, 255, 1);

      subst(line,0x01,CTRLZ); //also accept CTRLA as eof-marker
      if((eof=strchr(line, CTRLZ))!=0||stricmp(line,"/EX")==0)
      { *eof=0;

	if(strlen(line))
	{ fputs(line, f->ptr);
	  fputc('\n', f->ptr);
	}
      }
      else
      { fputs(line, f->ptr);
	fputc('\n', f->ptr);
      }
    }
    while(!eof);

    if(ftell(f->ptr)==0)
    { s_fclose(f->ptr);
      putf("No file stored.\n");
      xunlink(tmpname);
    }
    else
    { s_fclose(f->ptr);
      xunlink(f->fullname);      

      if(filetime(f->fullname)!=0) 
        putf("Unable to remove old file, tempfile is %s\n", tmpname);
      else
      {  if(xrename(tmpname, f->fullname)!=0)
          putf("Error writing output file\n");
        else
          putf("File %s ok.\n",f->fullname);
      }
    }
  }
  else
    putf("Can't create tempfile %s\n", tmpname);
}

/*---------------------------------------------------------------------------*/

fileio_abin::fileio_abin(void) : fileio()
{ *autobin_header=0;
  f->requires=REQ_CRCTHP;
}

/*---------------------------------------------------------------------------*/

void fileio_abin::set_header(char *str)
{ strncpy(autobin_header, str, 80);
  autobin_header[80]=0;
}

/*---------------------------------------------------------------------------*/

void fileio_abin::_tx(void)
//*************************************************************************
//
//  Sends a file using the AutoBIN protocol
//
//*************************************************************************
{ int a;
  unsigned long int bps;
  time_t startt, txtime;

  if(f->size==0)
  { putf("File is 0 bytes long.\n");
    return;
  }

  f->ptr=s_fopen(f->fullname, "lrb");
  if(f->ptr)
  { if(f->eraseflag)
      s_fsetopt(f->ptr, 2);

    fseek(f->ptr, f->offset, SEEK_SET);

    if(*autobin_header)
      putf("%s", autobin_header);
    else

    //printf("kurz: %s\nlang: %s\n",f->name,f->fullname);
    putf("#BIN#%ld#|%05u#$%s#%s\n", f->size, f->crc, f->dostime, f->name);
    // putf("#BIN#%ld#|%05u#$%s#%s\n", f->size, f->crc, f->dostime, 
    // f->fullname);  //F1ULQ

    startt=ad_time();

    while((a=fgetc(f->ptr))!=EOF)
      bputv(a);

    txtime=ad_time()-startt;
    if(txtime==0)
      txtime=1L;
    bps=f->size/txtime;
    putf("\nBIN-TX OK (%04Xh, %lu bit/s)\n",
         f->crc, bps*8L);

    s_fclose(f->ptr);
  }
  else
    putf("Can't open %s\n", f->fullname);
}

/*---------------------------------------------------------------------------*/

void fileio_abin::_rx(void)
//*************************************************************************
//
//  Receives a file using the AutoBIN protocol
//
//*************************************************************************
{ char *line=b->line;
  char tmpname[20];
  int a;
  off_t flen=0L, len;
  char rxed_crc=0;
  unsigned short int rx_crc=0;
  crcthp crc;
  unsigned long int bps;
  time_t startt=0, rxtime;

  sprintf(tmpname, TEMPPATH "/%s", time2filename(0));

  if(f->exists)
    putf("File exists, abort with CTRL-Z.\n");
  putf("Now waiting for binary file %s\n", f->fullname);
  putflush();

  f->ptr=s_fopen(tmpname, "lwb");
  if(f->ptr)
  { s_fsetopt(f->ptr, 1);

    // Wait for the #BIN#-Header, ignore empty lines
    do
      getline(line, 255, 1);
    while(!line[0]);

    if(strstr(line, "#BIN#")==line)
    { putf("#OK#\n");
      putflush();

      // Get the file length
      flen=len=atol(line+5);

      // Get the CRC
      if(strstr(line, "#|"))
      { rx_crc=(unsigned short int) atol(strstr(line, "#|")+2);
        rxed_crc=1;
      }

      startt=ad_time();

      while(len--)
      { a=getv();
        fputc(a, f->ptr);
        crc.update(a);
      }
    }

    if(ftell(f->ptr)==0)
    { s_fclose(f->ptr);
      putf("No file stored.\n");
      xunlink(tmpname);
      return;
    }

    s_fclose(f->ptr);

    if(rxed_crc && (rx_crc!=crc.result))
    { putf("CRC error, no file stored.\n");
      xunlink(tmpname);
      return;
    }

    rxtime=ad_time()-startt;
    if(rxtime==0)
      rxtime=1L;
    bps=flen/rxtime;
    putf("\nBIN-RX OK (%04Xh, %lu bit/s)\n",
         crc.result, bps*8L);

    if(f->exists && xunlink(f->fullname)!=0) // DH3MB
      putf("Unable to unlink old file, tempfile is %s\n", tmpname);
    else
      if(xrename(tmpname, f->fullname)!=0)
        putf("Error renaming tempfile %s\n", tmpname);
  }
  else
    putf("Can't create tempfile %s\n", tmpname);
}

/*---------------------------------------------------------------------------*/
