/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -----------
  File-Server
  -----------


  Copyright (C)       Johann Hanne, DH3MB, jonny@baycom.org
                      Falkenweg 6
                      D-85122 Hitzhofen

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

/*---------------------------------------------------------------------------*/

//19980419 OE3DZW fixed support for DOS,path "off",slash vs backslash
//                now using normal help
//19980420 OE3DZW fixed support for Win32
//                21:12
//19980504 DH3MB  fixed problems with long filename - filenames
//                now may generally not be longer than FNAMELEN
//19980505 OE3DZW now accepting #ok#
//19980526 OE3DZW could not change to "home" directory under dos
//19980610 OE3DZW added log for commands when in fs-mode
//19980619 OE3DZW added fsintro-text again
//19980830 OE3DZW fixed hang of fs on too long input lines
//19980916 OE3DZW added prefix to lastcmd again
//19980924 OE3DZW cleaned up overlapping strcpy's, limited inputline
//19980925 OE3DZW limited size of dir to ca. 167 under DOS
//19981001 OE3DZW fixed wyapp

#include "baycom.h"

#ifdef FILESURF

#ifdef __UNIX__
  //seperator for paths
  #define _SEP '/'
  #define _SEPS "/"
  //invalid seperator for paths
  #define _INVSEP '\\'
#else
  #define DRIVE_LETTERS
  #define _SEP '\\'
  #define _SEPS "\\"
  #define _INVSEP '/'
#endif

/*---------------------------------------------------------------------------*/

void filesurf::putwelcome()
//*************************************************************************
//
//  Shows the intro-text
//
//*************************************************************************
{ putf("Welcome to FileSurf. \n\n");
  readtext("fsintro");
}

/*---------------------------------------------------------------------------*/

void filesurf::printallowed()
//*************************************************************************
//
//  Lists the directories, which the users may read from
//
//*************************************************************************
{ char tempallowed[FSPATHLEN+1];
  char *pos;
  char *ptr;
  char accesslevel;

  putf("Paths available:\n");
  strcpy(tempallowed, m.fspath);
  strcat(tempallowed, " ");
  ptr=tempallowed;
  while((pos=strstr(ptr, " "))!=NULL)
  { *pos=0;
    accesslevel=1;
    if(*ptr=='+')
    { accesslevel=2;
      ptr++;
    }
    putf("        %s", ptr);
    if(accesslevel==2)
      putf(" (read/write) ");
    putf("\n");
    ptr=pos+1;
  }
}

/*---------------------------------------------------------------------------*/

int filesurf::pathallowed(char *path, int level)
//*************************************************************************
//
//  Checks, if a user may read from/write to a directory
//  If not, the allowed directories are shown
//
//  level 1..read only, 2..read/write
//
//  returns 1 if ok, 0 else
//
//*************************************************************************
{ char tempallowed[FSPATHLEN+1];
  char *pos;
  char *ptr;
  char accesslevel;
  char pathpart[FSPATHLEN+1];

  if(b->sysop) return 1; // Sysops may do everything

  strcpy(tempallowed, m.fspath);
  strcat(tempallowed, " ");
  ptr=tempallowed;
  while((pos=strstr(ptr, " "))!=NULL)
  { *pos=0;
    accesslevel=1;
    if(*ptr=='+')
    { accesslevel=2;
      ptr++;
    }
    strcpy(pathpart, ptr);
//    if(pathpart[strlen(pathpart)-1]!=_SEP) strcat(pathpart,_SEPS);
    if((strncmp(path, pathpart, strlen(pathpart))==0) &&
       (accesslevel>=level))
      return(1);
    ptr=pos+1;
  }

  putf("Access to ");
/*  if(path[strlen(path)-1]==_SEP)
    putf("Path");
  else
    putf("File");
  putf(" %s denied. ", path);
*/
  putf("%s denied. ",path);
  printallowed();

  return(0);
}

/*---------------------------------------------------------------------------*/

static near int fileexists(char *fname, char par)
//*************************************************************************
//
//  Checks the existence of a file and shows the according text
//
//*************************************************************************
{ //where comes that shit from???
  //if((fname))
  //{ putf("%s is a directory.\n", fname);
  //  return 0;
  //}

  if(!filetime(fname))
  { if(par==0)
      putf("No such file: %s.\n", fname);
    return 0;
  }
  else
  { if(par==1)
      putf("File %s already exists.\n", fname);
    return 1;
  }
}

/*---------------------------------------------------------------------------*/

int sortbydate(const signed long *num1, const signed long *num2)
{ waitfor(e_ticsfull);

  if(*num1>*num2)
    return(1);
  if(*num1<*num2)
    return(-1);
  return(0);
}

/*---------------------------------------------------------------------------*/

int sortbysize(const signed long *num1, const signed long *num2)
{ waitfor(e_ticsfull);

  if(*(num1+1)>*(num2+1))
    return(1);
  if(*(num1+1)<*(num2+1))
    return(-1);
  return(0);
}

/*---------------------------------------------------------------------------*/

int sortbyname(const char *s1, const char *s2)
{ waitfor(e_ticsfull);

  return(strcmp(s1+8, s2+8));
}

/*---------------------------------------------------------------------------*/

int filesurf::showdir(char *path, bitfeld options)
//*************************************************************************
//
//  Lists the files in a directory
//
//*************************************************************************
{ struct direntry
  { signed long date;
    signed long size;
    char name[40];
  };

  DIR *ptr;
  struct dirent *dirdat;
  struct stat info;
  struct direntry *dirinfo;
  char fullname[200];
  struct tm *time;
  unsigned int count;
  unsigned int counter=0;
  DIR *dummy;
  char maxlength=0;
  char smaxlength[6];
  char maxc=0;
  char filemask[80];
  
  if((ptr=opendir(path))==NULL)
  {
    #ifdef DRIVE_LETTERS   
	if (strlen(path)>3)
    #endif
      path[strlen(path)-1]=0; // Kill trailing slash
    strcpy(filemask, strrchr(path, _SEP)+1);
    *(strrchr(path, _SEP)+1)=0;
    if((ptr=opendir(path))==NULL)
    { putf("Path %s does not exist.\n", path);
      return 0;
    }
  }
  else
    strcpy(filemask, "*");
  strcpy(fullname, path);
  strcat(fullname, ".fs_text");
  if(filetime(fullname))
  { fileio_text fio;
    fio.usefile(fullname);
    fio.tx();
    putf("\n");
  }

  putf("Files in directory: %s\n", path);
  #ifdef __DOS16__
  //good old dos makes no difference between upper/lower case
  strupr(filemask);
  #endif
  putf("Filemask: %s\n\n", filemask);

  while((dirdat=readdir(ptr))!=NULL)
    if(jokcmp(dirdat->d_name, filemask, 0) &&
      (strcmp(dirdat->d_name, "TRANS.TBL")!=0) &&
      (dirdat->d_name[0]!='.'))
      { if ((counter++)*sizeof(direntry) > MAXDIRENTRIES)
        { putf("Sorry, directory too complex!\n");
          closedir(ptr);
          return 1;
        }
      }

  if(!counter)
  { closedir(ptr);
    putf("No such file found.\n");
    return(1);
  }

  dirinfo=(struct direntry *)t_malloc(counter*sizeof(direntry), "fsvr");

  #ifdef _WIN32
  closedir(ptr);
  ptr=opendir(path);
  #else
  rewinddir(ptr);
  #endif

  waitfor(e_ticsfull);
  counter=0;

  while(((dirdat=readdir(ptr))!=NULL) && (counter<=1300))
  { if(!jokcmp(/*(char[256])&*/dirdat->d_name, filemask, 0) ||
      (strcmp(dirdat->d_name, "TRANS.TBL")==0) ||
      (dirdat->d_name[0]=='.'))
      continue;

    strcpy(fullname, path);
    strcat(fullname, dirdat->d_name);
    if(stat(fullname, &info)!=0) continue;
    dirinfo->date=info.st_mtime;
    dirinfo->size=(signed long)info.st_size;
    strncpy(&dirinfo->name[1], dirdat->d_name, 38);
    dirinfo->name[38]=0;
    if(strlen(&dirinfo->name[1])>maxlength)
      maxlength=strlen(&dirinfo->name[1]);

    if((dummy=opendir(fullname))!=NULL)
    { closedir(dummy);
      if(!(options&o_l)) strcat(&dirinfo->name[1],_SEPS);
      dirinfo->name[0]=1;
      dirinfo->date=-dirinfo->date;
      dirinfo->size=-1;
    }
    else
      dirinfo->name[0]=2;

    counter++;
    dirinfo++;
  }

  closedir(ptr);

  dirinfo-=counter;

  if(options & o_s)
    qsort(dirinfo, counter, 48, (int(*)(const void *,const void *))sortbysize);
  else if(options & o_d)
    qsort(dirinfo, counter, 48, (int(*)(const void *,const void *))sortbydate);
  else if(!(options & o_n))
    qsort(dirinfo, counter, 48, (int(*)(const void *,const void *))sortbyname);

  waitfor(e_ticsfull);

  maxlength+=3;
  sprintf(smaxlength, "%%-%ds", maxlength);
  count=counter;
  for(counter=0;counter<count;counter++)
  { if(options & o_l)
    { putf("%-40s", &dirinfo->name[1]);
      if(dirinfo->size==-1)
      { dirinfo->date=-dirinfo->date;
        putf(" <DIR>     ");
      }
      else
        putf(" %10ld", dirinfo->size);
      time=localtime(&dirinfo->date);  //was gmtime
      putf(" %2.2d.%2.2d.%4.4d %2.2d:%2.2d:%2.2d\n",
           time->tm_mday, time->tm_mon+1, time->tm_year+1900,
           time->tm_hour, time->tm_min, time->tm_sec);
    }
    else
    { putf(smaxlength, &dirinfo->name[1]);

      maxc+=maxlength;
      if((maxc+maxlength)>=80)
      { putf("\n");
        maxc=0;
      }
    }

    dirinfo++;
  }

  if(maxc)
    putf("\n");

  putf("\n%d file(s) found.\n", count);

  dirinfo-=count;
  t_free(dirinfo);

  return(1);
}
 
/*---------------------------------------------------------------------------*/

void filesurf::processpath(char *path)
//*************************************************************************
//
//  - Replaces two or more slashes in a row by a single slash
//  - Replaces "/./" by "/"
//  - Removes "/../" and the preceding part of the path
//  - Puts a slash at the end, if there isn't one
//
//*************************************************************************
{ char *pointer, *pointer2;

  strcat(path, _SEPS);

  while((pointer=strstr(path, _SEPS""_SEPS))!=NULL)
    memmove(pointer,pointer+1,strlen(pointer+1)+1);
    //strcpy(pointer, pointer+1);

  while((pointer=strstr(path, _SEPS"."_SEPS))!=NULL)
    memmove(pointer,pointer+2,strlen(pointer+2)+1); 
    //strcpy(pointer, pointer+2);

  while((pointer=strstr(path, _SEPS".."_SEPS))!=NULL)
  { memmove(pointer,pointer+3,strlen(pointer+3)+1);
    //strcpy(pointer, pointer+3);
    if(pointer==path) break;

    pointer2=pointer;
    do pointer--; while(*pointer!=_SEP);
    memmove(pointer,pointer2,strlen(pointer2)+1);
    //strcpy(pointer, pointer2);
  }
}

/*---------------------------------------------------------------------------*/

void filesurf::addpath(char *path, char *addpath)
//*************************************************************************
//
//  Fuegt den zweiten angegebenen Pfad an den ersten Pfad an bzw. ersetzt
//  den ersten Pfad durch den zweiten, wenn dieser mit einem Slash beginnt
//
//*************************************************************************
{ strcpy(b->line, path);

#ifdef DRIVE_LETTERS
  if (addpath[1]==':')
    strcpy(b->line,addpath);
  else
#endif
  if(addpath[0]==_SEP)
#ifdef DRIVE_LETTERS
    strcpy(b->line+2, addpath);
#else
    strcpy(b->line, addpath);
#endif
  else
    strcat(b->line, addpath);
  processpath(b->line);

  if(strlen(b->line)>FNAMELEN)
    putf("Sorry - path/filename is too long, please contact your sysop.\n");
  else
    strcpy(path, b->line);  
}

/*---------------------------------------------------------------------------*/

void filesurf::putsyntax(char *cmd, char *para)
//*************************************************************************
//
//  Shows the syntax of a filesurf-command and puts a "filesurf " in
//  front of the command, if the user isn't in filesurf-mode
//
//*************************************************************************
{ putf("Syntax: ");
  if(!b->fsmode)
    putf("FileSurf ");
  putf("%s %s\n", cmd, para);
}

/*---------------------------------------------------------------------------*/

char filesurf::sysopcmd(char *command)
//*************************************************************************
//
//  Executes a filesurf-sysop-command
//
//*************************************************************************
{ char *befbuf;
  char temp[FNAMELEN+1];

  befbuf=command;

  static char *beftab[]=
    { "RM", "DEL", "MD", "MKDIR", "RD", "RMDIR", "CP", "COPY", "MV", "MOVE",
      NULL
    };

  enum befnum
    { unsinn,
      rm, del, md, mkdir_, rd, rmdir_, cp, copy, mv, move,
    } cmd=unsinn;

  cmd=(befnum)readcmd(beftab,&befbuf,0);
  befbuf+=blkill(befbuf);

  switch(cmd)
  {
  case md:
  case mkdir_:
    if(!befbuf[0])
    { putsyntax("MD", "<DirName>");
      break;
    }
    strcpy(temp, b->fspath);
    addpath(temp, befbuf);
    if (temp)
      temp[strlen(temp)-1]=0; //remove trailing (back)slash
    if(!filetime(temp))
      if(mkdir(temp)==0)
        putf("Directory %s created.\n", temp);
      else
        putf("Directory %s could not be created.\n", temp);
    else
      putf("File or directory %s already exists.\n", temp);
    break;

  case rd:
  case rmdir_:
    if(!befbuf[0])
    { putsyntax("RD", "<DirName>");
      break;
    }
    strcpy(temp, b->fspath);
    addpath(temp, befbuf);
    if (temp)
      temp[strlen(temp)-1]=0; //remove trailing (back)slash
    if(filetime(temp))
      if(rmdir(temp)==0)
        putf("Directory %s removed.\n", temp);
      else
        putf("Directory %s could not be removed.\n", temp);
    else
      putf("Directory %s does not exist.\n", temp);
    break;

  case cp:
  case copy:
  case mv:
  case move:
    if(!befbuf[0]||(strchr(befbuf, ' ')==NULL))
    { if((cmd==mv)||(cmd==move))
        putsyntax("MV", "<OldName> <NewName>");
      else
        putsyntax("CP", "<OldName> <NewName>");
      break;
    }
   {
    char name1[FNAMELEN+1], name2[FNAMELEN+1];

    strcpy(name1, b->fspath);
    strcpy(name2, b->fspath);
    addpath(name2, strrchr(befbuf, ' ')+1);
    *strrchr(befbuf, ' ')=0;
    addpath(name1, befbuf);
    #ifdef DRIVE_LETTERS
    if (strlen(name1) > 3)
    #endif
      name1[strlen(name1)-1]=0;
    #ifdef DRIVE_LETTERS
    if (strlen(name2) > 3)
    #endif
    name2[strlen(name2)-1]=0;

    if(!filetime(name1))
    { putf("File %s does not exist.\n", name1);
      break;
    }

    if(filetime(name2))
    { putf("File %s already exists.\n", name2);
      break;
   }

    if((cmd==mv)||(cmd==move))   
      putf("Moving");
    else
      putf("Copying");
    putf(" file %s to %s.\n", name1, name2);

    if (!filecopy(name1, name2))
    { 
      if((cmd==mv)||(cmd==move))
        unlink(name1);
    }
    else
      putf("Error.\n");
    }
    break;

  case rm:
  case del:
    if(!befbuf[0])
    { putsyntax("RM", "<FileName>");
      break;
    }
    strcpy(temp, b->fspath);
    addpath(temp, befbuf);
    #ifdef DRIVE_LETTERS
    if (strlen(temp) > 3)
    #endif
      temp[strlen(temp)-1]=0;
    if(filetime(temp))
      if(unlink(temp)==0)
        putf("File %s deleted.\n", temp);
      else
        putf("File %s could not be deleted.\n", temp);
    else
      putf("File %s does not exist.\n", temp);
    break;

  default:
    return 0;
  }

  return 1;
}

/*---------------------------------------------------------------------------*/

void filesurf::execute(char *command)
//*************************************************************************
//
//  Executes a filesurf-command
//
//*************************************************************************
{ char *befbuf;
  char temp[FNAMELEN+1];

  if (b->fsmode)
     trace(report,b->logincall,"fs %s",command);

  if(strncmp(b->fspath, "off", 3)==0)
    getfirstpath(b->fspath);

  if(strncmp(b->fspath,"off",3)==0)
  { putf("Sorry, FileSurf not enabled...\n");
    return;
  }

  if ( !b->sysop && !pathallowed(b->fspath,1))
    getfirstpath(b->fspath);

  befbuf=command;

  static char *beftab[]=
    { "PATH", "DIR", "LS", "CD", "CHDIR", "CD..", "READ", "TYPE",
      "GET", "CAT", "RTEXT", "PUT", "WTEXT", "BGET", "BIN", "RPRG",
      "BPUT", "WPRG", "HELP", "?", "QUIT", "BYE", "EXIT", "CMDMODE",
#ifdef FEATURE_YAPP
      "YGET", "RYAPP", "YPUT", "WYAPP",
#endif
      "#OK#", NULL
    };

  enum befnum
    { unsinn,
      path, dir, ls, cd, chdir, cd__, read, type, get, cat, rtext,
      put, wtext, bget, bin, rprg, bput, wprg, help, help_, quit,
      bye, exit, cmdmode,
#ifdef FEATURE_YAPP
      yget, ryapp, yput, wyapp,
#endif
      bin_ok
    }cmd=unsinn;


  //this is dirty, but it should help to ensure that the input line
  //is not too long..
  if (strlen(befbuf) > FNAMELEN)
  { befbuf[0]=0;
    putf("Sorry, input too long\n");
  }


  if(b->fsmode)
  { putf("\n");
    //oe3dzw: tmp-string is very short, cut off command if too long
    strcpy(temp,"fs: ");
    strncpy(temp+4,command,FNAMELEN-5);
    temp[FNAMELEN-1]=0;
    lastcmd(temp);
  }

  while(strchr(befbuf, _INVSEP))
    *strchr(befbuf, _INVSEP)=_SEP;

  if(strncmp(b->fspath, "off", 3)==0)
    getfirstpath(b->fspath);

#ifdef DRIVE_LETTERS
  if(befbuf[1]==':' && befbuf[2]==0)
  { char *pos;
    char *ptr;
    char tempallowed[FSPATHLEN+1];

    befbuf[0]|=32;
    strcpy(tempallowed, m.fspath);
    strcat(tempallowed, " ");
    ptr=tempallowed;
    while((pos=strstr(ptr, " "))!=NULL)
    { *pos=0;
      if(*ptr=='+')
        ptr++;
      if(strncmp(befbuf, ptr, 2)==0)
      { strcpy(b->fspath, ptr);
	processpath(b->fspath);
        return;
      }
      ptr=pos+1;
    }

    if(b->sysop)
    { strcpy(b->fspath, befbuf);
      processpath(b->fspath);
    }
    else
      putf("Drive %s not available.\n", befbuf);
    return;
  }
#endif

  cmd=(befnum)readcmd(beftab,&befbuf,0);
  befbuf+=blkill(befbuf);

  switch(cmd)
  {
  case path:
    printallowed();
    break;

  case dir:
  case ls:
    strcpy(temp, b->fspath);
    scanoptions(befbuf);
    if(cmd==dir) b->optplus+=o_l;
    befbuf+=blkill(befbuf);
    addpath(temp, befbuf);
    if(pathallowed(temp, 1))
      showdir(temp, b->optplus);
    break;

  case cd__:
    strcpy(befbuf, "..");
  case cd:
  case chdir:
    if(!befbuf[0])
    { putf("Current path: %s\n", b->fspath);
      break;
    }
    strcpy(temp, b->fspath);
    addpath(b->fspath, befbuf);
    #ifdef __MSDOS__
    if (strlen(b->fspath) > 3)
      b->fspath[strlen(b->fspath)-1]=0;
    #endif
    if((!pathallowed(b->fspath, 1)))
      strcpy(b->fspath, temp);
    else if(!isdir(b->fspath))
    { putf("Directory %s not found.\n", b->fspath);
      strcpy(b->fspath, temp);
    }
    #ifdef __MSDOS__
    if (b->fspath[strlen(b->fspath)-1]!=_SEP)
      strcat(b->fspath,_SEPS);
    #endif
  break;

  case get:
  case read:
  case type:
  case cat:
  case rtext:
    if(!befbuf[0])
    { putsyntax("GET", "<FileName>");
      break;
    }
    strcpy(temp, b->fspath);
    addpath(temp, befbuf);
    #ifdef DRIVE_LETTERS
    if(strlen(temp) > 3)
    #endif
      temp[strlen(temp)-1]=0;
    if(pathallowed(temp, 1) && fileexists(temp, 0))
    { fileio_text fio;
      fio.usefile(temp);
      fio.tx();
    }
    break;

  case bget:
  case bin:
  case rprg:
    if(!befbuf[0])
    { putsyntax("BGET", "<FileName>");
      break;
    }
    strcpy(temp, b->fspath);
    addpath(temp, befbuf);
    #ifdef DRIVE_LETTERS
    if (strlen(temp) > 3)
    #endif
      temp[strlen(temp)-1]=0;
    if(pathallowed(temp, 1) && fileexists(temp, 0))
    { fileio_abin fio;
      fio.usefile(temp);
      fio.tx();
    }
    break;

#ifdef FEATURE_YAPP
  case yget:
  case ryapp:
    if(!befbuf[0])
    { putsyntax("YGET", "<FileName>");
      break;
    }
    strcpy(temp, b->fspath);
    addpath(temp, befbuf);
    #ifdef DRIVE_LETTERS
    if (strlen(temp) > 3)
    #endif
      temp[strlen(temp)-1]=0;
    if(pathallowed(temp, 1) && fileexists(temp, 0))
    { fileio_yapp fio;
      fio.usefile(temp);
      fio.tx();
    }
    break;
#endif

  case put:
  case wtext:
    if(!befbuf[0])
    { putsyntax("PUT", "<FileName>");
      break;
    }
    strcpy(temp, b->fspath);
    addpath(temp, befbuf);
    #ifdef DRIVE_LETTERS
    if (strlen(temp) > 3)
    #endif
      temp[strlen(temp)-1]=0;
    if(pathallowed(temp, 2) && !fileexists(temp, 1))
    { fileio_text fio;
      fio.usefile(temp);
      fio.rx();
    }
    break;

  case bput:
  case wprg:
    if(!befbuf[0])
    { putsyntax("BPUT", "<FileName>");
      break;
    }
    strcpy(temp, b->fspath);
    addpath(temp, befbuf);
    #ifdef DRIVE_LETTERS
    if (strlen(temp) > 3)
    #endif
      temp[strlen(temp)-1]=0;
    if(pathallowed(temp, 2) && !fileexists(temp, 1))
    { fileio_abin fio;
      fio.usefile(temp);
      fio.rx();
    }
    break;

#ifdef FEATURE_YAPP
  case yput:
  case wyapp:
    if(!befbuf[0])
    { putsyntax("YPUT", "<FileName>");
      break;
    }
    strcpy(temp, b->fspath);
    addpath(temp, befbuf);
    #ifdef DRIVE_LETTERS
    if (strlen(temp) > 3)
    #endif
      temp[strlen(temp)-1]=0;
    if(pathallowed(temp, 2) && !fileexists(temp, 1))
    { fileio_yapp fio;
      fio.usefile(temp);
      fio.rx();
    }
    break;
#endif

  case help:
  case help_:
      if (befbuf[0])
        mbreadhelp(befbuf);
      else
        mbreadhelp("fs"); 
    break;

  case quit:
  case bye:
  case exit:
    if(b->fsmode)
      putf("FileSurf terminated. Good bye.\n");
    b->fsmode=0;
    break;

  case cmdmode:
    if(!b->fsmode)
    { putwelcome();
      b->fsmode=1;
      break;
    }

  case bin_ok:
    break;

  default:
    b->fsmode++; // b->fsmode must be set for start_runutil
    if(
#ifdef RUNUTILS
       !start_runutil(command) &&
#endif
       !((b->sysop)&&sysopcmd(command)))
      if(befbuf[0]) putf("Invalid command: '%s'\n", befbuf);
    b->fsmode--;
    break;
  }
}

 /*---------------------------------------------------------------------------*/

void filesurf::putprompt()
//*************************************************************************
//
//  Shows the FileSurf-prompt
//
//*************************************************************************
{ if(strncmp(b->fspath, "off", 3)==0)
    getfirstpath(b->fspath);

  putf("FileSurf:%s> ", b->fspath);
}

/*---------------------------------------------------------------------------*/

void filesurf::getfirstpath(char *string)
//*************************************************************************
//
//  Puts the first allowed path to the given pointer
//
//*************************************************************************
{ char tempallowed[FSPATHLEN+1];

  strcpy(tempallowed, m.fspath);
  strcat(tempallowed, " ");
  *strstr(tempallowed, " ")=0;

  if(tempallowed[0]=='+')
    strcpy(tempallowed, tempallowed+1);
  strcpy(string, tempallowed);
  processpath(string);
}

/*---------------------------------------------------------------------------*/

#endif
