/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  ---------------------------
  Verwaltung von Box-Adressen
  ---------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980115 OE3DZW utcoffset -> ad_timezone
//19980118 OE3DZW sec in timestr
//19980126 OE3DZW head -> long, output of p -s
//19980202 OE3DZW removed trailing \n in adr of hadr-file
//19980205 OE3DZW removed above fix, added \r fix for lastheader
//19980309 OE3DZW added \r,\n fix for headerline
//19980428 OE3DZW cutting off "ro" from ".euro"
//19980614 hrx    removed structure definitions hadr_t and hadr_old_t,
//                moved to mail.h.
//19990110 OE3DZW removed "newformat" from code, was commented out anyway
//19990214 OE3DZW added dummy function for future extensions
//19990817 JJ     removed static near (needed in afwd)

#include "baycom.h"

/*---------------------------------------------------------------------------*/

static handle uf=EOF;
static handle uhf=EOF;
static int is_null=0;

long hadr_len(int max)
{ if(max)
    return 65535L;
  else  
    return filesize(HADRNAME)/sizeof(hadr_t);
}

/*---------------------------------------------------------------------------*/
#ifdef _AUTOFWD
void
#else
static void near 
#endif
                  uclose(void)
{ if(uf!=EOF)
  { s_close(uf);
    uf=EOF;
  }
  if(uhf!=EOF)
  { s_close(uhf);
    uhf=EOF;
  }
}

/*---------------------------------------------------------------------------*/

void hadrfile_newformat(void)
{
  if(filetime(HADRNAME) < filetime(HADROLDNAME))
  { FILE *fi=s_fopen(HADROLDNAME,"srb");
    FILE *fo=s_fopen(HADRNAME,"swb");
    hadrold_t *hi=(hadrold_t *) t_malloc(sizeof(hadrold_t),"haol");  
    hadr_t *ho=(hadr_t*) t_malloc(sizeof(hadr_t),"hadr");

    trace(replog,"hadr","converting format");
    wdelay(334);
    if(fi && fo)
    { while(fread(hi,sizeof(hadrold_t),1,fi))
      { memset(ho,0,sizeof(hadr_t)); 


        ho->bulletins=hi->bulletins;
        ho->usermails=hi->usermails;
        memcpy(ho->delay,hi->delay,sizeof(hi->delay));
        memcpy(ho->lastupdate,hi->lastupdate,sizeof(hi->lastupdate));
        ho->lasthtime=hi->lasthtime;
        memcpy(ho->rel_mails,hi->rel_mails,sizeof(hi->rel_mails));
        memcpy(ho->hops,hi->hops,sizeof(hi->hops));
        ho->nextsamehash=hi->nextsamehash;
        memcpy(ho->adr,hi->adr,sizeof(hi->adr));
        memcpy(ho->adjacent,hi->adjacent,sizeof(hi->adjacent));
        memcpy(ho->lastheader,hi->lastheader,sizeof(hi->lastheader));
        memcpy(ho->lastbid,hi->lastbid,sizeof(hi->lastbid));
        memcpy(ho->lastboard,hi->lastboard,sizeof(hi->lastboard));
        memcpy(ho->lastuser,hi->lastuser,sizeof(hi->lastuser));
       
        fwrite(ho,sizeof(hadr_t),1,fo);
      }
    }
    if(fi)s_fclose(fi);
    if(fo)s_fclose(fo);
    t_free(hi);
    t_free(ho);
  }
}

/*---------------------------------------------------------------------------*/

static void near newhadrfile(void)
{ hadr_t *ha=(hadr_t*) t_malloc(sizeof(hadr_t),"hadr");

  xunlink(HADRHASHNAME);
  uf=s_open(HADRNAME,"sw+b");
  if(uf==EOF)
    trace(tr_abbruch,"hadr_tryopen","can't create");
  memset(ha,0,sizeof(&ha));
  _write(uf,ha,sizeof(hadr_t));
  s_close(uf);
  t_free(ha);
}

/*---------------------------------------------------------------------------*/
#ifdef _AUTOFWD
void
#else
static void near 
#endif
                  hadr_tryopen(void)
{ static int first=1;

  if(sizeof(hadr_t)!=1024 || is_null)
    trace(tr_abbruch,"hadr_t","size %d",sizeof(hadr_t));
  if(uf==EOF)
  {
    if(filetime(HADRNAME)==0 || filesize(HADRNAME)==0)
    { if(!first)
        return;
      xunlink(HADRHASHNAME);
      newhadrfile();
      uf=s_open(HADRNAME,"sr+b");
    }
    else
    { uf=s_open(HADRNAME,"sr+b");
      if(uf==EOF)
        return;
    }
  }
  if(uhf==EOF)
    uhf=s_open(HADRHASHNAME,"sr+b");
  if(uhf==EOF)
  { if(first)
      uhf=s_open(HADRHASHNAME,"sw+b");
    else
      return;
  }
  if(first)
    hashinit(uhf,HASH16);
  first=0;
}

/*---------------------------------------------------------------------------*/

static unsigned near newhadr(hadr_t *u,handle uf,handle uhf)
//*************************************************************************
//
//  legt einen hadr_t im HADR-File ab
//
//  Achtung: es muss an der Stelle sichergestellt sein, dass nicht
//           schon ein hadr_t mit gleichem Namen existiert!
//
//*************************************************************************
{ short unsigned hsh=strcrc(atcall(u->adr));
  short unsigned idx=0;

  lseek(uhf,(long)hsh*2,SEEK_SET);                 // Eintrag im Hashfile
  if(!_read(uhf,&idx,sizeof(short unsigned)))
    return 0;
  u->nextsamehash=idx;                             // verketten

  lseek(uf,0L,SEEK_END);
  lseek(uhf,(long)hsh*2,SEEK_SET);

  idx=(short unsigned)(filelength(uf)/sizeof(hadr_t));
  if(!_write(uhf,&idx,sizeof(short unsigned)))       // zurckschreiben
    return 0;
  if(!_write(uf,u,sizeof(hadr_t)))                   // User hinausschreiben
    return 0;
  return idx;
}

/*---------------------------------------------------------------------------*/

static int near hdrin(char *hadrcall,handle uf,handle uhf)
//*************************************************************************
//
//
//*************************************************************************
{ short unsigned idx;
  hadr_t *ux=(hadr_t *) t_malloc(sizeof(hadr_t),"hadr");  
  char atc[10];
  strcpy(atc,atcall(hadrcall));
  short unsigned hsh=strcrc(atc);

  lseek(uhf,(long)hsh*2,SEEK_SET);
  _read(uhf,&idx,sizeof(short unsigned));
  if(idx)
  { int num=0;
    while(idx)
    { lseek(uf,(long)idx*sizeof(hadr_t),SEEK_SET);
      if(!_read(uf,ux,sizeof(hadr_t)))
      { t_free(ux);
        return 0;
      }
      if((num++)>200)
      { t_free(ux);
        return 0;
      }
      if(strcmp(atc,atcall(ux->adr))==0)
      { t_free(ux);
        return idx;
      }
      idx=ux->nextsamehash;
    }
  }
  t_free(ux);
  return 0;
}

/*---------------------------------------------------------------------------*/
#ifdef _AUTOFWD
int
#else
static int near 
#endif
                  loadhadr(char *hadrcall,hadr_t *u,int anlegen)
//*************************************************************************
//
//
//*************************************************************************
{ short unsigned idx;
  unsigned retwert=0;
  char atc[10];
  strcpy(atc,atcall(hadrcall));
  short unsigned hsh=strcrc(atc);

  if(mbhadrok(hadrcall)==1)
  { lseek(uhf,(long)hsh*2,SEEK_SET);
    _read(uhf,&idx,sizeof(short unsigned));
    if(idx)
    { int num=0;
      while(idx)
      { lseek(uf,(long)idx*sizeof(hadr_t),SEEK_SET);
        if(!_read(uf,u,sizeof(hadr_t)))
          goto ende;
        if((num++)>500)
          goto ende;
        if(strcmp(atc,atcall(u->adr))==0)
        { retwert=idx;
          goto ende;
        }
        idx=u->nextsamehash;
      }
    }
    if(anlegen)
    { memset(u,0,sizeof(hadr_t));
      strcpy(u->adr,hadrcall);
      retwert=newhadr(u,uf,uhf);
    }
  }
  ende:
  return retwert;
}

/*---------------------------------------------------------------------------*/

void reorg_hadr(void)
{ handle ualt,uneu,uhneu;
  hadr_t *ux=(hadr_t *) t_malloc(sizeof(hadr_t),"hadr");  
  unsigned pos=0,errors=0;
  time_t ti=ad_time()+ad_timezone()/3600;  // current UTC time

  if(m.hadrstore)
  { trace(replog,"reorg","hadr");
    xunlink(HTMP);
    xunlink(HHTMP);
    ualt=s_open(HADRNAME,"lrb");
    if(dfree(".",0) < (filelength(ualt)/1024L+HOMEDISKFULL))
    { s_close(ualt);
      trace(serious,"reorghadr","disk full");
      t_free(ux);
      return;
    }
    uneu=s_open(HTMP,"lw+b");
    uhneu=s_open(HHTMP,"lw+b");
    if((ualt!=EOF) && (uneu!=EOF) && (uhneu!=EOF))
    { hashinit(uhneu,HASH16);
      _read(ualt,ux,sizeof(hadr_t));
      _write(uneu,ux,sizeof(hadr_t));
      while(1)
      { waitfor(e_ticsfull);
        if(!_read(ualt,ux,sizeof(hadr_t)))
          break;
        { char *a;  //fix trailing \n or \r
          if((a=strstr(ux->lastheader,"\r"))!=0)
            *a=0;
          if((a=strstr(ux->lastheader,"\n"))!=0)
            *a=0;
        }  
        if(ux->adr[0]==0)
          continue;
        if(ux->lasthtime<ti-(180L*24*3600)&&   //180d
           ux->lastwprcvd<ti-(720L*24*3600))   //2y if WP-received
          continue;
        if(ux->adr[HADRESSLEN] || mbhadrok(ux->adr)!=1 ||
          hdrin(ux->adr,uneu,uhneu))
        { errors++;
          continue;
        }
        if(strlen(ux->lastheader)<10)
          ux->lastheader[0]=0;        
        newhadr(ux,uneu,uhneu);
        pos++;
      }
      trace(replog,"reorghadr","%u hadrs, %u errors",
                   pos,errors);
      s_close(ualt);
      s_close(uneu);
      s_close(uhneu);
      xrename(HTMP,HADRNAME);
      xrename(HHTMP,HADRHASHNAME);
    }
  }
  t_free(ux);
}

/*---------------------------------------------------------------------------*/

int update_hadr(char *headerline,int hops,int uplink)
// used globals
//   b->bid       (if uplink != 0)
//   b->boardname (if uplink != 0)
//   b->usermail  (if uplink != 0)
//   b->frombox   (not if hops = -1)
//
{
  hadr_t *ha=(hadr_t*) t_malloc(sizeof(hadr_t),"hadr");

  int i;
  char *hpos;
  unsigned pos;                               // position within hadr-file
  char hadr[HADRESSLEN+1];                    // received H-Address
  
  { char *a;                                  // fix trailing \n or \r
    if((a=strstr(headerline,"\r"))!=0)
      *a=0;
    if((a=strstr(headerline,"\n"))!=0)
      *a=0;
  }  

  time_t htime=getheadertime(headerline);     // received time
  time_t mtime=ad_time()+ad_timezone()/3600;  // current UTC time
  time_t hdelay=(mtime-htime);                // delay of delivery
  int adj=(-1);                               // index of adjacent box call

  // if the mail is much newer than current time, the clock of the uplink-box
  // seems to be adjusted incorrectly. Add a time for always positive delay
  hdelay+=7200;

  hpos=getheaderadress(headerline);
  if(hpos && m.hadrstore)
    strcpy(hadr,hpos);
  else
  { t_free(ha);
    return 0;
  }

  waitfor(e_ticsfull);
  if(hadr)
  { strupr(hadr);
    hadr_tryopen();
    if(uf==EOF || uhf==EOF)
    { uclose();
      t_free(ha);
      return 0;
    }
    pos=loadhadr(hadr,ha,1);
    if(pos)
    { if(uplink)
      { if(b->usermail)
        { ha->usermails++;
          strcpy(ha->lastuser,b->boardname);
        }
        else
        { ha->bulletins++;
          strcpy(ha->lastboard,b->boardname);
        }
        if(b->bid[0] && strstr(b->bid,atcall(ha->adr)))
          strcpy(ha->lastbid,b->bid);
      }
      if(htime>mtime)
        htime=mtime;

      char adrname[CALLEN+1];
      strcpy(adrname,atcall(hadr));

      if(hops >= 0 && ha->lasthtime<=htime) 
        // only accept headers in ascending order
      {
        // look for an appropriate entry in list of adjacent box calls
        strcpy(ha->adr,hadr);
        for(i=0;i<ADJNUM;i++)
        { if(strcmp(b->frombox,ha->adjacent[i])==0)
          { adj=i;                           // found the current active call
            break;
          }
        }
        // if no such entry exists, look for an empty location in the list
        // of the adjacent box calls
        if(adj<0)
        { for(i=0;i<ADJNUM;i++)
          { if(ha->adjacent[i][0]==0)
            { strcpy(ha->adjacent[i],b->frombox);
              adj=i;
              break;
            }
          }
        }
        // if no empty position is left, remove the oldest position which
        // is reached through more hops or with a longer delivery time
        if(adj<0)
        { time_t last=mtime;

          for(i=0;i<ADJNUM;i++)
          { // search for the oldest entry with one of following criteria TRUE
            //  - the hop-count is greater than the current one
            //  - the delay timer is longer than the current one
            //  - the entry has not been updated for more than 30 days
            if(ha->lastupdate[i]<last && (hops<ha->hops[i]||
                                         hdelay<ha->delay[i])||
                                         ha->lastupdate[i]<(mtime-DAY*30))
            { last=ha->lastupdate[i];
              adj=i;
            }
          }
          if(adj>=0)
          { strcpy(ha->adjacent[adj],b->frombox); // change call
            ha->rel_mails[adj]=0;                 // call has changed, count new
          }
          else
          { uclose();
            t_free(ha);
            return 0;      // direction could not be updated, ignore the mail
          }
        }
        ha->rel_mails [adj]++;       // increase normalised mail counter
        if(ha->rel_mails[adj]>50)    // normalise the counters to values < 50
        { for(i=0;i<ADJNUM;i++)
            ha->rel_mails[i]>>=1;    // take the half
        }
        ha->delay     [adj]=hdelay;
        ha->lastupdate[adj]=mtime;
        ha->hops      [adj]=hops;

        ha->lasthtime=htime;

        // now search for the rest of the "received"-header, excluding date/time
        // and h-address. Both are stored in other members of ha.
        hpos=strstr(headerline,hadr);
        if(hpos!=NULL)
        { hpos+=strlen(hadr);
          while(hpos[0]==' ') hpos++;
          if(strlen(hpos)>10)
          { strncpy(ha->lastheader,hpos,HEADERLEN);
            ha->lastheader[HEADERLEN]=0;
          }
        }
      }
      if (strcmp(adrname,m.boxname)==0)
      { //dummy for local bbss
        sprintf(ha->lastheader,"[%s] "BCMLOGO VNUMMER,m.boxheader);        
        ha->lasthtime=htime;     
        ha->lastwprcvd=htime;
        strncpy(ha->sysopcall,m.sysopcall,sizeof(ha->sysopcall)-1);
        ha->sysopcall[sizeof(ha->sysopcall)-1]=0;
        strcpy(ha->protocol,"AX25");
        strncpy(ha->hwaddress,m.mycall[0],sizeof(ha->hwaddress)-1);
        ha->hwaddress[sizeof(ha->hwaddress)-1]=0;
        ha->bversion=BVERSION;
        strcpy(ha->bstatus,BSTATUS);
      }

      lseek(uf,(long)pos*sizeof(hadr_t),SEEK_SET);
      _write(uf,ha,sizeof(hadr_t));
    }
  }
  uclose();
  t_free(ha);
  return 1;
}

/*---------------------------------------------------------------------------*/

void browse_hadr(char *search)
{  hadr_t *ha=(hadr_t*) t_malloc(sizeof(hadr_t),"hadr");

  time_t upd;
  int i;
  unsigned count=0,found=0;
  long head=0;
  int ok;

  search+=blkill(search);
  strupr(search);

  if(m.hadrstore)
  { FILE *f=s_fopen(HADRNAME,"lrb");
    if(f)
    { setvbuf(f,NULL,_IOFBF,4096);    // buffer for increasing performance
      fseek(f,sizeof(hadr_t),SEEK_SET);
      while(fread(ha,sizeof(hadr_t),1,f))
      { if(ha->adr[0]==0) continue;
        count++;
        ok=0;
        if(!(b->optplus&o_p))
          ok += (stristr(ha->lastheader,search)!=NULL);
        if(!(b->optplus&o_h))
          ok += (strstr(ha->adr,search)!=NULL);
        if(ok)
        { found++;
          upd=0;
          if(!(b->optplus&o_c))
          { for(i=0;i<ADJNUM;i++)
            { if(ha->adjacent[i][0] && ha->lastupdate[i]>upd)
                upd=ha->lastupdate[i];
            }
            if(!(head++))
              putf("Mails Since Address\n");
            putf("%5ld%5s: %-22s %1.44s\n",ha->usermails+ha->bulletins,
                 zeitspanne(ad_time()-upd,zs_seconds),
                 ha->adr,ha->lastheader);
          }
        }
        waitfor(e_ticsfull);
        if(testabbruch()) break;
      }
      putf("%u searched, %u found.\n",count,found);
      s_fclose(f);
    }
  }
  t_free(ha);
}

/*---------------------------------------------------------------------------*/

void edit_hadr(bitfeld opt,char *line)
//*************************************************************************
//
//  Edit hadress entry
//  Only ONE of the following commands can be given at a time:
//
//  Syntax:
//    p -ea <h-address>        sets a new address
//                             (entry will be created if it does not exist)
//    p -ed <call>             deletes an entry
//    p -eh <call> <header>    sets a new R: header line
//    p -et <call> <date>      sets header received date to <date>
//
//*************************************************************************
{ hadr_t *ha=(hadr_t*) t_malloc(sizeof(hadr_t),"hadr");

  unsigned pos;
  char call[HADRESSLEN];
  time_t mtime=ad_time()+ad_timezone()/3600;        // current UTC time

  line=nexttoken(line,call,HADRESSLEN);
  strupr(call);

  if(m.hadrstore)
  { strupr(call);
    hadr_tryopen();
    pos=loadhadr(call,ha,0);
    if(pos)
    { if(opt & o_d)
      { ha->adr[0]=0;
        ha->lastheader[0]=0;
      }
      else if(opt & o_h)
      { strncpy(ha->lastheader,line,HEADERLEN);
        ha->lastheader[HEADERLEN]=0;
        ha->lasthtime=mtime;
      }
      else if(opt & o_a)
      { strncpy(ha->adr,call,HADRESSLEN);
        ha->adr[HADRESSLEN]=0;
        ha->lasthtime=mtime;
      }
      else if(opt & o_t)
      { ha->lasthtime=parse_time(line);
      }
      lseek(uf,(long)pos*sizeof(hadr_t),SEEK_SET);
      _write(uf,ha,sizeof(hadr_t));
    }
    else if(opt & o_a)
    { loadhadr(call,ha,1);
      ha->lasthtime=mtime;
    }
    else
      putf("edit_hadr: %s not found.\n",call);
    uclose();
    show_hadr(call);
  }
  t_free(ha);
}

/*---------------------------------------------------------------------------*/

int wpupdate_hadr(wpb_t *wpb)
//*************************************************************************
//
// Store WP data received in store and forward
//
//*************************************************************************
{ hadr_t *ha=(hadr_t*) t_malloc(sizeof(hadr_t),"hadr");

  unsigned pos;
  int isnew=0;
  time_t ti=ad_time()+ad_timezone()/3600;        // current UTC time

  if(m.hadrstore)
  { strupr(wpb->hadr);
    hadr_tryopen();
    pos=loadhadr(wpb->hadr,ha,0);
    if(pos)
    { if (ha->lastwprcvd < wpb->timestamp)
      { ha->lastwprcvd = wpb->timestamp;
        if (wpb->hadr[0])
          strcpy(ha->adr,wpb->hadr);
        strupr(wpb->sysopcall);
        if (wpb->sysopcall[0])
        { if(strcmp(ha->sysopcall,wpb->sysopcall))
            isnew++;
            strcpy(ha->sysopcall,wpb->sysopcall);
        }
        if (wpb->protocol[0] && wpb->hwaddress[0])
        { if (stricmp(ha->protocol,wpb->protocol) ||
              stricmp(ha->hwaddress,wpb->hwaddress))
            isnew++;
          strcpy(ha->protocol,wpb->protocol);
          strcpy(ha->hwaddress,wpb->hwaddress);
        }
        if (wpb->bstatus[0])
          strcpy(ha->bstatus,wpb->bstatus);
        ha->bversion=wpb->bversion;         
        if (isnew||(ha->lastwpsend<ti-86400L*30))
        { ha->lastwpsend=ti;
          isnew++;
        }
        if (wpb->hops>50)
          isnew=0;       
        lseek(uf,(long)pos*sizeof(hadr_t),SEEK_SET);
        _write(uf,ha,sizeof(hadr_t));
      }
    }
    uclose();
  }
  t_free(ha);
  return isnew;
}
/*---------------------------------------------------------------------------*/

void bupdate__hadr(char *call)
//*************************************************************************
//
// dummy for future extensions
//
//*************************************************************************
{ hadr_t *ha=(hadr_t*) t_malloc(sizeof(hadr_t),"hadr");

  unsigned pos;


  if(m.hadrstore)
  { strupr(call);
    hadr_tryopen();
    pos=loadhadr(call,ha,0);
    if(pos)
    {
      // TODO: do what you want to do with "call"

      lseek(uf,(long)pos*sizeof(hadr_t),SEEK_SET);
      _write(uf,ha,sizeof(hadr_t));
    }
    uclose();
    //DEBUG show_hadr(call);
  }
  t_free(ha);
}

/*---------------------------------------------------------------------------*/

void show_hadr(char *call)
{ hadr_t *ha=(hadr_t*) t_malloc(sizeof(hadr_t),"hadr");

  int i;
  time_t mtime=ad_time()+ad_timezone()/3600;        // current UTC time
  long mindelay=100*DAY;

  if(m.hadrstore)
  { strupr(call);
    hadr_tryopen();
    if(loadhadr(call,ha,0))
    { uclose();
      putf("\nH-Addr:      %s\n",ha->adr);
      putf("R-Header:    %s\n",ha->lastheader);
      if (ha->lastbid[0])
        putf("Last BID:    %s\n",ha->lastbid);
      if (ha->lastboard[0])
        putf("Last Board:  %s\n",ha->lastboard);
      if (ha->lastuser[0])
        putf("Last User:   %s\n",ha->lastuser);
      putf("Header Date: %s %s utc\n",datestr(ha->lasthtime,2),
                                  timestr(ha->lasthtime,1));
      if (ha->sysopcall[0])
        putf("Sysop:       %s %s\n",ha->sysopcall,get_name(ha->sysopcall,1));
      if (ha->protocol[0] && ha->hwaddress[0])
        putf("HW-Address:  %s: %s\n",ha->protocol,ha->hwaddress);
      if (ha->bversion && ha->bstatus[0] && ha->lastwprcvd)
        putf("BVersion:    %ld %s (%s %s utc)\n",ha->bversion,ha->bstatus,
              datestr(ha->lastwprcvd,2),timestr(ha->lastwprcvd,1)); 
      if (ha->bulletins)
        putf("Bulletins:   %ld\n",ha->bulletins);
      if (ha->usermails)
        putf("Usermails:   %ld\n",ha->usermails);
      int phead=0;
      for(i=0;i<ADJNUM;i++)
      { if(ha->adjacent[i][0])
        { if(ha->delay[i] < mindelay)
            mindelay=ha->delay[i];
        }
      }
      for(i=0;i<ADJNUM;i++)
      { if(ha->adjacent[i][0])
        { if (!phead)
          { phead++;
            putf("Neighbour Count Update +Delay Hops\n");
          }
          putf("%-9s%4d%8s",ha->adjacent[i],ha->rel_mails[i],
               zeitspanne(mtime-ha->lastupdate[i],zs_seconds));
          putf("%7s%5d\n",zeitspanne(ha->delay[i]-mindelay,zs_seconds),
               ha->hops[i]);
        }
      }
    }
    uclose();
  }
  t_free(ha);
}

/*---------------------------------------------------------------------------*/

void expand_hadr(char *adr,int force)
//*************************************************************************
//
//  Macht aus einer vom Benutzer angegebenen Adresse eine
//  (wenn moeglich) korrekte H-Adresse.
//  Wird das eigene Boxcall angegeben, so wird die eigene Boxadresse
//  eingesetzt, es sei denn es ist eine SSID ungleich dem ersten
//  Mycall angegeben.
//
//  force=1  nur eigenes Boxcall ergaenzen
//  force=2  nur ergaenzen, wenn keine komplette H-Adresse angegeben ist
//  force=3  immer ergaenzen, notfalls vorherige H-Adresse ueberschreiben
//
//*************************************************************************
{  hadr_t *ha=(hadr_t*) t_malloc(sizeof(hadr_t),"hadr");

  int notok=2;             // mehrere Versuche, File koennte gelockt sein

  strupr(adr);
  if(strcmp(atcall(adr),m.boxname)==0)
  { if(strchr(adr,'-') && strcmp(atcall(adr),m.mycall[0])!=0)
    { t_free(ha);
      return;
    }
    strcpy(adr,m.boxadress);
  }
  else if(force>1 && (!strchr(adr,'.') || force>2))
  { while(notok)
    { notok--;
      hadr_tryopen();
      if(loadhadr(adr,ha,0) && strlen(ha->adr)>9 && strchr(ha->adr,'.'))
      { strcpy(adr,ha->adr);
        if(notok==0)
          trace(replog,"expand_hadr","load retry %s",adr);
        notok=0;
      }
      uclose();
    }
  }
  //if address ends with ".euro" cut off "ro" so that ".eu" remains
  char *a;
  a=strstr(adr,".EURO");
  if (a && (strlen (a) == 5)) 
  *(a+3)=0; 
  t_free(ha);
}

/*---------------------------------------------------------------------------*/
