/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -------------------------------
  Online-Help fr Mailboxbenutzer
  -------------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

#include "baycom.h"

/*---------------------------------------------------------------------------*/

class mbhelp_t
{
public:
  void dohelp(char *subject);
  ~mbhelp_t();
private:
  int helpindex(void);
  int helpopen(char *speech);
  void charstr(int number,char character);

  FILE *helpf;
  FILE *idxf;
  char helpname[40];
  char idxname[40];
  char inbuf[82];
};

/*---------------------------------------------------------------------------*/

mbhelp_t::~mbhelp_t()
{ if(helpf) s_fclose(helpf);
  if(idxf)  s_fclose(idxf);
}

/*---------------------------------------------------------------------------*/

int mbhelp_t::helpindex(void)
{ int i=0;

  if(!helpf) return 0;

  idxf=s_fopen(idxname,"sw+t");
  if(!idxf) return 0;

  s_fsetopt(idxf,1);
  fprintf(idxf,"%ld\n",filetime(helpname));

  while(fgets(inbuf,80,helpf))
  { if((inbuf[0]=='\\')&&(inbuf[1]=='\\'))
    { inbuf[strlen(inbuf)-1]=0;                         // delete newline
      fprintf(idxf,"%1.30s %ld\n",inbuf+2,ftell(helpf));
      i++;
    }
  }
  rewind(idxf);
  trace(replog,"helpindex","%d subjects in %s",i,helpname);
  return 1;
}

/*---------------------------------------------------------------------------*/

int mbhelp_t::helpopen(char *speech)
{ time_t helptime=0;

#if NODE
  sprintf(helpname,MSGPATH"/%s.%s",b->node?"n_help":"help",speech);
#else
  sprintf(helpname,MSGPATH"/help.%s",speech);
#endif
  strlwr(helpname);
  helpf=s_fopen(helpname,"srt");
  if(helpf)
  {
#if NODE
    sprintf(idxname,MSGPATH"/%s.%s",b->node?"n_helpid":"helpidx",speech);
#else
    sprintf(idxname,MSGPATH"/helpidx.%s",speech);
#endif
    strlwr(idxname);
    idxf=s_fopen(idxname,"srb");
    { if(idxf)
      { fscanf(idxf,"%ld",&helptime);
	if(filetime(helpname)!=helptime)
	{ s_fclose(idxf);
          idxf=NULL;
        }
      }
      if(idxf==NULL)
      { if(!helpindex())     // generate index file
        { s_fclose(helpf);     // do not loose filehandles if no correct index
          helpf=NULL;
          return 0;
        }
      }
      fgets(inbuf,80,idxf);  // skip the first line containing filetime
      return 1;
    }
  }
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

void mbhelp_t::charstr(int number,char character)
{ int i;
  putv('\n');
  for(i=0;i<number;i++)
    putv(character);
  putv('\n');
}

/*---------------------------------------------------------------------------*/

void mbhelp_t::dohelp(char *subject)
{ int i=0;
  long seekpos=0;
  int helpall=(stricmp(subject,"ALL")==0);
  int found=0;
  char *idx="INDEX";
  char *subsubject=strchr(subject,' ');
  char *link=NULL;
  char *sublink=NULL;
  char linkstr[40]="";
  char sublinkstr[40]="";
  int linkcnt=0;
  char oldsubject[60]="";

  helpf=NULL;
  idxf=NULL;

  if(subsubject==NULL)
    subsubject=strchr(subject,'.');

  if(subsubject)
  { subsubject[0]=0;
    subsubject++;
  }

  if(!subject[0])
    subject="HELP";

  if(helpopen(ms(m_helpext))==0)
  { if(helpopen("GB")==0)
    { if(helpopen("DL")==0)
      { putf("No help file found.\n");
        trace(serious,"dohelp","no help file");
        return;
      }
    }
  }

  if(strstr(idx,subject)==idx)
  { putf(ms(m_helpkeywords));
    while(fscanf(idxf,"%s %ld",inbuf,&seekpos)==2)
    { if(strchr(inbuf,'='))
        strchr(inbuf,'=')[0]=0;
      putf("%-18s",inbuf);
      if(((i+1)%4)==0)
        putv('\n');
      i++;
    }
    putv('\n');
  }
  else
  { while(fscanf(idxf,"%s %ld\n",inbuf,&seekpos)==2)
    { if((strstr(inbuf,subject)==inbuf)||helpall)
      { if(subsubject && !helpall)
        { char *dot=strchr(inbuf,'.');
          if(dot)
          { dot++;
            if(strstr(dot,subsubject)!=dot)
              continue;
          }
          else
            continue;
        }
        link=strchr(inbuf,'=');
        if(link)
        { if(helpall)
            continue;
          link[0]=0;
          link++;
          strcpy(linkstr,link);
          sublink=strchr(linkstr,'.');
          sprintf(oldsubject,"%s -> ",inbuf);
          if(sublink)
          { sublink[0]=0;
            strcpy(sublinkstr,sublink+1);
            subsubject=sublinkstr;
          }
          else
            subsubject=NULL;
          subject=linkstr;
          rewind(idxf);
          fgets(inbuf,80,idxf);  // skip the first line containing filetime
          if(++linkcnt > 10)
          { trace(serious,"help","cyclic link %s.%s",subject,subsubject);
            break;
          }
          continue;
        }
        found++;
        if(strchr(inbuf,'.'))      strchr(inbuf,'.')[0]=' ';
        if(strchr(oldsubject,'.')) strchr(oldsubject,'.')[0]=' ';
        charstr(strlen(inbuf)+6+strlen(oldsubject),':');
	putf(":: %s%s ::",oldsubject,inbuf);
	charstr(strlen(inbuf)+6+strlen(oldsubject),':');
	putv('\n');

	fseek(helpf,seekpos,SEEK_SET);
        while(fgets(inbuf,80,helpf))
	{ if((inbuf[0]=='\\')&&(inbuf[1]=='\\'))
	    break;
	  putf("%s",inbuf);
	  if(b->abbruch) break;
	}
	if(!helpall) break;
      }
      if(b->abbruch) break;
    }
    //testabbruch();
    if(!found)
      putf(ms(m_helpnotfound),subject,subsubject?subsubject:"*");
  }
}

/*---------------------------------------------------------------------------*/

void mbreadhelp(char *cmd)
{ mbhelp_t help;

  strupr(cmd);
  help.dohelp(cmd);
}
