/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -------------------
  HTTP-Server for BCM   Reference: RFC 1945
  -------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980115 OE3DZW utcoffset -> ad_timezone, increased expire by 800s
//19980329 OE3DZW fixed send
//19980428 OE3DZW fixed conversion of to/subject
//19980505 OE3DZW l baybox -30 statt st l in Leiste, logout als 
//                remove cookie, "home" statt back
//19980525 DG9MHZ added support for MS IE ;-((( (fixed parser bug)
//19980916 OE3DZW added signature
//19981013 OE3DZW added debug output to find bug in http-post (send mails)

#include "baycom.h"
#define FRAME 0
#define BGCOL "\"#ffffdd\""
#define HTTPSIGSTR "BCMHTTPD"

/*---------------------------------------------------------------------------*/

class httpd
{ 
public:
  httpd(void);
  ~httpd(void);
  void start_http(char *name);

private:  
  // data
  char signature[sizeof(HTTPSIGSTR)];
  enum { GET,HEAD,POST } method;
  enum { NONE, BIN, TXT, GIF, JPG, WAV} _mimetype;
  int version;                          // 9 -> HTTP/0.9   10 -> HTTP/1.0 etc
  char inbuf[400];
  char uri[200];
  char login[30];
  char cookie_login[30];
  char pw[NAMELEN+1]; //was 12
  char cookie_pw[NAMELEN+1]; //was 12
  int  status; 
  char file[80];
  char userpass[50];
  char referer[100];
  char host[100];
  long contentlength;
  char *content;
  int nocookie;
  int mimetype;
  char header[1024];
 
  // methods
  char *ht_time(time_t t);
  void base64bin(char *in,char *out,int maxlen);
  void get_authorization(char *buf,int cookie);
  void get_contentlength(char *buf);
  void get_field(char *buf,char *dest,int maxlen);
  void form_referer(void);
  void get_request(void);
  void get_postarea(char *tag,char *out,int max,int wrap=0);
  int  prepare_header(void);
  void put_header(char *header,int frame);
  void put_homepage(char *cmd);
  void put_frame(void);
  void put_sendform(char *to,char *subj,char *read);
  void postmsg(char *in,char *out,int outlen);
};

/*---------------------------------------------------------------------------*/

httpd::httpd(void)
//*************************************************************************
//
//  Initialization
//
//*************************************************************************
{          
  strcpy(signature,HTTPSIGSTR);
}

/*---------------------------------------------------------------------------*/

httpd::~httpd(void)
//*************************************************************************
//
//  Deinitialization
//
//*************************************************************************
{
  if (strcmp(signature,HTTPSIGSTR))
    trace(fatal,"httpd","sig broken");
}

/*---------------------------------------------------------------------------*/

char *httpd::ht_time(time_t t)
{ static char zeitdatum[40];

  strftime(zeitdatum,sizeof(zeitdatum),
           "%a, %d-%b-%Y %H:%M:%S GMT",localtime(&t));
  return zeitdatum;
}

/*---------------------------------------------------------------------------*/

int isamprnet(char *ip)  
//Stellt fest, ob es sich um eine Ampr-Net Adresse handelt.
{

 if (strncmp(ip,"44.",3)==0)
   return 2;
 else if (strncmp(ip,"127.",4)==0)
   return 1;
 else
   return 0;
}

/*---------------------------------------------------------------------------*/

// highly simplified base64-decoder, according RFC1521
// constraints: - no line feeds
//              - no inappropriate characters (lead to abort of conversion)
//              - no zero bytes within data
//
void httpd::base64bin(char *in,char *out,int maxlen)
{ int i,a,end=0;
  long outword=0;
 
  while(in[0] && end==0 && maxlen>3)
  { for(i=0;i<4;i++)
    { if(in[i]==0) end=1;
      outword<<=6;
      a=in[i];
      if(isalpha(a) && isupper(a)) a-=('A');
      else if(isalpha(a) && islower(a)) a-=('a'-26);
      else if(isdigit(a)) a+=4;
      else if(a=='+') a=62;  
      else if(a=='/') a=63;  
      else a=0;                  // text only! Padding with 0-Bytes
      outword|=a;
    }
    out[0]=(outword>>16)&255;    // slow, but portable (byte order!) 
    out[1]=(outword>>8)&255;     
    out[2]=(outword)&255;     
    in+=4;
    out+=3;maxlen-=3;
  }
  out[0]=0;
}

/*---------------------------------------------------------------------------*/

void httpd::get_contentlength(char *buf)
{
  char *dop=strchr(buf,':');
  if(dop)
    contentlength=atol(dop+1);
}

/*---------------------------------------------------------------------------*/

void httpd::get_field(char *buf,char *dest,int maxlen)
{
  char *dop=strchr(buf,':');
  if(dop)
  { dop++; 
    while(dop[0]==' ') 
      dop++;
    if(strlen(dop)>=maxlen) 
      dop[maxlen-1]=0;
    
    strcpy(dest,dop);
  }
  else
    dest[0]=0;
}

/*---------------------------------------------------------------------------*/

void httpd::form_referer(void)
{
  char tmp[80];
  char *hostpos=stristr(referer,host);
  if(referer[0] && host[0] && hostpos)
  {
    if ((strlen(hostpos+strlen(host))>79))
      trace(fatal,"form_referer","str too long");
    strcpy(tmp,hostpos+strlen(host));
    strcpy(referer,tmp); 
  }
  if(referer[0]==0)
    strcpy(referer,"/");
  //printf("referer: \"%s\"\n",referer);
}

/*---------------------------------------------------------------------------*/

void httpd::get_authorization(char *buf,int cookie)
{
  char *search;
  char *basic;
  if(cookie)
    search=m.boxname;
  else
    search="Basic";
    
  basic=strstr(buf,search);

  if(basic)
  { char loginpw[50];
    char *locpw;
    basic+=(strlen(search)+1);
    if(strlen(basic)>50)
      basic[50]=0;
    
    strcpy(userpass,basic);
    base64bin(basic,loginpw,49);
    locpw=strchr(loginpw,':');
    if(locpw)
    { locpw[NAMELEN+1]=0;
      if(cookie)
        strcpy(cookie_pw,locpw+1);
      else
        strcpy(pw,locpw+1);
      locpw[0]=' ';
    }
    locpw=strchr(loginpw,' ');
    if(locpw)
      locpw[0]=0;
    loginpw[NAMELEN]=0;  
    if(cookie)
      strcpy(cookie_login,loginpw);
    else
      strcpy(login,loginpw);
  }
  //printf("http login: \"%.50s\", pw: \"%s\"\n",login,pw);
}

/*---------------------------------------------------------------------------*/

void httpd::get_request(void)
{ char locinbuf[301];
  getline(locinbuf,300,1);
  strcpy(inbuf,locinbuf);
  char *locmethod=locinbuf;
  char *locuri=blankweg(locmethod);
  char *locprotocol=blankweg(locuri);
  blankweg(locprotocol);
  
  contentlength=0L;
  content=NULL;
  status=200;
  version=9;             // default if no protocol is given
  file[0]=0;
  login[0]=0;
  pw[0]=0;
  cookie_login[0]=0;
  cookie_pw[0]=0;
  nocookie=0;
  referer[0]=0;  //deti 16-jul-97
  mimetype=NONE; //deti 15-jul-97
  
  if(locprotocol)
  { char *slash=strchr(locprotocol,'/');
    if(slash && isdigit(slash[1]))
      version=(slash[1]-'0')*10;
    if(slash && isdigit(slash[3])) // DH3MB: Added "slash &&"
      version+=(slash[3]-'0');
  }
  uri[0]=0;
  if(locuri)
    strcpy(uri,locuri);
  if(uri[0]!='/')
    status=400;

  //printf("http: m='%s' u='%s' p='%s'\n",locmethod,locuri,locprotocol);
  if(stricmp(locmethod,"GET")==0)
    method=GET;
  else if(stricmp(locmethod,"HEAD")==0)
    method=HEAD;
  else if(stricmp(locmethod,"POST")==0)
    method=POST;
  else
    status=501;

  // check if we should send a file
  if(strchr(uri,'.') && strchr(uri,'?')==NULL)  
  { // hm, yes it is a dirty hack...
    sprintf(file,"http%s",locuri);
    if(file[strlen(file)-1]=='/')
      file[strlen(file)-1]=0;
#ifndef _WIN32
    if(access(file,R_OK)!=0)  
    { status=404;
      file[0]=0;
    }
#endif
  }
  
  if(stricmp(uri,"/logout")==0)
    status=401;
  else if(locprotocol && stristr(locprotocol,"HTTP/")==locprotocol)
  {
    // determine HTTP version
    do 
    { getline(locinbuf,sizeof(locinbuf),1);
      if(locinbuf[0])
      { //printf("http:\"%s\"\n",locinbuf);
        if(stristr(locinbuf,"Authorization")==locinbuf)
          get_authorization(locinbuf,0);
        if(stristr(locinbuf,"Cookie")==locinbuf)
          get_authorization(locinbuf,1);
        if(stristr(locinbuf,"Referer")==locinbuf)
          get_field(locinbuf,referer,sizeof(referer));
        if(stristr(locinbuf,"Host")==locinbuf)
          get_field(locinbuf,host,sizeof(host));
        if(stristr(locinbuf,"Content-length")==locinbuf)
          get_contentlength(locinbuf);
      }    
    }
    while(locinbuf[0]);
    if(contentlength)
    { long i;    
      if (contentlength > 10000000L)
          trace(fatal,"httpd::start_http","content too long");
  
      content=(char *)t_malloc(contentlength*2,"hcon");  
      for(i=0;i<contentlength;i++)
        content[i]=getv();
      content[i]=0;  
    }
  }
  if(status!=200)
    uri[0]=0;
  if(file[0]==0 && status!=401)
  {   
  // oe3dzw ttypw nur beim Drahtzugang, sonst Name des Users
    char userpw[NAMELEN+5];
    if (isamprnet(b->peerip) > 0)
      strcpy(userpw,get_httppw(login));
    else
      get_ttypw(login,userpw);
    if(pw[0]==0 || stricmp(userpw,pw)!=0)
    { if (isamprnet(b->peerip) > 0)
        strcpy(userpw,get_httppw(cookie_login));
      else
        get_ttypw(cookie_login,userpw);
      
      if(cookie_pw[0]==0 || stricmp(userpw,cookie_pw)!=0)
      { status=401;
        pwlog(b->peerip,b->uplink,"bad cookie");
      }
      else
      { strcpy(pw,cookie_pw);
        strcpy(login,cookie_login);
        nocookie=1;
      }
    }
    strupr(login);
    if(login[0] && status!=401)
      sprintf(t->name,"/%s",login);
  }
  locuri=strchr(uri,'?');
  if(locuri)
  { if(content==NULL)
    { locuri[0]=0;  
      content=locuri+1;
    }  
  }
  form_referer();
}

/*---------------------------------------------------------------------------*/

void httpd::get_postarea(char *tag,char *result,int max,int wrap)
{ 
  int i;
  result[0]=0;
  char *firstresult=result;

  if(content==NULL)
    return;
  char *found=stristr(content,tag);
  if(found)
  { //printf("http: \"%.70s\"\n",found);
    found+=strlen(tag);
    while(found[0] && found[0]!='&' && max)
    {
      if(found[0]=='+')
        result[0]=' ';
      else if(found[0]=='%')
      { char hex[3];
        unsigned val;
        hex[0]=found[1];
        hex[1]=found[2];
        hex[2]=0;
        sscanf(hex,"%2X",&val);
        found+=2;
        if(val==13)
        { found++; 
          continue;
        }
        result[0]=val;
      }
      else
        result[0]=found[0];
      result++;
      found++;
      max--;
      result[0]=0;
    }
  }
  // do word wrapping since the browser seems not to be able to do it 
  if(wrap)
  { 
    result=firstresult;
    while(result[0])
    { if(result[0]=='\n')
      { result++;
        continue;
      }
      for(i=0;i<wrap && result[i] && result[i]!='\n';i++);
      if(i==wrap)
      { while(i && result[i]!=' ')
          i--;
        if(i==0)
          i=wrap;
        else
          result[i]='\n';
      }
      result+=i;
    }
  }
  result=firstresult;
  // convert ANSI-Umlaut to IBM
  for(;result[0];result++)
  { switch(result[0])
    { case 0xe4: result[0]=0x84; break; //ae
      case 0xf6: result[0]=0x94; break; //oe
      case 0xfc: result[0]=0x81; break; //ue
      case 0xc4: result[0]=0x8e; break; //Ae
      case 0xd6: result[0]=0x99; break; //Oe
      case 0xdc: result[0]=0x9a; break; //Ue
      case 0xdf: result[0]=0xe1; break; //ss
    }
  }
}

/*---------------------------------------------------------------------------*/

int httpd::prepare_header(void)
{ struct { int status; char *phrase; } st_tab[] = {
  { 200,"OK" },
  { 201,"Created" },
  { 202,"Accepted" },
  { 204,"No Content" },
  { 301,"Moved Permanently" },
  { 302,"Moved Temporarily" },
  { 304,"Not Modified" },
  { 400,"Bad Request" },
  { 401,"Unauthorized" },
  { 403,"Forbidden" },
  { 404,"Not Found" },
  { 500,"Internal Server Errror" },
  { 501,"Not Implemented" },
  { 502,"Bad Gateway" },
  { 503,"Service Unavailable" },
  { 0,"Unknown" }
  };
  int st;
  char* mimestr;
  char* h=header;
 
  for(st=0;st_tab[st].status;st++)
  { if(st_tab[st].status==status)
      break;
  }
  if(version>9)
  { 
    h+=sprintf(h,"HTTP/1.0 %d %s\n",status,st_tab[st].phrase);
    h+=sprintf(h,"Date: %s\n",ht_time(ad_time()+ad_timezone()));
    h+=sprintf(h,"Server: BayCom-Mailbox/"VNUMMER"\n");
    if(status==401)
    { h+=sprintf(h,"WWW-Authenticate: Basic realm=\"%s\"\n",m.boxname);
      h+=sprintf(h,"Set-Cookie: %s=; path=/; expires=%s\n",m.boxname,ht_time(0));
    }
    else if(strcmp(uri,"/")==0 && status==200 && pw[0] && nocookie==0)
      h+=sprintf(h,"Set-Cookie: %s=%s; path=/; expires=%s\n",
           m.boxname,userpass,ht_time(ad_time()+3600*24*30));

    if(mimetype==NONE)
    {
      if(stristr(uri,".jpg"))
        mimetype=JPG;
      if(stristr(uri,".gif"))
        mimetype=GIF;
      if(stristr(uri,".wav"))
        mimetype=WAV;
    }
    
    switch(mimetype)
    {
      case JPG: mimestr=MIME_JPG;break;
      case GIF: mimestr=MIME_GIF;break;
      case BIN: mimestr=MIME_BIN;break;
      case WAV: mimestr=MIME_WAV;break;
      default: mimestr=MIME_TXT;
    }
    h+=sprintf(h,"Content-type: %s\n",mimestr);
    
    if(file[0] && !access(file,0))
    {
        h+=sprintf(h,"Content-length: %ld\n",filesize(file));
        h+=sprintf(h,"Last-modified: %s\n",ht_time(filetime(file)+ad_timezone()));
    }
    else {
      char cmd[30];
      int expsec=180;
      get_postarea("cmd=",cmd,29);
      if(cmd[0] && cmd[0]!='d' && cmd[0]!='c' && cmd[0]!='h')
        expsec=(-10000);
      if(stristr(uri,"/send")==uri)
        expsec=8000; //OE3DZW was 7200
      h+=sprintf(h,"Expires: %s\n",ht_time(ad_time()+ad_timezone()+expsec));
    }  
    h+=sprintf(h,"\n");
  }
  if(status!=200)
  { //h+=sprintf(h,"<html><head><title>%s</title></head>\n",st_tab[st].phrase);
    h+=sprintf(h,"<body><h1>%s (%d)</h1>\n",st_tab[st].phrase,status);
    h+=sprintf(h,"<p>An error ocurred while processing your query.</p>\n");
    h+=sprintf(h,"<p>Rejected request: <tt>'%s'</tt></p><hr>\n",inbuf);
    h+=sprintf(h,"<b>BayCom-Mailbox</b> - Version "VNUMMER" - <i>httpd</i>\n");
    h+=sprintf(h,"</body></html>\n");
    put_header(st_tab[st].phrase,1);
    //printf("length: %d<br>",h-header); 
    return 0;
  }
  if(method==GET || method==POST)
  return 1;
  put_header("",1);
  return 0;
}

/*---------------------------------------------------------------------------*/

void httpd::put_header(char *head, int frame)
{ int ht=b->http;
  char *s=header;
  b->http=1;
  while(*s)
    putv(*(s++));
  b->http=ht;

  if(!frame)
    return;
  html_putf("<html><head><title>%s - %s</title></head>\n",m.boxname,head);
        html_putf("<body BGCOLOR=" BGCOL " background=\"/back.jpg\">\n");
}

/*---------------------------------------------------------------------------*/

void httpd::put_sendform(char *to,char *subj,char *read)
{ 
  if(to==NULL)   to="";
  if(subj==NULL) subj="";
  html_putf("<h2><i>Send Message</i></h2>\n");
  html_putf("<pre><form method=post action=\"/sendok\">\n");
  html_putf("     To: <input type=\"text\" name=\"to\" "
            "value=\"%s\" size=40><BR>",to);
  html_putf("Subject: <input type=\"text\" name=\"subj\" "
            "value=\"%s\" size=70>\n",subj);
  html_putf("<br>");
  html_putf("<tt><textarea name=\"text\" rows=15"
            " cols=79 hscroll=\"no\" wrap=\"virtual\">\n");
  if(read && read[0])
  {  
    setfullpath(read);
    b->job=j_read;
    b->optgroup=o_ir;
    scanoptions("");
    read_file(2);
  }
  html_putf("</textarea></tt>\n");
  html_putf("<BR><input type=\"submit\" value=\"Send Message\">");
  html_putf("&nbsp;<input type=\"reset\" value=\"Clear Form\">\n");
  html_putf("</form></pre>\n");
}

/*---------------------------------------------------------------------------*/

void httpd::put_homepage(char *cmd)
{
  char *tar="";
  if(cmd==NULL) cmd="";

#ifdef FRAME
  tar=" target=\"txt\"";
  html_putf(
     "<B><FONT SIZE=+2>BayCom-Mailbox V"VNUMMER" ("OPSYSTEM")</FONT></B> <FONT SIZE=0> &copy; by\n"
           "<a href=\"http://www.baycom.org/flori/\">"
           "DL8MBT</a></FONT>\n");
  html_putf("<br><tt><b>Login: %s@%s [%s]</b></tt>\n",
           login,m.boxadress,m.boxheader);
#endif  
  html_putf("<form method=get action=\"/cmd\"%s>\n",tar);
  html_putf("Command: <input name=\"cmd\" value=\"%s\" size=40>",
            cmd);
  html_putf("  <input type=\"submit\" value=\"Execute\">\n");
  html_putf("</form>\n");

#ifdef FRAME
  html_putf("<a href=\"%s\" target=\"_parent\">home</a> |\n",referer);
#else
  html_putf("<a href=\"%s\"%s>back</a> |\n",referer,tar);
#endif
  html_putf("<a href=\"/cmd?cmd=c+100\"%s>newest info</a> |\n",tar);
  html_putf("<a href=\"/cmd?cmd=d+-20\"%s>newest own</a> |\n",tar);
  html_putf("<a href=\"/cmd?cmd=d+b\"%s>boards</a> |\n",tar);
  //html_putf("<a href=\"/cmd?cmd=h+index\"%s>help index</a> |\n",tar);
  html_putf("<a href=\"/cmd?cmd=log\"%s>log</a> |\n",tar);
  html_putf("<a href=\"/cmd?cmd=ps+-ru\"%s>ps</a> |\n",tar);
  html_putf("<a href=\"/send\"%s>send</a> |\n",tar);
  html_putf("<a href=\"/cmd?cmd=sl\"%s>slog</a> |\n",tar);
  html_putf("<a href=\"/cmd?cmd=st+f\"%s>status forward</a> |\n",tar);
  html_putf("<a href=\"/cmd?cmd=l+baybox+-30\"%s>bcm news</a> |\n",tar);
  html_putf("<a href=\"/cmd?cmd=u\"%s>users</a> |\n",tar);
  html_putf("<a href=\"/cmd?cmd=v\"%s>version</a> |\n",tar);
  html_putf("<a href=\"/logout\">remove cookie</a>\n");
#ifndef FRAME
  html_putf("<hr>\n");
#endif

}

/*---------------------------------------------------------------------------*/

void httpd::put_frame(void)
{
  html_putf("<html><head><title>%s - BayCom-Mailbox V"VNUMMER"\n",m.boxname);
  html_putf("</title><frameset rows=\"150,*\">\n");
  html_putf("<frame scrolling=no src=\"/command\" name=\"cmd\">\n");
  html_putf("<frame scrolling=auto src=\"/login\" name=\"txt\">\n");
  html_putf("</frameset></head>\n");
  html_putf("<noframe><body BGCOLOR=" BGCOL " background=\"/back.jpg\">\n");
  html_putf("<h1>Please use a browser with frame capability</h1><p>\n");
  put_homepage("");
  html_putf("</body></noframe>\n");
  html_putf("</html>\n");
}

/*---------------------------------------------------------------------------*/

void httpd::postmsg(char *in,char *out,int outlen)
{ int n=0;
  char *pin=in;
  out[0]=0;
  while(in[0])
  { out[1]=0; 
    if (n+2>outlen)
    { trace(serious,"postmsg","too short %s (%d)",pin,outlen);
      return;
    }   
    switch(in[0])
    { case '#':
      case '<':
      case '>':
      case '+':
      case '&':
      case '%':
      case '?':
      case '"':
      case 10: 
      case 13:
         sprintf(out,"%%%02X",(unsigned)in[0]);
         out+=2;
         n+=2;
         break;
      default:
         if(in[0]==' ')
           out[0]='+';
         else  
           out[0]=in[0];
    }
    out++;  
    in++;
    n++;
  }
}

/*---------------------------------------------------------------------------*/

void httpd::start_http(char *name)
{
  char sign1[]="STARTHTTP";
  //b->http=1;
  cookie_login[0]=cookie_pw[0]=0; //oe3dzw, Initialisierung

  get_request();

  char cmd[200];
  char *head;
  char to[80];
  char subj[80];
  char read_cmd[30];
  char *sl;
  int do_erase=0;
  char post_subj[BETREFFLEN*2+5];
  char *eptr;
  long nrep;
  
  if(prepare_header())
  {
    if(file[0])
    { FILE *f=s_fopen(file,"lrb");
      put_header("",1);
      if(f)
      { int a;
        while((a=fgetc(f))!=EOF)
          putv_sock(a);
      }
      s_fclose(f);
    }
    else
    {
      get_postarea("cmd=",cmd,59);
      if(cmd[0])
        head=cmd;
      else
        head="BayCom-Mailbox V"VNUMMER;
      b->continous=1;
      b->http=2; 
#ifdef FRAME
      if(strcmp(uri,"/")==0)
      { put_header(head,0);
  put_frame();
        return;
      }
      else if(stricmp(uri,"/command")==0)
      { put_header(head,1);
  	put_homepage("");
  	html_putf("</body></html>");
        return;
      }
#endif

      //trace(report,"mbhttp","rx %d tx %d",b->rxbytes,b->txbytes);
#ifdef FRAME
      if(stricmp(uri,"/login")==0)
#else
      {
  put_header(head,1);
        html_putf("<h1>BayCom-Mailbox V"VNUMMER" ("OPSYSTEM")</h1>\n"
                "<p><tt><b>Login: %s@%s [%s]</b></tt></p>\n",
                login,m.boxadress,m.boxheader);
        put_homepage("");
      }
      if(strcmp(uri,"/")==0)
#endif     
      {
        put_header(head,1);
        html_putf("<pre>");
        mblogin(login,login_standard,"HTTP");
        html_putf("</pre>");
      }
      else if(stricmp(uri,"/cmd")==0)
      {
        put_header(head,1);
        html_putf("<pre>");
        mblogin(login,login_silent,"HTTP");
        mailbef(cmd,0);
        html_putf("</pre>");
      }
      else if(stristr(uri,"/read/")==uri)
      {
        put_header(head,1);
        html_putf("<pre>");
        mblogin(login,login_silent,"HTTP");
        strcpy(read_cmd,uri+6);
        setfullpath(read_cmd);
        b->job=j_read;
        b->optgroup=o_ir;
        u->opt[b->optgroup]&=~(o_q | o_x);
  //b->http=1;
        scanoptions("");
        read_file(0);
        markread(0,0);
        html_putf("</pre><hr>");
        strcpy (b->line, b->betreff);
        sprintf (b->line, "Re:%s", b->betreff);
        if (toupper (b->betreff[0]) == 'R' &&
            toupper (b->betreff[1]) == 'E' &&
            b->betreff[2] == ':')
          sprintf (b->line, "Re^2:%s", b->betreff + 3);
        else if (toupper (b->betreff[0]) == 'R' &&
             toupper (b->betreff[1]) == 'E' &&
             b->betreff[2] == '^')
        {
          nrep = strtol (b->betreff + 3, &eptr, 10);
          if (nrep > 0 && nrep < 10000)
          sprintf (b->line, "Re^%ld%s", nrep + 1, eptr);
        }
        sprintf (b->betreff, "%.50s", b->line);       
        postmsg(b->betreff,post_subj,BETREFFLEN*2);
        strlwr(b->herkunft);
        strlwr(b->ziel);
        strlwr(b->at);
        postmsg(b->at,to,80);
        html_putf("<a href=\"%s\">Go back</a> |\n",referer);
        html_putf("<a href=\"/send?to=%s&subj=%s\">Reply</a> |\n",
             b->herkunft,post_subj);
        html_putf("<a href=\"/send?to=%s&subj=%s&read=%s\">"
             "Reply with text</a> |\n",b->herkunft,post_subj,read_cmd);
        html_putf("<a href=\"/send?to=%s+@+%s&subj=%s\">Comment</a> |\n",
             b->ziel,to,post_subj);
        html_putf("<a href=\"/send?to=%s+@+%s&subj=%s&read=%s\">"
             "Comment with text</a>\n",
             b->ziel,to,post_subj,read_cmd);
        html_putf("| <a href=\"/send?to=%s&subj=%s&era=%s\">"
             "Erase+Reply</a>\n",b->herkunft,post_subj,read_cmd);
        sl=strchr(read_cmd,'/');
        if(sl)
        { sl[0]=0;
          sprintf(post_subj,"e+%s+%%22%s%%22",read_cmd,sl+1);
          html_putf("| <a href=\"/cmd?cmd=%s\">Erase</a>\n",post_subj);
        }  
      }
      else if(stristr(uri,"/bread/")==uri)
      {
        char read[256];

        get_postarea("read=",read,sizeof(read));

        for(int i = 0; i != strlen(read); i++)
          read[i] = (read[i]==':')?'/':read[i];

        if(stristr(uri+7,".jpg"))
          mimetype=JPG;
        else
        if(stristr(uri+7,".gif"))
          mimetype=GIF;
        else
        if(stristr(uri+7,".wav"))
          mimetype=WAV;
        else
          mimetype=BIN;
        //  mimetype=TXT;
        prepare_header();
        put_header(head,0);
        mblogin(login,login_silent,"HTTP");
        strcpy(read_cmd,read);
        setfullpath(read_cmd);
        //printf("read_cmd [%s]",read_cmd);
        b->job=j_read;
        b->optgroup=o_ir;
        u->opt[b->optgroup]&=~(o_q | o_x);
        b->http=1;
        scanoptions("");
        //printf("%lX,%lX %s\n",b->optplus,b->optminus,t);
        b->optplus=0x10000;
        read_file(0);
        markread(0,0);
	return;
      }
      else if(stricmp(uri,"/send")==0)
      { 
        put_header(head,1);
        mblogin(login,login_silent,"HTTP");
        get_postarea("to=",to,79);
        get_postarea("subj=",subj,79);
        get_postarea("read=",read_cmd,29);
        if(read_cmd[0]==0)
        { get_postarea("era=",read_cmd,29);
          if(read_cmd[0])
            do_erase=1;
        }
        put_sendform(to,subj,read_cmd);
        if(do_erase && read_cmd[0])
        {
          sl=strchr(read_cmd,'/');
          if(sl)
          { sl[0]=0;
            sprintf(post_subj,"%s \"%s\"",read_cmd,sl+1);
            mbchange(post_subj,w_erase,1);
          }  
        }
      }
      else if(stricmp(uri,"/sendok")==0)
      { if (contentlength > 10000000L)
          trace(fatal,"httpd::start_http","content too long");
        char *sendtext=(char *)t_malloc(contentlength*2,"smsg");
        char tmpname[30];
        FILE *f;
        put_header(head,1);
        html_putf("<pre>");
        mblogin(login,login_silent,"HTTP");
        get_postarea("text=",sendtext,contentlength,79);        
        trace(report,"http","debug: contentlength %ld len %ld",
           contentlength,strlen(sendtext));
        get_postarea("to=",to,79);
        get_postarea("subj=",subj,79);

        sprintf(tmpname, "%s/%s.imp",TEMPPATH,strlwr (time2filename (0)));
        f=s_fopen(tmpname,"swt");
        if(f) //oe3dzw: nnnn at start of line
        { s_fsetopt(f,1);
          fprintf(f,"%s \nnnnn\nend\n",sendtext);
          s_fclose(f);
          t_free(sendtext);
          trace(report,"http","debug: sysimport %s",to);
          sysimport(tmpname);
          sprintf(cmd,"s %s %s",to,subj);
          mailbef(cmd,0);
          trace(report,"http","debug: mail sent %s",to);
        }
        else
          putf("\nCannot open file: %s\n",strerror(errno));
        unlink(tmpname);
      }
      else
      {
        put_header(head,1);
        putf("\nunknown request URI\n");
      }
      html_putf("</body></html>");
    }
  }

  if (strcmp(sign1,"STARTHTTP"))
    trace(serious,"http","debug: sig starthttp broken");
}

/*---------------------------------------------------------------------------*/

void html_putf(char *format, ...)
{ va_list argpoint;
  char cbuf[260];
  char *s=cbuf;
  int ht=b->http;
  if(ht==0)
    return;
  va_start(argpoint,format);
  vsprintf(cbuf,format,argpoint);
  va_end(argpoint);

  if(strlen(s)>257)
  { s[50]=0;
    trace(tr_abbruch,"http_putf","long %s",s);
  }

  b->http=1; 
  while(*s)
    putv(*(s++));
  b->http=ht;
}

/*---------------------------------------------------------------------------*/

void mblogin_http(char *name)
{ httpd ht;
//oe3dzw
  strcpy(b->uplink,"HTTP");
  strcpy(b->peerip,name);
  b->logincall[0]=0;
  ht.start_http(name);
  putflush_sock();
  disconnect_sock();
}

/*---------------------------------------------------------------------------*/
