/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  --------------------------------------------
  Auswertung/Erzeugung des Konfigurationsfiles
  --------------------------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980115 OE3DZW removed utcoffset, added pcisutc, stimeoffset
//19980202 OE3DZW added callformat
//                infolifetime 0 -> 999 (was 10)
//19980227 hrx    added 'guestcall' to beftab/cases/init.bcm - default is off
//19980311 DG9MHZ changes for Linux AX25-Kernel support
//19980324 OE3DZW fixed output of fwdssid
//19980404 hrx    support for tty-fwding
//19980408 hrx    autotrcwin added to beftab/befnum/cases/mbinit()/mbparsave()
//                commented out line 557, callformat should always be saved
//19980614 hrx    paclen added to beftab/befnum/cases/mbinit()/mbparsave(),
//                limited to dos16. added createboardfwd to beftab/befnum
//                cases/mbinit()/mbparsave()
//19980831 OE3DZW control-characters in the inifile will be ignored
//19980913 OE3DZW added savebroken
//19980924 Jan    added oldestbeacon
//19980924 OE3DZW fixed oldestbeacon,removed old parameters, again
//19981001 OE3DZW fixed oldestbeacon, (changed oldestfwd)
//19981015 OE3DZW added parameters for wxstation
//19981022 OE3DZW translated name of variables
//19990110 OE3DZW hadr will always be updates with correct own data
//19990215 OE3DZW removed TELL-server
//19990221 OE3DZW changed limit for maxbids, infolife,userlife
//19990721 hpk    added support for defpaclen, minpaclen, maxpaclen
//19990723 hpk    added support for defrea
//19991003 hpk    added support for pwonly
//19991021 hpk    added usertimeout
//19991115 hpk    in CB_OPT are infolife and userlife=800, watchdog=2, 
//                maillistserv=1, readlock=1, createboard=1 at first "boot".
//19991202 DB1RAS added support for deffhold and usertimeout
//19991216 DB1RAS in CB_OPT are userpw=0, defcheck+Q, deflt=1, maxlogins=2,
//		  fwdssid=9, fwdtrace=2 and eraselog=127 at first boot
//		  in CB_OPT are callformat=2, remerase=1-3, oldestfwd=>30,
//		  fwdtimeout=>30, usertimeout=>30 and createboard=0-1
//	          added support for deffdelay, working under _DEFFDELAY
//19991225 DB1RAS added support for fwdonly, working under _FWDONLY

#include "baycom.h"
#include "l2_appl.h"

/*---------------------------------------------------------------------------*/

static char *beftab[]=
  { "BOXADRESS","BOXADDRESS","BOXHEADER","SYSOPCALL","SYSOPBELL",
    "INFOLIFE","USERLIFE","OLDESTFWD","OLDESTBEACON","TRACELEVEL","FWDTRACE",
    "USERPATH","INFOPATH","TTYMODE","TTYPORT","FWDSSID",
    "USERQUOTA","USVSENSE","WATCHDOG","MAXLOGINS","DISABLE",
    "DEFPROMPT","DEFHELP","DEFLINES","DEFLF","DEFCMD","DEFSPEECH",
    "DEFIDIR","DEFUDIR","DEFILIST","DEFULIST","MAILBEACON","PURGEHOUR",
    "DEFIREAD","DEFUREAD","DEFCHECK","DEFGREP","READLOCK",
    "NEXTBID","NEXTFILE","SCROLLDELAY","TIMESLOT","DOSINPUT",
    "HADRSTORE","USERPW","REMERASE","ERASELOG","CREATEBOARD","MAXBIDS",
    "FWDTIMEOUT","CRTSAVE","MYCALL","NOPURGE","CALLFORMAT","SAVEBROKEN",
#ifdef _LCF
    "LOGINCALLFORMAT",
#endif
#ifdef _AUTOFWD
    "AUTOFWDTIME", 
#endif
#ifdef _PACLEN
    "DEFPACLEN","MINPACLEN","MAXPACLEN",
#endif
#ifdef _INIT_PACLEN
    "PACLEN", 
#endif
#ifdef __FLAT__
    "SMTP_PORT","POP3_PORT","NNTP_PORT","HTTP_PORT","TELNET_PORT","SERV_PORT",
    "RADIO_PORT",
#endif
#ifdef __DOS16__
    "PCISUTC","STIMEOFFSET",
#endif
#ifdef FILESURF // DG4MMI
    "FSPATH",
#endif
#ifdef MAILSERVER // DG4MMI
    "MAILLISTSERV",
#endif
#ifdef _AX25K
    "AX25K_IF", 
#endif
#ifdef _GUEST
    "GUESTCALL",
#endif
#ifdef HRX_OPT
    "AUTOTRCWIN",
#endif
#ifdef _TTY
    "TTYDEVICE",
#endif
#ifdef _WXSTN
    "WXSTNNAME","WXQTHALTITUDE","WXSENSORALTITUDE","WXTTY","WXPATH",
#endif
#ifdef _DEFREA
    "DEFREA",
#endif
#ifdef _EXDLOGIN
    "PWONLY",
    "DEFFHOLD",
#endif
#ifdef _DEFFDELAY	//db1ras
    "DEFFDELAY",
#endif
#ifdef _FWDONLY		//db1ras
    "FWDONLY",
#endif
#ifdef _UTIMEOUT
    "USERTIMEOUT",
#endif
#ifdef _EXT_WATCHDOG	//db1ras
    "DOSBEFORE",
    "DOSAFTER",
#endif
    NULL
  };

/*---------------------------------------------------------------------------*/

static enum befnum
  { unsinn,
    boxadress,boxaddress,boxheader,sysopcall,sysopbell,
    infolife,userlife,oldestfwd,oldestbeacon,tracelevel,fwdtrace,
    userpath,infopath,ttymode,ttyport,fwdssid,
    userquota,usvsense,watchdog,maxlogins,disable_,
    defprompt,defhelp,deflines,deflf,defcmd,defspeech,
    defidir,defudir,defilist,defulist,purgehour,mailbeacon,
    defiread,defuread,defcheck,defgrep,readlock,
    nextbid,nextfile,scrolldelay,timeslot,dosinput,
    hadrstore,userpw,remerase,eraselog,createboard,maxbids,
    fwdtimeout,crtsave,mycall,nopurge,callformat,savebroken
#ifdef _LCF              // JJ
    ,logincallformat     // JJ  
#endif                   // JJ
#ifdef _AUTOFWD
    ,autofwdtime 
#endif
#ifdef _PACLEN
    ,defpaclen,minpaclen,maxpaclen
#endif
#ifdef _INIT_PACLEN
    ,paclen
#endif
#ifdef __FLAT__
    ,smtp_port,pop3_port,nntp_port,http_port,telnet_port,serv_port
    ,radio_port
#endif
#ifdef __DOS16__
    ,pcisutc,stimeoffset
#endif
#ifdef FILESURF
    ,fspath
#endif
#ifdef MAILSERVER
    ,maillistserv
#endif
#ifdef _AX25K
    ,ax25k_if 
#endif
#ifdef _GUEST
    ,guestcall
#endif
#ifdef HRX_OPT
    ,autotrcwin 
#endif
#ifdef _TTY
    ,ttydevice
#endif
#ifdef _WXSTN
    ,wxstnname,wxqthaltitude,wxsensoraltitude,wxtty,wxpath
#endif
#ifdef _DEFREA
    ,defrea
#endif    
#ifdef _EXDLOGIN
   ,pwonly
   ,deffhold
#endif
#ifdef _DEFFDELAY	//db1ras
   ,deffdelay
#endif
#ifdef _FWDONLY		//db1ras
   ,fwdonly
#endif
#ifdef _UTIMEOUT
   ,usertimeout
#endif
#ifdef _EXT_WATCHDOG	//db1ras
   ,dosbefore
   ,dosafter
#endif
  }cmd=unsinn;

/*---------------------------------------------------------------------------*/

int mbinitbef(char *befbuf,int inifile)
{ int optgroup=0;

  befbuf+=blkill(befbuf);
  if((befbuf[0]==0) || (befbuf[0]==';') || (befbuf[0]=='['))
    return 1;

  if (inifile) 
  {
    if (strchr(befbuf,';')) // OE3DZW cut off comment
       *strchr(befbuf,';')=0;

    //replace all controll-characters by space (tab,linefeed etc.)
    int i;
    for (i=0;befbuf[i];i++)
    { 
      if (befbuf[i] < 0x20)
        befbuf[i]=0x20;
    } 
  }
  cmd=(befnum)readcmd(beftab,&befbuf,0);
  if(befbuf[0]=='=') befbuf++;
  befbuf+=blkill(befbuf);

  if(befbuf[0])
  { switch(cmd)
    { case unsinn:
	return 0;

      case boxaddress:
      case boxadress:
	strupr(befbuf);
	befbuf[HADRESSLEN]=0;
        // set address
	strcpy(m.boxadress,befbuf);
	if(strchr(befbuf,'.'))
	  strchr(befbuf,'.')[0]=0;
	befbuf[6]=0;
        // set name
	strcpy(m.boxname,befbuf);
        if(!inifile && mbcallok(m.boxname))
          m.disable=0;
	break;

      case boxheader:
	befbuf[59]=0;
	strcpy(m.boxheader,befbuf);
	break;

#ifdef __FLAT__
      case smtp_port:
        m.smtp_port=atoi(befbuf);
        break;
      case pop3_port:
        m.pop3_port=atoi(befbuf);
        break;
      case nntp_port:
        m.nntp_port=atoi(befbuf);
        break;
      case radio_port:
        m.radio_port=atoi(befbuf);
        break;
      case telnet_port:
        m.tcp_port=atoi(befbuf);
        break;
      case serv_port:
        m.serv_port=atoi(befbuf);
        break;
      case http_port:
        m.http_port=atoi(befbuf);
        break;
#endif
#ifdef _AX25K
      case ax25k_if:
        befbuf[80]=0;
        strupr(befbuf);
        strcpy(m.ax25k_if,befbuf);
      break; 
#endif
#ifdef _GUEST
      case guestcall:               
        strupr(befbuf);            
        befbuf[6]=0;               
        strcpy(m.guestcall,befbuf); 
        break;                     
#endif

#ifdef __DOS16__
      case pcisutc:
        m.pcisutc=atoi(befbuf);
        break;

      case stimeoffset:
        m.stimeoffset=atoi(befbuf);
        break;

#endif

#ifdef _INIT_PACLEN
      case paclen:             
        m.paclen=atoi(befbuf); 
        break;                
#endif

#ifdef HRX_OPT
      case autotrcwin:              
        m.autotrcwin=atoi(befbuf);  
        if (m.autotrcwin > 1U)     
          m.autotrcwin = 0;        
        break;                     
#endif

#ifdef _TTY
      case ttydevice:
        m.ttydevice=atoi(befbuf);
        break;
#endif

#ifdef _WXSTN
      case wxstnname:
        befbuf[30]=0;
        strcpy(m.wxstnname,befbuf);
        break;

      case wxqthaltitude:
        m.wxqthaltitude=atol(befbuf);
        break;
 
      case wxsensoraltitude:
        m.wxsensoraltitude=atol(befbuf);
        break;

      case wxtty:
        befbuf[20]=0;
        strcpy(m.wxtty,befbuf);
        break;
     
      case wxpath:
        strlwr(befbuf);
        killbackslash(befbuf); 
        befbuf[28]=0;
	if(befbuf[strlen(befbuf)-1]=='/')         // remove trailing slash
	  befbuf[strlen(befbuf)-1]=0;
        strcpy(m.wxpath,befbuf);
        xmkdir(befbuf);
        break;          
#endif

      case sysopcall:
	strupr(befbuf);
	befbuf[6]=0;
	strcpy(m.sysopcall,befbuf);
	break;

      case callformat:   // 0..amateur, 1..cb, 2..both
#ifndef CB_OPT
        m.callformat=atoi(befbuf);
        if (m.callformat > 2U)
	  m.callformat = 0;
#else
	m.callformat=2;
#endif
        break;

#ifdef _LCF                                                   // JJ
      case logincallformat:   // 0..amateur, 1..cb, 2..both   // JJ
        m.logincallformat=atoi(befbuf);                       // JJ
        if (m.logincallformat > 2U)                           // JJ
        m.logincallformat = 0;                                // JJ
#endif

      case savebroken:
        m.savebroken=atoi(befbuf);
        break;

      case infolife:
	m.infolife=atoi(befbuf);
	if(m.infolife==0)
	  m.infolife=999;
	break;

      case userlife:
	m.userlife=atoi(befbuf);
	if(m.userlife==0)
	  m.userlife=999;
	break;

      case dosinput:
	m.dosinput=atoi(befbuf);
	break;

      case nopurge:
	m.nopurge=atoi(befbuf);
	break;

      case maxbids:
	m.maxbids=atol(befbuf);
        if(m.maxbids<131072L) //2^17
          m.maxbids=131072L;
	break;

      case crtsave:
	m.crtsave=atoi(befbuf);
	break;

      case createboard:
	m.createboard=atoi(befbuf);
#ifdef CB_OPT		//db1ras
	if(m.createboard > 1)
	  m.createboard = 1;
#endif
	break;

      case remerase:
	m.remerase=atoi(befbuf);
#ifdef CB_OPT		//db1tas
	if(m.remerase < 1 || m.remerase > 3)
	  m.remerase=3;
#endif
	break;

      case eraselog:
	m.eraselog=atoi(befbuf);
	break;

      case oldestfwd:

	m.oldestfwd=atoi(befbuf);
	if(m.oldestfwd==0)
	  m.oldestfwd=999;
#ifdef CB_OPT		//db1ras
	if(m.oldestfwd < 30)
	  m.oldestfwd=30;
#endif
	break;

      case oldestbeacon:

	m.oldestbeacon=atoi(befbuf);
	if(m.oldestbeacon==0U||m.oldestbeacon > 1000U)
	  m.oldestbeacon=1;
	break;

      case userquota:
	m.userquota=atoi(befbuf);
	break;

      case readlock:
	m.readlock=atoi(befbuf);
	break;

      case usvsense:
	m.usvsense=atoi(befbuf);
	break;

      case watchdog:
	m.watchdog=atoi(befbuf);
	break;

      case maxlogins:
	m.maxlogins=atoi(befbuf);
	break;

      case disable_:
	m.disable=atoi(befbuf);
	break;

      case tracelevel:
	m.tracelevel=atoi(befbuf);
	break;

      case fwdtrace:
	m.fwdtrace=atoi(befbuf);
	break;

      case sysopbell:
	m.sysopbell=atoi(befbuf);
	break;

      case userpath:
	strlwr(befbuf);
        killbackslash(befbuf);
	befbuf[28]=0;
	if(befbuf[strlen(befbuf)-1]=='/')         // remove trailing slash
	  befbuf[strlen(befbuf)-1]=0;
	strcpy(m.userpath,befbuf);
        xmkdir(befbuf);
	break;

      case infopath:
	strlwr(befbuf);
        killbackslash(befbuf);
	befbuf[28]=0;
	if(befbuf[strlen(befbuf)-1]=='/')         // remove trailing slash
	  befbuf[strlen(befbuf)-1]=0;
	strcpy(m.infopath,befbuf);
        xmkdir(befbuf);
	break;

      case ttymode:
	if(strlen(befbuf)>5)
	{ fncase(befbuf);
	  befbuf[19]=0;
	  strcpy(m.ttymode,befbuf);
	}
	else
	  m.ttymode[0]=0;
	break;

      case fwdssid:
	m.fwdssid=atoi(befbuf);
	break;

      case fwdtimeout:
	m.fwdtimeout=atoi(befbuf);
#ifdef CB_OPT		//db1ras
	if(m.fwdtimeout < 30)
	  m.fwdtimeout=30;
#endif
	break;

#ifdef _AUTOFWD
      case autofwdtime:             
        m.autofwdtime=atoi(befbuf); 
        break;                      	
#endif

      case defprompt:
	befbuf[PROMPTLEN]=0;
	strcpy(m.prompt,befbuf);
	break;

      case defcmd:
	strupr(befbuf);
	befbuf[FIRSTCMDLEN]=0;
	strcpy(m.firstcmd,befbuf);
	break;

      case defhelp:
	m.helplevel=atoi(befbuf);
	break;

      case deflines:
	m.zeilen=atoi(befbuf);
	break;

      case userpw:
	m.userpw=atoi(befbuf);
	break;

      case mycall:
      { strupr(befbuf);
        int i=sscanf(befbuf,"%s%s%s%s",
                     m.mycall[0],m.mycall[1],m.mycall[2],m.mycall[3]);
        if(i>0 && i<5)
          m.mycalls=i;
        else
          m.mycalls=0;
        align_mycalls();
      } break;

#ifdef FILESURF
      case fspath:
	befbuf[FSPATHLEN]=0;
	strcpy(m.fspath,befbuf);
	break;
#endif

#ifdef MAILSERVER
      case maillistserv:
	m.maillistserv=atoi(befbuf);
	break;
#endif

      case deflf:
	m.deflf=atoi(befbuf);
	break;

      case defgrep:
	optgroup++;
      case defcheck:
	optgroup++;
      case defuread:
	optgroup++;
      case defiread:
	optgroup++;
      case defulist:
	optgroup++;
      case defilist:
	optgroup++;
      case defudir:
	optgroup++;
      case defidir:
	while(befbuf[0])
	{ if(isalpha(befbuf[0]))
	    m.opt[optgroup]|=(1L<<(befbuf[0]-'A'));
	  befbuf+=1;
	}
	break;

#ifdef _DEFREA
      case defrea:
	m.defrea=atoi(befbuf);
	break;
#endif
#ifdef _EXDLOGIN
      case pwonly:
	m.pwonly=atoi(befbuf);
	break;
      case deffhold:
	m.deffhold=atoi(befbuf);
	break;
#endif
#ifdef _DEFFDELAY	//db1ras
      case deffdelay:
	m.deffdelay=atoi(befbuf);
	if(m.deffdelay > 30)
	  m.deffdelay = 30;
	break;
#endif
#ifdef _FWDONLY		//db1ras
      case fwdonly:
	m.fwdonly=atoi(befbuf);
	break;
#endif
#ifdef _UTIMEOUT
      case usertimeout:
	m.usertimeout=atoi(befbuf);
	if (m.usertimeout && m.usertimeout < 30)
	  m.usertimeout = 30;
	break;
#endif
#ifdef _EXT_WATCHDOG	//db1ras
      case dosbefore:
	if (strlen(befbuf) > 110)
	  befbuf[110]=0;
	strcpy(m.dosbefore,strupr(befbuf));
	break;
      case dosafter:
	if (strlen(befbuf) > 110)
	  befbuf[110]=0;
	strcpy(m.dosafter,strupr(befbuf));
	break;
#endif
#ifdef _PACLEN
      case defpaclen:
	m.defpaclen=atoi(befbuf);
	break;

      case minpaclen:
        m.minpaclen=atoi(befbuf);
        break;

      case maxpaclen:
        m.maxpaclen=atoi(befbuf);
        break;
#endif      
    }
  }
  return 1;
}

/*---------------------------------------------------------------------------*/

void mbinit(void)
{ FILE *f;
  int line=0;
  char s[256];

  memset(&m,0,sizeof(m));
  strcpy(m.boxname,"MYCALL");
  strcpy(m.boxadress,"MYCALL.#BAY.DEU.EU");
  strcpy(m.boxheader,STD_BOXHEADER);
  strcpy(m.sysopcall,"DL8MBT");
#ifdef _GUEST
  strcpy(m.guestcall,"off");
#endif
  strcpy(m.userpath,"user");
  strcpy(m.infopath,"info");
#ifdef FILESURF 
  strcpy(m.fspath, "off");
#endif
#ifdef _WXSTN
  strcpy(m.wxstnname,"BayCom");
  m.wxqthaltitude=585L;
  m.wxsensoraltitude=11L;
  strcpy(m.wxtty,"/dev/wxbcm");
  strcpy(m.wxpath,"wx");
#endif
#ifdef __FLAT__
  m.tcp_port =DEFTELNPORT;
  m.smtp_port=DEFSMTPPORT;
  m.pop3_port=DEFPOP3PORT;
  m.nntp_port=0;//DEFNNTPPORT;
  m.serv_port=DEFSERVPORT;
  m.http_port=DEFHTTPPORT;
  m.radio_port=0;//DEFRADIOPORT;
#endif
#ifdef _AX25K 
  strcpy(m.ax25k_if,"off");
#endif
#ifdef __DOS16__
  m.pcisutc = 0;
  m.stimeoffset = (-1);
#endif
#ifdef _PACLEN
  #ifdef CB_OPT
    m.defpaclen = 80;
    m.minpaclen = 40;
    m.maxpaclen = 128;
  #else
    m.defpaclen = 0;
    m.minpaclen = 40;
    m.maxpaclen = 256;
  #endif
#endif
#ifdef _DEFREA
  m.defrea = 2; // Readlock 2 fuer neue User
#endif
#ifdef _INIT_PACLEN
  m.paclen = 0; 
#endif
  m.scrolldelay=1;
#ifndef CB_OPT
  m.callformat=0; //default is amateur radio calls
#else
  m.callformat=2; // in cb-version always amateur radio and cb-calls allowed
#endif  
#ifdef _LCF             // JJ
  m.logincallformat=0;  // JJ
#endif                  // JJ
  m.savebroken=0;
  m.timeslot=2;
#ifndef CB_OPT
  m.infolife=999;
  m.userlife=999;
#else
  m.infolife=800;
  m.userlife=800;
#endif  
  m.oldestfwd=30;
  m.oldestbeacon=1;
#ifndef CB_OPT		//db1ras
  m.fwdssid=8;
#else
  m.fwdssid=9;
  m.fwdtrace=2;
  m.eraselog=127;
#endif
  m.fwdtimeout=60;
  m.tracelevel=1;
#ifndef CB_OPT		//db1ras
  m.dosinput=1;
#else
  m.dosinput=0;
#endif
  m.nopurge=0;
  m.usvsense=0;
#ifndef CB_OPT		//db1ras
  m.maxlogins=0;
#else
  m.maxlogins=2;
#endif
  m.sysopbell=0;
#ifdef HRX_OPT
  m.autotrcwin=0;
#endif
#ifndef CB_OPT
  m.userpw=1;
  m.watchdog=6;
#else
  m.userpw=0;
  m.watchdog=2;
  m.readlock=1;
  m.createboard=1;
  m.deflf=1;
  #ifdef MAILSERVER
    m.maillistserv=1;
  #endif
#endif  
#ifdef _EXDLOGIN
  m.pwonly=0;	// ausschliesslicher Passwortlogin (0=abgeschaltet)
  m.deffhold=0;	// default for alter fhold
#endif
#ifdef _DEFFDELAY	//db1ras
  m.deffdelay=10;
#endif
#ifdef _FWDONLY		//db1ras
  m.fwdonly=0;
#endif
#ifdef _UTIMEOUT
  m.usertimeout=60;
#endif
#ifdef _EXT_WATCHDOG	//db1ras
  strcpy(m.dosbefore,"AUTOBOOT /DOS 0");
  strcpy(m.dosafter,"AUTOBOOT /DOS 1");
#endif
#ifdef _TTY
  m.ttydevice=0;
#endif
  m.crtsave=0;
  m.hadrstore=2;
#ifndef CB_OPT		//db1ras
  m.remerase=1;
#else
  m.remerase=3;
#endif
  m.maxbids=262144L;  //2^18
  strcpy(m.prompt,"(%b)-->");
  strcpy(m.firstcmd,"D");
  m.helplevel=2;
  m.opt[o_id]=o_a+    o_d+o_e+o_k+    o_l+o_m+	          o_w+o_y+o_z;
  m.opt[o_ud]=o_a+    o_d+    o_k+    o_l+o_m+    o_t+    o_w+o_y+o_z;
  m.opt[o_il]=o_a+o_b+o_d+o_j+o_q+           	  o_t+    o_w+o_x+o_y;
  m.opt[o_ul]=o_a+o_b+o_d+o_j+o_q+                o_t+    o_w+o_x+o_y;
  m.opt[o_ir]=o_a+o_b+o_d+o_e+o_i+o_l+o_m+o_p+o_r+o_t+o_u+o_w+o_y+o_z;
  m.opt[o_ur]=o_a+o_b+o_d+o_e+o_i+o_l+o_m+o_p+o_r+o_t+o_u+o_w+o_y+o_z;
#ifndef _CHECK_Q	//db1ras
  m.opt[o_ch]=o_a+o_b+o_d+o_e+o_j+o_l+o_m+                o_w+o_y;
#else
  m.opt[o_ch]=o_a+o_b+o_d+o_e+o_j+o_l+o_m+    o_q+        o_w+o_y;
#endif
  m.opt[o_gr]=o_i;

  if(filetime(MBINITOLDNAME) > filetime(MBINITNAME))  // only if newer
    xrename(MBINITOLDNAME,MBINITNAME);
  f=s_fopen(MBINITNAME,"srt");
  if(f)
  { while(fgets(s,255,f))
    { if(s[strlen(s)-1]=='\n')
	s[strlen(s)-1]=0;
      if(s[strlen(s)-1]=='\r')
	s[strlen(s)-1]=0;
      if(!mbinitbef(s,1))
	trace(replog,"ini","line %d: %s",line,s);
      line++;
    }
    s_fclose(f);
  }
  if(m.mycalls==0 && mbcallok(m.boxname))
  { sprintf(m.mycall[0],"%s-8",m.boxname);
    m.mycalls=1;
  }
  if(m.mycalls==1 && mbcallok(m.boxname))
  { sprintf(m.mycall[1],"%s-7",m.boxname);
    m.mycalls=2;
  }
  align_mycalls();
  
  update_hadr(makeheader(1),-1,0);
}

/*---------------------------------------------------------------------------*/

void mbparsave(void)
{ FILE *f;
  int i;

  if(!mbcallok(m.boxname))
  { trace(serious,"parsave","BOXADDR not specified, not saved");
    return;
  }
  f=s_fopen(INITTMPNAME,"swt");
  if(f)
  { s_fsetopt(f,1);
    fprintf(f,"; --- mailbox ---\n");
    fprintf(f,"boxaddress %s\n",m.boxadress);
    fprintf(f,"boxheader %s\n",m.boxheader);
    fprintf(f,"sysopcall %s\n",m.sysopcall);
#ifdef _GUEST
    fprintf(f,"guestcall %s\n",m.guestcall);   
#endif
    if (m.callformat) //only save this parameter if not default
      fprintf(f,"callformat %d\n",m.callformat);
#ifdef _LCF                                                             // JJ
    if (m.logincallformat) //only save this parameter if not default    // JJ
      fprintf(f,"logincallformat %d\n",m.logincallformat);              // JJ
#endif                                                                  // JJ
    if (m.savebroken)
      fprintf(f,"savebroken %d\n",m.savebroken);  
    fprintf(f,"infopath %s\n",m.infopath);
    fprintf(f,"userpath %s\n",m.userpath);
#ifdef FILESURF // DG4MMI, 30 Aug 1997
    fprintf(f,"fspath %s\n", m.fspath);
#endif
    fprintf(f,"remerase %u\n",m.remerase);
    fprintf(f,"maxbids %lu\n",m.maxbids);
    fprintf(f,"; --- packet interface ---\n");
    fprintf(f,"mycall");
    for(i=0;i<m.mycalls;i++) fprintf(f," %s",m.mycall[i]);
    fprintf(f,"\n");
    fprintf(f,"fwdssid %u\n",m.fwdssid);
#ifdef _AX25K
    fprintf(f,"ax25k_if %s\n",m.ax25k_if);
#endif
#ifdef _PACLEN
    fprintf(f,"minpaclen %i\n",m.minpaclen);
    fprintf(f,"maxpaclen %i\n",m.maxpaclen);    
    fprintf(f,"defpaclen %i\n",m.defpaclen);
#endif
#ifdef __FLAT__
    fprintf(f,"; --- tcp/ip interface ---\n");
    fprintf(f,"http_port %d\n",m.http_port);
    fprintf(f,"smtp_port %d\n",m.smtp_port);
    fprintf(f,"pop3_port %d\n",m.pop3_port);
	if (m.nntp_port) //only save this parameter if activated
      fprintf(f,"nntp_port %d\n",m.nntp_port);
    fprintf(f,"telnet_port %d\n",m.tcp_port);
    fprintf(f,"serv_port %d\n",m.serv_port);
    fprintf(f,"radio_port %d\n",m.radio_port);
#endif
#ifdef __DOS16__
    fprintf(f,"pcisutc %d\n",m.pcisutc);
    fprintf(f,"stimeoffset %d\n",m.stimeoffset);
#endif
#ifdef _INIT_PACLEN
    fprintf(f,"paclen %i\n",m.paclen); 
#endif
    fprintf(f,"; --- timers ---\n");
    fprintf(f,"infolife %u\n",m.infolife);
    fprintf(f,"userlife %u\n",m.userlife);
    fprintf(f,"oldestfwd %u\n",m.oldestfwd);
    if (m.oldestbeacon!=1U)
      fprintf(f,"oldestbeacon %u\n",m.oldestbeacon);
    fprintf(f,"fwdtimeout %u\n",m.fwdtimeout);
#ifdef _UTIMEOUT
    fprintf(f,"usertimeout %i\n",m.usertimeout);
#endif
#ifdef _AUTOFWD
    fprintf(f,"autofwdtime %u\n",m.autofwdtime); 
#endif
    if (m.crtsave)
      fprintf(f,"crtsave %u\n",m.crtsave);
    fprintf(f,"; --- logging ---\n");
    fprintf(f,"tracelevel %u\n",m.tracelevel);
    fprintf(f,"fwdtrace %u\n",m.fwdtrace);
    fprintf(f,"eraselog %u\n",m.eraselog);
    fprintf(f,"; --- system ---\n");
    fprintf(f,"disable %d\n",m.disable);
    if (m.usvsense)
      fprintf(f,"usvsense %d\n",m.usvsense);
    fprintf(f,"watchdog %d\n",m.watchdog);
#ifdef _TTY
    fprintf(f,"ttydevice %d\n",m.ttydevice);
#endif    
    fprintf(f,"ttymode %s\n",m.ttymode[0] ? m.ttymode : "OFF");
#ifndef CB_OPT		//db1ras
    if (!m.dosinput)
#endif
      fprintf(f,"dosinput %d\n",m.dosinput);
#ifdef _EXT_WATCHDOG	//db1ras
      fprintf(f,"dosbefore %s\n",m.dosbefore);
      fprintf(f,"dosafter %s\n",m.dosafter);
#endif
#ifdef HRX_OPT
    fprintf(f,"autotrcwin %d\n",m.autotrcwin); 
#endif
    if (m.sysopbell)
      fprintf(f,"sysopbell %u\n",m.sysopbell);
    fprintf(f,"; --- user constraints ---\n");
    fprintf(f,"userquota %d\n",m.userquota);
    if (m.maxlogins) 
      fprintf(f,"maxlogins %d\n",m.maxlogins);
    fprintf(f,"userpw %d\n",m.userpw);
    fprintf(f,"readlock %d\n",m.readlock);
    fprintf(f,"createboard %d\n",m.createboard);
    fprintf(f,"nopurge %u\n",m.nopurge);
#ifdef MAILSERVER
    fprintf(f,"maillistserv %d\n",m.maillistserv);
#endif
    fprintf(f,"; --- user defaults ---\n");
    fprintf(f,"defprompt %s\n",m.prompt);
    fprintf(f,"defcmd %s\n",m.firstcmd);
    fprintf(f,"defhelp %u\n",m.helplevel);
    fprintf(f,"deflines %u\n",m.zeilen);
    fprintf(f,"deflf %u\n",m.deflf);
    fprintf(f,"defidir %s\n",optstr(m.opt[o_id]));
    fprintf(f,"defudir %s\n",optstr(m.opt[o_ud]));
    fprintf(f,"defilist %s\n",optstr(m.opt[o_il]));
    fprintf(f,"defulist %s\n",optstr(m.opt[o_ul]));
    fprintf(f,"defiread %s\n",optstr(m.opt[o_ir]));
    fprintf(f,"defuread %s\n",optstr(m.opt[o_ur]));
    fprintf(f,"defcheck %s\n",optstr(m.opt[o_ch]));
    if(fprintf(f,"defgrep %s\n",optstr(m.opt[o_gr]))<0) goto fehler;;
#ifdef _DEFREA
    fprintf(f,"defrea %i\n",m.defrea);
#endif
#ifdef _DEFFDELAY	//db1ras
    fprintf(f,"deffdelay %i\n",m.deffdelay);
#endif
#ifdef _WXSTN    
    fprintf(f,"; --- wx station ---\n");
    fprintf(f,"wxstnname %s\n",m.wxstnname);
    fprintf(f,"wxqthaltitude %ld\n",m.wxqthaltitude);
    fprintf(f,"wxsensoraltitude %ld\n",m.wxsensoraltitude);
    fprintf(f,"wxtty %s\n",m.wxtty);
    fprintf(f,"wxpath %s\n",m.wxpath);
#endif
#ifdef _BCMNET
    fprintf(f,"; --- special options for CB-BCMNET ---\n");
#endif
#ifdef _EXDLOGIN
    fprintf(f,"pwonly %i\n",m.pwonly);
    fprintf(f,"deffhold %i\n",m.deffhold);
#endif
#ifdef _FWDONLY		//db1ras
    if (m.fwdonly)
      fprintf(f,"fwdonly %i\n",m.fwdonly);
#endif
    s_fclose(f);
    xunlink(INITBAKNAME);
    xrename(MBINITNAME,INITBAKNAME);
    xrename(INITTMPNAME,MBINITNAME);
  }
  else
    trace(serious,"mbparsave","fopen errno=%d",errno);
    
  update_hadr(makeheader(1),-1,0);


  return;

  fehler:
    trace(serious,"mbparsave","errno=%d",errno);
    s_fclose(f);
}
