/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  --------------------------
  Mailinglist-Server for BCM
  --------------------------


  Copyright (C)       Johann Hanne, DH3MB, jonny@baycom.org
                      Falkenweg 6
                      D-85122 Hitzhofen

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980223 OE3DZW fixed fsetopt
//19980429 DH3MB  fixed several things...
//19980430 OE3DZW info/list are equal, both will show lists in detail
//                when called
//                removed texts under dos
//                included normal help
//                b->continous is always set
//                made commands shorter
//19980505 DH3MB  fixes for DOS-version, mailfile was corrupt
//19980505 OE3DZW several fixes 
//19980527 DH3MB  fixes

#include "baycom.h"

#ifdef MAILSERVER

/*---------------------------------------------------------------------------*/

void mailserv::parse_listheader(char *line, char **name, char **number,
                                        char **options, char **description)
//*************************************************************************
//
//  Extracts the data from a mailing list header and stores the field
//  to the according pointers
//
//*************************************************************************
// eg: TEST 21 DESCRIPTION
{ *name=line;

  if(!strchr(line, ' '))  //append number 1 if no number in list
    strcat(line, " 1 ");

  rmcrlf(line);

  *(*number=strchr(line, ' '))=0;
  strupr(*name);

  while(*(++*number)==' ')
    **number=0;

  if((*options=strchr(*number, ' '))!=0)
  { **options=0;
    while(*(++*options)==' ')
      **options=0;
    if(**options=='-')
    { if((*description=strchr(*options, ' '))!=0)
      { **description=0;
        while(*(++*description)==' ')
          **description=0;
      }
      else
        *description=line+strlen(line);
    }
    else
    { *description=*options;
      *options=line+strlen(line);
    }
  }
  else
    *options=*description=line+strlen(line);

  if(**description)
    while((*description)[strlen(*description)-1]==' ')
      (*description)[strlen(*description)-1]=0;
}

/*---------------------------------------------------------------------------*/

char mailserv::seek_list(FILE *fold, FILE *f, char what, char *parameter)
//*************************************************************************
//
//  Moves the file-pointer to the searched string and copies the
//  file to f, if f!=0
//
//  If searching an user, the file-pointer must point to the header
//  of the mailing list
//
//  If searching an mailing list header, the options are automatically
//  stored to b->optplus
//
//*************************************************************************
{ char s[128], line[128];
  char *s2;
  long posbuf;

  if(what==ma_seek_user)
  { fgets(line,127,fold);
    if(f)
      fprintf(f, "%s", line);
  }
  posbuf=ftell(fold);

  while(fgets(line,127,fold)&& !feof(fold))
  { strcpy(s, line);
    rmcrlf(s);

    switch(what)
    {
    case ma_seek_list:
      if(s[0]!=' ')
      { if((s2=strchr(s, ' '))!=0)
          *s2++=0;
        if(stricmp(s, parameter)==0)
        { b->optplus=0;
          if(s2)
          { while(*s2==' ')
              s2++;
            while(isdigit(*s2))
              s2++;
            while(*s2==' ')
              s2++;
            if(*s2=='-')
              scanoptions(s2);
          }
          fseek(fold, posbuf, SEEK_SET);
          return 1;
        }
      }
      if(f)
        fprintf(f, "%s", line);
      break;

    case ma_seek_user:
      if(s[0]==' ')
      { while(*s==' ')
          strcpy(s, s+1);
        if(strchr(s, ' ')) *strchr(s, ' ')=0;
        strupr(s);
        if(strcmp(s, parameter)==0)
        { fseek(fold, posbuf, SEEK_SET);
          if(line[1]==' ')
            return 2;
          else
            return 1;
        }
        else
          if(f)
            fprintf(f, "%s", line);
      }
      else
      { fseek(fold, posbuf, SEEK_SET);
        return 0;
      }
      break;

    case ma_seek_end:
      if(f)
        fprintf(f, "%s", line);
      break;

    default:
      trace(serious, "maillistserv", "invalid seek-code");
      break;
    }

    posbuf=ftell(fold);
    waitfor(e_ticsfull);
  }

  return 0;
}

/*---------------------------------------------------------------------------*/

void mailserv::modify_list(char what, char *listname, char *parameter)
//*************************************************************************
//
//  Changes something in the mailing list settings
//
//*************************************************************************
{ FILE *f=s_fopen(MAILLISTTMPNAME, "swt");
  FILE *fold=s_fopen(MAILLISTNAME, "srt");
  char mybbs[HADRESSLEN+1]="";

  #ifndef __FLAT__  
  char monly[]="Sorry, only the maintainer may modify/set this parameter\n";
  #endif
  char nexis[]="A mailing list named %s does not exist.\n";
  //char *buffer=(char*)t_malloc(90,"msvd");
  char buffer[90];
  char *name, *number, *options, *description;
  char *s;
  char numok=1;
  char i;

  if(f)
    s_fsetopt(f,1);

  switch(what)
  { case ma_createlist:
      if((m.maillistserv==2) && !b->sysop)
      { putf("Sysop only.\n");
        break;
      }

      if(seek_list(fold, f, ma_seek_list, listname))
        putf("%s already exists.\n", listname);
      else
      { fprintf(f, "%s 1 \n", listname);
        // if (!b->sysop)
          fprintf(f,"  %s\n", b->logincall);
        putf("%s has been created.\n", listname);
      }
      break;

    case ma_deletelist:
      if(!b->sysop)
      { putf("Sysop only.\n");
        break;
      }

      if(seek_list(fold, f, ma_seek_list, listname))
      { seek_list(fold, 0, ma_seek_user, "dummy"); // seek to end of this list
        putf("Removed %s.\n", listname);
      }
      else
      { putf(nexis, listname);
        fseek(fold, 0, SEEK_SET);
      }
      break;

    case ma_setdescription:
      if(seek_list(fold, 0, ma_seek_list, listname))
      { if(!b->sysop && (seek_list(fold, 0, ma_seek_user, u->call)!=2))
        { 
          #ifdef __FLAT__
          putf("Only mailing list-maintainers may set the description.\n");
          #else
          putf(monly);
          #endif
          rewind(fold);
          break;
        }

        rewind(fold);
        seek_list(fold, f, ma_seek_list, listname);

        fgets(buffer,sizeof(buffer)-1,fold);
        rmcrlf(buffer);

        parse_listheader(buffer, &name, &number, &options, &description);

        while(*parameter=='-')
          parameter++;

        fprintf(f, "%s %s%s%s %s\n", name, number, *options ? " " : "",
                                      options, parameter);
        putf("New description of %s:\n%s\n",
             listname, parameter);
      }
      else
      { putf(nexis, listname);
        fseek(fold, 0, SEEK_SET);
      }
      break;

    case ma_setoptions:
      if(seek_list(fold, 0, ma_seek_list, listname))
      { if(!b->sysop && (seek_list(fold, 0, ma_seek_user, u->call)!=2))
        { 
          #ifdef __FLAT__            
          putf("Only mailing list maintainers may set the options.\n");
          #else
          putf(monly);
          #endif
          rewind(fold);
          break;
        }

        rewind(fold);
        seek_list(fold, f, ma_seek_list, listname);

        // Clean up the given options
        qsort(parameter, strlen(parameter), 1,
              (int(*)(const void *,const void *))strcmp);
        s=parameter;
        while(*s)
          if(strchr(s+1, *s) || (*s<'A') || (*s>'Z'))
            strcpy(s, s+1);
          else
            s++;
        // parameter now only consists of letters in alphabetical order

        fgets(buffer,sizeof(buffer)-1,fold);
        rmcrlf(buffer);

        parse_listheader(buffer, &name, &number, &options, &description);

        if(*parameter)
        { fprintf(f, "%s %s -%s %s\n", name, number, parameter,
                                       description);
          putf("Options of %s set to '%s'.\n",
               listname, parameter);
        }
        else
        { fprintf(f, "%s %s %s\n", name, number, description);
          putf("Options of %s deleted.\n",
               listname);
        }
      }
      else
      { putf(nexis, listname);
        fseek(fold, 0, SEEK_SET);
      }
      break;

    case ma_setnumber:
      if(seek_list(fold, 0, ma_seek_list, listname))
      { if(!b->sysop && (seek_list(fold, 0, ma_seek_user, u->call)!=2) &&
           (strcmp(b->uplink, "Mailsv")!=0))
        { 
          #ifdef __FLAT__
          putf("Only mailing list maintainers may set the serial number.\n");
          #else
          putf(monly);
          #endif
          rewind(fold);
          break;
        }
        for(i=0;i<strlen(parameter);i++)
          if(!isdigit(parameter[i]))
            numok=0;

        if(!numok)
        { putf("Invalid number.\n");
          rewind(fold);
          break;
        }

        rewind(fold);
        seek_list(fold, f, ma_seek_list, listname);

        fgets(buffer,sizeof(buffer)-1,fold);
        rmcrlf(buffer);

        parse_listheader(buffer, &name, &number, &options, &description);

        fprintf(f, "%s %s%s%s %s\n", name, parameter, *options ? " " : "",
                                      options, description);
        putf("The serial number of the mailing list %s has been set to %s.\n",
             listname, parameter);
      }
      else
      { putf(nexis, listname);
        fseek(fold, 0, SEEK_SET);
      }
      break;

    case ma_subscribe:
    case ma_adduser:
 
      if (!strchr(parameter,'@') && mbcallok(parameter))
      { get_mybbs(parameter,mybbs,0);
        if (!mybbs[0])
        { putf(ms(m_a_unknown),parameter);
          break;
        }
      }

      if(seek_list(fold, 0, ma_seek_list, listname))
      { if((b->optplus&o_c) &&
           (what==ma_subscribe) &&
           !(b->sysop && (strcmp(b->uplink, "Import")!=0)))
        { 
          #ifdef __FLAT__
          putf("Only mailing list maintainers ");
          putf("may add an user to the list %s.\n", listname);
          #else
          putf(monly);
          #endif
          rewind(fold);
          break;
        }

        if((what==ma_adduser) && !b->sysop &&
           (seek_list(fold, 0, ma_seek_user, u->call)!=2))
        { 
          #ifdef __FLAT__
          putf("Only mailing list maintainers may add an user.\n");
          #else
          putf(monly);
          #endif
          rewind(fold);
          break;
        }

        rewind(fold);
        seek_list(fold, f, ma_seek_list, listname);

        if(seek_list(fold, f, ma_seek_user, parameter))
        { if(what==ma_adduser)
            putf("%s is already a user of the mailing list %s.\n",
                 parameter, listname);
          else
            putf("You are already a user of the mailing list %s.\n",
                 listname);
        }
        else
        { fprintf(f, " %s\n", parameter);
          if(what==ma_adduser)
            putf("%s is now a user of the mailing list %s.\n",
                 parameter, listname);
          else
            putf("You are now a user of the mailing list %s.\n",
                 listname);
        }
      }
      else
      { putf(nexis, listname);
        fseek(fold, 0, SEEK_SET);
      }
      break;

    case ma_unsubscribe:
    case ma_deleteuser:
      if(seek_list(fold, 0, ma_seek_list, listname))
      { if((what==ma_adduser) && !b->sysop &&
           (seek_list(fold, 0, ma_seek_user, u->call)!=2))
        { 
          #ifdef __FLAT__
          putf("Only mailing list maintainers may delete an user.\n");
          #else
          putf(monly);
          #endif
          rewind(fold);
          break;
        }

        rewind(fold);
        seek_list(fold, f, ma_seek_list, listname);

        if(seek_list(fold, f, ma_seek_user, parameter))
        { fgets(buffer,sizeof(buffer)-1,fold);
          if(what==ma_deleteuser)
            putf("User %s has been deleted from the mailing list %s.\n",
                 parameter, listname);
          else
            putf("You have been deleted from the mailing list %s.\n",
                 listname);
        }
        else
        { if(what==ma_deleteuser)
            putf("No user %s in the mailing list %s.\n", parameter, listname);
          else
            putf("You aren't a user of the mailing list %s.\n", listname);
        }
      }
      else
      { putf(nexis, listname);
        fseek(fold, 0, SEEK_SET);
      }
      break;

    case ma_addmaintainer:
      if (!strchr(parameter,'@') && mbcallok(parameter))
      { get_mybbs(parameter,mybbs,0);
        if (!mybbs[0])
        { putf(ms(m_a_unknown),parameter);
          break;
        }
      }


      if(seek_list(fold, 0, ma_seek_list, listname))
      { if(!b->sysop && (seek_list(fold, 0, ma_seek_user, u->call)!=2))
        { 
          #ifdef __FLAT__
          putf("Only mailing list maintainers may add another maintainer.\n");
          #else
          putf(monly);
          #endif
          rewind(fold);
          break;
        }

        rewind(fold);
        seek_list(fold, f, ma_seek_list, listname);

        switch(seek_list(fold, f, ma_seek_user, parameter))
        {
        case 1:
          fgets(buffer,sizeof(buffer)-1,fold);
          // fall through

        case 0:
            putf("%s is now a maintainer of the mailing list %s.\n",
                 parameter, listname);
          break;

        case 2:
          putf("%s is already a maintainer of the mailing list %s.\n",
               parameter, listname);
          fgets(buffer,sizeof(buffer)-1,fold);
          break;
        }

          fprintf(f, "  %s\n", parameter);
      }
      else
      { putf(nexis, listname);
        fseek(fold, 0, SEEK_SET);
      }
      break;

    case ma_deletemaintainer:
      if(!b->sysop)
      { 
        #ifdef __FLAT__
        putf("Only mailing list maintainers may delete a maintainer.\n");
        #else    
        putf(monly);
        #endif
        break;
      }

      rewind(fold);
      if(seek_list(fold, f, ma_seek_list, listname))
      { if(seek_list(fold, f, ma_seek_user, parameter)==2)
        { fgets(buffer,sizeof(buffer)-1,fold);
          fprintf(f, " %s\n", parameter);
          putf("User %s is no longer a maintainer of the mailing list %s.\n",
               parameter, listname);
        }
        else
          putf("%s is not a maintainer of the mailing list %s.\n",
               parameter, listname);
      }
      else
      { putf(nexis, listname);
        fseek(fold, 0, SEEK_SET);
      }
      break;

    default:
      trace(serious, "maillistserv", "illegal modify-code");
      break;
  }

  seek_list(fold, f, ma_seek_end, "");
  s_fclose(fold);
  s_fclose(f);
  xunlink(MAILLISTNAME);
  xrename(MAILLISTTMPNAME, MAILLISTNAME);
}

/*---------------------------------------------------------------------------*/

void mailserv::list_mailinglists(void)
//*************************************************************************
//
//  Shows all mailing lists
//
//*************************************************************************
{ FILE *f=s_fopen(MAILLISTNAME, "srt");
  char s[255];
  char *p;
  int userc=0, maintc=0;

  if(f)
  { while(fgets(s,255,f) || (maintc+userc))
    { if(s[0])
        rmcrlf(s);
      if(s[0]!=' ')
      { if(maintc+userc)
          putf("Users: %4d, Maintainers: %4d\n", userc, maintc);
        userc=0;
        maintc=0;
        if((p=strchr(s, ' ')+1)!=(char *)1)
        { while(*p==' ')
            p++;
          *(p-1)=0;
        }
        if(s[0])
        { strupr(s);
          putf("%-10s", s);
        }
        if((p!=(char *)1) && ((p=strchr(p, ' ')+1)!=(char *)1))
        { scanoptions(p);
          p+=blkill(p);
          if(*p!=0)
            putf(" - %s", p);
        }
        if(s[0])
          putf("\n");
      }
      else
      { if(s[1]==' ')
          maintc++;
        else
          userc++;
      }
      s[0]=0;
      waitfor(e_ticsfull);
    }
    s_fclose(f);
  }
  else
    putf("No mailing lists active.\n");
}

/*---------------------------------------------------------------------------*/

void mailserv::show_info(char *name, char longformat)
//*************************************************************************
//
//  Shows users, maintainers and options of a mailing list
//
//*************************************************************************
{ char s[256];
  char format[6];
  char cols;
  char row;
  int userc=0, maintc=0, i;
  char maxlen=6;
  char *userlist;
  char *listname, *number, *options, *description;
  FILE *f=s_fopen(MAILLISTNAME, "srt");

  if(f && seek_list(f, 0, ma_seek_list, name))
  { fgets(s, 255, f);
    rmcrlf(s);

    parse_listheader(s, &listname, &number, &options, &description);

    if(longformat)
    { putf("Name of the mailing-list: %s\n", listname);
      putf("Next mail-number: %s\n", number);
    }

    if(longformat && *options)
    { scanoptions(options);
      if(b->optplus&(o_c|o_m|o_u))
      { putf("Options: ");
        if(b->optplus&o_c) putf("C");
        if(b->optplus&o_m) putf("M");
        if(b->optplus&o_u) putf("U");
        putf("\n");
      }
    }

    if(*description)
      if(longformat)
        putf("Description: %s\n", description);
      else
        putf("Description of the mailing list: %s\n", description);

    while(fgets(s, 50, f))
    { rmcrlf(s);
      if(s[0]!=' ') break;
      if(s[1]==' ')
        maintc++;
      else
      { userc++;
        while(s[0]==' ')
          strcpy(s, s+1);
        if(strlen(s)>maxlen)
          maxlen=strlen(s);
      }
    }

    if(maintc+userc)
    { maxlen+=2; // for the 0-byte and the user/maintainer-byte
      rewind(f);
      seek_list(f, 0, ma_seek_list, name);
      userlist=(char *)t_malloc(maxlen*(userc+maintc), "msvs");
      fgets(s, 255, f);
      for(i=0;i<(userc+maintc);i++)
      { fgets(s, 50, f);
        rmcrlf(s);
        if(s[1]==' ')
          *(userlist+i*maxlen)=0;
        else
          *(userlist+i*maxlen)=1;
        while(s[0]==' ')
          strcpy(s, s+1);
        strcpy(userlist+i*maxlen+1, s);
      }
      qsort(userlist, userc+maintc, maxlen,
            (int(*)(const void *,const void *))strcmp);
    
      cols=79/(maxlen-1);
      sprintf(format, "%%-%ds", maxlen-1);

      if(maintc)
      { putf("Maintainers (%d):\n", maintc);
        row=1;
        for(i=0;i<maintc;i++)
        { if(!(i % 11) && i)
          { putf("\n");
            row++;
            if(!longformat && row==6)
            { putf("(Not all listed)");
              break;
            }
          }
          putf("%-7s", userlist+i*maxlen+1);
        }
        putf("\n");
      }

      if(userc)
      { putf("Users (%d):\n", userc);
        row=1;
        for(i=0;i<userc;i++)
        { if(!(i % cols) && i)
          { putf("\n");
            row++;
            if(!longformat && row==6)
            { putf("(Not all listed)");
              break;
            }
          }
          putf(format, userlist+(i+maintc)*maxlen+1);
        }
        putf("\n");
      }

      t_free(userlist);
    }

    s_fclose(f);
  }
  else
    putf("No such mailing list: %s\n", name);
}

/*---------------------------------------------------------------------------*/

void mailserv::execute(char *command)
//*************************************************************************
//
//  Executes an user-command destined to the mailing
//  list server subsystem
//
//*************************************************************************
{ if(!m.maillistserv)
  { putf("Sorry, mailing list server not enabled.\n");
    return;
  }

  char *befbuf;
  char *para2;

  befbuf=command;

  static char *beftab[]=
    {  "NEWLIST", "+L","DELLIST","-L", 
       "DESCRIPTION", "OPTIONS", 
       "ADDUSER","+U", "DELUSER","-U",
       "ADDMAINTAINER","+M", "DELMAINTAINER", "-M",
       "INFO", "SETNUMBER", "LIST", 
       "SUBSCRIBE","UNSUBSCRIBE",
       "HELP",NULL
    };

  enum befnum
    { unsinn, 
      newlist, newlist_, dellist, dellist_,
      description, options, 
      adduser, adduser_, deluser,deluser_,
      addmaintainer, addmaintainer_, delmaintainer, delmaintainer_,
      info, setnumber, list, 
      subscribe, unsubscribe, 
      help
    }cmd=unsinn;

  cmd=(befnum)readcmd(beftab,&befbuf,0);
  befbuf+=blkill(befbuf);
 
  switch(cmd)
  {
  case newlist:
  case newlist_:
    if(!befbuf[0] || strchr(befbuf,' ') || strlen(befbuf) > 20)
    { putf("Syntax: MAILSERVER +LIST <ListName>\n");
      break;
    }
    strupr(befbuf);
    modify_list(ma_createlist, befbuf,"");
    break;

  case dellist:
  case dellist_:
    if(!befbuf[0])
    { putf("Syntax: MAILSERVER -LIST <ListName>\n");
      break;
    }
    strupr(befbuf);
    modify_list(ma_deletelist, befbuf, "");
    break;

  case description:
    if(!strchr(befbuf, ' '))
    { putf("Syntax: MAILSERVER DESCRIPTION <ListName> <Text>\n");
      break;
    }
    para2=strchr(befbuf, ' ')+1;
    *strchr(befbuf, ' ')=0;
    strupr(befbuf);
    modify_list(ma_setdescription, befbuf, para2);
    break;

  case options:
    if(!befbuf[0])
    { putf("Syntax: MAILSERVER OPTIONS <ListName> [<Options>]\n");
      break;
    }
    strupr(befbuf);
    if(strchr(befbuf, ' '))
    { para2=strchr(befbuf, ' ')+1;
      *strchr(befbuf, ' ')=0;
    }
    else
      para2=&befbuf[strlen(befbuf)]; // empty string
    modify_list(ma_setoptions, befbuf, para2);
    break;

  case adduser:
  case adduser_:
    if(!strchr(befbuf, ' '))
    { putf("Syntax: MAILSERVER +USER <ListName> <Call/Board>\n");
      break;
    }
    strupr(befbuf);
    para2=strchr(befbuf, ' ')+1;
    *strchr(befbuf, ' ')=0;
    modify_list(ma_adduser, befbuf, para2);
    break;

  case deluser:
  case deluser_:
    if(!strchr(befbuf, ' '))
    { putf("Syntax: MAILSERVER -USER <ListName> <Call/Board>\n");
      break;
    }
    strupr(befbuf);
    para2=strchr(befbuf, ' ')+1;
    *strchr(befbuf, ' ')=0;
    modify_list(ma_deleteuser, befbuf, para2);
    break;

  case addmaintainer:
  case addmaintainer_:
    if(!strchr(befbuf, ' '))
    { putf("Syntax: MAILSERVER +MAINTAINER <ListName> <Call/Board>\n");
      break;
    }
    strupr(befbuf);
    para2=strchr(befbuf, ' ')+1;
    *strchr(befbuf, ' ')=0;
    modify_list(ma_addmaintainer, befbuf, para2);
    break;

  case delmaintainer:
  case delmaintainer_:
    if(!strchr(befbuf, ' '))
    { putf("Syntax: MAILSERVER -MAINTAINER <ListName> <Call/Board>\n");
      break;
    }
    strupr(befbuf);
    para2=strchr(befbuf, ' ')+1;
    *strchr(befbuf, ' ')=0;
    modify_list(ma_deletemaintainer, befbuf, para2);
    break;

  case info:
  case list:
    if(befbuf[0])     
    { strupr(befbuf);
      show_info(befbuf,1);
    }
    else
      list_mailinglists();
    break;

  case setnumber:
    if(!strchr(befbuf, ' '))
    { putf("Syntax: MAILSERVER SETNUMBER <ListName> <Number>\n");
      break;
    }
    strupr(befbuf);
    para2=strchr(befbuf, ' ')+1;
    *strchr(befbuf, ' ')=0;
    modify_list(ma_setnumber, befbuf, para2);
    break;

  case subscribe:
    if(!befbuf[0])
    { putf("Syntax: MAILSERVER SUBSCRIBE <ListName>\n");
      break;
    }
    strupr(befbuf);
    modify_list(ma_subscribe, befbuf, u->call);
    break;

  case unsubscribe:
    if(!befbuf[0])
    { putf("Syntax: MAILSERVER UNSUBSCRIBE <ListName>\n");
      break;
    }
    strupr(befbuf);
    modify_list(ma_unsubscribe, befbuf, u->call);
    break;

  case help:
    if (befbuf[0])
      mbreadhelp(befbuf);
    else
      mbreadhelp("mailserver"); 
     //readtext("mahelp");
    break;

  default:
      putf("Syntax: MAILSERVER +L|-L|+U|-U|+M|-M|INFO|SUBSCRIBE|...\n");
      //readtext("mahelp");
    break;
  }
}

/*---------------------------------------------------------------------------*/

void mailserv::examine_mail(void)
//*************************************************************************
//
//  Checks, if a mail is desting the the mailing list server
//  subsystem and processes the mail (called within mbsend.cpp)
//
//*************************************************************************
{ int usercontinous=0;
  char *command;
  char *listname;
  char *newtitle;
  char nexis[]="A mailing list named %s does not exist.\n";
  //char *tempfilename=(char*)t_malloc(FNAMELEN+1,"msvc");
  char tempfilename[FNAMELEN+1];
  FILE *serverfile;
  FILE *mailfile;
  unsigned long int lastrline=0L;
  char distribnum[11];
  char *name, *number, *options, *description;
  char *betreffbuf=(char *)t_malloc(BETREFFLEN+1,"msva");
  char *betreffbak=(char *)t_malloc(BETREFFLEN+1,"msvb");
  //char betreffbuf[BETREFFLEN+1];
  //char betreffbak[BETREFFLEN+1];

  strcpy(betreffbuf, b->betreff);
  strcpy(betreffbak, b->betreff);

  command=betreffbuf;

  listname=strchr(betreffbuf, ' ');
  if(listname)
    *(listname++)=0;
  else
    return;

  newtitle=strchr(listname, ' ');
  if(newtitle)
    *(newtitle++)=0;
  else
    newtitle=command+strlen(command); // Move pointer to a 0-byte

  strupr(command);
  strupr(listname);

  waitfor(e_ticsfull);

  if((strcmp(command, "MAILTO")==0) ||
     (strcmp(command, "SUBSCRIBE")==0) ||
     (strcmp(command, "UNSUBSCRIBE")==0))
  { lastcmd("Mailing list server");
    strcpy(b->uplink, "Mailsv");
    strcpy(t->name, ".");
    b->forwarding=fwd_none;

    markerased('M', 0);
    sprintf(tempfilename,TEMPPATH"/%s", time2filename(0));
    strlwr(tempfilename);
    if((b->outputfile=s_fopen(tempfilename, "swt"))==0)
    { trace(serious,"maillistserv","fopen %s errno=%d",tempfilename,errno);
      return;
    }
    usercontinous=b->continous;
    b->continous=1;
    b->oldoutput=t->output;
    t->output=io_file;

    switch(command[0])
    {
    case 'M':
      if(!(serverfile=s_fopen(MAILLISTNAME, "srt")))
      { putf("SEND %s Re:%s\n", b->herkunft, betreffbak);
        putf("Sorry - there are no mailing lists available at this mailbox.\n");
        break;
      }

      if(!seek_list(serverfile, 0, ma_seek_list, listname))
      { putf("SEND %s Re:%s\n", b->herkunft, betreffbak);
        putf(nexis, listname);
        s_fclose(serverfile);
        break;
      }

      if(b->optplus&o_m)
      { if(seek_list(serverfile, 0, ma_seek_user, b->herkunft)!=2)
        { putf("SEND %s Re:%s\n", b->herkunft, betreffbak);          
          putf("Only maintainers may write a mail to the list %s.\n",
               listname);
          s_fclose(serverfile);
          break;
        }
        else
        { fseek(serverfile, 0, SEEK_SET);
          seek_list(serverfile, 0, ma_seek_list, listname);
        }
      }

      if(b->optplus&o_u)
      { if(!seek_list(serverfile, 0, ma_seek_user, b->herkunft))
        { putf("SEND %s Re:%s\n", b->herkunft, betreffbak);
          putf("Only users may write a mail to the list %s.\n", listname);
          s_fclose(serverfile);
          break;
        }
        else
        { fseek(serverfile, 0, SEEK_SET);
          seek_list(serverfile, 0, ma_seek_list, listname);
        }
      }

      show_info(listname,0);

      putf("\nFor server redistribution write a mail to %s with\n",
           m.boxname);
      putf("title \"MAILTO %s <title of the mail to be distributed>\".\n",
           listname);

      putf("----------------------------------------");
      putf("---------------------------------------\n");

      mailfile=s_fopen(b->mailpath, "srt");

      // Throw away the header and all the R:-lines, but the last one
      do
        fgets(b->line, 255, mailfile);
      while(strncmp(b->line, "R:", 2)!=0);
      do
      { if(strncmp(b->line, "R:", 2)==0)
          lastrline=ftell(mailfile)-strlen(b->line);
        fgets(b->line, 255, mailfile);
      }
      while(strncmp(b->line, "R:", 2)==0);

      fseek(mailfile, lastrline, SEEK_SET);
      readmail(mailfile, 0); // mailfile is closed within readmail()

      s_fclose(b->outputfile);

      fgets(b->line, 250, serverfile);
      parse_listheader(b->line, &name, &number, &options, &description);
      strncpy(distribnum, number, 10);
      distribnum[10]=0;

      strcpy(b->logincall, b->herkunft);

      t->output=io_dummy;

      while(fgets(b->line, 50, serverfile))
      { if(b->line[0]!=' ') // End of mailing list?
          break;
        rmcrlf(b->line);

        b->inputfile=s_fopen(tempfilename, "srt");
        b->oldinput=t->input;
        t->input=io_file;        // Will be restored by mbsend()

        sprintf(b->line + strlen(b->line), " (%s %s) %s",
                                           listname, distribnum, newtitle);
        if(!mbsend(b->line, 0))
        { s_fclose(b->inputfile);
          t->input=(io_t)b->oldinput;
          b->oldinput=io_dummy;
          trace(serious,"mailsv:ex","err %s",b->line);
        }

        waitfor(e_ticsfull);
      }

      s_fclose(serverfile);

      xunlink(tempfilename);
      sprintf(b->line,"%ld",atol(distribnum)+1);
      modify_list(ma_setnumber, listname, b->line);

      // The rest of the function needs this file to be open
      b->outputfile=s_fopen(tempfilename, "swt");
      break;

    case 'S':
    case 'U':
      putf("SEND %s Re:%s\n", b->herkunft, betreffbak);
      if(command[0]=='S')
        modify_list(ma_subscribe, listname, b->herkunft);
      else
        modify_list(ma_unsubscribe, listname, b->herkunft);
      break;
    }

    putf("nnnn\nimpdel\n");
    s_fclose(b->outputfile);
    mbimport(tempfilename);
    t->output=(io_t)b->oldoutput;
    xunlink(tempfilename); 
  }
  b->continous=usercontinous;
}

/*---------------------------------------------------------------------------*/

#endif
