/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  ------------
  NNTP for BCM   Reference: RFC 977, RFC 850
  ------------


  Copyright (C)       Deti Fliegl, deti@lrz.de, DG9MHZ
                      Guardinistr. 47
                      81375 Muenchen

  Alle Rechte vorbehalten / All Rights Reserved

***************************************************************/

#include "baycom.h"
#define MAX_RCPTCNT 100

#ifdef __UNIX__
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#define closesocket close
#endif
#ifdef _WIN32
#include "winsock.h"
#endif

/*---------------------------------------------------------------------------*/

static char *cmdlist[] =
  {"ARTICLE","BODY","GROUP","HEAD","LAST","LIST","NEXT","POST","QUIT",
  "STAT","NEWGROUPS","HELP","IHAVE","NEWNEWS","SLAVE","DATE", "MODE",
  "XOVER",NULL};

char *not_supp="%d not supported\n";

class nntp
{
public:
  void start_nntp (char *name);
private:
  enum
  {
    _ERR = -1, _ARTICLE, _BODY, _GROUP, _HEAD, _LAST, _LIST, _NEXT, _POST, _QUIT,
      _STAT, _NEWGROUPS, _HELP, _IHAVE, _NEWNEWS, _SLAVE, _DATE, _MODE,
      _XOVER
  }
  _cmdlist;
  void charset (char *s);
  int checkcmd (char *cmd, char **cmdlist);
  int extract_argument (char *s, int size);
  void list_groups(void);
  void put_date(void );
  void set_group(char *s);
  void put_head(int mode,char *s);
  void read_file (void);
  char* putreadpath (FILE * f, char *line);

  FILE *f;
  char tmpname[40];
  char buffer[1024];
  char cmd[32];
  char group[256];
  int finish;
  int badcmd;
  int currentmsg;
  int msgcnt;
};

/*---------------------------------------------------------------------------*/

void nntp::charset (char *s)
{ char *result = s;
  for (; result[0]; result++)
  {
    switch (result[0])
    {
    case 0x84:
      result[0] = 0xe4;
      break;                    //ae
      
    case 0x94:
      result[0] = 0xf6;
      break;                    //oe
      
    case 0x81:
      result[0] = 0xfc;
      break;                    //ue
      
    case 0x8e:
      result[0] = 0xc4;
      break;                    //Ae
      
    case 0x99:
      result[0] = 0xd6;
      break;                    //Oe
      
    case 0x9a:
      result[0] = 0xdc;
      break;                    //Ue
      
    case 0xe1:
      result[0] = 0xdf;
      break;                    //ss
      
    }
  }
}

/*---------------------------------------------------------------------------*/

void nntp::list_groups(void)
{ int i;
  handle fh;
  char* group;
  
  putf("215 Newsgroups in form \"group high low flags\".\n");
  for (i=0;i<treelen;i++)
  { 
    if((strlen(tree[i].name)!=1))
    { 
      fh=preparedir(tree[i].name,0);
      msgcnt=0;
      if(fh!=EOF)
      { 
        msgcnt=lseek(fh,0,SEEK_END)/128;
        s_close(fh);
      }
      group=strdup(tree[i].name);
      strlwr (group);
      subst(group,'/','.');
      putf("ampr.bbs.%s %d 1 y\n", group,msgcnt);
      free(group);
    }
  }
  putf(".\n");
  msgcnt=0;
}

/*---------------------------------------------------------------------------*/

void nntp::set_group(char *s)
{ char *t="411 Invalid group name.\n";
  //currentmsg=0;
  if(!s[0])
    putf(t);
  else
  {
    handle fh;
    char *tmp=strdup(s);
    strupr(tmp);
    subst(tmp,'.','/');
    if(finddir(tmp,b->sysop)==0)
      putf(t);
    else
    {
      //strcpy(b->prompt,b->boardfullname);
      fh=preparedir(b->boardfullname,0);
      msgcnt=0;
      if(fh!=EOF)
      { 
        msgcnt=lseek(fh,0,SEEK_END)/128;
        s_close(fh);
      }
      putf("211 %d 1 %d %s\n",msgcnt,msgcnt,s);
      strcpy(group,s);
      if(msgcnt)    
        currentmsg=1;
      else
        currentmsg=0;
    }
    free(tmp);
  }
}

/*---------------------------------------------------------------------------*/

char* nntp::putreadpath (FILE * f, char *line)
{ char *lsave;
  int pathshown = 0, pos = 0;
  while (((lsave = fgets (line, 116, f)) != NULL) && (line[0] == 'R') && (line[1] == ':'))
  {
    char *adr;
    if (b->headermode!=2)
    {
      if(!pathshown)
      {
        putf ("Path: ");
        pathshown++;
      }
      else
        putf("!");
      adr = getheaderadress (line);
      if (adr)
      {
        if (strchr (adr, '.'))
          strchr (adr, '.')[0] = 0;
        putf ("%s", adr);
        pos += strlen (adr);
      }
    }
    strcpy (line + 128, line);
  }
  if(pathshown)
    putf("\n");
  return lsave;
}

/*---------------------------------------------------------------------------*/

void nntp::read_file (void)
//****************************************************************
//
//  gibt ein READ-File zum User aus
//
//****************************************************************
{
  //long time = filename2time (b->mailfname);
  int end;
  long mimeboundary = 0;
  int header = 1;
  int i = 0;

  FILE *f;

  strlwr (b->mailpath);
  f = s_fopen (b->mailpath, "lrt");
  if (f)
  {
    fgets (buffer, 255, f);
    mbsend_parse (buffer, 0);
    formoptions ();
    waitfor (e_ticsfull);
    fgets (buffer, 255, f);
    fgets (buffer, 255, f);
    end = 0;
    while ((end < 252) && (buffer[end] != '.'))
      end++;
    buffer[end] = 0;
    end = strlen (buffer) - 1;
    if (buffer[end] == '\n')
      buffer[end] = 0;
    if (strlen (buffer))
    {
      char *l2 = buffer;
      while (l2[0])
      {
        int i = strlen (l2);
        if (strlen (l2) > 73)
        {
          i = 66;
          while (l2[i] != ' ')
            i++;
          l2[i] = 0;
          i++;
        }
        //if(b->headermode!=2)
        //  putf ("X-Read: %s\n", l2);
        l2 += i;
      }
    }
    fgets (buffer, 100, f);
    end = strlen (buffer) - 1;
    if (buffer[end] == '\n')
      buffer[end] = 0;
    buffer[79] = 0;
    strcpy (b->betreff, buffer);
    if(b->headermode!=2)
      b->opt|= o_p;
    else
      b->opt&= ~o_p;

    if(b->headermode!=2)
    {
      putf("Xref: db0aab-bbs.ampr.org %s:%d\n",group,currentmsg);
      putf("Newsgroups: %s\n",group);
    }
    if (putreadpath (f, buffer))
    {
      mimeboundary = scanmimetype (f);
      if(b->headermode!=2)
      {
        putmimeheader (buffer, mimeboundary);
        putf("References: <%s@%s>\n", b->bid, m.boxname);
      }
    }
    do
    {
      waitfor (e_ticsfull);
      if ( buffer[0]=='#' || mimeboundary <= ftell(f))
        if (testreadbin (f, mimeboundary,0))
        {
          f = NULL;
          break;
        }
      if (!strcmp (buffer, ".\n"))
      {
        memmove (buffer + 1, buffer, sizeof (buffer) - 1);
        buffer[0] = '.';
      }
      charset (buffer);
      if (header)
      {
        if (!strcmp (buffer, "\n"))
          continue;

        if (!strncasecmp (buffer, "from: ", 6))
            continue;

        if (!strncasecmp (buffer, "to: ", 4) || !strncasecmp (buffer, "to  : ", 4))
            continue;

        if (!strncasecmp (buffer, "reply-to: ", 10))
        {
          char *s = strchr (buffer + 10, ' ');
          if (s && s[1] == '@' && s[2] == ' ')
          {
            memmove (s, s + 1, strlen (s));
            memmove (s + 1, s + 2, strlen (s));
          }
        }

        if (!strncasecmp (buffer, "message-id: ", 12))
            continue;

        if (!strstr (buffer, ": ") && !strstr (buffer, "Transfer"))
        {
          header = 0;
          if(b->headermode<2)
            goto raus;
          putf ("\n");
        }
      }
      if(b->headermode!=2||!header)
        for (i = 0; buffer[i]; i++)
          putv (buffer[i]);
    }while (fgets(buffer,sizeof(buffer)-1,f));
    raus:;
    if (f)
      s_fclose (f);
    putf (".\n");
    waitfor (e_ticsfull);
  }
  else
    putf ("can't open: %s\n", b->mailpath);
}

/*---------------------------------------------------------------------------*/

void nntp::put_head(int mode,char *s)
{ char* t=strchr(s,'<');
  char* u=t;
  char* msg=NULL;
  int msgnum=atoi(s);

  if(!group[0])
  {
    putf("412 no newsgroup has been selected\n");
    return;
  }
  if(!s[0])
    msgnum=currentmsg;

  if(t && strchr(t,'>') && 
    (u=strchr(t,'@'))
    )
  {
    u[0]=0;
    msg=bidvorhanden(t+1);
    if(!msg)
      goto inv;
    
    handle fh=preparedir(b->boardfullname,0);
    if (fh != EOF)
    {
      char ok;
      seek_fname(fh,strchr(msg,'/')+1,&ok);

      if(ok)
        //msgnum=(tell(fh)/128)+1;
        msgnum=(lseek(fh,0,SEEK_CUR)>>7)+1;
      else
        msgnum=0;

      s_close(fh);
    }
  }
  
  if(msgnum)
  {
      if(msgnum<0 || (msgnum>msgcnt))
        goto inv;

      handle fh=preparedir(b->boardfullname,0);
      if (fh != EOF)
      {
        lseek (fh, ((long) (msgnum - 1)) << 7, SEEK_SET);
        _read (fh, buffer, 128);
        dirline_data(buffer);
        strcpy (b->mailfname, buffer);
        strcpy (b->mailpath, b->mask);
        strcpy (strstr (b->mailpath, "*.*"), b->mailfname);
        s_close(fh);
        currentmsg=msgnum;
      }
  }
  else
  {
    inv:putf("430 no such article found\n");
    return;
  }
  b->headermode=mode;
  switch(mode)
  {
    case 0: 
      putf("223 %d <%s@%s> article retrieved - request text separately\n",msgnum,b->bid,m.boxname);
      return;
    case 1: 
      putf("221 %d <%s@%s> article retrieved - head follows\n",msgnum,b->bid,m.boxname);
      break;
    case 2: 
      putf("222 %d <%s@%s> article retrieved - body follows\n",msgnum,b->bid,m.boxname);
      break;
    case 3: 
      putf("220 %d <%s@%s> article retrieved - head and body follows\n",msgnum,b->bid,m.boxname);
      break;
  }
  read_file();
}

/*---------------------------------------------------------------------------*/

void nntp::put_date(void)
{
  time_t t=ad_time();
  struct tm* ti=localtime(&t);
  putf("111 %04d%02d%02d%02d%02d%02d\n",ti->tm_year+1900,ti->tm_mon+1,ti->tm_mday,
    ti->tm_hour,ti->tm_min,ti->tm_sec);
}

/*---------------------------------------------------------------------------*/

int nntp::checkcmd (char *cmd, char **cmdlist)
{
  int i = atoi (cmd);
  if (i >= 200 && i < 600)
    return i;
  i = 0;
  while (cmdlist[i] != NULL)
  {
    if (!stricmp (cmd, cmdlist[i]))
    {
      badcmd = 0;
      return i;
    }
    i++;
  }
  return _ERR;
}

/*---------------------------------------------------------------------------*/

int nntp::extract_argument (char *s, int size)
{
  char help[256];
  char *t = strstr (buffer, cmd) + strlen (cmd);
  sprintf (help, "%%%ds", size);
  s[0] = 0;
  sscanf (t, help, s);
  //  putf("[%s] [%s]\n",t,s);
  if (s[0])
    return 1;
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

void nntp::start_nntp (char *name)
{
  finish = 0;
  badcmd = 0;
  currentmsg =0;
  msgcnt=0;
  group[0]=0;
  
  if (myhostname (buffer, sizeof (buffer) - 1) == -1)
    return;
  
  mblogin(m.boxname,login_silent,"NNTP");
  
  putf ("201 %s BayCom Mailbox V" VNUMMER " NNTP Server\n", buffer);
  
  while (!runterfahren)
  {
    getline (buffer, sizeof (buffer) - 1, 1);
    printf ("%s\n", buffer);
    if (buffer[0])
    {
      sscanf (buffer, "%31s", cmd);
      switch (checkcmd (cmd, cmdlist))
      {
      case _LAST:
        if(currentmsg>2)
        {
          currentmsg--;
          put_head(0,"");
        }
        else
          putf("422 No previous article to retrieve.\n");
        break;
      case _NEXT:
        if(currentmsg<msgcnt)
        {
          currentmsg++;
          put_head(0,"");
        }
        else
          putf("421 No next article to retrieve.\n");
        break;
      case _STAT:
        extract_argument(tmpname,sizeof(tmpname));
        put_head(0,tmpname);
        break;
      case _HEAD:
        extract_argument(tmpname,sizeof(tmpname));
        put_head(1,tmpname);
        break;
      case _BODY:
        extract_argument(tmpname,sizeof(tmpname));
        put_head(2,tmpname);
        break;
      case _ARTICLE:
        extract_argument(tmpname,sizeof(tmpname));
        put_head(3,tmpname);
        break;
      case _XOVER:
      /*
      extract_argument(tmpname,sizeof(tmpname));
      put_head(4,tmpname);
        */
        putf(not_supp,500);
        break;
      case _GROUP:
        extract_argument(tmpname,sizeof(tmpname));
        set_group(tmpname);
        break;
      case _LIST: 
        if(!extract_argument(tmpname,sizeof(tmpname)))
          list_groups();
        else
          putf(not_supp,501);
        break;
      case _POST:
        putf(not_supp,501);
        break;
      case _QUIT:
        finish = 1;
        putf ("205 nntp server signing off\n");
        break;
      case _NEWGROUPS:
        putf(not_supp,501);
        break;
      case _HELP:
        putf("100 BayBox NNTP Server.\n.\n");
        break;
      case _IHAVE:
        putf(not_supp,501);
        break;
      case _NEWNEWS:
        putf(not_supp,501);
        break;
      case _SLAVE:
        putf(not_supp,202);
        break;
      case _DATE:
        put_date();
        break;
      case _MODE:
        if(!extract_argument(tmpname,sizeof(tmpname)))
          putf(not_supp,501);
        else
          if(stricmp(tmpname,"READER"))
            putf(not_supp,501);
          else
            putf("201 ok\n");
          break;
      default:
        if (badcmd > 30)
        {
          putf ("205 Too many bad commands.\n");
          finish = 1;
        }
        putf ("500 Unknown command: \"%s\".\n", cmd);
        badcmd++;
      }
    }
    if (finish)
      break;
  }
}

/*---------------------------------------------------------------------------*/

void mblogin_nntp (char *name)
{ nntp p;
  strcpy (b->uplink, "nntp");
  strcpy (b->peerip, name);
  b->logincall[0]=0;
  b->headermode=_NNTP;
  p.start_nntp (name);
  putflush_sock ();
  disconnect_sock ();
}

/*---------------------------------------------------------------------------*/
