/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  ------------
  POP3 for BCM   Reference: RFC 1725
  ------------


  Copyright (C)       Deti Fliegl, deti@lrz.de, DG9MHZ
                      Guardinistr. 47
                      81375 Muenchen

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980505 DG9MHZ added pop3db
//19980506 OE3DZW changes to pop3db
//19980619 OE3DZW show_pop - added fclose

#include "baycom.h"
static char *cmdlist[] =
{"USER", "PASS", "QUIT", "STAT", "LIST", 
 "RETR", "DELE", "NOOP", "RSET", "UIDL", NULL};

/*---------------------------------------------------------------------------*/

void pop3::onkill(void)
{ if (login)
    unlink (lockfile);
}

/*---------------------------------------------------------------------------*/

static void onkill(void *p)
{ ((pop3 *)p)->onkill();
}

/*---------------------------------------------------------------------------*/

void pop3::add_to_popdb(void)
{ char peeraddr[32]="";
  char name[32];
  time_t lastpop;
  time_t now=ad_time();
  char added=FALSE;
  f=s_fopen(POPDBNAME,"srt");
  FILE *g=s_fopen(POPDBTMPNAME,"swt");
  if(f)
  {
    while(!feof(f))
    {
      fscanf(f,"%31s\t%08lX\t%31s\n",peeraddr,&lastpop,name);
      if(!strcmp(peeraddr,b->peerip) && !stricmp(user,name))
      {
        lastpop=now;
        added=TRUE;
      }
      if ((now-lastpop) < 60*60*24*365  &&  //keep data max. 1 year
          (lastpop-now) < 60*60 )           //max 1h 
        fprintf(g,"%s\t%08lX\t%s\n",peeraddr,lastpop,name);
    }
     s_fclose(f);
  }
  if(!added)
    fprintf(g,"%-15.15s\t%08lX\t%s\n",b->peerip,now,user);
  s_fclose(g);
  xunlink(POPDBNAME);
  xrename(POPDBTMPNAME,POPDBNAME);
}

/*---------------------------------------------------------------------------*/

void pop3::charset (char *s)
{ char *result = s;
  for (; result[0]; result++)
  {
    switch (result[0])
    {
    case 0x84:
      result[0] = 0xe4;
      break;                    //ae

    case 0x94:
      result[0] = 0xf6;
      break;                    //oe

    case 0x81:
      result[0] = 0xfc;
      break;                    //ue

    case 0x8e:
      result[0] = 0xc4;
      break;                    //Ae

    case 0x99:
      result[0] = 0xd6;
      break;                    //Oe

    case 0x9a:
      result[0] = 0xdc;
      break;                    //Ue

    case 0xe1:
      result[0] = 0xdf;
      break;                    //ss

    }
  }
}

/*---------------------------------------------------------------------------*/

int pop3::retr_msg (int msg)
{
  int ret = 0;
  int header = 1;
  //      int emptyline=0;
  //      int begin=1;
  int from = 0;
  int to = 0;
  int msgid = 0;
  int repl = 0;
  int i = 0;

  msg = search_msg (msg);
  if (msg)
  {
    b->job = j_read;
    b->optgroup = o_ur;
    u->opt[b->optgroup] = o_w | o_r;

    b->oldoutput = (char)t->output;
    sprintf(tmpname, TEMPPATH"/%s.imp",strlwr (time2filename (0)));
    t->output = io_file;
    b->outputfile = s_fopen (tmpname, "swb");
    if (b->outputfile == NULL)
    {
      putf ("-ERR Sorry, I could not open a tmpfile.\n");
      return 0;
    }

    sprintf (buffer, "%s %d", user, msg);
    ret = mbread (buffer, 3);

    s_fclose (b->outputfile);

    t->output = (io_t) b->oldoutput;
  }
  if (ret)
  {
    f = s_fopen (tmpname, "srb");
    if (f == NULL)
    {
      putf ("-ERR Sorry, I lost my tmpfile... shit happens.\n");
      unlink (tmpname);
      return 0;
    }
    putf ("+OK %d octets\n", filelength (fileno (f)));

    while (fgets (buffer, sizeof (buffer) - 10, f) != NULL)
    {
      //printf("%s",buffer);
      if (!strcmp (buffer, ".\n"))
      {
        //printf("stuffed a period\n");
        memmove (buffer + 1, buffer, sizeof (buffer) - 1);
        buffer[0] = '.';
      }
      charset (buffer);
      if (header)
      {
        if (!strcmp (buffer, "\n"))
        {
          /*if(!begin)
             emptyline++;
             putf("emptyline %d\n",emptyline);
           */
          continue;
        }
        //begin=0;

        if (!strncasecmp (buffer, "from: ", 6))
        {
          if (from)
            continue;
          from++;
        }

        if (!strncasecmp (buffer, "to: ", 4) || !strncasecmp (buffer, "to  : ", 4))
        {
          if (to)
            continue;
          to++;
        }

        if (!strncasecmp (buffer, "reply-to: ", 10))
        {
          if (repl)
            continue;

          char *s = strchr (buffer + 10, ' ');
          if (s && s[1] == '@' && s[2] == ' ')
          {
            memmove (s, s + 1, strlen (s));
            memmove (s + 1, s + 2, strlen (s));
          }
          repl++;
        }

        if (!strncasecmp (buffer, "message-id: ", 12))
        {
          if (msgid)
            continue;
          msgid++;
        }

        if (!strstr (buffer, ": ") && !strstr (buffer, "Transfer"))
        {
          header = 0;
          //putf("\n--eof--\n");
          putf ("\n");
        }
      }
      for (i = 0; buffer[i]; i++)
        putv (buffer[i]);
    }
    s_fclose (f);
    putf ("\n.\n");
  }
  else
    putf ("-ERR no such message.\n");  //.\n");

  unlink (tmpname);
  return 1;
}

/*---------------------------------------------------------------------------*/

int pop3::search_msg (int msg)
{ int dummy;
  int msgcnt = 0;
  int msgnum = 0;

  f = s_fopen (lockfile, "srt");
  if (f == NULL)
    return 0;

  while (fscanf (f, "%d %d %s", &msgnum, &dummy, buffer) == 3)
  {
    msgcnt++;
    if (msg == msgcnt)
      break;
  }
  s_fclose (f);
  if (msg == msgcnt)
    return msgnum;
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

int pop3::put_uidl (int msg)
{ int dummy;
  int msgcnt = 0;

  f = s_fopen (lockfile, "srt");
  if (f == NULL)
  {
    putf ("-ERR Sorry, I lost my lockfile... shit happens.\n");
    return 0;
  }
  putf ("+OK ");

  while (fscanf (f, "%d %d %s", &dummy, &dummy, buffer) == 3)
  {
    msgcnt++;
    if (!msg)
      putf ("\n");
    if (!msg || msg == msgcnt)
      putf ("%d %s", msgcnt, buffer);
  }
  s_fclose (f);
  putv ('\n');
  if (!msg)
    putf (".\n");
  return 1;
}

/*---------------------------------------------------------------------------*/

// messages normally are to be deleted in update state (which is entered
// right after the quit command) we wont do this here.
int pop3::dele_msg (int msg)
{
  int ret = 0;
  msg = search_msg (msg);
  if (msg)
  {
    b->oldoutput = (char) t->output;
    t->output = io_dummy;
    sprintf (buffer, "%s %d", user, msg);
    ret = mbchange (buffer, w_erase, 1);
    t->output = (io_t) b->oldoutput;
  }
  if (ret)
    putf ("+OK message deleted\n");
  else
    putf ("-ERR message not deleted\n");
  return ret;

}

/*---------------------------------------------------------------------------*/

int pop3::init_list (void)
{ b->job = j_search;
  b->optgroup = o_ud;
  u->opt[b->optgroup] = o_i | o_h | o_b;  // | o_v;

  b->oldoutput = (char)t->output;
  t->output = io_file;
  b->outputfile = s_fopen (lockfile, "swt");
  if (b->outputfile == NULL)
  {
    putf ("-ERR Sorry, I could not open a lockfile.\n");
    return 0;
  }
  mbdir (user, 0);
  s_fclose (b->outputfile);
  t->output = (io_t) b->oldoutput;
  return put_list (-1);
}

/*---------------------------------------------------------------------------*/

int pop3::put_list (int msg)
{ FILE *f;
  int bytes;
  int rdmsg = 0;

  f = s_fopen (lockfile, "srt");

  if (f == NULL)
  {
    putf ("-ERR Sorry, I have lost my lockfile... shit happens.\n");
    return 0;
  }

  if (!msg)
    putf ("+OK %d messages (%d octets)\n", msgcnt, bytecnt);

  bytecnt = 0;
  msgcnt = 0;
  while (fscanf (f, "%d %d %32s", &rdmsg, &bytes, buffer) == 3)
  {
    bytecnt += bytes;
    msgcnt++;
    if (!msg)
      putf ("%d %d\n", msgcnt, bytes);
 
    if (msg && msg == msgcnt)
      break;
  }
  s_fclose (f);

  if(msg==-1)
    return 1;

  if (!msg)
    putf (".\n");
  else if (msg == msgcnt)
    putf ("+OK %d %d\n", msgcnt, bytes);
  else
    putf ("-ERR no such message\n");

  return 1;
}

/*---------------------------------------------------------------------------*/

int pop3::checkcmd (char *cmd, char **cmdlist)
{ int i = atoi (cmd);
  if (i >= 200 && i < 600)
    return i;
  i = 0;
  while (cmdlist[i] != NULL)
  {
    if (!stricmp (cmd, cmdlist[i]))
    {
      if (!login && (i != _USER) && (i != _PASS) && (i != _QUIT))
        return _ERR;
      badcmd = 0;
      return i;
    }
    i++;
  }
  return _ERR;
}

/*---------------------------------------------------------------------------*/

int pop3::extract_argument (char *s, int size)
{ char help[256];
  char *t = strstr (buffer, cmd) + strlen (cmd);
  sprintf (help, "%%%ds", size);
  s[0] = 0;
  sscanf (t, help, s);
//  putf("[%s] [%s]\n",t,s);
  if (s[0])
    return 1;
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

void pop3::start_pop3 (char *name)
{ finish = 0;
  badcmd = 0;
  msgcnt = 0;
  user[0] = 0;
  pass[0] = 0;
  lockfile[0] = 0;
  msg = 0;
  login = 0;
  waslocked = 0;
  atkill(::onkill,(void *)this);
 
  if (myhostname (buffer, sizeof (buffer) - 1) == -1)
    return;
  putf ("+OK %s BayCom Mailbox V" VNUMMER " POP3 Server\n", buffer);
  
  while (!runterfahren)
  { getline (buffer, sizeof (buffer) - 1, 1);
    //printf ("%s\n", buffer);
    if (buffer[0])
    {
      sscanf (buffer, "%31s", cmd);
      switch (checkcmd (cmd, cmdlist))
      {
      case _USER:
        if (user[0])
          goto err;
        if (!extract_argument (user, sizeof (user) - 1))
          putf ("-ERR Too few arguments for the user command.\n");
        else if (mbcallok (user))
          putf ("+OK Password required for %s\n", user);
        else
        {
          putf ("-ERR %s not known here.\n", user);
          user[0] = 0;
        }
        break;
      case _PASS:
        if (!pass[0] && user[0])
        {
          if (!extract_argument (pass, sizeof (pass) - 1))
            putf ("-ERR Too few arguments for the pass command.\n");
          else
          {
            char *s;
            if (isamprnet (b->peerip) > 0)
              s = get_httppw (user);
            else
              s = get_ttypw (user, buffer);

            if (!s[0] || stricmp (s, pass))
            {
              trace (report, "pop3", "Authorization for %s failed [%s]", user, pass);
              putf ("-ERR Authorization failed.\n");
              user[0] = 0;
              pass[0] = 0;
              break;
            }
            sprintf (lockfile, TEMPPATH"/lock.%s", user);
            if (!waslocked && !access (lockfile, 0))
            {
              putf ("-ERR Maildrop allready locked.\n");
              waslocked++;
              user[0] = 0;
              pass[0] = 0;
              break;
            }
            add_to_popdb();
            login = 1;
            mblogin (user, login_silent, "POP3");
            b->continous = 1;
            b->sysop = 1;
            b->oldoutput = (char)t->output;
            t->output = io_dummy;
            t->output = (io_t) b->oldoutput;
            init_list ();
            goto rset;
          }
        }
        else
          goto err;
        break;
      case _QUIT:
        finish = 1;
        putf ("+OK POP3 server signing off\n");
        break;
      case _STAT:
        putf ("+OK %d %d\n", msgcnt, bytecnt);
        break;
      case _LIST:
        extract_argument (tmpname, sizeof (tmpname) - 1);
        msg = atoi (tmpname);
        put_list (msg);
        break;
      case _RETR:
        if (!extract_argument (tmpname, sizeof (tmpname) - 1))
          putf ("-ERR Too few arguments for the retr command.\n");
        else
        {
          msg = atoi (tmpname);
          retr_msg (msg);
        }
        break;
      case _DELE:
        if (!extract_argument (tmpname, sizeof (tmpname) - 1))
          putf ("-ERR Too few arguments for the dele command.\n");
        else
        {
          msg = atoi (tmpname);
          dele_msg (msg);
        }
        break;
      case _NOOP:
        putf ("+OK\n");
        break;
      case _RSET:
      rset:putf ("+OK %s maildrop has %d messages (%d octets).\n", user, msgcnt, bytecnt);
        break;
      case _UIDL:
        if (!extract_argument (tmpname, sizeof (tmpname) - 1))
          msg = 0;
        else
          msg = atoi (tmpname);
        put_uidl (msg);
        break;
      default:
      err:
        if (badcmd > 30)
        {
          putf ("-ERR Too many bad commands.\n");
          finish = 1;
        }
        putf ("-ERR Unknown command: \"%s\".\n", cmd);
        badcmd++;
      }
    }
    if (finish)
      break;
  }
  
  if (login)
    unlink (lockfile);
}

/*---------------------------------------------------------------------------*/

void pop3::show_popdb (void)
{ char peeraddr[32]="";
  char name[32];
  time_t lastpop;
  int kopf=FALSE;
  FILE *f=s_fopen(POPDBNAME,"lrt");

  if (f)
  { while (!feof(f))
    { fscanf(f,"%31s\t%08lX\t%31s\n",peeraddr,&lastpop,name);
      if (!kopf)
      { kopf = TRUE;
        putf("Peer IP         Call     Time\n");
      }
      putf("%-15.15s %-8.8s %s\n",peeraddr,name,rfc_date(lastpop));
    }
    s_fclose(f);
  }
  if (!kopf)
    putf("No records found.\n");
}

/*---------------------------------------------------------------------------*/

void mblogin_pop3 (char *name)
{ pop3 p;
  strcpy (b->uplink, "POP3");
  strcpy (b->peerip, name);
  b->logincall[0]=0;
  b->headermode=_POP;
  p.start_pop3 (name);
  putflush_sock ();
  disconnect_sock ();
}

/*---------------------------------------------------------------------------*/
