/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  ------------------------
  READ und ERASE von Mails
  ------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980118 OE3DZW corrected binbug again, should be ok now..
//                time without sec in read-output
//19980119 DG9MHZ ifdef was set wrong for Linux, fixed
//19980120 OE3DZW ifdef was set wrong for DOS, fixed
//19980121 DG9MHZ new string-compare at scanmimetype
//19980125 OE3DZW Debugging wrong time in read-header -> read opt j
//                puts filename into that header
//19980126 OE3DZW fixed string compare at scanmimetype again
//                MIME is only recognised if the is only 1 empty line
//19980202 OE3DZW another fix to scanmimetype, tested now ;-)
//19980215 OE3DZW cosmetic changes to mbchange
//19980224 OE3DZW added Read (instead of X-Read), sneefers need it...
//19980302 OE3DZW renamed change_addess(), rub out old BIDs in header
//19980304 OE3DZW on comment "@" only if b->at is known
//19980307 OE3DZW added opt_h for head/kopf (long form of r-line-header)
//19980404 hrx    fix - mails to sysop will be marked as read,too
//19980407 OE3DZW bytes corrected
//19980408 OE3DZW r -5 should work now
//19980307 OE3DZW removed opt_h for head/kopf (long form of r-line-header)
//19980903 OE3DZW fix: comment to local mails caused invalid addr
//19980924 OE3DZW cleaned up code - removed markreply 
//19990427 OE3DZW added eraseloglevel for WP-erase
//19990808 DH3MB  Added statistics after AutoBIN-TX
//                Now calculating crcthp while sending AutoBIN-Mail

#include "baycom.h"

/*---------------------------------------------------------------------------*/

#ifdef __FLAT__

char *mimestr(char *s)
{ if (stristr(s, ".jpg"))
    return MIME_JPG;

  if (stristr(s, ".gif"))
    return MIME_GIF;

  if (stristr(s, ".wav"))
    return MIME_WAV;

  if (stristr(s, ".txt"))
    return MIME_TXT;

  return MIME_BIN;
}

/*---------------------------------------------------------------------------*/

long scanmimetype(FILE * f)
{ char buffer[257];
  long oldpos = ftell(f);
  long ret = 0;
  long emptyline = 0;

  while (fgets(buffer, sizeof(buffer) - 1, f) != NULL)
  {
    if (buffer[0] == '\n')
      emptyline++; // it's no longer mime when beeing transfered etc.

    if (strstr(buffer, "X-Info: Received") &&
	strstr(buffer, "SMTP-gate") && (emptyline <= 1))
    { ret = -1;
      break;
    }

    if (!strncasecmp(buffer, "#bin#", 5))
    { ret = ftell(f);
      break;
    }

    if (!strncmp(buffer, START7PFLAG, 8))
    { ret = ftell(f) - strlen(buffer);
      break;
    }

  }
  fseek(f, oldpos, SEEK_SET);
  return ret;
}

#endif

/*---------------------------------------------------------------------------*/

static int owner(void)
//*************************************************************************
//
//  Stellt fest, ob eine Nachricht geloescht werden darf
//
//*************************************************************************
{ if (b->sysop || b->forwarding == fwd_standard)
    return 1;

  if (strcmp(b->ziel, b->logincall) == 0)
    return 1;

  if (strcmp(b->herkunft, b->logincall) == 0)
    return 1;

  return 0;
}

/*---------------------------------------------------------------------------*/

static void putreadheader(long mtime)
//*************************************************************************
//
//  Gibt die erste Zeile beim READ aus
//
//*************************************************************************
{ unsigned days_old = (unsigned) ((ad_time() - mtime) / 86400L);
  if (b->opt & o_a)
    putf("%-6s ", b->herkunft);
  if (b->opt & o_e)
    putf("> %-8s ", b->ziel);
  if (b->opt & o_d)
  { if (u->dirformat)
      putf("%s ", datestr(mtime, 2));
    else
      putf("%s ", datestr(mtime, !!(b->opt & o_y)));
  }
  if (b->opt & o_j)
    putf("%s ", b->mailfname);
  if (b->opt & o_t)
    putf("%s ", timestr(mtime, 0));
  if (b->opt & o_z)
    putf("%u Lines ", b->lines);
  if (b->opt & o_b)
  { if (b->bytes < 1000000L)
      putf("%lu Bytes ", b->bytes);
    else
      putf("%lu kBytes ", b->bytes / 1024L);
  }
  if (b->opt & o_l)
  { if (b->lifetime == 999)
      putf("#999 ");
    else if (b->lifetime)
      putf("#%d ", b->lifetime - days_old);
    else
      putf("#%d ", b->boardlife_max - days_old);
  }
  if (b->opt & o_f)
    putf("%s", b->frombox);
  if (b->opt & o_m)
    putf("@%s", b->at);
  putf("\n");
  if (b->opt & o_i)
//oe3dzw: Unterscheidung BID/MID
    if (b->bid[0])
    { if (b->mailtype == 'B')
	putf("BID : %s\n", b->bid);
      else
	putf("MID : %s\n", b->bid);
    }
}

/*---------------------------------------------------------------------------*/

#ifdef __FLAT__

void putmimeaddress(char *s, char *t)
{ if (s[0] && t[0])
    putf("<%s@%s>\n", s, t);
  else
    putf("<%s>\n", s);
}

/*---------------------------------------------------------------------------*/

void putmimeheader(char *line, long mimeboundary)
{ char *s = getheaderadress(line + 128);
  strlwr(s);

  if (b->bid[0])
    putf("Message-ID: <%s@%s>\n", b->bid, m.boxname);
  if (mimeboundary != -1)
  { putf("From: ");
    putmimeaddress(b->herkunft, s);
    putf("To: ");
    putmimeaddress(b->ziel, b->at);
    putf("Date: %s\n", rfc_date(getheadertime(line + 128)));
    putf("Subject: %s\n", b->betreff);
  }

  if (mimeboundary > 0)
  { putf("MIME-Version: 1.0\n");
    putf("Content-Type: multipart/mixed; boundary=\"----%08X\"\n\n", mimeboundary);
    putf("This is a multi-part message in MIME format.\n------%08X\n", mimeboundary);
    putf("Content-Type: text/plain; charset=us-ascii\n");
    putf("Content-Transfer-Encoding: 7bit\n");
  }

  //putf ("Content-Type: text/plain; charset=iso-8859-1\n");
}

#endif

/*---------------------------------------------------------------------------*/

char *putreadpath(FILE * f, char *line, int kopf)
//*************************************************************************
//
//  analysiert die R:-Zeilen und gibt sie ggf. aus
//
//*************************************************************************
{ char *lsave;
  int pathshown = 0, pos = 0;
#ifdef __FLAT__
  char next[128];
  int cnt = 0;
#endif

  if (kopf == 3)
  { strlwr(b->herkunft);
    strlwr(b->ziel);
    strlwr(b->at);
    //myhostname(last,sizeof(last));
  }

  while (((lsave = fgets(line, 116, f)) != NULL) && (line[0] == 'R') && (line[1] == ':'))
  {
#ifdef __FLAT__
    if (kopf == 3)
    { if (cnt)
      { strcpy(next, getheaderadress(line + 128));
	char *adr = getheaderadress(line);
	char *info = strstr(line + 128, next) + strlen(next) + 1;
	char *s = strchr(info, '\n');
	char minibuff[80];

	if (s)
	  s[0] = 0;

	if (info[0])
	{
	  sprintf(minibuff, "with %s", info);
	  s = minibuff;
	}
	else
	  s = "";

	putf("Received: from %s by %s%s; %s\n", adr, next, s,
	     rfc_date(getheadertime(line + 128)));
      }
      cnt++;
    }
    else
#endif
    if (b->opt & o_h)
    { putf("%s", line);
      pathshown++;
    }
    else if ((b->opt & o_p) && kopf != 2)
    { char *adr;
      if (!pathshown)
      { putf("Path: !");
        pathshown++;
      }
      if ((pos++) > 63)
      { putf("\n      !");
	pos = 0;
      }
      // R:920503/1216z @DB0AAB.DB0PV.DEU.EU [Muenchen-Nord]
      adr = getheaderadress(line);
      if (adr)
      { if (strchr(adr, '.'))
          strchr(adr, '.')[0] = 0;
	putf("%s", adr);
	pos += strlen(adr);
      }
      putf("!");
    }
    strcpy(line + 128, line);
  }

  if (pathshown)
  { if ((b->opt & o_p) && !(b->opt & o_h) && kopf != 2)
      putf("\n");
    if ((b->opt & o_u) && !(b->opt & o_h))
    { int l = strlen(line + 130) + 129;
      if (line[l] == LF)
	line[l] = 0;
      if (kopf < 2)
	putf("Sent: %0.73s\n", line + 130);
    }
  }
  return lsave;
}

/*---------------------------------------------------------------------------*/

static void eraselog(char reason, char *eraser)
{ char buf[120];
  handle fh;

  if (reason == ' ')
    reason = 'U';
  if ((reason == 'E') && b->sysop)
    reason = 'S';

  switch (reason)
  {
  case 'E':
    if ((m.eraselog & 4) == 0)
      return;
    break;
 
 case 'F':
    if ((m.eraselog & 8) == 0)
      return;
    break;

  case 'K':
  case 'X':
  case 'L':
    if ((m.eraselog & 1) == 0)
      return;
    break;

  case 'S':
    if ((m.eraselog & 2) == 0)
      return;
    break;

  case 'T':
    if ((m.eraselog & 32) == 0)
      return;
    break;

  case 'U':
    if ((m.eraselog & 16) == 0)
      return;

  case 'W':
    if ((m.eraselog & 64) == 0)
      return;
    break;
  }

  waitfor(e_ticsfull);
  fh = s_open(TRACEPATH "/" ERASELOGNAME, "sab");
  if (fh == EOF)
    fh = s_open(TRACEPATH "/" ERASELOGNAME, "swb");
  if (fh != EOF)
  { sprintf(buf, "%s %6s: #%c %6s>%-8s %s %.40s",
	    zeitstring(ad_time(), 0), eraser, reason, b->herkunft,
	    b->ziel, b->bid, b->betreff);
    buf[79] = 0;
    strcat(buf, "\r\n");
    _write(fh, buf, strlen(buf));
    s_close(fh);
  }
  waitfor(e_ticsfull);
}

/*---------------------------------------------------------------------------*/

int markerased(char reason, int unerase)
//*************************************************************************
//
//  Vermerkt in einer Nachricht, dass sie geloescht ist
//
//*************************************************************************
{ int len;
  char found[1];
  handle fh;
  FILE *f;
  int i;
  char buf[60];
  char eraser[7];
  int doerase = 1;
  int remerase = 0;
  long fwdpos;

  if (reason == 'L')
    strcpy(eraser, b->herkunft);
  else
    strcpy(eraser, b->logincall);

  if (b->sysop && (reason == 'K') && (strcmp(b->herkunft, b->logincall) != 0))
  { putf("\007Warning: Don't erase in foreign boxes without a strict reason!\n");
    if (janein("Do you really want to erase this message globally") != JA)
      return 0;
  }
  strlwr(b->mailpath);
  f = s_fopen(b->mailpath, "sr+t");
  if (f)
  { fgets(b->line, 255, f);	// Headerzeile wegmachen

    mbsend_parse(b->line, 0);
    if (m.eraselog || reason == 'L')
    { fgets(b->line, 255, f);	// Forwardzeile einlesen

      if (b->line[0] == '*' && !unerase)
      { s_fclose(f);
	return 0;
      }
      fgets(b->line, 255, f);	// READ-Zeile einlesen

      fgets(b->line, 80, f);	// Betreff-Zeile einlesen

      if (b->line[0])
	b->line[strlen(b->line) - 1] = 0;	// newline entfernen

      strcpy(b->betreff, b->line);
      if (reason == 'L')
      {	if (strstr(b->bid, eraser) == NULL)
	{ if (m.remerase == 2)
	    doerase = 0;
	  reason = 'X';
	}
	if (m.remerase == 0)
	  doerase = 0;
      }
      eraselog(reason, eraser);
      if (doerase == 0)
      { s_fclose(f);
	return 0;
      }
      fseek(f, 0, SEEK_SET);
      fgets(b->line, 255, f);	// Headerzeile wegmachen

    }
    fwdpos = ftell(f);
    fgets(b->line, 255, f);	// Forwardzeile einlesen

    len = strlen(b->line);
    remerase = (strchr(b->line + 1, '*') != NULL);
    // nur Nachrichten fuer Fernloeschen freigeben, die tatsaechlich
    // per Forward rausgegangen sind
    for (i = 0; i < (len - 1); i++)
      b->line[i] = '.';		// Forwarden der Nachricht abstellen

    if (!unerase)
    { b->line[0] = '*';
      b->line[1] = reason;	// Grund fr Lschen merken
    }

    fseek(f, fwdpos, SEEK_SET);
    fwrite(b->line, len, 1, f);	// Forwardzeile zurckschreiben

    s_fclose(f);
  }
  waitfor(e_reschedule);
  strlwr(b->listpath);
  fh = s_open(b->listpath, "sr+b");
  if (fh != EOF)
  { seek_fname(fh, b->mailfname, found);
    if (found[0])
    { _read(fh, b->line, 128);
      if (unerase)
	b->line[14] = '>';
      else
	b->line[14] = reason;
      lseek(fh, -128L, SEEK_CUR);
      _write(fh, b->line, 128);
    }
    else
      trace(report, "merase", "not found %s", b->mailpath);
    s_close(fh);
  }
  else
    trace(report, "merase", "%s", b->listpath);
  waitfor(e_reschedule);
  if (!b->usermail)
  { fh = s_open(CHECKNAME, "sr+b");
    if (fh != EOF)
    { seek_fname(fh, b->mailfname, found);
      if (found[0])
      { _read(fh, b->line, 128);
	if (unerase)
	  b->line[14] = '>';
	else
	  b->line[14] = reason;
	lseek(fh, -128L, SEEK_CUR);
	_write(fh, b->line, 128);
      }
      s_close(fh);
    }
    else
      trace(serious, "merase", "check");
  }
  waitfor(e_reschedule);
  if (((strcmp(b->herkunft, b->logincall) == 0) || (reason == 'K')))
  { if ((b->mailtype == 'B') && remerase)
    { switch (reason)
      {
      case 'E':
      case 'K':
	{ char ebid[BIDLEN + 1];
	  strcpy(ebid, newbid(0));
	  char at2[NAMELEN + 1];
	  strncpy(at2, b->at, NAMELEN);
	  at2[NAMELEN] = 0;
	  subst(at2, '.', 0);

	  char reason_str[4];
	  sprintf(reason_str, "#%c", reason);
	  addwp_e(at2, b->logincall, ebid, b->bid, b->logincall, 0,
		  reason_str, b->logincall);
	  sprintf(buf, "E @ THEBOX < %s $%s %s",
		  m.boxname, ebid, b->bid);
	  add_emfile(buf, b->logincall, "M", "E", "THEBOX");
	}
      }
    }
  }
  return 1;
}

/*---------------------------------------------------------------------------*/

static void marklifetime(int tage)
//*************************************************************************
//
//  Setzt die Lifetime einer Nachricht neu
//
//*************************************************************************
{ int len, i;
  char found[1];
  FILE *f;
  handle fh;
  char *lt;
  char nlt[4];

  tage += (unsigned) ((ad_time() - filename2time(b->mailfname)) / 86400L);
  if (tage > 999)
    tage = 999;
  sprintf(nlt, "%-3d", tage);

  strlwr(b->mailpath);
  f = s_fopen(b->mailpath, "sr+t");
  if (f)
  { fgets(b->line, 255, f);	// Headerzeile wegmachen

    len = strlen(b->line);
    lt = strstr(b->line, " #");
    if (lt)
    { lt += 2;
      if ((lt[1] != '%') && (lt[2] != '%') && (lt[3] != '%'))
      { for (i = 0; i < 3; i++)
	  lt[i] = nlt[i];
      }
    }
    fseek(f, 0, SEEK_SET);
    fwrite(b->line, len, 1, f);	// Forwardzeile zurckschreiben

    s_fclose(f);
  }
  waitfor(e_reschedule);
  strlwr(b->listpath);
  fh = s_open(b->listpath, "sr+b");
  if (fh != EOF)
  { seek_fname(fh, b->mailfname, found);
    if (found[0])
    { _read(fh, b->line, 128);
      lt = b->line + 51;
      for (i = 0; i < 3; i++)
	lt[i] = nlt[i];
      lseek(fh, -128L, SEEK_CUR);
      _write(fh, b->line, 128);
    }
    else
      trace(serious, "msetl", "not found");
    s_close(fh);
  }
  else
    trace(serious, "msetl", "%s", b->listpath);

  waitfor(e_reschedule);

  if (!b->usermail)
  { fh = s_open(CHECKNAME, "sr+b");
    if (fh != EOF)
    { seek_fname(fh, b->mailfname, found);
      if (found[0])
      { _read(fh, b->line, 128);
	lt = b->line + 51;
	for (i = 0; i < 3; i++)
	  lt[i] = nlt[i];
	lseek(fh, -128L, SEEK_CUR);
	_write(fh, b->line, 128);
      }
      s_close(fh);
    }
    else
      trace(serious, "msetl", "check");
  }
  waitfor(e_reschedule);
}

/*---------------------------------------------------------------------------*/

static void change_address(void)
//*************************************************************************
//
//  Changes the forward-address of a mail
//
//*************************************************************************
{ char found[1];
  FILE *f;
  handle fh;
  int i;
  char *at, *db;
  long pos1, pos2;

#ifndef _EXDLOGIN	//db1ras
  if (b->usermail || (b->optplus & o_f)) // Change address in the mail file
  {
#else
    int change = 0;
    if ((b->guestlevel || u->fhold) && !b->sysop)
    {
      char ath[HADRESSLEN+2];
      sprintf(ath,"~%s",b->at);
      safe_strcpy(b->at,ath);
    }
#endif
    strlwr(b->mailpath);
    f = s_fopen(b->mailpath, "sr+t");
    if (f)
    { fread(b->line, 300, 1, f); // Read the first two lines

      // Change the address
      at = strchr(b->line, '@');
#ifdef _EXDLOGIN	//db1ras
     if (at[1] == '~' || b->usermail || (b->optplus & o_f))
     { change = 1;
#endif
#ifndef _LONG_HROUTE
      if (at && (strlen(at + 1) >= 24))
      { sprintf(at + 1, "%-24s", b->at);
        at[strlen(at)] = ' ';	// replace the 0-byte
      }
#else
      if (at && (strlen (at + 1) >= 24))
      {
        if (strchr (b->line, '$') - at < 41)
        //backward compatibility for old mail headers
        { //old mail header
          if (strlen (b->at) > 24) strcpy (b->at, atcall (b->at));
          sprintf (at + 1, "%-24s", b->at);
        }
        else
        { //new mail header
          sprintf (at + 1, "%-39s", b->at);
        }
        at [strlen (at)] = ' '; // replace the 0-byte
      }
#endif
      // Change the forward trigger line (no longer used as trigger)
      at = strchr(b->line, '\n');
      db = b->destboxlist;
      while (at && at[1] != '\n' && db[0])
      { at[1] = db[0];
	at++;
	db++;
      }

      // If it's a usermail, we will change the BID
      at = strchr(b->line, '$');
      if (b->usermail && at)
      { at++;
	at += blkill(at);
	db = newbid(0);
	if (isalnum(at[0]) && at[strlen(db)] == ' ')	// enough space?
        { strcpy(at, db);
	  at[strlen(at)] = ' ';
	}
      }

      fseek(f, 0L, SEEK_SET);
      fwrite(b->line, 300, 1, f);	// write back the changed stuff

      fseek(f, 0L, SEEK_SET);
      for (i = 0; i < 6; i++)
      { pos1 = ftell(f);
	fgets(b->line, 255, f);
      }

      while (b->line[0] == 'R' && b->line[1] == ':')
      { char *ha = getheaderadress(b->line);
	int save_line = FALSE;
	pos2 = ftell(f);
	if (ha && strstr(ha, m.boxadress) && b->line[13] == 'z')
	{ b->line[13] = 'Z';
	  save_line = TRUE;
	}
	if (strstr(b->line, "$:"))
	{ strstr(b->line, "$:")[0] = '_';
	  save_line = TRUE;
	}
	if (strstr(b->line, "B:"))
	{ strstr(b->line, "B:")[0] = '_';
	  save_line = TRUE;
	}
	if (save_line)
	{ fseek(f, pos1, SEEK_SET);
	  fputs(b->line, f);
	  fseek(f, pos2, SEEK_SET);
	}
	pos1 = ftell(f);
	if (fgets(b->line, 255, f) == NULL)
	  break;
      }
#ifdef _EXDLOGIN	//db1ras
     }
#endif
      s_fclose(f);
    }

#ifdef _EXDLOGIN	//db1ras
  if (change)
  {
#endif
    // Change the address in the list file (only save the BBS-call, not the
    // whole hierarchical address)
    if (strchr(b->at, '.'))
      *strchr(b->at, '.') = 0;

#ifdef _EXDLOGIN	//db1ras
    if (b->at[0]=='~' && strlen(b->at)==7)
      b->at[6] = 0;
#endif

    if (strlen(b->at) > 6)
    { trace(serious, "change_address", "call %s too long", b->at);
      return;
    }
    while (strlen(b->at) < 6)
      strcat(b->at, " ");

    waitfor(e_reschedule);
    strlwr(b->listpath);
    fh = s_open(b->listpath, "sr+b");
    if (fh != EOF)
    { seek_fname(fh, b->mailfname, found);
      if (found[0])
      { _read(fh, b->line, 128);
	db = b->line + 31;
	for (i = 0; i < 6; i++)
	  db[i] = b->at[i];
	lseek(fh, -128L, SEEK_CUR);
	_write(fh, b->line, 128);
      }
      else
	trace(serious, "change_address", "not found %s", b->mailfname);
      s_close(fh);
    }
    else
      trace(serious, "change_address", "%s", b->listpath);
  }
  waitfor(e_reschedule);

  add_fwdfile("", get_fdelay(b->herkunft));
}

/*---------------------------------------------------------------------------*/

void markread(int resetread, int dummy)
//*************************************************************************
//
//  Vermerkt in einer Nachricht, dass sie gelesen worden ist.
//
//  dummy was used for replys in earlier versions
//
//*************************************************************************
{ char line[256];
  char *nxtpos;
  int len;
  FILE *f;
  handle fh;
  long pos;

  strlwr(b->mailpath);
  f = s_fopen(b->mailpath, "sr+b");
  if (f)
  { fgets(line, 255, f);	// Headerzeile wegmachen

    fgets(line, 255, f);	// Forwardzeile wegmachen

    pos = ftell(f);
    fgets(line, 255, f);	// 'Gelesen' Zeile einlesen

    len = strlen(line);
    if (resetread)
    { int i;
      for (i = 0; i < (len - 1); i++)
	line[i] = '.';
      fseek(f, pos, SEEK_SET);
      fwrite(line, len, 1, f);
    }
    else
    { nxtpos = strstr(line, ".......");
      if ((strstr(line, b->logincall) == NULL) && nxtpos)
      {
	strcpy(nxtpos, b->logincall);
	nxtpos[strlen(b->logincall)] = ' ';
	fseek(f, pos, SEEK_SET);
	fwrite(line, len, 1, f);
      }
    }
    s_fclose(f);
  }

  if (strcmp(b->boardname, b->logincall) == 0)
  { strlwr(b->listpath);
    fh = s_open(b->listpath, "sr+b");
    if (fh != EOF)
    { strupr(b->mailfname);
      seek_fname(fh, b->mailfname, line);
      if (line[0])
      { _read(fh, b->line, 128);
	if (!resetread && (b->line[22] == ' '))
	  b->line[22] = 'R';
	if (resetread && (b->line[22] == 'R'))
	  b->line[22] = ' ';
	lseek(fh, -128L, SEEK_CUR);
	_write(fh, b->line, 128);
      }
      s_close(fh);
    }
  }
}

/*---------------------------------------------------------------------------*/

static void mbreply(int comment)
//*************************************************************************
//
//
//*************************************************************************
{ FILE *f;
  char befbuf[150];
  long nrep;
  char *eptr;

  char *bbuf;
  bbuf = (char *) t_malloc(sizeof(tvar_t) - sizeof(task_t), "repl");
  if (!bbuf)
    return;
  memcpy(bbuf, (char *) b + sizeof(task_t), sizeof(tvar_t) - sizeof(task_t));
  strlwr(b->mailpath);
  f = s_fopen(b->mailpath, "sr+t");
  if (f)
  { fgets(b->line, 255, f);	// Headerzeile einlesen

    mbsend_parse(b->line, 0);
    // oe3dzw Reply ist immer eine Personal-Mail
    // andere Flags sind immer auf defaults gesetzt
    if (!comment)
      b->mailtype = 'P';
    b->fwdhold = b->fwdprop = b->replied = '!';
    fgets(b->line, 255, f);	// Forwardzeile einlesen

    fgets(b->line, 255, f);	// READ-Zeile einlesen

    fgets(b->betreff, 78, f);	// Betreff-Zeile einlesen

    if (b->betreff[0])
      b->betreff[strlen(b->betreff) - 1] = 0;	// newline entfernen

    s_fclose(f);
    if (comment)
    { sprintf(befbuf, "%s ", b->ziel);
      if (*b->at)
	sprintf(befbuf + strlen(befbuf), "@ %s ", b->at);
      sprintf(befbuf + strlen(befbuf), "# %d ", b->lifetime);
    }
    else
      sprintf(befbuf, "%s ", b->herkunft);
    sprintf(b->line, "Re:%s", b->betreff);
    if (toupper(b->betreff[0]) == 'R' &&
	toupper(b->betreff[1]) == 'E' &&
	b->betreff[2] == ':')
      sprintf(b->line, "Re^2:%s", b->betreff + 3);
    else if (toupper(b->betreff[0]) == 'R' &&
	     toupper(b->betreff[1]) == 'E' &&
	     b->betreff[2] == '^')
    { nrep = strtol(b->betreff + 3, &eptr, 10);
      if (nrep > 0 && nrep < 10000)
	sprintf(b->line, "Re^%ld%s", nrep + 1, eptr);
    }
    sprintf(befbuf + strlen(befbuf), "%.50s", b->line);
    mbsend(befbuf, 0);
  }
  memcpy((char *) b + sizeof(task_t), bbuf, sizeof(tvar_t) - sizeof(task_t));
  t_free(bbuf);
  b->sentfiles++;
  if ((b->mailtype == 'P') && (strcmp(b->ziel, b->logincall) == 0))
    b->replied = '1';
  writemailflags();
}

/*---------------------------------------------------------------------------*/

int testreadbin(FILE *f, long mimeboundary, int is7plus)
//*************************************************************************
//
//  Checks current message for box-bin and outputs binary part
//
//*************************************************************************
// 7plus only for HTTP !
{ int cnt = 0;
  char bincmd[BINCMDLEN+1];
  int c;
  long posx;
  crcthp crc;
  unsigned long int bps;
  time_t startt, txtime;

#ifdef __FLAT__
  char *fname = "";
  long length = 0;
  char buffer[16];

  if (mimeboundary || is7plus)
  { if (is7plus)
      fseek(f, -1L, SEEK_CUR); // seek on space before go_7+.

    long pos = ftell(f);
    fgets(bincmd, BINCMDLEN, f);
    //printf("7plus %s\n",bincmd);
    char parts[8];

    if (strncmp(bincmd, START7PFLAG, 8)==0)
    { if (b->http != 1)
      { strncpy(parts, bincmd + 8, 3);
	strcat(parts, "/");
	strncat(parts, bincmd + 15, 3);

	fname = "unknown.7pl";

	length += strlen(bincmd);

	while (fgets(bincmd, BINCMDLEN, f) != NULL)
	{ length += strlen(bincmd);
	  if (strncmp(bincmd, STOP7PFLAG, 10)==0)
	  { char *s;

            s = strchr(bincmd, '(');
	    if (s)
	      *s = ' ';

	    s = strchr(bincmd, ')');
	    if (s)
	      *s = 0;

	    if (sscanf(bincmd + 11, "%15s", buffer) == 1)
	      fname = buffer;

	    break;
	  }
	}
      }

      if (mimeboundary)
      { fseek(f, mimeboundary, SEEK_SET);
        goto label;
      }
      else if (b->http == 2)
      { if (strchr(fname, '/'))
          *strchr(fname, '/') = 0;

        char tmp[256];
        int i;
        for (i = 0; i != strlen(b->mailpath); i++)
          tmp[i] = (b->mailpath[i] == '/') ? ':' : b->mailpath[i];

        html_putf("7plus file: <A HREF=\"/bread/%s?read=%s\">%s</A> "
                  "(%ld kBytes) - %s",
                  fname, tmp, fname, length / 1024L, parts);
        s_fclose(f);
        return 1;
      }
      else
      { fseek(f, pos, SEEK_SET);
	while ((c = fgetc(f)) != EOF)
	{ bputv(c);
	  if (((cnt++) & 255) == 0)
	    waitfor(e_ticsfull);
	}
	s_fclose(f);
	return 1;
      }
    }

    fseek(f, pos, SEEK_SET);
  }
#endif

  if (b->binstart)
  { posx = ftell(f);

    if (fread(bincmd, 4, 1, f) != 1)
      bincmd[0] = 0;		// if eof

#ifndef __LINUX__
    fseek(f, 0, SEEK_SET);
    while (ftell(f) < posx)
      getc(f);			// find old position
#else
    fseek(f, posx, SEEK_SET);
#endif

    if (strstr(bincmd, "BIN#"))
    { s_fclose(f);
      f = s_fopen(b->mailpath, "lrb");
      fseek(f, b->binstart, SEEK_SET);
      fread(bincmd, BINCMDLEN, 1, f);
#ifdef __FLAT__
      if (b->http == 2 || mimeboundary)
      { char *s = bincmd;

	if (strlen(bincmd))
	  bincmd[strlen(bincmd) - 1] = 0;
	if (bincmd[strlen(bincmd) - 2] != '#')
	{
	  for (c = strlen(bincmd); bincmd[c] != '/' && bincmd[c] != '#' && bincmd[c] != '\\' && c > 0; c--) ;
	  fname = bincmd + c + 1;
	}
	if (!strlen(fname))
	  fname = "unknown.bin";

	strtok(s, "#");
	length = atol(strtok(NULL, "#"));
      }

      if (b->http == 2)
      { char tmp[256];
	int i;
	for (i = 0; i != strlen(b->mailpath); i++)
	  tmp[i] = (b->mailpath[i] == '/') ? ':' : b->mailpath[i];

	if (stristr(fname, ".jpg") || stristr(fname, ".gif"))
	  html_putf("<image src=\"/bread/%s?read=%s\" "
                    "ALT=\"%s (%ld kBytes)\">",
                    fname, tmp, fname, length / 1024L);
	else
	  html_putf("Binary file: <A HREF=\"/bread/%s?read=%s\">%s</A> "
                    "(%ld kBytes)", fname, tmp, fname, length / 1024L);

	s_fclose(f);
	return 1;
      }
#endif
      if (b->forwarding == fwd_none && (b->opt & o_x))
      { putf("*** %s", bincmd);
	s_fclose(f);
	return 1;
      }

      // Send the BIN-header
      if (!mimeboundary && !(b->opt & o_q && b->forwarding == fwd_none))
	putf("%s", bincmd);

#ifdef HRX_OPT
      if (b->opt & o_v)
      { char binok[80];
        long oklength = 0;
	getline(binok, 80, 1);
	if (strncmp(binok, "#OK#", 4) == 0)
	{ strtok(binok, "#");
	  oklength = atol(strtok(NULL, "#"));
	  trace(report, "AUTOBIN", "size OK %i", oklength);
	}
	else
	  return 1;
      }
#endif

#ifdef __FLAT__
    label:
      if (mimeboundary)
      { static char *b64 = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdef"
                           "ghijklmnopqrstuvwxyz0123456789+/";
	long in24 = 0;
	int chr = 0;
	int out = 0;
	char f2name[255];
	strncpy(f2name, fname, 255);
	f2name[254] = 0;
	if (strchr(f2name, '/') != NULL)
	  *strchr(f2name, '/') = 0;

	putf("------%08X\n", mimeboundary);
	putf("Content-Type: %s; name=\"%s\"\n", mimestr(fname), f2name);
	putf("Content-Transfer-Encoding: base64\n\n");
	//putf ("Content-Length: %d Bytes\n\n",length);
	//putf ("Content-Disposition: inline; filename=\"%s\"\n\n",f2name);
	while (((c = fgetc(f)) != EOF) && (!length || length--))
	{ in24 <<= 8;
	  in24 |= c;
	  //printf("in24 %d %08lx %02x\n",chr,in24,c);
	  chr++;

	  if (chr == 3)
	  { //printf("%08lx %02lx %02lx %02lx %02lx\n",in24,in24&63,(in24>>6)&63,(in24>>12)&63,(in24>>18)&63);
	    putv(b64[(in24 >> 18) & 63]);
	    putv(b64[(in24 >> 12) & 63]);
	    putv(b64[(in24 >> 6) & 63]);
	    putv(b64[in24 & 63]);
	    in24 = 0;
	    chr = 0;
	    out += 4;
	  }

	  if (out == 76)
	  { putv('\n');
	    out = 0;
	  }

	  if (((cnt++) & 255) == 0)
	  { waitfor(e_ticsfull);
	    if (b->forwarding != fwd_none)
	      timeout(m.fwdtimeout);
	  }
	}

	if (chr)
	{ in24 <<= 8 << (2 - chr);
	  putv(b64[(in24 >> 18) & 63]);
	  putv(b64[(in24 >> 12) & 63]);
	}

	switch (chr)
	{
	case 1:
	  putf("==");
	  break;

	case 2:
	  putf("%c=", b64[(in24 >> 6) & 63]);
	  break;
	}

	putf("\n------%08X--\n", mimeboundary);
      }
      else
#endif
      { startt = ad_time();

	while ((c = fgetc(f)) != EOF)
	{ bputv(c);
	  crc.update(c);
	  if (((cnt++) & 255) == 0)
	  { waitfor(e_ticsfull);
	    if (b->forwarding != fwd_none)
	      timeout(m.fwdtimeout);
	  }
	}

	if (b->forwarding == fwd_none && !(b->opt & o_q))
	{ txtime = ad_time() - startt;
	  if (txtime == 0)
	    txtime = 1L;
	  bps = cnt / txtime;
	  putf("\nBIN-TX OK (CRC: %u/%04Xh, %lu bit/s, %lu byte/s)\n",
	       crc.result, crc.result, bps * 8L, bps);
	}
      }

      s_fclose(f);
      return 1;
    }
  }

  return 0;
}

/*---------------------------------------------------------------------------*/

void read_file(int mode)
//*************************************************************************
//
//  Gibt ein READ-File zum User aus
//
//*************************************************************************
{ long fntime = filename2time(b->mailfname);
  int end;
  char *line = b->line;
  int c, last = 0;
  long mimeboundary = 0;
  FILE *f;

  strlwr(b->mailpath);

  f = s_fopen(b->mailpath, "lrb");	//binary now, was text
  if (f)
  { fgets(line, 255, f);
    mbsend_parse(line, 0);
    formoptions();

    if (mode == 1)
      b->opt |= o_a + o_b + o_d + o_e + o_f + o_i + o_l +
	o_m + o_p + o_r + o_t + o_u + o_w + o_y + o_z;
    else
      b->readfiles++;

    if (mode < 2 && !(b->opt & o_q))
      putreadheader(fntime);

    waitfor(e_ticsfull);

    fgets(line, 255, f);
    if (mode == 1 && !(b->opt & o_q))
    { end = 0;
      while ((end < 73) && (line[end] != '.'))
	end++;
      line[end] = 0;
      end = strlen(line) - 1;
      if (line[end] == '\n')
	line[end] = 0;
      if (strlen(line))
	putf("Forw: %s\n", line);
    }

    fgets(line, 255, f);
    end = 0;
    while ((end < 252) && (line[end] != '.'))
      end++;
    line[end] = 0;
    end = strlen(line) - 1;
    if (line[end] == '\n')
      line[end] = 0;
    if (strlen(line) && (b->opt & o_r))
    {
      char *l2 = line;
      while (l2[0])
      {
	int i = strlen(l2);
	if (strlen(l2) > 73)
	{
	  i = 66;
	  while (l2[i] != ' ')
	    i++;
	  l2[i] = 0;
	  i++;
	}
	if (mode != 2 && !(b->opt & o_q))
	  if (mode < 3)
	    putf("Read: %s\n", l2);	//some sneefers depend on it
	  else
	    putf("X-Read: %s\n", l2);
	l2 += i;
      }
    }

    fgets(line, 100, f);
    end = strlen(line) - 1;
    if (line[end] == '\n')
      line[end] = 0;
    line[79] = 0;
    strcpy(b->betreff, line);

    if (b->opt & o_w && mode != 2 && !(b->opt & o_q))
      if (mode != 3)
	putf("Subj: %s\n", line);

    if (mode == 2 && !(b->opt & o_q))
      putv('>');

    if (!(b->opt & o_q))
      if (putreadpath(f, line, mode))
      {
#ifdef __FLAT__
        if (mode == 3)
        { mimeboundary = scanmimetype(f);
          putmimeheader(line, mimeboundary);

          /*
          if (!strncasecmp (line, "de", 2))
          { int j;
            for (j = 2; line[j]; j++)
              if (line[j] != ' ' && line[j] != ':')
                break;

            if (line[j])
            { char *t = "from: ";
              memmove (line + 6, line + j, strlen (line + j) + 1);
              for (j = 0; t[j]; j++)
                line[j] = t[j];
            }
          }
	  */
	}
#endif
	putf("%s", line);
      }

    if (mode == 1 && !(b->opt & o_q))
      putf("...\n");
    else
    { if (mode == 2 && !(b->opt & o_q))
	putv('>');
      long cnt = ftell(f);

      // This loop sends the mail
      while ((c = fgetc(f)) != EOF)
      { cnt++;

        // Test for AutoBIN-part
	if (((last == '\n' && c == '#') || (mode == 3 && mimeboundary == cnt)))
	{ if (testreadbin(f, mimeboundary, 0))
	  { f = NULL;
	    break;
	  }
	}

        // Test for 7plus-part (http only)
	if (b->http && (b->msgty == '7' || b->msgty == '6') &&
	    last == '\n' && c == ' ') // better check would be complicated a bit
	{ if (testreadbin(f, 0, 1))   // mimeboundary not used for http
	  { f = NULL;
	    break;
	  }
	}

	if (!(b->opt & o_q))
	  putv(c);

	last = c;

	if (c == '\n')
	{
	  if (mode == 2 && !(b->opt & o_q))
	    putv('>');

	  if (testabbruch())
	    break;

	  waitfor(e_ticsfull);
	}
      }

      end = strlen(line) - 1;
      if ((line[end] != '\n' || b->binstart) && !(b->opt & o_q))
	putf("\n");
      loaduser(b->logincall, u, 1);
      u->mailread++;
      saveuser(u);
    }

    if (f)
      s_fclose(f);

    waitfor(e_ticsfull);
  }
  else
    putf("Can't open mail file %s\n", b->mailpath);
}

/*---------------------------------------------------------------------------*/

int rmemptydir(char *mask)
//*************************************************************************
//
//  Loescht eine leere Rubrik als Directory auf der Platte
//  Gibt 1 zurueck, wenn eine Rubrik unter TMP entfernt wurde
//
//*************************************************************************
{
  int fileanzahl = 0;
  char *wildcard;
  DIR *d;
  struct dirent *di;

  strlwr(mask);
  sprintf(b->listpath, "%s/", mask);
  wildcard = b->listpath + strlen(b->listpath);

  if (strlen(mask) > 40 || mask[strlen(mask) - 1] == '/')
  {
    trace(serious, "rmemptydir", "format %s", mask);
    return 0;
  }

  d = opendir(mask);
  while (d && (di = readdir(d)) != NULL)
  {
    if (di->d_name[0] != '.')
    {
      if (strlen(di->d_name) == 7)
	fileanzahl++;
    }
  }
  if (d)
    closedir(d);

  if (fileanzahl == 0)
  { 
    strcpy(wildcard, DIRLISTNAME);
    xunlink(b->listpath);
    *(wildcard - 1) = 0;
    rmdir(b->listpath);
    wildcard = strstr(b->listpath, "tmp/");
    if (wildcard)
    { 
      if (rmboard(wildcard) != 0)
      {
	trace(serious, "rmemdir", "%s rmboard", wildcard);
	return 0;
      }
      return 1;
    }
  }
  return 0;
}

/*---------------------------------------------------------------------------*/

int mbchange(char *selektor, int thiscmd, int interactive)
//*************************************************************************
//
// Erases,forwards,transfers,replies and sets lifetime
//
// thiscmd:     w_erase,w_unerase,w_forward,w_transfer,w_setlifetime,
//              w_reply,w_comment (enum mail_modify_t, mail.h)
//
// selektor:    eg. command-line buffer
//
// interactive: 1..interactive i/o 0..no interactive i/o
//
//*************************************************************************
{
  char *line = b->line;
  unsigned files = 0, pos = 0;
  long lastpos;
  char *jobtext = m.sysopcall;	//dummy init

  char *ziel;
  char zielbuf[61] =
  {0};
  handle fh;
  char limiter;
  char searchitem[31] = "";

  switch (thiscmd)
  {
  case w_forward:
    limiter = '@';
    break;

  case w_transfer:
    limiter = '>';
    break;

  default:
    limiter = '#';
  }

  findbereich(selektor);
  scanoptions(selektor);

  if (get_searchitem(selektor, searchitem, 30))
    b->bereich = 1;

  strupr(selektor);
  selektor += blkill(selektor);

  ziel = strchr(selektor, limiter);
  if (ziel == 0)
    ziel = blankweg(selektor);
  else
  {
    ziel[0] = 0;
    ziel++;
  }

  if (ziel)
  {
    ziel += blkill(ziel);
    if (ziel[0] == limiter)
      ziel++;
    ziel += blkill(ziel);
    if (thiscmd == w_forward)
    {
      if (strlen(ziel) > HADRESSLEN)	// DG4MMI

	ziel[HADRESSLEN - 1] = 0;
      strcpy(b->at, ziel);
      if (weiterleiten(0, "") == unbekannt)
      {
	if (interactive)
	  putf(ms(m_adrunknown), b->at);
	return 0;
      }
      while (strlen(b->destboxlist) < 139)
	strcat(b->destboxlist, ".");
    }
    if (strlen(ziel) > 60)
      ziel[60] = 0;
    strcpy(zielbuf, ziel);
  }
  selektor += blkill(selektor);
  blankweg(selektor);

  if ((b->beginn == 0) || (!b->bereich))
  {
    if (b->replynum && selektor[0] == 0)
    {
      b->beginn = b->ende = b->replynum;
      strcpy(selektor, b->replyboard);
    }
    else
    {
      if (interactive)
	putf(ms(m_givenumber));
      return 0;
    }
  }

  if (selektor[0] == 0)
    strcpy(selektor, b->prompt);

  fh = preparedir(selektor, 1);
  if (fh != EOF)
  {
    strcpy(b->prompt, b->boardfullname);
    lastpos = filelength(fh) >> 7;

    if (b->ende > lastpos)
      b->ende = (unsigned) lastpos;

    while (_read(fh, line, 128))
    {
      waitfor(e_ticsfull);
      pos++;
      if ((pos >= b->beginn) && (pos <= b->ende))
      {
	line[126] = 0;

	if (searchitem[0] && !stristr(line, searchitem))
	  continue;

	dirline_data(line);
	lastpos = ltell(fh);
	s_close(fh);
	fh = EOF;
	line[7] = 0;
	strcpy(b->mailfname, line);
	strcpy(b->mailpath, b->mask);
	if (!strstr(b->mailpath, "*.*"))
	  trace(fatal, "mbchange", "mask %s", b->mailpath);
	strcpy(strstr(b->mailpath, "*.*"), b->mailfname);
	if (owner() || thiscmd == w_reply || thiscmd == w_comment)
	{
	  switch (thiscmd)
	  {
	  case w_erase:
	    if (!b->geloescht)
	    {
	      if (b->optplus & o_f)
		files += markerased('K', 0);
	      else
		files += markerased('E', 0);
	    }
	    jobtext = ms(m_erased);
	    break;

	  case w_unerase:
	    if (b->geloescht)
	    {
	      files++;
	      markerased(' ', 1);
	    }
	    jobtext = ms(m_unerased);
	    break;

	  case w_forward:
	    if (zielbuf[0])
	    {
	      if (!b->geloescht)
	      {
		strcpy(b->at, zielbuf);
		// OE3DZW: Abfrage nach Holdflag eingebaut
		if ((b->mailtype == 'B' || mbhadrok(b->at) == 1)
		    && (b->fwdhold == '!' || b->optplus & o_h))
		{
		  files++;
		  change_address();
		}
	      }
	      jobtext = ms(m_forwarded);
	    }
	    else
	    {
	      putf(ms(m_syntaxforward));
	      return 0;
	    }
	    break;

	  case w_setlifetime:
	    if (zielbuf[0])
	    {
	      if (!b->geloescht)
	      {
		files++;
		marklifetime(atoi(zielbuf));
	      }
	      jobtext = ms(m_lifetimeset);
	    }
	    else
	    {
	      putf(ms(m_syntaxlifetime));
	      return 0;
	    }
	    break;

	  case w_transfer:
	    if (zielbuf[0])
	    {
	      if (!b->geloescht)
	      {
		if (mbtransfer(zielbuf) == OK)
		{
		  files++;
		  if (!b->usermail)
		    markerased('T', 0);
		}
	      }
	      jobtext = ms(m_transferred);
	    }
	    else
	    {
	      putf(ms(m_syntaxtransfer));
	      return 0;
	    }
	    break;

	  case w_reply:
	  case w_comment:
	    mbreply(thiscmd == w_comment);
	    files++;
	    break;
	  }
	}
	else
	{
	  if (!files)
	  {
	    if (interactive)
	      putf(ms(m_permissiondenied));
	    return 0;
	  }
	}
	waitfor(e_reschedule);
	strlwr(b->listpath);
	fh = s_open(b->listpath, "srb");
	lseek(fh, lastpos, SEEK_SET);
	if (testabbruch())
	  break;
      }
    }
    s_close(fh);
    if (interactive && thiscmd != w_reply && thiscmd != w_comment)
    {
      if (files == 1)
	putf(ms(m_messagewillbe), jobtext);
      else if (files)
	putf(ms(m_messageswillbe), files, jobtext);
    }
  }
  if (!files && interactive)
    putf(ms(m_nomessagefound));
  return files;
}

/*---------------------------------------------------------------------------*/

int mbread(char *selektor, int mode)
//*************************************************************************
//
//  Outputs one or more mails (parses the command line)
//
//  mode: 0 = Usual output of mail to user (command READ)
//        1 = Only output of the header (commands HEADER, KOPF)
//        2 = Output for HTTP-reply: Put a '>' in front of every line
//        3 = Mime V1.0 compliant headers for POP3 service
//
//*************************************************************************
{
  char *line = b->line;
  unsigned kopf = 0, wargeloescht = 0;
  long lastpos;
  int nachrichtgekommen = 0;
  handle fh;
  char searchitem[31] = "";
  int readlock = 0;

  findbereich(selektor);

  scanoptions(selektor);

  if (mode != 3 && !(b->optplus & o_q))
  {
    putf("\n");
    leerzeile();
  }

  if (get_searchitem(selektor, searchitem, 30))
    b->bereich = 1;

  selektor += blkill(selektor);
  if (selektor[0] == 0)
    strcpy(selektor, b->prompt);

  fh = preparedir(selektor, 1);
  if (fh != EOF)
  {
    strcpy(b->prompt, b->boardfullname);
    if (b->usermail || !checkquota())
    {
      if (!b->bereich)		// Wenn kein Bereich angegeben worden ist, dann

      {
	b->beginn = 0;		// die neueste Nachricht ausgeben.

	b->ende = 1;
      }
      if ((b->beginn) > 0)
	lseek(fh, ((long) (b->beginn - 1)) << 7, SEEK_SET);
      else
      {
	seek_lastentry(fh, b->ende);	//was 1

	b->ende = MAXBEREICH;
      }

      if (mbcallok(b->boardname) && strcmp(b->boardname, b->logincall) != 0)
      {
	readlock = get_readlock(b->boardname);
	if (readlock != 1 && u->rlimit)
	  readlock = u->rlimit;
      }

      while (_read(fh, line, 128) && ((unsigned) (ltell(fh) >> 7) <= b->ende))
      {
	if (searchitem[0] && !stristr(line, searchitem))
	  continue;
	if (readlock)
	{
	  char fromcall[10];
	  strncpy(fromcall, line + 8, 6);
	  fromcall[6] = 0;
	  blankweg(fromcall);
	  if (strcmp(fromcall, b->logincall) != 0 && !b->sysop)
	  {
	    if (readlock == 2 || line[22] != 'R')
	      continue;
	  }
	}
	if (!b->usermail && checkquota())
	  break;

#ifdef _MAILFILTER	//db1ras
       if ((b->sysop) || (line[31]!='~') ||
           (strncmp(line+8,b->logincall,strlen(b->logincall))==0))
       {
#endif
	if (line[14] == '>' || b->sysop)
	{
	  lastpos = ltell(fh);
	  s_close(fh);
	  fh = EOF;
	  line[7] = 0;
	  strcpy(b->mailfname, line);
	  strcpy(b->mailpath, b->mask);
	  if (!strstr(b->mailpath, "*.*"))
	    trace(fatal, "mbread", "mask %s", b->mailpath);
	  strcpy(strstr(b->mailpath, "*.*"), b->mailfname);
	  if (nachrichtgekommen && !(b->optplus & o_q))
	    putf("------------------------------------------------------\n");
	  read_file(mode);
	  b->replynum = (unsigned) (lastpos >> 7);
	  strcpy(b->replyboard, b->ziel);
	  nameconvert(b->replyboard);
	  nachrichtgekommen++;
	  if (!mode && !(owner() && (b->optplus & o_g)))
	    markread(0, 0);

	  if ((b->opt & o_n) && owner() && !mode && !(b->optplus & o_q))
	  {
	    leerzeile();
	    if (janein(ms(m_erasemessage)) == JA)
	    {
	      markerased('E', 0);
	      putf(ms(m_messageerased));
	    }
	  }
	  kopf++;
	  if (testabbruch())
	    break;
	  fh = s_open(b->listpath, "srb");
	  if (fh == EOF)
	    break;
	  lseek(fh, lastpos, SEEK_SET);
	}
	else
	  wargeloescht++;
#ifdef _MAILFILTER	//db1ras
       }
#endif
      }
    }
    else
      kopf = 1;
    if (fh != EOF)
      s_close(fh);
  }

  if (!kopf && !(b->optplus & o_q))
  {
    if (wargeloescht)
      putf(ms(m_messageiserased));
    else
      putf(ms(m_nomessagefound));
    return 0;
  }

  return 1;
}
