/***************************************************************

  BayCom(R)           Packet-Radio fr IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fr Mailboxsystem


  ----------------------------------------------
  Einbindungsmoeglichkeit fuer externe Programme
  ----------------------------------------------


  Copyright (c)       Florian Radlherr
                      Taubenbergstr. 32 
		      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19970915 DG4MMI Continued working on mbrun.cpp
//19980125 OE3DZW option -p also for sysop (not only for pw-holders..)
//19980307 OE3DZW removed .run from temporary file names
//19980318 OE3DZW fixed segfault due to invalid string-pointer
//19980428 DG4MMI fixed erase of bin_output
//19980505 DG4MMI fixed user_imp (was not erased, was not correct)
//19980707 DH3MB  made runutil-interface modular (#define RUNUTILS)
//19980724 DH3MB  added DPBOXUSERIMPOUT, activated option -D for Win32
//19980831 OE3DZW fixed -D for Win32, could not even be compiled
//19990206 OE3DZW added fspath

#include "baycom.h"

#ifdef RUNUTILS

/* sample runutil.bcm
;Befehl Programm     Parameter Kommentar
fwda*dd "fwdadd"     -sq       Hilfsmittel zum Eintragen von Forwardadressen 
o*shell "command /c" -sqi      Shell starten
*/

#define RUN_CMDLEN 11
#define RUN_EXELEN 19
#define RUN_COMLEN 65
#define RUN_PARAMLEN 130  //max. length of parameters for run-util

typedef struct mbrun_t
{ char command[RUN_CMDLEN+1];
  char executable[RUN_EXELEN+1]; 
  int abbreviation;
  bitfeld opt;
  // o_s: Only executable by sysop
  // o_f: Only executable in FileSurf-mode
  // o_q: Do NOT create "rundat.bcm"
  // o_c: Do NOT pass the given parameters to the external program
  // o_i: Input from user is NOT passed to the external program
  //      (Linux/Win32 only)
  // o_p: Only executable by users, who have set a password
  // o_d: Enable DPBOX-compatibility, i.e. set some variables in the 
  //      environment-table (Linux/Win32 only)
  // o_t: Even pass |, > and < to the external programm (Linux only)
  char comment[RUN_COMLEN+1];
} mbrun_t;

mbrun_t *runs=NULL;
int run_num;

/*---------------------------------------------------------------------------*/

void parse_runfile(char *s,mbrun_t *r)
{ int i=0;

  r->abbreviation=0;
  while(s[0] && (s[0]!=' ') && (i<RUN_CMDLEN))
  { if(s[0]=='*')
      r->abbreviation=i;
    else 
      r->command[i++]=s[0];
    s++;
  } 
  r->command[i++]=0;
  strupr(r->command);
  if(!r->abbreviation) r->abbreviation=strlen(r->command);

  s=nexttoken(s,r->executable,RUN_EXELEN);

  scanoptions(s);
  r->opt=b->optplus;

  while((s[0]==' ') && (s[0]!=0) && (i<RUN_CMDLEN)) s++;
  strncpy(r->comment, s, RUN_COMLEN);
  r->comment[RUN_COMLEN]=0;
}

/*---------------------------------------------------------------------------*/

void read_runfile(void)
{ FILE *f=s_fopen(RUNNAME,"srt");
  char s[255];

  run_num=0;

  if(f)
  { if(runs)
    { t_free(runs);
      runs=NULL; 
    }
    while(fgets(s,255,f))
      if(s[0]!=';') run_num++;

    runs=(struct mbrun_t *)t_malloc(run_num*sizeof(mbrun_t),"*run");
    rewind(f);

    run_num=0;
    while(fgets(s,255,f))
    { if(s[0]==';') 
        continue;
      if(s[0])
        s[strlen(s)-1]=0;
      parse_runfile(s,runs+run_num);  
      run_num++;
    }
    s_fclose(f);
    trace(report,"runutils","%d runutils activated",run_num);
    if (*b->logincall && run_num)
      putf("%d runutils.\n",run_num);
  }
}

/*---------------------------------------------------------------------------*/

void list_runutils(char *befbuf)
{ int c;
  mbrun_t *r;
  char cmdtemp[RUN_CMDLEN+3];

  if(run_num==0)
  { putf("No runutils active.\n");
    return;
  }

  scanoptions(befbuf);
  r=runs;

  putf("%-14s", "Command");

  if(b->optplus&o_l)
    putf(" %-19s SFQCIPDT", "External program");

  putf("\n");

  for(c=0;c<run_num;c++)
  { if(!(r->opt&o_s) || (b->optplus&o_s))
    { if(b->optplus&o_l)
        putf("\n");
      strcpy(cmdtemp, r->command);
      if(r->abbreviation!=strlen(r->command))
      { cmdtemp[r->abbreviation]='(';
        strcpy(&cmdtemp[r->abbreviation+1], &r->command[r->abbreviation]);
        strcat(cmdtemp, ")");
      }
      putf("%-14s ", cmdtemp);

      if(b->optplus&o_l)
      { putf("%-19s ", r->executable);
        if(r->opt&o_s) putv('1'); else putv('0');
        if(r->opt&o_f) putv('1'); else putv('0');
        if(r->opt&o_q) putv('1'); else putv('0');
        if(r->opt&o_c) putv('1'); else putv('0');
        if(r->opt&o_i) putv('1'); else putv('0');
        if(r->opt&o_p) putv('1'); else putv('0');
        if(r->opt&o_d) putv('1'); else putv('0');
        if(r->opt&o_t) putv('1'); else putv('0');
        putf("\n");
      }

      putf("%s\n", r->comment);
    }

    r++;
  }
  putf("\n");
  putf("%d runutils active\n", run_num);
}

/*---------------------------------------------------------------------------*/

int start_runutil(char *befbuf)
{ 
#ifdef __FLAT__
  char command[2*RUN_PARAMLEN];
  char temp[RUN_PARAMLEN];
#else
  char command[RUN_PARAMLEN];
#endif
  char yapptmp[20];
  char bintmp[20];
  char abintmp[20];
  char txttmp[20];
  char imptmp[20];
  char uimptmp[20];
  char *origbef;
  int c;
  mbrun_t *r;
  FILE *f;

  if(run_num==0) return 0;

  r=runs;

  origbef=befbuf;
  befbuf=nexttoken(befbuf,command,RUN_CMDLEN);
  if (strlen(befbuf) >= RUN_PARAMLEN) //cut input string if too long
    befbuf[RUN_PARAMLEN-1]=0; 
 
  if(command[0])
  { strupr(command);

    for(c=0;c<run_num;c++)
    { if((strncmp(r->command,command,strlen(command))==0)&&
	 (strlen(command)>=r->abbreviation))
      { strcpy(command, r->executable);
        break;
      }
      if(c==(run_num-1)) command[0]=0;
      r++;
    }

    if((r->opt&o_s) && (!b->sysop))
      return 0;
    if((r->opt&o_p) && !(u->password[0]>1) && (!b->sysop))
      return 0;

#ifdef FILESURF
    if((r->opt&o_f) && (!b->fsmode))
      return 0;
    if(!(r->opt&o_f) && (b->fsmode))
      return 0;
#endif

    if(command[0])
      { if(!(r->opt&o_q) || (r->opt&o_d))
        { sprintf(txttmp,TEMPPATH"/%s",time2filename(0));
          strlwr(txttmp);
          sprintf(bintmp,TEMPPATH"/%s",time2filename(0));
          strlwr(bintmp);
          sprintf(abintmp,TEMPPATH"/%s",time2filename(0));
          strlwr(abintmp);
          sprintf(yapptmp,TEMPPATH"/%s",time2filename(0));
          strlwr(yapptmp);
          sprintf(imptmp,TEMPPATH"/%s",time2filename(0));
          strlwr(imptmp);
          sprintf(uimptmp,TEMPPATH"/%s",time2filename(0));
          strlwr(uimptmp);
        }

        if(!(r->opt&o_q))
        { f=s_fopen(RUNDATNAME,"swt");
          if(f)
          { s_fsetopt(f,1);
            fprintf(f,"%s %s %s %s %d %d %d %d %s\n",u->call,u->mybbs,u->name,
                    u->sprache,u->password[0]>1,b->sysop,u->status,
                    u->helplevel,u->uplink);
            fprintf(f,"call       %s\n",u->call);
            fprintf(f,"mybbs      %s\n",u->mybbs);
            fprintf(f,"name       %s\n",u->name);
            fprintf(f,"language   %s\n",u->sprache);
            fprintf(f,"password   %d\n",u->password[0]>1);
            fprintf(f,"sysop      %d\n",b->sysop);
            fprintf(f,"status     %d\n",u->status);
            fprintf(f,"helplevel  %d\n",u->helplevel);
            fprintf(f,"digi       %s\n",u->uplink);
            fprintf(f,"txt_output %s\n",txttmp);
            fprintf(f,"bin_output %s\n",bintmp);
            fprintf(f,"abinoutput %s\n",abintmp);
            fprintf(f,"yappoutput %s\n",yapptmp);
            fprintf(f,"imp_output %s\n",imptmp);
            fprintf(f,"user_imp   %s\n",uimptmp);
            fprintf(f,"cmdline    %s\n",origbef);
            fprintf(f,"login_time %ld\n",b->logintime);
            fprintf(f,"board      %s\n", b->prompt);
#ifdef FILESURF // DG4MMI
          if (b->fspath)
            fprintf(f,"fspath     %s\n", b->fspath);
#endif
            s_fclose(f);
          }
        }

        if(!(r->opt&o_c))
        { strcat(command, " ");

#ifdef __LINUX__ // Running LinuX, even the characters |, > and < can be passed
                 // to the external programm (if the parameter -T is given)
          if(!(r->opt&o_t))
#endif
          { if(strchr(befbuf,'|'))
              *strchr(befbuf,'|')=0;
            if(strchr(befbuf,'>'))
              *strchr(befbuf,'>')=0;
            if(strchr(befbuf,'<'))
              *strchr(befbuf,'<')=0;
          }

#ifdef __LINUX__ // Running LinuX, propose a backslash to every character of
                 // the parameters (not to the spaces, of course)
          while(befbuf[0])
          { if (befbuf[0]==' ')
              sprintf(command+strlen(command),"%c",befbuf[0]);
            else
              sprintf(command+strlen(command),"\\%c",befbuf[0]);
            befbuf++;
          }
#else
          strcat(command,befbuf);
#endif
        }

#ifdef __FLAT__
        if(r->opt&o_d)
        { setenv("TERM", "dumb", 1);
          setenv("LOGNAME", u->call, 1);
          setenv("CALLSIGN", u->call, 1);
          setenv("DPBOXUSERCALL", u->call, 1);
          setenv("DPBOXUSERMYBBS", u->mybbs, 1);
          setenv("DPBOXUSERNAME", u->name, 1);
          setenv("DPBOXUSERLANGUAGE", u->sprache, 1);
          setenv("DPBOXBOARD", b->boardname, 1);
          setenv("DPBOXLASTDATE", zeitstring(u->lastboxlogin, 1), 1);
          sprintf(temp, "%ld", u->lastboxlogin);
          setenv("DPBOXIXLASTDATE", temp, 1);
          setenv("DPBOXSENDERASEOK", "1", 1);        // dummy
          setenv("DPBOXHIDDEN", "0", 1);             // dummy
          sprintf(temp, "%d", u->password[0]>1);
          setenv("DPBOXPWOK", temp, 1);
          sprintf(temp, "%d", b->sysop);
          setenv("DPBOXBOARDSYSOP", temp, 1);        // dummy
          sprintf(temp, "%d", b->sysop);
          setenv("DPBOXSYSOP", temp, 1);
          sprintf(temp, "%d", b->sysop * 127);
          setenv("DPBOXUSERLEVEL", temp, 1);         // dummy
          setenv("DPBOXBOXCALL", m.boxname, 1);
          sprintf(temp, "%d", b->fsmode);
          setenv("DPBOXSERVERMODE", temp, 1);
          setenv("DPBOXSERVERROOT", "/", 1);
          setenv("DPBOXSERVERPATH", b->fspath, 1);
          strcpy(temp, u->call);
          strcat(temp, " ");
          strcat(temp, b->uplink);
          setenv("DPBOXCONNECTPATH", temp, 1);
          setenv("DPBOXUNPROTOOK", "0", 1);          // dummy
          if(strncmp(u->notvisible+1, "-R ", 3)==0)
          { setenv("DPBOXNOCHECK", "", 1);
            setenv("DPBOXWANTCHECK", u->notvisible+4, 1);
          }
          else
          { setenv("DPBOXNOCHECK", u->notvisible+1, 1);
            setenv("DPBOXWANTCHECK", "", 1);
          }
          setenv("DPBOXLASTCMD", origbef, 1);
          setenv("DPBOXDEBUGLEVEL", "0", 1);         // dummy
          setenv("DPBOXUSERTIMEOUT", "300", 1);      // dummy
          setenv("DPBOXVERSION", "5.07.00", 1);      // dummy
          sprintf(temp, "%d", gettaskid());
          setenv("DPBOXUSERNUMBER", temp, 1);
          sprintf(temp, "%d", listusers(0));
          setenv("DPBOXUSERCOUNT", temp, 1);
          strcpy(temp, globalpath);
          strcat(temp, txttmp);
          setenv("DPBOXTEXTOUT", temp, 1);
          strcpy(temp, globalpath);
          strcat(temp, bintmp);
          setenv("DPBOXTRANSOUT", temp, 1);
          strcpy(temp, globalpath);
          strcat(temp, abintmp);
          setenv("DPBOXBINOUT", temp, 1);
          strcpy(temp, globalpath);
          strcat(temp, yapptmp);
          setenv("DPBOXYAPPOUT", temp, 1);
          strcpy(temp, globalpath);
          strcat(temp, imptmp);
          setenv("DPBOXIMPOUT", temp, 1);
          strcpy(temp, globalpath);
          strcat(temp, uimptmp);
          setenv("DPBOXUSERIMPOUT", temp, 1);
#ifdef FILESURF // DG4MMI
          if (b->fspath)
            setenv("DPBOXFSPATH",b->fspath,1); 
#endif

        }
#endif

        putf("\n"); //add linebreak before start of runutil (OE3DZW)
        if(r->opt&o_i)
          oshell(command, sh_noinput);
        else
          oshell(command, sh_ifmultitask);

        if(!(r->opt&o_q) || (r->opt&o_d))
        { if(filetime(txttmp))
          { fileio_text fio;
            fio.usefile(txttmp);
            fio.doerase();
            fio.tx();
          }

          if(filetime(bintmp))
          { int a;
            f=s_fopen(bintmp, "lrb");
            s_fsetopt(f, 2);
            while((a=fgetc(f))!=EOF)
              bputv(a);
            s_fclose(f);
          }

          if(filetime(abintmp))
          { fileio_abin fio;
            fio.usefile(abintmp);
            fio.doerase();
            fio.tx();
          }

#ifdef FEATURE_YAPP
	  if(filetime(yapptmp))
	  { fileio_yapp fio;
	    fio.usefile(yapptmp);
	    fio.doerase();
	    fio.tx();
	  }
#endif

	  if((t->input!=io_file||t->output==io_dummy) && t->output!=io_file
             && filetime(uimptmp)) // Only if no file-i/o
          { b->inputfile=s_fopen(uimptmp,"lrt");
            s_fsetopt(b->inputfile,2);
            b->oldinput=t->input;
            t->input=io_file;
          }

          if(filetime(imptmp))
            fork(P_BACK|P_MAIL,0,mbimport,imptmp);
        }

        return 1;
      }
  }

  return 0;
}

/*---------------------------------------------------------------------------*/

#endif
