/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  ------------
  SERV for BCM
  ------------


  Copyright (C)       Deti Fliegl, deti@lrz.de, DG9MHZ
                      Guardinistr. 47
                      81375 Muenchen

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980211 OE3DZW added option for SERV

#include "baycom.h"

#ifdef SERVIF

#define MAX_RCPTCNT 100

#ifdef __UNIX__
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#define closesocket close
#endif
#ifdef _WIN32
#include "winsock.h"
#endif
#define BUFLEN 300

//extern char lasttrace[90];

/*---------------------------------------------------------------------------*/

static char *cmdlist[] = {"TRACE", "MONITOR", "CMD","QUIT", NULL};

class serv
{
  public:
  void start_serv (char *name);
  private:
  char buffer[1024];
  char cmd[80];
  int badcmd;
  int finished;
  int monitor;
  int trace;
  int a;
  FILE* f;
  unsigned long pos;

  enum
  {
    _ERR = -1, _TRACE, _MONITOR, _CMD, _QUIT
  }
  _cmdlist;
  enum
  {
    _SIGTRACE, _SIGMONITOR, _SIGCMD, _SIGSTAT
  }
  _sig;
  int checkcmd (char *cmd, char **cmdlist);
  int extract_argument (char *s, int size);
  void putsig(int type);
};
  
/*---------------------------------------------------------------------------*/

int serv::checkcmd (char *cmd, char **cmdlist)
{
  int i = 0;
  while (cmdlist[i] != NULL)
  {
    if (!stricmp (cmd, cmdlist[i]))
    {
      badcmd = 0;
      return i;
    }
    i++;
  }
  return _ERR;
}

/*---------------------------------------------------------------------------*/

int serv::extract_argument (char *s, int size)
{
  char help[256];
  char *t = strstr (buffer, cmd) + strlen (cmd);
  sprintf (help, "%%%ds", size);
  s[0] = 0;
  sscanf (t, help, s);
  if (s[0])
    return 1;
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

void serv::putsig(int type)
{
   switch(type)
   {
     case _SIGTRACE:
          putf("\n%c%c\n",0xff,0xff);
          break;

     case _SIGMONITOR:
          putf("\n%c%c\n",0xff,0xfe);    
          break;

     case _SIGCMD:
          putf("\n%c%c\n",0xff,0xfd);
          break;
     case _SIGSTAT:
     default:
          putf("\n%c%c\n",0xff,0xfc);

   }
}

/*---------------------------------------------------------------------------*/

void serv::start_serv (char *name)
{
  unsigned c=0,d='\n';
  t->port=0;
  badcmd=0;
  finished=0;
  monitor=0;
  trace=0;
  b->sysop=1;
  //char tracebuf[90];
  
  f=s_fopen(TRACEPATH"/syslog_r.bcm","lrt");
  if(f)
  {
	  fseek(f,0,SEEK_END);
	  pos=ftell(f);
	  if(pos>1000)
	  {
		  fseek(f,-1000,SEEK_END);		  
		  while((a=fgetc(f))!='\n' && a!=EOF);
	  }
  }
  
  mblogin(m.boxname,login_silent,"SERV");
  putsig(_SIGSTAT);
  putf("BayCom Mailbox V"VNUMMER" Service Interface\n");
  while (!finished)
  {
    if(getvalid_sock())
    {
      getline (buffer, sizeof (buffer) - 1, 1);
      if (sscanf (buffer, "%31s", cmd) == 1)
        switch (checkcmd (cmd, cmdlist))
        {
          case _TRACE:
            if(extract_argument (cmd, sizeof (cmd) - 1))
            if(!stricmp(cmd,"on"))
              trace=1;
            else
              trace=0;
            putsig(_SIGSTAT);
            putf("TRACE %s\n",trace?"ON":"OFF");
            break;
          case _MONITOR:
            if(extract_argument (cmd, sizeof (cmd) - 1))
            if(!stricmp(cmd,"on"))
              monitor=1;
            else
              monitor=0;
            putsig(_SIGSTAT);
            putf("MONITOR %s\n",monitor?"ON":"OFF");
            break;
          case _CMD:
            putsig(_SIGCMD);
            mailbef(strstr(buffer,cmd)+strlen(cmd),0);
            break;
          case _QUIT:
            finished=1;
            putsig(_SIGSTAT);
            putf("Goodbye\n");
            break;
        }
    }
    
    if(trace && f)
    {
      if((a=fgetc(f))!=EOF)
      {
        putsig(_SIGTRACE);
        do
          putv(a);
        while((a=fgetc(f))!=EOF);
      }
    }
    if(monitor && getvalid_tnc())
    { 
      putsig(_SIGMONITOR);
      while(getvalid_tnc())
      {
        c=(unsigned char)getv_tnc();
        if(c==7)
          c='#';
        if(c>=(unsigned char)0xfd)
        { if(c==(unsigned char)0xff)
            c='\n';
          else
            c=0;
        }
        if(c=='\r')
          c='\n';      
        if(c)
        {
          putv(d);
          d=c;
        }
      }
    }
    wdelay (77);
  }
}

/*---------------------------------------------------------------------------*/

void mblogin_serv (char *name)
{
  serv s;
  strcpy (b->uplink, "SERV");
  strcpy (b->peerip, name);
  b->logincall[0]=0;
  s.start_serv (name);
  putflush_sock ();
  disconnect_sock ();
}

/*---------------------------------------------------------------------------*/

#endif
