/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  ----------------
  Sendmail for BCM   Reference: RFC 821
  ----------------


  Copyright (C)       Deti Fliegl, deti@lrz.de, DG9MHZ
                      Guardinistr. 47
                      81375 Muenchen

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980121 DG9MHZ Struktur msg wurde nicht korrekt geloescht,
//                im Subj war gegegentlich Gammel hinten dran (fixed)
//19980126 OE3DZW fixed Subj again, Struktur
//19980505 DG9MHZ smtp only after pop3
//19980506 OE3DZW changes to pop3db
//19980514 OE3DZW changed smtp-message, string was wrong
//19990818 OE3DZW user may disable pop before smtp

#include "baycom.h"
#define MAX_RCPTCNT 100

#ifdef __UNIX__
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#define closesocket close
#endif
#ifdef _WIN32
#include "winsock.h"
#endif

/*---------------------------------------------------------------------------*/

static char *cmdlist[] =
{"HELO", "EHLO", "MAIL", "RCPT", "DATA", "RSET", "NOOP", "QUIT", NULL};
typedef struct
{
    char *rcpt[MAX_RCPTCNT];
    char from[65];
    char subject[256];
    char partnername[65];
    int rcptcnt;
} msg_t;

class smtp
{
  public:
  void start_smtp (char *name);
  void onkill(void);

    private:
  enum
  {
    _ERR = -1, _HELO, _EHLO, _MAIL, _RCPT, _DATA, _RSET, _NOOP, _QUIT
  }
  _cmdlist;
  int checkcmd (char *cmd, char **cmdlist);
  int popped_before ();
  int send_msg (void);
  char *extract_call (char *line,int from);
  void rset(void);
  void charset (char *s);
  FILE *f;
//  char username[256];
  char tmpname[40];
  char buffer[1024];
  char cmd[32];
  msg_t msg;
  int finish;
  int badcmd;
};

/*---------------------------------------------------------------------------*/

void smtp::charset (char *s)
{
  char *result = s;
  for (; result[0]; result++)
  {
    switch (result[0])

    {
    case 0xe4:
      result[0] = 0x84;
      break;                    //ae

    case 0xf6:
      result[0] = 0x94;
      break;                    //oe

    case 0xfc:
      result[0] = 0x81;
      break;                    //ue

    case 0xc4:
      result[0] = 0x8e;
      break;                    //Ae

    case 0xd6:
      result[0] = 0x99;
      break;                    //Oe

    case 0xdc:
      result[0] = 0x9a;
      break;                    //Ue

    case 0xdf:
      result[0] = 0xe1;
      break;                    //ss

    }
  }
}

/*---------------------------------------------------------------------------*/

int smtp::popped_before(void)
{
  char peeraddr[32]="";
  char name[32];
  time_t lastpop;
  char *s = extract_call (msg.from,1);
  int yes=FALSE;

  if (get_smtp_security(s)==1) //dzw
    return TRUE;

  f=s_fopen(POPDBNAME,"srt");
  if(f==NULL)
    return FALSE;

  while(fscanf(f,"%31s\t%08lX\t%31s",peeraddr,&lastpop,name)==3)
  {
    //printf("hier0: %s %s\n",peeraddr,name);
    if(!strcmp(peeraddr,b->peerip))
    {
      //printf("hier1: %s %s %s %lX %lX\n",peeraddr,name,s,ad_time(),lastpop);
      if(s[0] && !stricmp(s,name) && ((ad_time()-lastpop) < 60*60) ) 
      {
        //printf("hier2: %s %s\n",peeraddr,name);
        yes=TRUE;
        break;
      }
    }
  }

  s_fclose(f);
  return yes;
}

/*---------------------------------------------------------------------------*/

int smtp::checkcmd (char *cmd, char **cmdlist)
{
  int i = atoi (cmd);

  if (i >= 200 && i < 600)
    return i;

  i = 0;
  while (cmdlist[i] != NULL)
  {
    if (!stricmp (cmd, cmdlist[i]))
    {
      badcmd = 0;
      return i;
    }
    i++;
  }
  return _ERR;
}

/*---------------------------------------------------------------------------*/

char *smtp::extract_call (char *line,int from)
{
  static char backup[65];
  char *t,*u,*pkt;
  strncpy (backup, line, 64);
  backup[64]=0;
  char *s = strchr (backup, '<');
  if (s == NULL)
    return "";
  s++;
  if (t = strchr (s, '>'))
    t[0] = 0;
  else
    return "";
  t = strchr (s, '@'); //eg . oe3dzw@oe3xsr>
  u = strchr (s, '>'); 
  if (t == NULL)
    return s;
  if (from)
  { t[0]=0;
    return s;
  }
  if (t && u && u > t) // ">" at end
     u[0] = 0; 
  // remove @?
  pkt = strchr(t+1,'.');
  if (pkt) 
     pkt[0]=0; // eg. oe3xsr.xx.xx -> oe3xsr
  
  return s;
}

/*---------------------------------------------------------------------------*/

int smtp::send_msg (void)
{
  char login[32];
  char *s;
  int i;
  int j = 0;
  io_t tty = t->output;
  s = extract_call (msg.from,1);
  if (!s[0])
    return 0;
 
  strncpy (login, s, 31);
  login[31]=0;

  t->output = io_dummy;
  mblogin (login, login_silent, "SMTP");
  for (i = 0; i < msg.rcptcnt; i++)
  {
    s = extract_call (msg.rcpt[i],0);
    if (s[0])
    {
      b->oldinput = (char)t->input;
      sysimport (tmpname);
      sprintf (buffer, "%s %s", s, msg.subject);
      b->mailtype='?';
      b->job=j_send;
      mbsend (buffer, 1);  
      j++;
      s_fclose (b->inputfile);
      t->input = (io_t) b->oldinput;
    }
  }
  t->output = tty;
  if (j)
    return 1;
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

int myhostname (char *s, int len)
{
  if (gethostname (s, len - 2) == -1)
    return -1;
  return 1;
}

/*---------------------------------------------------------------------------*/

void smtp::rset(void)
{
  for (int i = 0; i < msg.rcptcnt; i++)
    t_free (msg.rcpt[i]);
  memset(&msg,0,sizeof(msg_t));
  /*
  msg.from[0] = 0;
  msg.partnername[0] = 0;
  msg.rcptcnt = 0;
  msg.subject[0] = 0;
  */
}

/*---------------------------------------------------------------------------*/

void smtp::onkill(void)
{
  if (f && tmpname[0])
  {
    s_fclose (f);
    unlink (tmpname);
  }
}

/*---------------------------------------------------------------------------*/

static void onkill(void *p)
{
  ((smtp *)p)->onkill();
}

/*---------------------------------------------------------------------------*/

void smtp::start_smtp (char *name)
{
  msg.from[0] = 0;
  msg.partnername[0] = 0;
  msg.subject[0] = 0;
  msg.rcptcnt = 0;
  f=NULL;
  finish = 0;
  badcmd = 0;
  tmpname[0]=0;

  if (myhostname (buffer, sizeof (buffer) - 1) == -1)
    return;
  
  rset(); //OE3DZW clear buffer

  atkill(::onkill,(void *)this);
  putf ("220 %s BayCom Mailbox V" VNUMMER " SMTP Server\n", buffer);
  
  while (!runterfahren)
  { getline (buffer, sizeof (buffer) - 1, 1);
    //printf("%s\n",buffer);
    if (sscanf (buffer, "%31s", cmd) == 1)
      switch (checkcmd (cmd, cmdlist))
      {
      case _EHLO:
      case _HELO:
        sscanf (strstr (buffer, cmd) + strlen (cmd), "%64s", msg.partnername);
        if (msg.partnername[0])
        {
          myhostname (buffer, sizeof (buffer) - 1);
          putf ("250 %s Hello, pleased to meet you\n", buffer);
        }
        else
          putf ("501 HELO requires domain address\n");
        break;

      case _MAIL:
        if (msg.from[0])
        {
          putf ("503 Sender already specified\n");
          break;
        }
        strncpy(msg.from,strstr (buffer, cmd) + strlen (cmd),63);
        msg.from[63]=0;
        //if (!sscanf (strstr (buffer, cmd) + strlen (cmd), "%64s", msg.from))
        //  break;
        if (stristr(msg.from, "FROM:"))
        {
          char *t=strchr(msg.from,'<');
          if(t==NULL)
           goto err;
          char *s = extract_call (t,1);
          if (s[0] && mbcallok (s))
          { char sendercall[CALLEN+1];
            strncpy(sendercall,s,sizeof(sendercall)-1);
            sendercall[sizeof(sendercall)-1]=0;
            if(popped_before())
            {
              putf ("250 %s Sender ok\n", sendercall);
              break;
            }
            else
            { pwlog(s,b->peerip,"smtp w/o pop");
              putf ("552 %s has to POP before SMTP.\n", sendercall);
            }
          }
          else
            putf ("552 %s contains no valid ham-call\n", msg.from);
        }
        else
          err:putf ("501 Syntax error in parameters scanning \"%s\"\n", msg.from);
        msg.from[0] = 0;
        break;

      case _RCPT:
        if (msg.rcptcnt < MAX_RCPTCNT)
        {
          char localbuf[65];
          localbuf[0]=0;
          strncpy(localbuf,strstr(buffer,cmd)+strlen(cmd),63);
          localbuf[63]=0;
          //if (!sscanf (strstr(buffer,cmd)+strlen(cmd), "%64s", localbuf))
          //  break;
          if (stricmp (localbuf, "TO:"))
          {
            char *s = msg.rcpt[msg.rcptcnt] = (char *) t_malloc (65, "SMTP:RCPT line");
            if (s)
            {
              strcpy (s, localbuf);
              msg.rcptcnt++;
              putf ("250 OK\n");
            }
            else
              goto tomany;
          }
          else
            putf ("501 Syntax error in parameters scanning \"%s\"\n",localbuf);
        }
        else
        tomany:putf ("552 To many recipients.\n");
        break;

      case _DATA:
        if (msg.rcptcnt && msg.from[0])
        {
          char *s;
          int header = 1;
          sprintf(tmpname, "%s/%s.imp",TEMPPATH,strlwr (time2filename (0)));
          f = s_fopen (tmpname, "swt");
          if (f == NULL)
          {
            putf ("452 Insufficient system storage\n");
            break;
          }
          putf ("354 Start mail input; end with <CRLF>.<CRLF>\n");
          
          getline (buffer, sizeof (buffer) - 1, 1);
          //printf("erstes getline: %s\n",buffer);
          if (buffer[0])
            while (1)
            {
              if (!buffer[0])
                header = 0;
              if (strlen (buffer) == 1 && !strcmp (buffer, "."))
                break;
              if (buffer[0] == '.' && strlen (buffer) > 1)
                memmove (buffer, buffer + 1, strlen (buffer) - 1);
              if (header && (s = stristr (buffer, "subject: ")) != NULL)
              { strncpy (msg.subject, s + 9, 81);
                msg.subject[81]=0;
              }

              charset (buffer);
              fprintf (f, "%s\n", buffer);
              getline (buffer, sizeof (buffer) - 1, 1);
              //inputline_sock(buffer,sizeof (buffer) - 1);
              //printf("%s\n",buffer);
            }
          
          fprintf (f, "\nnnnn\nend\n");
          s_fclose (f);
          f=NULL;
          if (send_msg ())
            putf ("250 Message accepted for delivery\n");
          else
            putf ("451 Local error in processing\n");
          unlink (tmpname);
          rset();
        }
        else
          putf ("503 Bad sequence of commands\n");
        break;
      case _RSET:
       rset();
      case _NOOP:
        putf ("250 OK\n");
        break;
      case _QUIT:
        myhostname (buffer, sizeof (buffer) - 1);
        putf ("221 %s Server closing transmission channel\n", buffer);
        finish = 1;
        break;
      default:
        if (badcmd > 30)
        {
          putf ("421 Too many bad commands; closing connection\n");
          finish = 1;
        }
        putf ("500 Command unrecognized\n");
        badcmd++;
      }
    if (finish)
      break;
  }
 
  for (int i = 0; i < msg.rcptcnt; i++)
    t_free (msg.rcpt[i]);
}

/*---------------------------------------------------------------------------*/

void mblogin_smtp (char *name)
{
  smtp sm;
  strcpy (b->uplink, "SMTP");
  strcpy (b->peerip, name);
  b->headermode=_SMTP;
  b->logincall[0]=0;
  sm.start_smtp (name);
  putflush_sock ();
  disconnect_sock ();
}

/*---------------------------------------------------------------------------*/
