/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  --------------------------------
  Interface zur socket-Ansteuerung
  --------------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980124 DG9MHZ/OE3DZW send()->-1 means buffer full -> no err
//19980211 OE3DZW added option for SERVIF
//19980223 DG2FEF/OE3DZW fixed send() again, hope this helps
//19980301 OE3DZW removed suicide, therefore changed trace to "fatal"
//19980310 DG9MHZ fixed EAGAIN,EINTR
//19980311 OE3DZW more nice err-msges
//19980318 OE3DZW fixed peerips - trailing 0x00 was missing
//19980323 DG9MHZ fixed Win32-version
//19980418 OE3DZW added show_sockets
//19980506 OE3DZW update show_sokets
//19980609 OE3DZW removed "feature" for db0rgb
//19980831 OE3DZW addrlen is signed again (was unsigned in bcm140b)
//19980905 OE3DZW fix from 19980301 was no good idea, removed
//19980914 OE3DZW added unsigned addrlen for glibc, from bcm1.40c (dh3mb)
//19990215 OE3DZW default rhosts.bcm will be generated; comments allowed (;)
//19990220 DK2UI  added define for tcp/ip timeout

#include "baycom.h"
#define TIMEOUT 120
/*
   class test
   {
   public:
   ~test();
   test();
   };

   test::~test()
   {
   }

   test::test()
   { 
   }
 */

/*---------------------------------------------------------------------------*/

#ifdef __DOS16__
int putfree_sock (void)
{ return 1;
}

void putv_sock (int c)
{
}

void putflush_sock (void)
{
}

int getvalid_sock (void)
{ return 0;
}

int getv_sock (void)
{ return EOF;
}

void inputline_sock (char *s, int maxlen, char cut)
{
}

void logindaemon_sock (char *name)
{
}

void disconnect_sock (void)
{
}

void show_sock(void)
{
}

void html_putf (char *,...)
{
}
#else

/*---------------------------------------------------------------------------*/

#ifdef __UNIX__
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#define closesocket close
#endif
#ifdef _WIN32
#include "winsock.h"
#undef errno
#define errno h_errno
WORD wVersionRequested;
WSADATA wsaData;
u_long mode = FIONBIO;
#endif

/*---------------------------------------------------------------------------*/

#define TXLEN 512
#define RXLEN 512

/*---------------------------------------------------------------------------*/

typedef struct sock_t
{
  char rxbuf[RXLEN];
  char txbuf[TXLEN];
  int handle;
  int rxc;
  int txc;
  int rxlen;
  int wasput;
  int wasda;
  char dasda;
  time_t lastflush;
  time_t tcreate; //time when socket was created
}
sock_t;

#define MAXSOCK 256
static sock_t so[MAXSOCK];

/*---------------------------------------------------------------------------*/

char *in_ntoa (unsigned long in)
//*************************************************************************
//
//  Display an IP address in readable format
//
//  from Linux-Kernel net/utils.c
//
//*************************************************************************
{ static char buff[18];
  char *p;

  p = (char *) &in;
  sprintf (buff, "%d.%d.%d.%d",
           (p[0] & 255), (p[1] & 255), (p[2] & 255), (p[3] & 255));
  return (buff);
}

/*---------------------------------------------------------------------------*/

int find_free_sock (void)
{
  for (int i = 1; i < MAXSOCK; i++)
    if (so[i].handle == EOF)
      return i;
  return -1;
}

/*---------------------------------------------------------------------------*/

void show_sock(void)
{ int head=FALSE;
  for (int i=1;i< MAXSOCK;i++)
    if (so[i].handle != EOF)
    { if (!head)
      { putf("Nr ha rxc txc created\n"); 
        head=TRUE;
      }
      putf("%2d %2d %3d %3d %s\n",
             i,so[i].handle,so[i].rxc,so[i].txc,rfc_date(so[i].tcreate));
    }
  if (!head)
    putf("No sockets active.\n");
}

/*---------------------------------------------------------------------------*/

int putfree_sock (void)
{
  return 1;
}

/*---------------------------------------------------------------------------*/

void putv_sock (int c)
//*************************************************************************
//
//
//
//*************************************************************************
{ sock_t *s = &so[t->socket];
  s->txbuf[s->txc++] = c;
  s->wasput = 1;
  if (s->txc >= TXLEN)
    putflush_sock ();
}

/*---------------------------------------------------------------------------*/

void putflush_sock (void)
//*************************************************************************
//
//
//
//*************************************************************************
//#ifdef __LINUX__
{ sock_t *s = &so[t->socket];
  int ok = 0;
  int sum = 0;

  if (s->txc)
  {
    timeout (TIMEOUT);
    while (1)
    {
      fd_set wfds;
      struct timeval tv;
      tv.tv_sec = 0;
      tv.tv_usec = 0;

      FD_ZERO(&wfds);
      FD_SET(s->handle, &wfds);

      if(select(s->handle + 1, NULL, &wfds , NULL ,&tv))
      {
        //ok = write(s->handle, s->txbuf + sum, s->txc - sum);
		  ok = send(s->handle, s->txbuf + sum, s->txc - sum,0);
        if (ok < 0)   
        { 
#ifdef _WIN32
			if (errno && (errno != EWOULDBLOCK))
#endif
#ifdef __LINUX__
			if (errno!=EAGAIN && errno!=EINTR)
#endif
         { trace(report,"putflush_sock","kill %d: %s",errno,strerror(errno));
           suicide();
         }
        }
        else 
          sum += ok;
        if (sum == s->txc)
          break;
      }
      wdelay (11);
    }
    s->txc = 0;
  }
}
//#endif
/*
 #ifdef _WIN32
{
  sock_t *s = &so[t->socket];
  int ok = 0;
  int sum = 0;

  if (s->txc)
  {
    timeout (TIMEOUT);
    t->last_input=ad_time();
    while (1)
    {
      ok = send (s->handle, s->txbuf + sum, s->txc - sum, 0);
        if (errno && (errno != EWOULDBLOCK))
        {
          trace (report, "sock_send", "kill: %d %s", errno, strerror (errno));
          suicude();
        }
      if (ok > 0)
        sum += ok;
      if (sum == s->txc)
        break;
      wdelay (11);
    }
    s->txc = 0;
  }
}
#endif
*/

/*---------------------------------------------------------------------------*/

int holsock (sock_t * s)
{ int ret;
  if (s->rxc >= s->rxlen)
  {
    fd_set rfds;
    struct timeval tv;
    tv.tv_sec = 0;
    tv.tv_usec = 0;

    FD_ZERO(&rfds);
    FD_SET(s->handle, &rfds);

    int ret=select(s->handle + 1, &rfds, NULL , NULL ,&tv);
    if(!ret)
		return EOF;
    
	//if ((s->rxlen = read (s->handle, s->rxbuf, RXLEN)) < 1)
	if ((s->rxlen = recv (s->handle, s->rxbuf, RXLEN,0)) < 1)
    {
        if(!s->rxlen && ret)
          suicide(); 
      
      if ((
#ifdef __LINUX__
            s->rxlen == EOF ||
#endif
            errno == ENOTCONN || errno == EBADF)
#ifdef __LINUX__
          && errno != EAGAIN
#endif
        )
      {
        trace (report, "holsock", "kill: %d %s", errno, strerror (errno));
        suicide();
      }
      return EOF;
    }
    s->rxc = 0;
  }
  ret = s->rxbuf[s->rxc++];
  if (ret == 10)
    return EOF;
  else
    return ret;
}

/*---------------------------------------------------------------------------*/

int getvalid_sock (void)
//*************************************************************************
//
//  prueft ob ein neues Zeichen vom Socket gekommen ist
//
//*************************************************************************
{ int a;
  sock_t *s = &so[t->socket];

  if (s->txc && s->lastflush != ad_time ())
  {
    s->lastflush = ad_time ();
    putflush_sock ();
  }

  if (s->wasda)
  {
    if (s->dasda == 13)
      return CR;
    else
      return 1;
  }
  //a = holsock (s->handle);
  a = holsock (s);
  if (a != EOF)
  {
    s->wasda = 1;
    s->dasda = (char) a;
  }
  if (s->wasda)
  {
    if (s->dasda == 13)
      return CR;
    else
      return 1;
  }
  return 0;
}

/*---------------------------------------------------------------------------*/

int getv_sock (void)
{ int a;
  sock_t *s = &so[t->socket];
  timeout (TIMEOUT);
  t->last_input=ad_time();
  if (s->wasda)
  {    
    s->wasda = 0;
    return s->dasda;
  }
  while ((a = holsock (s)) == EOF)
    wdelay (77);
  return a;
}

/*---------------------------------------------------------------------------*/

void inputline_sock(char *s, int maxlen, char cut)
{ int i=0;
  int semi=0;

  if(maxlen<0)
  { if(cut)
      semi=1;
    maxlen=(-maxlen);
  }

  if(so[t->socket].wasput && !getvalid_sock())
    putflush_sock();

  while(1)
  { if(!cut && i==maxlen)
      break;

    do
    { s[i]=getv_sock();  
      //trace(report,"inputline_sock","s[%d]=%02X=%c",i,s[i],s[i]);
    }
    while(s[i]==LF);

    if(semi && s[i]==';')
      break;

    if(s[i]==CR)
    { if(!cut)
        s[i++]=LF;
      break;
    }

    if((s[i]==8) || (s[i]==127))
    { if(i)
        i--;
      continue;
    }

    if(i<maxlen)
      i++;
  }

  s[i]=0;
}

/*---------------------------------------------------------------------------*/

void disconnect_sock (void)
{ closesocket (so[t->socket].handle);
  so[t->socket].handle = EOF;
  t->socket = 0;
}

/*---------------------------------------------------------------------------*/

void mblogin_socket (char *name)
{ char s[10];
  char s1[10]="";
  int unknown = name[0] == '*';
  if (unknown)
    name++;
  char *up;
  //if (strcmp (m.boxname, "DB0RGB") == 0)
  //  up = "DB0RGB";
  //else
    up = "TCP/IP";

  int ok=getlogin (s, unknown, up);
  if (ok)
  {
    b->sysop = !unknown;
    strcpy (b->peerip, name);
    strcpy (t->name, s);
    if (ok==2)
      strcpy(s1,m.mycall[1]);    
    mbmain (s, up, s1);
  }
  wdelay (320);
  disconnect_sock ();
}

/*---------------------------------------------------------------------------*/

int checkrhost (long addr)
{ FILE *f = s_fopen (RHOSTSNAME, "srt");
  char s[100];

  static char rhosts_init[]=
  "; sample rhosts.bcm file\n"
  "; might be unsecure!\n"
  ";\n" 
  "127.0.0.1\n";

  if (!f)
  {
    f=s_fopen(RHOSTSNAME,"sw+t");
    if (f)
    { fwrite(rhosts_init,sizeof(rhosts_init)-1,1,f);
      trace(replog,"checkrhost","sample "RHOSTSNAME" generated");
      rewind(f);
    }   
  }
  if (f)
  {
    struct hostent *hp;

    while (fgets(s,99,f))
    { s[99]=0;
      subst(s,';',0); //remove comments
      subst(s,'\n',0);
      subst(s,'\r',0);
      if (s[0]==0)
        continue;
      hp = gethostbyname (s);
      if (hp && addr == ((struct in_addr *) (hp->h_addr))->s_addr)
      {
        trace (report, "checkrhost", "%s found", s);
        s_fclose (f);
        return 1;
      }
    }
    s_fclose (f);
  }
  return 0;
}

/*---------------------------------------------------------------------------*/

#ifdef _WIN32

void cleanup (void)
{ WSACleanup ();
}

/*---------------------------------------------------------------------------*/

void ws_init (void)
{ static int first = 1;
  if (first)
  {
    first = 0;
    wVersionRequested = MAKEWORD (1, 1);
    if (WSAStartup (wVersionRequested, &wsaData) != 0)
      trace (tr_abbruch, "ws_init", "winsock.dll not found");
    atexit (cleanup);
    if (LOBYTE (wsaData.wVersion) != 1 || HIBYTE (wsaData.wVersion) != 1)
      trace (tr_abbruch, "ws_init", "wrong winsocket implementation");
    trace (report, "ws_init", "Sockets %u, UDP size %u",
           wsaData.iMaxSockets, wsaData.iMaxUdpDg);
  }
}
#else
#define ws_init()
#endif

/*---------------------------------------------------------------------------*/

void logindaemon_sock (char *name)
{ static int sockinit = 0;

#ifdef __GLIBC__  
  unsigned
#endif
  int addrlen;   
  int i, s=0, ls;
  struct servent *sp;           /* pointer to service information */

  struct sockaddr_in myaddr_in; /* for local socket address */
  struct sockaddr_in peeraddr_in;  /* for peer socket address */
  long strue = 1;
  unsigned long peerip = 0;
  char peerips[17];
  int service = _TELNET;

  if (!sockinit)
  {
    sockinit++;
    for (i = 0; i < MAXSOCK; i++)
      so[i].handle = EOF;
  }

  if (strcmp (name, "httpd") == 0)
    service = _HTTP;

  if (strcmp (name, "sendmail") == 0)
    service = _SMTP;

  if (strcmp (name, "popper") == 0)
    service = _POP;

  if (strcmp (name, "nntpd") == 0)
    service = _NNTP;


  if (strcmp (name, "radiod") == 0)
    service = _RADIO;

  if (strcmp (name, "serv") == 0)
    service = _SERV;

//  wdelay(1198);

  ws_init ();

  /* clear out address structures */
  memset ((char *) &myaddr_in, 0, sizeof (struct sockaddr_in));
  memset ((char *) &peeraddr_in, 0, sizeof (struct sockaddr_in));

  myaddr_in.sin_family = AF_INET;
  myaddr_in.sin_addr.s_addr = INADDR_ANY;

  switch (service)
  {
  case _HTTP:
    if (m.http_port == 0)
    {
      trace (report, name, "http connectivity disabled");
      return;
    }
    else
      myaddr_in.sin_port = htons (m.http_port);
    break;

  case _SMTP:
    if (m.smtp_port == 0)
    {
      trace (report, name, "smtp connectivity disabled");
      return;
    }
    else
      myaddr_in.sin_port = htons (m.smtp_port);
    break;

  case _POP:
    if (m.pop3_port == 0)
    {
      trace (report, name, "pop3 connectivity disabled");
      return;
    }
    myaddr_in.sin_port = htons (m.pop3_port);
    break;

  case _NNTP:
    if (m.nntp_port == 0)
    {
      trace (report, name, "nntp connectivity disabled");
      return;
    }
    else
      myaddr_in.sin_port = htons (m.nntp_port);
    break;


  case _RADIO:
    #ifdef RADIOIF
    if (m.radio_port == 0)
    {
      trace (report, name, "radio connectivity disabled");
      return;
    }
    else
      myaddr_in.sin_port = htons (m.radio_port);
    break;
    #else
      return;
    #endif

  case _SERV:
    #ifdef SERVIF
    if (m.serv_port == 0)
    {
      trace (report, name, "remote service connectivity disabled");
      return;
    }
    else
      myaddr_in.sin_port = htons (m.serv_port);
    break;
    #else
    return;
    #endif

  case _TELNET:
    if (m.tcp_port == 0)
    {
      trace (report, name, "telnet connectivity disabled");
      return;
    }
    else if (m.tcp_port == 1)
    {
      m.tcp_port = DEFTELNPORT;
#ifdef _WIN32
      sp = NULL;
#else
      sp = getservbyname ("bcm", "tcp");
#endif
      if (sp == NULL)
      {
        trace (report, name, "bcm not found in /etc/services");
        myaddr_in.sin_port = htons (DEFTELNPORT);
      }
      else
        myaddr_in.sin_port = sp->s_port;
    }
    else
      myaddr_in.sin_port = htons (m.tcp_port);
    break;
  }

//------------------
  ls = socket (AF_INET, SOCK_STREAM, 0);
  if (ls == -1)
  {
    trace (serious, name, "create: d% %s", errno,strerror (errno));
    return;
  }
#ifdef _WIN32
  //WINDOWS Specific!!
  if (ioctlsocket (ls, FIONBIO, &mode) != 0)
  {
    trace (serious, name, "ioctlsocket: %d %s", errno, strerror (errno));
    return;
  }
#else                                                          
  if (fcntl (ls, F_SETFL, fcntl(s,F_GETFL) | O_NONBLOCK) < 0) //dg2fef
  {
    trace (serious, name, "fcntl: %d %s", errno,strerror (errno));
    return;
  }
  if (setsockopt (ls, SOL_SOCKET, SO_REUSEADDR, &strue, 4) == -1)
  {
    trace (tr_abbruch, name, "setsockopt: %d %s", errno,strerror (errno));
    return;
  }
#endif
  if (bind (ls, (struct sockaddr *) &myaddr_in, sizeof (struct sockaddr_in)) == -1)
  {
    trace (tr_abbruch, name, "bind: %d %s", errno, strerror (errno));
    return;
  }
  if (listen (ls, 5) == -1)
  {
    trace (serious, name, "listen: %d %s", errno, strerror (errno));
    return;
  }
  trace (report, name, "initialised to port %d", ntohs (myaddr_in.sin_port));

  while (!runterfahren)
  {
    addrlen = sizeof (struct sockaddr_in);
    fd_set rfds;
    struct timeval tv;
    tv.tv_sec = 0;
    tv.tv_usec = 0;
    
    wdelay (79);
    
    FD_ZERO(&rfds);
    FD_SET(ls, &rfds);
    
    if(!select(ls + 1, &rfds, NULL , NULL ,&tv))
      continue;
    
    s = accept (ls, (struct sockaddr *) &peeraddr_in, &addrlen);
    if (s != EOF && (i = find_free_sock ()) != -1)
    {
      memset (&so[i], 0, sizeof (sock_t));
      so[i].handle = s;
      so[i].tcreate = ad_time();
#ifdef _WIN32
      //WINDOWS Specific!!
      if (ioctlsocket (s, FIONBIO, &mode) != 0)
      {
       trace (serious, name, "fcntl2: %d %s", errno, strerror (errno));
       continue;
      }
#else
      if (fcntl (s, F_SETFL, O_NONBLOCK) < 0)
      {
        trace (serious, name, "fcntl2: %d %s", errno, strerror (errno));
        continue;
      }
#endif
      peerip = ((struct sockaddr_in) peeraddr_in).sin_addr.s_addr;
      strcpy (peerips, in_ntoa (peerip));
      trace (report, name, "socket %d / ip: [%s]", s, peerips);
#ifdef __UNIX__
      struct linger linger;     /* allow a lingering, graceful close; */
      linger.l_onoff = 1;
      linger.l_linger = 1;
      if (setsockopt (s, SOL_SOCKET, SO_LINGER, &linger, sizeof (linger)) == -1)
      {
        trace (serious, name, "setsockopt: %d %s", errno, strerror (errno));
        continue;
      }
#endif

      switch (service)
      {

      #ifdef SERVIF
      case _SERV:
        if (checkrhost (((struct sockaddr_in) peeraddr_in).sin_addr.s_addr))
          fork (P_MAIL, -i, mblogin_serv, peerips);
        break;
      #endif

      case _HTTP:
        fork (P_MAIL, -i, mblogin_http, peerips);
        break;

      case _SMTP:
        fork (P_MAIL, -i, mblogin_smtp, peerips);
        break;

      case _POP:
        fork (P_MAIL, -i, mblogin_pop3, peerips);
        break;

      case _NNTP:
        fork (P_MAIL, -i, mblogin_nntp, peerips);
        break;

      #ifdef RADIOIF
      case _RADIO:
        if (checkrhost (((struct sockaddr_in) peeraddr_in).sin_addr.s_addr))
          fork (P_MAIL, -i, mblogin_radio, peerips);
        break;
      #endif

      case _TELNET:
        int wellknown = checkrhost (((struct sockaddr_in) peeraddr_in).sin_addr.s_addr);
        if (wellknown == 0)
        {
          memmove (peerips + 1, peerips, strlen (peerips)+1);
          peerips[0] = '*';
          peerips[17] = 0 ;
        }
        fork (P_MAIL, -i, mblogin_socket, peerips);
        break;
      }
    }

  }
  closesocket (ls);
}

#endif
