/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Mailbox


  ----------------------
  Multitasking-Scheduler
  ----------------------


  Copyright (c)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

#include "baycom.h"

//19980202 OE3DZW added " " in "users" for fbb
//19980301 hrx    added dashes to ps-command
//19980307 OE3DZW moved "t->port=0" to position after disconnect -> 
//                fwdlog will show port of logout
//19980311 DG9MHZ changes for support of Linux AX25 Kernel
//19980329 OE3DZW ax25kshutdown only for linux
//19980420 OE3DZW removed dashes after protests of flori,deti,chris and me
//19980428 DG4MMI update of mailserver for dos (task cloning)
//19990323 OE3DZW changed define for ax25kshutdown (was linux)
//19990422 OE3DZW removed some debug tjob code, left 92/93
//19991028 Jan    fixed parm handling for new tasks
//19991111 OE3DZW removed ax25kshutdown from post-mortem-dump

/*---------------------------------------------------------------------------*/

// global sichtbare Variablen
//
task_t *tfeld[TASKS]; // Zeiger aus die Task-Kontrollblcke
task_t *t=NULL;       // Zeiger auf aktuellen TCB
tvar_t *b;
user_t *u;
int allupdate=0;   // gesetzt wenn alle Fenster neu geschrieben werden
int newkeywin=0;   // enthlt neues Fenster, Umschaltung im Scheduler
int deskkey=0;     // enthlt Taste, dievom Desktop ausgewertet wird.
int keytaskid=0;   // Task-Nummer, zu der die Tastatur geleitet wird.
int init_off=0;    // Flag zum stoppen des Schedulers

int      runterfahren=0;
char     *stopreason="??";
char     abortreason[60];

/*---------------------------------------------------------------------------*/

// Scheduler lokale Variablen
//
static int      taskrunning=NOTASK;        // Index fuer laufende Task
static jmp_buf jb;

static int waitstack[TASKS+1];  // Warteschlange fuer laufende Prozesse
static int waitsp=0;            // Zeiger in Warteschlange
static int nextkill=0;          // task-id die gelscht werden soll.

static unsigned updtic=0;       // Timer fr zwangsweisen Fenster-update
static unsigned alltic=0;       // Timer fr zwangsweisen Bildschirm-update

/*---------------------------------------------------------------------------*/

// Parameterblock fr neue Tasks. Diese mssen statisch gespeichert werden,
// damit sie das Umschalten des Stack-Kontextes berleben
static int      newtask;           // Flag: Task mu erzeugt werden
static int     *newtasknum;        // Pointer auf TID von neu erzeugter Task
static int      newport;           // ggf. TNC-Verbindug erzeugen
static unsigned newkind;           // Bitfeld fr Task-Resourcen
static void     (*start)(char *);  // Zeiger auf Startfunktion
static char     *param;            // Zeiger auf Parameter der Startfunktion
static task_t   *newtaskclone;     // Zeiger auf TCB des zu clonenden Tasks
                                  
static unsigned tjob=0;   // Merker fr Stelle, die momentan bearbeitet wird
                          // (Dient nur zum Debugging)

static int maxstackdepth=TASKSTACK;       // Schleppzeiger fuer Stacktiefe

/*---------------------------------------------------------------------------*/

void t_lock(void)
//**************************************************************************
//
//  Sperrt einen Prozess gegen Kill und Taskumschaltung
//
//**************************************************************************
{ if(t)
    t->lock++;
}

/*---------------------------------------------------------------------------*/

void t_unlock(int force)
//*************************************************************************
//
//  Gibt einen Prozess wieder zum Kontextwechsel frei
//
//*************************************************************************
{ if(t)
  { if(force)
      t->lock=0;
    else
    { if(t->lock)
        t->lock--;
      else
        trace(serious,"t_unlock","no lock %d \"%s\"",t->taskid,t->name);
    }
  }
}

/*---------------------------------------------------------------------------*/

void suicide(void)
//*************************************************************************
//
//  killt den momentan laufenden Prozess
//
//*************************************************************************
{ if(taskrunning!=NOTASK)
  { while(nextkill) wdelay(249);   // Warten bis Funktion frei ist
    nextkill=taskrunning;
    wdelay(24096);                 // Rescheduling erzwingen
  }                                // (kommt nie zurck)
}

/*---------------------------------------------------------------------------*/

char *gettaskname(int tid)
{ if(tid>0 && tid<TASKS)
  {  if(tfeld[tid])
       return tfeld[tid]->name;
  }
  return "SYSTEM";
}

/*---------------------------------------------------------------------------*/

int gettaskid(void)
//*************************************************************************
//
//  momentane TASK-ID zurckliefern (zur Kapselung)
//
//*************************************************************************
{ return taskrunning;
}

/*---------------------------------------------------------------------------*/

static void near inittcb(int taskid)
//*************************************************************************
//
//  allokiert einen Task-Kontrollblock
//  besetzt den Stack-Bereich mit 0xaa vor, um
//  im Betrieb die Stacktiefe feststellbar zu machen.
//
//*************************************************************************
{ task_t *t;
  int size=sizeof(task_t);
  if(newkind & P_MAIL)       // Wenn Mailbox, dann erweiterten TCB
    size=sizeof(tvar_t);

  tfeld[taskid]=(task_t *)t_malloc(size,"task");
  t=tfeld[taskid];

  if(newtaskclone)
  { memcpy(t,newtaskclone,size);               // Copy TCB
    t->port=0;
    t->socket=0;
    t->processkind&=~P_AX25K;
    t->input=io_dummy;
    t->output=io_dummy;
    newtaskclone=0;
  }
  else
    memset(t,0,size);                          // definiert vorbesetzen
  memset(t->stack,0xaa,TASKSTACK);
  t->stack[TASKSTACK]=0;
  t->taskid=taskid;
}

/*---------------------------------------------------------------------------*/

int taskkbhit(void)
//*************************************************************************
//
//  Gibt einen Tastendruck abhngig vom Task-Kontext zurck
//
//*************************************************************************
{ if(t && (t->keybufin!=t->keybufout))
  { if(t->keybuffer[t->keybufout]&255)
      return t->keybuffer[t->keybufout]&255;
    else
      return t->keybuffer[t->keybufout]>>8;
  }
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

int iskeytask(void)
//*************************************************************************
//
//  Gibt zurck, ob momentanes Fenster die Tastatur besitzt
//
//*************************************************************************
{ if(t)
    return keytaskid==t->taskid;
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

int keywindow(void)
//*************************************************************************
//
//  Gibt Window-ID des momentan mit der Tastatur assozierten Fensters zur.
//
//*************************************************************************
{ if(tfeld[keytaskid])
  { struct edstruct *ee=(edstruct *)tfeld[keytaskid]->e;
    return ee->wa;
  }
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

static void near setkeytask(int tid)
//*************************************************************************
//
//  Weist die Tastatur einer Task-ID zu. Fenster kommt in den Vordergrund
//
//*************************************************************************
{ if(tfeld[tid])
  { edstruct *ee=(edstruct *)tfeld[tid]->e;
    if(ee)
    { keytaskid=tid;
      nachvorne(ee->wa);
      setzcurs(ee->curx,ee->cury,ee);
      allupdate++;
      return;
    }
  }
}

/*---------------------------------------------------------------------------*/

int setkeywindow(int fenster)
//*************************************************************************
//
//  Weist die Tastatur einer Window-ID zu. Fenster kommt in den Vordergrund
//
//*************************************************************************
{ int i;
  for(i=1;i<TASKS;i++)
  { if(tfeld[i])
    { edstruct *ee=(edstruct *)tfeld[i]->e;
      if(ee && (ee->wa==fenster))
      { keytaskid=i;
	nachvorne(ee->wa);
	setzcurs(ee->curx,ee->cury,ee);
	allupdate++;
	return 1;
      }
    }
  }
  return 0;
}

/*---------------------------------------------------------------------------*/

void setkeyhere(void)
//*************************************************************************
//
//  Legt Tastatureingabe auf laufende Task um
//  Beim fork() muss die Tastatur-Berechtigung P_KEYB eingeschaltet sein
//
//*************************************************************************
{ if((taskrunning!=NOTASK) && (t->processkind&P_KEYB))
    setkeytask(t->taskid);
}

/*---------------------------------------------------------------------------*/

static int near scankey(void)
//*************************************************************************
//
//  Holt Taste und schaut, ob ALT-Tastenkombination gedrckt ist
//  Wenn nein, wird die Taste im zustzlichen Tastaturpuffer abgelegt
//
//*************************************************************************
{ 
#ifndef __FLAT__
  int alt;
  unsigned key;
  task_t *t;

  tjob=1;
  while((ad_peek(0x40,0x1a)!=ad_peek(0x40,0x1c)) && bioskey(1))
  { video_switchon();
    key=bioskey(0);

    alt=f_taste(key);
    if(alt)
    { if(alt<11)
      {	if(!deskkey && !boxopen)
	{ setkeywindow(alt);
	}
      }
      else
	deskkey=alt;
    }
    else
    { alt=alt_taste(key);
      if(alt)
      { if(alt=='K')
	  return 1;
	else if(!deskkey)
	  deskkey=alt;
      }
      else
      { if(deskkey)
          t=tfeld[0];
        else
          t=tfeld[keytaskid];
        if(((t->keybufin+1)%32)!=t->keybufout)
	{ t->keybuffer[t->keybufin]=key;
	  t->keybufin=(t->keybufin+1)%32;
	}
      }
    }
  }
  if(newkeywin)
  { if(!setkeywindow(newkeywin))
      setkeywindow(1);
    newkeywin=0;
  }
#endif
  return 0;
}

/*---------------------------------------------------------------------------*/

static int near testevent(event_t event)
//*************************************************************************
//
//  Prueft auf das Eintreten eines Ereignisses. Es wird lediglich
//  der Zustand geprueft, aber sonst nichts unternommen.
//
//  return: 0=Event nicht eingetreten, >0 Event eingetreten
//
//*************************************************************************
{ tjob=2;
  switch(event)
  { case e_kbhit:
      return taskkbhit();
    case e_ticsfull:
      return TRUE;
    case e_reschedule:
    case e_nothing:  //fall through
    case e_delay:    //fall through
      return FALSE;
  }
  return FALSE;
}

/*---------------------------------------------------------------------------*/

static void global_process(void)
//*************************************************************************
//
//  legt den Stack nach Aufruf von waitfor() wieder
//  zum Hauptprogramm um. Die Funktion ist als interrupt
//  deklariert, damit alle Register vor dem Umschalten auf
//  dem Stack gesichert werden.
//
//*************************************************************************
{ if(setjmp(t->j)==0)
  {
    /*
    if(heapcheck()<0)
    { post_mortem_dump();
      trace(tr_abbruch,"global_process","heap corrupted");
    }
    */
    if(e)
    { e->buf=buf;
      e->wa=wa;
    }
    t->e=e;
    t->b=b;
    taskrunning=NOTASK;
    t=NULL;
    tjob=4;
    longjmp(jb,1);
  }
}

/*---------------------------------------------------------------------------*/

int waitfor(event_t event)
//*************************************************************************
//
//  setzt das laufende Programm aus, bis der Event im Argument
//  eingetreten ist. Ist die Bedingung bereits beim Einsprung
//  in waitfor erfllt, so wird ganz normal weitergemacht, ohne
//  zu unterbrechen.
//  Tritt whrend der Warteposition ein Disconnect auf dem
//  korrespondierenden Port auf, so wird der Prozess gelscht,
//  und ein offenes File geschlossen
//
//*************************************************************************
{ static int  i;                      // 'lokale Variablen', nicht am Stack!!

  if(taskrunning==NOTASK) return OK;
  if(((unsigned)systic<(tfeld[taskrunning]->begintic+m.timeslot)) &&
    testevent(event))
    return OK;                        // Event ist bereits gesetzt
  if(tfeld[taskrunning]->lock)
    return OK;
  for(i=waitsp;i;i--)                 // Prozess in die Warteschlange
    waitstack[i+1]=waitstack[i];      // aufnehmen. Warteschlange aufrcken
  waitsp++;
  waitstack[1]=taskrunning;           // vorne ist die niedrigste Prioritt
  if(event==e_reschedule) event=e_ticsfull;
  t->event=event;
  t->cputics+=(unsigned)((unsigned)systic-(unsigned)t->begintic);
  t->begintic=(unsigned)(systic+1);   // Zeit zum Aussetzen speichern
  tjob=5;
  global_process();
  tjob=6;
  return NO;
}

/*---------------------------------------------------------------------------*/

unsigned waitkey(void)
//*************************************************************************
//
//  suspendiert einen laufenden Prozess, bis eine Taste gedrckt wird und
//  das zugehrige Fenster dem momentenan Tastaturfenster entspricht
//
//*************************************************************************
{ unsigned key;
  waitfor(e_kbhit);

  key=t->keybuffer[t->keybufout];
  t->keybufout=(t->keybufout+1)%32;
  mausaus();
  mausein();
  return key;
}

/*---------------------------------------------------------------------------*/

int wdelay(unsigned zeit)
//*************************************************************************
//
//  suspendiert einen laufenden Prozess fr die angegebene Zeit
//  (in Millisekunden) Die tatschlich vergangene Zeit ist minimal gleich
//  bzw. grer als die angegebene Zeit.
//
//*************************************************************************
{ static int  i;                // 'lokale Variablen', nicht am Stack!!

  if(taskrunning==NOTASK || t->lock) return OK;
  tjob=23;
  t->cputics+=(unsigned)((unsigned)systic-(unsigned)t->begintic);
  t->begintic=(unsigned)(systic+(zeit/MS_PER_TIC)+1);
  t->event=e_delay;
  t->lastdelay=zeit;
  for(i=waitsp;i;i--)                 // Prozess in die Warteschlange
    waitstack[i+1]=waitstack[i];      // aufnehmen. Warteschlange aufrcken
  waitsp++;
  waitstack[1]=taskrunning;           // vorne ist die niedrigste Prioritt
  tjob=7;
  global_process();
  tjob=8;
  return NO;
}

/*---------------------------------------------------------------------------*/

static void dealloctask(int tid)
//*************************************************************************
//
//  Lscht eine komplette Task-Umgebung. Diese Routine ist das Herz von
//  jedem kill()-Aufruf und wird auch nach einer regulren Beendigung
//  aufgerufen.
//
//*************************************************************************
{ task_t   *tt=t;                   // momentanen Kontext sichern
  user_t   *uu=u;
  edstruct *ee=e;
  tvar_t   *bb=b;
  char   *buff=buf;
  int      waa=wa;
  int     trun=taskrunning;

  tjob=9;
  taskrunning=NOTASK;
  t=tfeld[tid];
  if(t)
  { int j=0;
    t_lock();
    while((unsigned char)(t->stack[j])==0xaa) 
      j++;                             //belegte Stacktiefe feststellen
    if(j<maxstackdepth) maxstackdepth=j;

    e=(edstruct *)t->e;                // Window-Struktur feststellen
    b=(tvar_t *)t->b;                  // Feststellen, ob Box-Kontrollblock
    if(b)
      u=&b->u;                         // Userdaten-Kontext herstellen
    for(j=0;j<t->atkillcnt;j++)
    (*t->atkill[j])(t->atkill_p[j]);
    if(e)
    { buf=e->buf;                      // Window-Kontext herstellen
      wa=e->wa;
      if(wa)
	wclose();                      // Fenster schlieen
    }
    if(t->port)                        // ist eine AX25-Connection aktiv?
      mbdisconnect(0);                 // ... trennen.
#ifndef __DOS16__
    if(t->socket)
      disconnect_sock();

#ifdef __UNIX__
    if(t->childpid)
      kill(-(t->childpid),9);
#endif
#ifdef _WIN32
    if(t->childpid)
      _beginthread(kill,0,(void*)t->childpid);
#endif

#endif      
    tjob=92;
    //t->port=0;
    if(b && !(t->processkind&P_CLON))  // oe3dzw: added ()
      mblogout(1);                     // ordentlichen Logout durchfhren
    tjob=93;
    t->port=0;
    t->event=e_nothing;                // kein Wartezustand mehr.
    close_task(tid);                   // alle Files und Semaphoren schlieen
    t_allfree(tid);                    // alle Speicherblcke freigeben
    tfeld[tid]=NULL;                   // TCB ist nun inaktiv.
    for(j=0;j<TASKS;j++)           // ggf. Monitor-Server schlieen.
    { if(tfeld[j] && tfeld[j]->monitor)
      { monbuf_t *mm=tfeld[j]->monitor;
	if(mm->tid==tid)
	  tfeld[j]->monitor=NULL;
      }
    }
  }
  t=tt;                                // laufenden Kontext wiederherstellen
  b=bb;
  e=ee;
  u=uu;
  buf=buff;
  wa=waa;
  taskrunning=trun;
  tjob=10;
}

/*---------------------------------------------------------------------------*/

static void start_newprocess(void)
{
  if(param)
  {
    char* prm=(char *)t_malloc(strlen(param)+1,"parm"); 
    //t_malloc param so it will be freed when task is killed
    strcpy(prm,param);
    free(param); //free copy from strdup
    param=prm;
  }
  t->begintic=(unsigned)systic;
  t->processkind=newkind;
  if(newport>0)
  { t->port=newport;
    t->input=io_tnc;
    t->output=io_tnc;
  }
#ifndef __DOS16__  
  else if(newport<0)
  { t->socket=(-newport);

      t->input=t->output=io_socket;
  }
#endif  
  else
  { t->input=io_dummy;
    t->output=io_dummy;
  }
  t->created=ad_time();
  t->e=e=NULL;
  t->b=b=NULL;
  strncpy(t->name,param,15);
  t->name[15]=0;

  if(newkind & P_WIND)
  { e=(edstruct *)t_malloc(sizeof(edstruct),"edst");
    memset(e,0,sizeof(edstruct));
    e->frame=1;
    t->e=e;
    t->input=t->output=io_window;
  }
  if(newkind & P_MAIL)
  { b=(tvar_t *)t;
    t->b=b;
    u=&b->u;
    b->job=j_idle;
  }
  tjob=11;
  (*start)(param);                            // Prozess starten
  tjob=12;
  suicide();
  trace(serious,"start_newprocess","%d not terminated",taskrunning);
}

/*---------------------------------------------------------------------------*/

static void newprocess(void)
//*************************************************************************
//
//  startet einen Prozess, der bisher noch keinen Stack reserviert
//  hatte. Dabei muss in taskrunning bereits eine Task-Nummer
//  vermerkt sein, die auf einen freien TASK-Kontrollblock zeigt.
//
//*************************************************************************
{
  if(setjmp(jb)==0)
  {
    t=tfeld[taskrunning];
    ad_setsp(t->stack+TASKSTACK); // Stackpointer umschalten (makro)
    start_newprocess();
    longjmp(jb,1);
  }
  t_checkfree();
}

/*---------------------------------------------------------------------------*/

void atkill(atkill2_t func,void *p)
{ if(t->atkillcnt==MAX_ATKILL)
  { trace(serious,"atkill","table full");
    return;
  }
  t->atkill[t->atkillcnt]=func;
  t->atkill_p[t->atkillcnt++]=p;
}

/*---------------------------------------------------------------------------*/

void atkill(atkill_t func)
{ if(t->atkillcnt==MAX_ATKILL)
  { trace(serious,"atkill","table full");
    return;
  }                               
  t->atkill[t->atkillcnt]=(atkill2_t)func;
  t->atkill_p[t->atkillcnt++]=NULL;
}

/*---------------------------------------------------------------------------*/

int fork(int typ,int port,void(*stfunc)(char *),char *stpar)
//*************************************************************************
//
//  Verzweigt zu einem neuen Prozess. Dieser luft so lange, bis entweder
//  ein KILL kommt oder sich das angegebene Programm beendet.
//
//*************************************************************************
{ int newtasknum_local=0;
  if((typ&(P_CLON|P_MAIL))==(P_CLON|P_MAIL) &&
     t->processkind&P_MAIL) // DG4MMI
    newtaskclone=t;
  else
    newtaskclone=NULL;

  start=stfunc;
  param=(stpar==NULL?(char*)NULL:strdup(stpar));
  newkind=typ;
  newport=port;
  newtask=1;
  newtasknum=&newtasknum_local;
  waitfor(e_reschedule);
  return newtasknum_local;
}

/*---------------------------------------------------------------------------*/

static void near showtasknum(int tnum,char job)
//*************************************************************************
//
//  zeigt am oberen Bildschirmrand den Prozesstatus an
//
//*************************************************************************
{ lastjob=job;
  if(video_off)
    return;
    
  static unsigned shtic=0;
  static int lasttnum=0;
  int wasave=wa;
  wa=0;
  if(!shtic || (((int)((unsigned)systic-shtic))>0))
  {
    wf(0,0,7,"%30s",memfree(0));
    shtic=(unsigned)(systic+8);
  }
  video_putc(lasttnum,0,7,' ');
  if(tnum<47)
  { video_putc(tnum,0,7,job);
    lasttnum=tnum;
  }
  wa=wasave;
}

/*---------------------------------------------------------------------------*/

int kill(int tid)
//*************************************************************************
//
//  Loescht den angegebenen Prozess,
//  gibt allen belegten Speicher frei und schliesst ggf. offene Files
//  Ist ein Mailbox-Login vorhanden, so wird ein ordentlicher LOGOUT
//  veranlasst.
//
//*************************************************************************
{ int j,i=waitsp;

  if(newtaskclone==t) // DG4MMI: Zu klonende Task wird gekillt
    newtaskclone=0;

  tjob=14;
  i=waitsp;
  //trace(report,"kill","task %d",tid);
  while(i)
  { if(tid==waitstack[i])
    { for(j=i;j<waitsp;j++)          // Task aus der Warteschlange entfernem
	waitstack[j]=waitstack[j+1];
      waitsp--;
      if(tfeld[tid])
	dealloctask(tid);
      tjob=15;
      t_checkfree();
      return 1;
    }
    i--;
  }
  //trace(report,"kill","%d not found",tid);
  tjob=16;
  return 0;
}

/*---------------------------------------------------------------------------*/

int killport(int port)
//*************************************************************************
//
//  Lscht einen Prozess mit angegebenem Funkport. Wird bei erkanntem
//  Disconnect aufgerufen
//
//*************************************************************************
{ int i;
  if(port)
  { for(i=0;i<TASKS;i++)
    { if(tfeld[i] && (tfeld[i]->port==port))
	return kill(i);
    }
  }
  return 0;
}

/*---------------------------------------------------------------------------*/

int taskport0(int port)
//*************************************************************************
//
//  Teilt einem Prozess die Portnummer 0 zu
//
//*************************************************************************
{ int i;
  if(port)
  { for(i=0;i<TASKS;i++)
    { if(tfeld[i] && (tfeld[i]->port==port))
      { tfeld[i]->port=0;
	return 1;
      }
    }
  }
  trace(report,"taskport0","%d not found",port);
  return 0;
}

/*---------------------------------------------------------------------------*/

void maintask(void)
//*************************************************************************
//
//  wird vom Hauptprogramm aufgerufen. Hier wird nachgesehen, ob eine
//  Anforderung in der Vorbereiteschlange (scanstack) ansteht, ggf.
//  wird ein neuer Prozess gestartet.
//
//*************************************************************************
{ int i,j;
  event_t ev;
#ifndef _AX25K_ONLY
  extern void call_l2(int);
#endif
  int active=0;

  memset(&m,0,sizeof(m));
  sema_init();
#ifdef __FLAT__
  call_l2(0);
#endif

  set_watchdog(1);
  updtic=(unsigned)systic;
  alltic=(unsigned)systic;

  fork(P_WIND,0,desktop,"Desktop");    // Erste Task, alles andere wird von
				       // dort aus gestartet.
  while(newtask || waitsp)
  { active=0;

    if(newtask)               // Ist eine Anforderung vorhanden?
    { tjob=17;
      active=1;
      ev=e_nothing;
      for(i=0;i<TASKS;i++)    // freien TASK-Block suchen
      { if(!tfeld[i])
	{ taskrunning=i;      // dieser Block wird genommen
          *newtasknum=i;
	  ev=e_reschedule;
	  inittcb(i);
	  break;
	}
      }
      if(ev==e_nothing)
      	trace(serious,"maintask","tasks full");
      else
      { newtask=0;
        showtasknum(taskrunning,'N');
        if(taskrunning!=NOTASK)       // war noch einer frei?
	  newprocess();               // und starten
      }
    }
    else
    { tjob=18;
      i=waitsp;
      while(i)
      { t=tfeld[waitstack[i]];
	ev=t->event;
	if((ev==e_delay)&&(((int)((unsigned)systic-t->begintic))>=0))
	  break;                                          // Delay Vorrang
	if(testevent(ev) || !ev)
	  break;
	i--;
      }
      t=NULL;

      if(i)                       // wurde einer mit gltigem Event gefunden ?
      { tjob=19;
        active=1;
        taskrunning=waitstack[i];
	for(j=i;j<waitsp;j++)          // Task aus der Warteschlange entfernem
	  waitstack[j]=waitstack[j+1];
	waitsp--;
	if(ev)
	{ showtasknum(taskrunning,'R');
          if(setjmp(jb)==0)
          {
            t=tfeld[taskrunning];          // Ausgesetzten Prozess fortsetzen
            t->begintic=(unsigned)systic;
            e=(edstruct *)t->e;
            if(e)
            { buf=e->buf;
              wa=e->wa;
            }
            b=(tvar_t *)t->b;
            if(b)
              u=&b->u;
            tjob=3;
            longjmp(t->j,1);
          }
	  t_checkfree();
	}
	else
	{ taskrunning=NOTASK;
	  trace(serious,"maintask","no event");
	}
      }
      if(!video_off)
      {
        if(allupdate&& !boxopen&&(!alltic||(((int)((unsigned)systic-alltic))>0)))
        { tjob=20;
          active=1;
          updtic=(unsigned)(systic+5);
          alltic=allupdate=0;
          vorne_reorg();
          for(i=0;i<=waitsp;i++)          // Alle Fenster update
          { t=tfeld[waitstack[i]];
            if(t)
            { showtasknum(waitstack[i],'U');
              taskrunning=NOTASK;
              if(t->e)
              { e=(edstruct *)t->e;
                wa=e->wa;
                buf=e->buf;
                if(e->update)
                { if(!(*e->update)(1))
                  { allupdate++;
                    alltic=(unsigned)(systic+8);
                  }
                }
              }
            }
          }
          mausaus();
          video_update();
          mausein();
          t=NULL;
        }
        else if(((int)((unsigned)systic-updtic))>0)
        { tjob=21;
          active=1;
          int updates=0;
          updtic=(unsigned)(systic+(m.scrolldelay?(m.scrolldelay):2));
          for(i=0;i<=waitsp;i++)          // Alle Fenster update
          { t=tfeld[waitstack[i]];
            taskrunning=NOTASK;
            if(t)
            { e=(edstruct *)t->e;
              if(e && t->event && e->update && e->shouldupdate)
              { e->shouldupdate=0;
                updates++;
                showtasknum(waitstack[i],'M');
                wa=e->wa;
                buf=e->buf;
                if(!(*e->update)(0))
                  e->shouldupdate++;
                t=NULL;
              }
            }
          }
          if(updates)
          { mausaus();
            video_update();
            mausein();
          }
        }
      }
    }
    if(scankey())           // Tastatur abfragen und
      break;                // bei ALT-K hart abbrechen
    if(nextkill)
    { kill(nextkill);
      nextkill=0;
    }
    showtasknum(0,'D');
    tjob=22;
    task_idle(active + (init_off?2:0));
    showtasknum(0,'I');
    if(init_off)
      break;
  }
}

/*---------------------------------------------------------------------------*/

void test_killtime(void)
{ int i;
  time_t act_time=ad_time();

  for(i=0;i<TASKS;i++)
  { if(tfeld[i])
    { task_t *tt=tfeld[i];

#ifdef _UTIMEOUT	//db1ras
      if (tt->timeout && (tt->last_input || tt->last_output) &&
//	 ^ alles gesetzt? ^
         tt->last_input  + tt->timeout < act_time &&
//	 ^ und last_input Zeitueberschreitung? ^
	 tt->last_output + tt->timeout < act_time &&
//	 ^ und last_output Zeitueberschreitung? ^
	 strcmp(tt->name,"Sysop"))
//	 ^ und kein Task mit Namen "Sysop"? ^
#else
      if(tt->kill_time && act_time>tt->kill_time)
#endif
      { trace(report,"timeout","%s: \"%s\"",tt->name,tt->lastcmd);
        //TODOincrease fwd-timeout-counter
        kill(i);
      }
    }
  }
}

/*---------------------------------------------------------------------------*/

void timeout(int minutes)
#ifdef _UTIMEOUT	//db1ras
{ t->timeout= 60*minutes; }
#else
{ if(minutes)
    t->kill_time=ad_time()+60*minutes;
  else
    t->kill_time=0;
}
#endif

/*---------------------------------------------------------------------------*/

static int convin(int tid,tvar_t *bb,char *fromcall,char *msg,int first)
{ conv_t *newc=(conv_t *)t_malloc(sizeof(conv_t),"conv");
  int pending=1;

  t_settaskid(newc,tid);
  newc->next=NULL;
  if (first)
  { if (strlen(msg) > 68)
      snprintf(newc->line,sizeof(newc->line),"*%s*:\n%s",fromcall,msg);
    else
      snprintf(newc->line,sizeof(newc->line),"*%s*: %s",fromcall,msg);
  }
  else 
    snprintf(newc->line,sizeof(newc->line),"%s",msg);
  if(bb->conv)
  { conv_t *cp=bb->conv;
    while(cp->next)
    { cp=cp->next;
      pending++;
    }
    pending++;
    if(pending<20)
      cp->next=newc;
    else
      t_free(newc);
  }
  else
    bb->conv=newc;
  return pending;
}

/*---------------------------------------------------------------------------*/

int mbtalk(char *fromcall,char *tocall,char *message)
{ int i;
  task_t *tt;
  tvar_t *bb;
  int pending=0;
  int first=1;
  int toall;
  char *current_msg=message;
  char *next_msg=NULL;

  strupr(fromcall);
  strupr(tocall);
  toall=(stricmp(tocall,"ALL")==0);

  while(current_msg)
  { next_msg=NULL;
    if(strlen(current_msg)>80)
    { next_msg=current_msg+79;
      i=79;
      while(i && (current_msg[i]!=' ')) i--;
      if(i)
        next_msg=current_msg+i;
      next_msg[0]=0;
      next_msg++;
    }

    for(i=0;i<TASKS;i++)
    { tt=tfeld[i];
      if(tt && tt->b && (i!=taskrunning))
      { bb=(tvar_t *)tt->b;
        if((strcmp(bb->logincall,tocall)==0)||toall)
        { 
          pending=convin(i,bb,fromcall,current_msg,first);
        }
      }
    }
    first=0;
    if (next_msg)
      current_msg=next_msg;
    else
      current_msg=NULL;
  }
  return pending;
}

/*---------------------------------------------------------------------------*/
#ifdef _TTY
int taskexists(int tid)                   // JJ
{ task_t *tt;                             // JJ
  tt=tfeld[tid];                          // JJ
  if(tt) return 1; else return 0;         // JJ
}                                         // JJ
#endif
/*---------------------------------------------------------------------------*/

int monitor(int tid,bitfeld opt)
{ task_t *tt;
  monbuf_t *mm;
  int c=0;

  tt=tfeld[tid];

  if(!tt || tid==gettaskid())
  { putf("monitor: invalid task-id.\n");
    return 0;
  }
  if(tt->monitor)
  { putf("monitor: task monitor busy.\n");
    return 0;
  }

  mm=(monbuf_t *)t_malloc(sizeof(monbuf_t),"monb");
  tt->monitor=mm;

  mm->tid=gettaskid();
  mm->in=0;
  mm->out=0;
  mm->congested=0;
  mm->opt=opt;
  putf("monitor: task %d (%s):\n",tid,tt->name);
  while(!getvalid() && tfeld[tid])
  { while(mm->in != mm->out)
    { c=mm->buf[mm->out++];
      if(mm->out>=MONBUFLEN)
        mm->out=0;
      putv(c);
    }
    if(mm->congested)
    { putf("\nmonitor: data discarded due to buffer congestion.");
      c=0;
      break;
    }
    wdelay(274);
  }
  if(c!='\n')
    putv('\n');
  if(!tfeld[tid])
    putf("monitor: server task died.\n");
  t_free(mm);
  if(tfeld[tid])
    tfeld[tid]->monitor=NULL;
  return 1;
}

/*---------------------------------------------------------------------------*/

int eingelogt(char *logincall,int job)
{ int i;
  task_t *t;
  tvar_t *b;
  int logins=0;
  int nochmal=0;

  do
  { nochmal=0;
    for(i=0;i<TASKS;i++)
    { t=tfeld[i];
      if(t && t->b && (i!=taskrunning))
      { b=(tvar_t *)t->b;
#ifdef HRX_OPT
	if((strcmp(b->logincall,logincall)==0 || strcmp(b->name,logincall)==0)&& (!job || (job==b->job)))   // JJ
#else
        if((strcmp(b->logincall,logincall)==0)&&(!job || (job==b->job)))
#endif
        { if(!b->sysop)
	    logins++;
        }
      }
    }
    if(job==j_fwdsend) { job=j_fwdsendf; nochmal=1; }
    if(job==j_fwdrecv) { job=j_fwdrecvf; nochmal=1; }
  } while(nochmal);
  return logins;
}

/*---------------------------------------------------------------------------*/

void listprocess(long opt)
//*************************************************************************
//
//  gibt die Task-Kontrollblcke aller momentan aktiven Prozesse aus.
//
//*************************************************************************
{ int i,j,width=44;
  task_t *t;
  char *evstr;
  char evdummy[]="     ";

  putf(" Task ");
  if(opt & o_t)
    putf("TP ");
  if(opt & o_a)
    putf("Event Kdeb ");// 12345678  
  putf(" CPUt Create Input Name    ");
  if(opt & o_n) putf("User ID       ");      
  if(opt & o_m) putf("Mycall   ");
  if(opt & o_u) putf("Uplink         ");
#ifdef HRX_OPT
  if(opt & o_d) putf("Downlink       "); 
#endif
  if(opt & o_c) putf("Bytes "); // JJ 
  if(opt & o_r) putf("Board   ");
  if((opt & (o_u|o_r|o_m))==0) putf(" "); 
  putf(" Command\n");
#ifdef HRX_OPT2
  putf("---------------------------------------"); 
  putf("--------------------------------------\n"); 
#endif
  if(opt&o_r) width-=8;
  if(opt&o_t) width-=3;
#ifdef HRX_OPT
  if(opt&o_d) width-=14; // hrx
#endif
  if(opt&o_c) width-=6;
  if(opt&o_u) width-=14;
  if(opt&o_m) width-=8;
  if(opt&o_a) width-=11;
  if(opt&o_n) width-=14;    

  for(i=0;i<TASKS;i++)
  { t=tfeld[i];
    if(t)
    { tvar_t *bb=(tvar_t *)t->b;

      if(bb || (opt & o_b))
      {	if(bb && bb->sysop) 
          putf("s");
        else
          putf(" ");
        putf("%c%3d ",(t->taskid==taskrunning)?'*':' ',i);
 	if(opt & o_t)
	  putf("%2d ",t->port);
	if(opt & o_a)
	{ switch(t->event)
	  { case e_ticsfull:      evstr="slice";   break;
	    case e_delay:         evstr=evdummy;
				  sprintf(evstr,"%u",t->lastdelay);
				  break;
	    case e_kbhit:         evstr="kbhit";   break;
	    default:              evstr="other";
	  }
	  putf("%-6s%2d%d%d ",evstr,t->processkind,!!t->e,!!t->b);
	}
	putf("%5s",zeitspanne(t->cputics,zs_cputime));
	putf("%6s",zeitspanne(ad_time()-t->created,zs_seconds));
	putf("%6s  ",zeitspanne(ad_time()-t->last_input,zs_seconds));
        if (*t->name && strstr(t->name,"."))
             // 12345678
          putf("(system)");
        else            
          putf("%-8.8s",t->name);
        if(opt & o_n)                         
        { if(mbcallok(t->name))               
          putf("%-14.14s",get_name(t->name,0)); 
          else                                 
           putf("              ");             
        }                                      
        if(opt & o_m)
        { if(bb)
            putf("%-9.9s",bb->mycall);
          else
            putf("         ");
        }
        if(opt & o_u)
//oe3dzw: Ausgabe der IP-Nummer
        { if(bb && strcmp(bb->uplink,m.boxname)!=0)
          { if ((strcmp(bb->uplink,"TCP/IP")==0)||
                (strcmp(bb->uplink,"HTTP")==0))
              putf("%-15.15s",bb->peerip);
            else  
              putf("%-15.15s",bb->uplink);
          }
          else // 123456789012345
            putf("               ");        
        }
#ifdef HRX_OPT
        if(opt & o_d)                  
         putf("%-15.15s",t->downlink); 
#endif
        if (opt & o_c)
        { if (bb)
          { long tbytes=bb->rxbytes+bb->txbytes;
            if (tbytes < 10000L)
              putf("%5lu ",tbytes);
            else if (tbytes < 10000L * 1024L)
              putf("%4luk ",tbytes/1024L);
            else 
              putf("%4luM ",tbytes/(1024L*1024L));
          }
          else
            putf("      "); 
        }
        if(opt & o_r)
        { if(bb && bb->forwarding==fwd_none)
          { char *board=strchr(bb->prompt,'/');
            if(board) 
              board++; 
            else 
              board=bb->prompt;
            putf("%-8.8s",board);
          }
          else
            putf("        ");
        }
	if((opt & (o_u|o_r|o_m))==0)
          putf(" "); 
        int working=(t->event==e_ticsfull);
        if(bb) switch(bb->job)
        { case j_idle:
          case j_fwdsend:
          case j_fwdlink:
          case j_fwdrecv: break;
          default: working=1;
        }
        if(working)
          putv('*');
        else
          putv(' ');
        
	putf("%.*s\n",width,t->lastcmd);
      }
    }
  }
  if(opt & o_s)
  { putf("Wait:  ");
    for(i=1;i<=waitsp;i++)
      putf("%d ",waitstack[i]);
    putf("\nStack: ");
    for(i=0;i<TASKS;i++)
    { if(tfeld[i])
      { j=0;
	while((unsigned char)(tfeld[i]->stack[j])==0xaa) j++;
	putf("%d ",TASKSTACK-j);
      }
    }
    putf(" (max=%d limit=%d)\n",TASKSTACK-maxstackdepth,TASKSTACK);
  }
#ifdef HRX_OPT2
  putf("---------------------------------------"); 
  putf("--------------------------------------\n");  
#endif
}

/*---------------------------------------------------------------------------*/

void lastcmd(char *cmd)
{ if(t && cmd[0])
  { while(cmd[0]==' ') cmd++;
    strncpy(t->lastcmd,cmd,59);
    t->lastcmd[59]=0;
  }
}

/*---------------------------------------------------------------------------*/

void post_mortem_dump(char *problem)
{ char reason[70];
  extern char lasttrace[];

  enable();
  sprintf(reason,"#W %s state=%c tid=%d tjob=%d maxs=%d",problem,
          lastjob,taskrunning,tjob,TASKSTACK-maxstackdepth);
  tracelog(reason,_s_serious);
  lasttrace[1]='W';
  tracelog(lasttrace,_s_serious);
  if(taskrunning!=NOTASK)
  { task_t *t=tfeld[taskrunning];
    int j=0;
    while(t->stack[j]==0xaa) j++;
    sprintf(reason,"#W %s: stk=%d ev=%d dly=%u \"%s\"",t->name,
            TASKSTACK-j,t->event,t->lastdelay,t->lastcmd);
    tracelog(reason,_s_serious);
    if(t->b)
    { b=(tvar_t *)t->b;

      sprintf(reason,"#W board=%s userjob=%d",b->prompt,b->job);
      tracelog(reason,_s_serious);
    }
  }
  #ifndef __FLAT__
  delay(5000);  // if smartdrv.exe is still alive give it a chance to
                // write outstanding data if write behind cache is on
  #endif
}

/*---------------------------------------------------------------------------*/

int listusers(int putit)
//*************************************************************************
//
//  gibt die momentan aktiven Benutzer aus
//
//*************************************************************************
{ int i,pos=putit;
  task_t *t;
  tvar_t *b;
  int logins=0;

  for(i=0;i<TASKS;i++)
  { t=tfeld[i];
    if(t && t->b)
    { b=(tvar_t *)t->b;
      if(b->logincall[0])
      { logins++;
	if(putit)
	{ if(putit==1 || putit==2)
	  { if(t->taskid==taskrunning)
	      putf("You    ");
	    else switch(b->job)
	    { case j_idle:    putf("Idle   ");
			      break;
              case j_fwdsendf:
	      case j_fwdsend: putf("FwdSend");
			      break;
	      case j_fwdlink: putf("FwdLink");
			      break;
              case j_fwdrecvf:
	      case j_fwdrecv: putf("FwdRecv");
			      break;
	      case j_send:    putf("Send   ");
			      break;
	      case j_read:    putf("Read   ");
			      break;
	      case j_cmd:     putf("Cmd    ");
			      break;
	      case j_search:  putf("Search ");
			      break;
  	      case j_convers: putf("Convers");
			      break;
#ifdef FBBFWD
              case j_fbb_send_propose:
              case j_fbb_send_mail:
              case j_fbb_send_delay:
              case j_fbb_recv_propose:
              case j_fbb_recv_mail:
              case j_fbb_end: 
              case j_fbb_error:   
                          putf("FBBfwd ");
                              break;
#endif
	    }
	    putf("  %-6s",b->logincall);
            if(putit==2)
            {
              if(b->mycall[0])
                putf(" > %-9s",b->mycall);
              else
                putf(" > (n/a)    ");
            }
	    if(b->uplink[0])
	      putf("  via %s",b->uplink);
              if ((strcmp(b->uplink,"TCP/IP")==0)||
                  (strcmp(b->uplink,"HTTP")==0))
                putf(" [%s]",b->peerip);
	    putf("\n");
	  }
	  else
	  { if(pos<putit)
	      putf(" ");
	    else
	    { pos=2;
	      putf("\n");
	    }
	    pos++;
	    putf("%d:%-6s",b->job,b->logincall);
	  }
	}
      }
    }
  }
  return logins;
}
