/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  ------------------------------------
  Time-conversion functions UNIX<->DOS
  ------------------------------------


  Copyright (C)       Johann Hanne, DH3MB, jonny@baycom.org
                      Falkenweg 6
                      D-85122 Hitzhofen

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

#include "baycom.h"

//19980206 OE3DZW changed time from int to long, now correct under dos
//19980308 OE3DZW added typecasts, no warnings under dos

/*---------------------------------------------------------------------------*/

/* MS-DOS time/date conversion routines derived from: */

/*
 *  linux/fs/msdos/misc.c
 *
 *  Written 1992,1993 by Werner Almesberger
 */

/* Linear day numbers of the respective 1sts in non-leap years. */

static int day_n[] = { 0,31,59,90,120,151,181,212,243,273,304,334,0,0,0,0 };
		  /* JanFebMarApr May Jun Jul Aug Sep Oct Nov Dec */


/* Convert a MS-DOS time/date pair to a UNIX date (seconds since 1.1.1970). */

time_t date_dos2unix(unsigned short time,unsigned short date)
{
	int month,year,secs;

	month = ((date >> 5) & 15)-1;
	year = date >> 9;
	secs = (time & 31)*2+60*((time >> 5) & 63)+(time >> 11)*3600+24*3600*
	    ((date & 31)-1+day_n[month]+(year/4)+year*365-((year & 3) == 0 &&
	    month < 2 ? 1 : 0)+3653);
			/* days since 1.1.70 plus 80's leap day */
	return secs;
}


/* Convert linear UNIX date to a MS-DOS time/date pair. */

void date_unix2dos(time_t unix_date,unsigned short *time,unsigned short *date)
{
	int year,nl_day,month;
        long day;

	*time = (unix_date % 60)/2+(((unix_date/60) % 60) << 5)+
	    (((unix_date/3600) % 24) << 11);
	day = unix_date/(3600L*24L)-3652L;
	year = (int) day/365;
	if ((year+3)/4+365*year > day) year--;
	day -= (year+3)/4+365*year;
	if (day == 59 && !(year & 3)) {
		nl_day = (int) day;
		month = 2;
	}
	else {
		nl_day = ((int) year & 3) || (int) day <= 59 
                   ? (int) day : (int) day-1;
		for (month = 0; month < 12; month++)
			if (day_n[month] > nl_day) break;
	}
	*date = nl_day-day_n[month-1]+1+(month << 5)+(year << 9);
}

/*---------------------------------------------------------------------------*/

time_t time_dos2unix(char *dostime)
//*************************************************************************
//
//  Convert 8 character DOS time hex format to unix time
//
//*************************************************************************
{ char i;
  unsigned short time, date;

  if(strlen(dostime)!=8)
    return 0;

  for(i=0;i<8;i++)
    if(!isxdigit(dostime[i]))
      return 0;

  time=(unsigned short)strtoul(&dostime[4],(char **)NULL,16);
  dostime[4]=0;
  date=(unsigned short)strtoul(dostime,(char **)NULL,16);

  return(date_dos2unix(time,date));
}

/*---------------------------------------------------------------------------*/

void time_unix2dos(time_t unix_date, char *buffer)
//*************************************************************************
//
//  Convert unix time to 8 character DOS time hex format
//
//*************************************************************************
{ unsigned short int time, date;

  date_unix2dos(unix_date,&time,&date);
  sprintf(buffer,"%04X%04X",date,time);
}

/*---------------------------------------------------------------------------*/
