/***************************************************************


  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  --------------------------------------------
  Erzeugung und Behandlung des Bulletin-Baumes
  --------------------------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980124 OE3DZW text from "Maximum" to "Limit" in status limits
//                added +10k to maxbids, is caused stress
//19980202 OE3DZW fixed bug when reloading empty convname/life
//19980205 OE3DZW removed debug output
//19980913 OE3DZW users will usually not see temperory boards

#include "baycom.h"

/*---------------------------------------------------------------------------*/

unsigned treelen = 0;
bulletin_t *tree = NULL;

/*---------------------------------------------------------------------------*/

static unsigned convlen = 0;
static unsigned clifelen = 0;
static convname_t *convnames = NULL;
static convlife_t *convlifes = NULL;

/*---------------------------------------------------------------------------*/

void
limit_list (void)
{
  struct dirent *di;
  DIR *d = opendir (m.userpath);
  unsigned umailcalls = 0;

  while ((di = readdir (d)) != NULL)
    {
      if (di->d_name[0] != '.')
	umailcalls++;
    }
  closedir (d);

  putf ("Item          Current      Limit\n");
  putf ("boardnames  %8u %11u\n", treelen, MAXTREEENTRIES);
  putf ("convnames   %8u %11u\n", convlen, MAXCONVNAME);
  putf ("convlifes   %8u %11u\n", clifelen, MAXCONVLIFE);
  putf ("fwdpartners %8u %11u\n", fwdpartners, MAXFWD);
  putf ("fwddestlen  %8u %11u\n", fwddestlen, MAXDESTLEN);
  putf ("h-addrs     %8lu %11lu\n", hadr_len (0), hadr_len (1));
  putf ("bids        %8lu %11lu\n", bid_len (), m.maxbids + 10000);
  putf ("users       %8lu         -\n", filesize (USERNAME) / sizeof (user_t));
  putf ("infomails   %8lu %11lu\n", filesize (CHECKNAME) / 128,
	filesize (CHECKNAME) / 128 + dfree (m.infopath, 0) / 8);
  putf ("u-mailcalls %8u %11u\n", umailcalls, MAXUSERMAILS);
  putf ("languages   %8u %11u\n", msg_limit (1), msg_limit (0));
}

/*---------------------------------------------------------------------------*/

static void near
intree (struct dirent *p, char *path, char *mask)
//*************************************************************************
//
//  Legt eine Zeile des Directorybaumes im tree-Array ab
//
//*************************************************************************
{
  char fname[80];
  sprintf (fname, "%s/%s", mask, p->d_name);

  if (isdir (fname) && p->d_name[0] != '.')
    {
      strcpy (tree[treelen].name, path);
      if (path[0])
	strcat (tree[treelen].name, "/");
      strcat (tree[treelen].name, p->d_name);
      if (strlen (tree[treelen].name) >= MAXTREELENGTH)
	trace (tr_abbruch, "intree", "name too long");
      strupr (tree[treelen].name);
      char *sl=strrchr(tree[treelen].name,'/');
      if (sl)
        tree[treelen].flatoffset=(byte)(sl+1-tree[treelen].name);
      else
        tree[treelen].flatoffset=0;
      tree[treelen].newestmail = 0;
      tree[treelen].lifetime_min = 0;
      tree[treelen].lifetime_max = 999;
      treelen++;
      if (treelen > MAXTREEENTRIES)
	trace (tr_abbruch, "intree", "too many boards");
    }
}

/*---------------------------------------------------------------------------*/

static void near
search (char *path, char *root)
//*************************************************************************
//
//  Durchsucht das aktuelle Directory nach weiteren Subdirectories
//
//*************************************************************************
{
  char mask[80];
  struct dirent *p;
  DIR *d;

  strcpy (mask, root);
  if (path[0])
    {
      strcat (mask, "/");
      strcat (mask, path);
    }
  strlwr (mask);

  d = opendir (mask);
  while (d && (p = readdir (d)) != NULL)
    {
      waitfor (e_ticsfull);
      intree (p, path, mask);
    }
  closedir (d);
  waitfor (e_ticsfull);
}

/*---------------------------------------------------------------------------*/

static void near
treesort (void)
//*************************************************************************
//
//  Sortiert den Baum alphabetisch
//
//*************************************************************************
{
    qsort ((void *) tree, treelen, sizeof (bulletin_t),
	 (int (*)(const void *, const void *)) strcmp);

  tree = (bulletin_t *) t_realloc (tree, sizeof (bulletin_t) * (treelen + 2));
}

/*---------------------------------------------------------------------------*/

static int near
mbtreeload (void)
//*************************************************************************
//
//  Laedt eine Bulletin-Struktur von der Datei TREENAME
//  Rueckgabe: Bulletin-Liste gueltig
//  (ignoriert die Verzeichnisse auf der Platte)
//*************************************************************************
{
  FILE *f;
  char s[40];
  char *name, *life, *n2;
  int blanks, linesok = 0;
  char lastpath[MAXTREELENGTH];
  bulletin_t *tr = NULL;

  lastpath[0] = 0;

  f = s_fopen (TREENAME, "srt");
  if (f)
    {
      while (fgets (s, 39, f))
	{
	  if (s[0] != ';')
	    {
	      blanks = 0;
	      strupr (s);
	      name = s;
	      while (name[0] == ' ')
		{
		  name++;
		  blanks++;
		}
	      life = name;
	      while (isgraph (life[0]))
		life++;
	      if (life[0])
		life[0] = 0;
	      life++;
	      life += blkill (life);
	      if (strlen (name) > 8)
		name[8] = 0;
	      if (finddir (name,1) > 0)
		continue;
	      tr = &tree[treelen];	// zum einfacheren Zugriff

	      tr->lifetime_max = nextdez (&life);
	      tr->lifetime_min = nextdez (&life);

	      if (tr->lifetime_max > 999)
		tr->lifetime_max = 999;
	      if (tr->lifetime_min > 999)
		tr->lifetime_min = 999;
	      if (tr->lifetime_max == 0)
		tr->lifetime_max = 999;
	      if (tr->lifetime_max < tr->lifetime_min)
		tr->lifetime_min = tr->lifetime_max;
              

	      if (name[0] && (strlen (name) < 9))
		{
		  linesok++;
		  n2 = lastpath;
		  while (blanks)
		    {
		      if (n2[0] == '/')
			n2++;
		      while (isalnum (n2[0]))
			n2++;
		      blanks--;
		    }
		  n2[0] = 0;

		  if (lastpath[0])
		    strcat (lastpath, "/");
		  strcat (lastpath, name);
		  if (strlen (lastpath) >= MAXTREELENGTH)
		    {
		      trace (serious, "mbtreeload", "path %s too long", lastpath);
		      continue;
		    }
		  strcpy (tr->name, lastpath);
                  char *sl=strrchr(tr->name,'/');
                  if (sl)
                    tr->flatoffset=(byte)(sl+1-tr->name);
                  else
                    tr->flatoffset=0;
		  tr->newestmail = 0;
		  treelen++;
		  if (treelen >= MAXTREEENTRIES)
		    {
		      trace (serious, "mbtreeload", "too many directories");
		      treelen--;
		      break;
		    }
		}
	    }
	}
      s_fclose (f);
      return linesok;
    }
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

void
mbtreesave (void)
//*************************************************************************
//
//  Speichert den Bulletin-Baum auf Platte ab
//  Dabei wird darauf geachtet, dass zu keinem Zeitpunkt ein
//  fehlerhaftes File auf der Platte ist.
//
//*************************************************************************
{
  FILE *f;
  unsigned int i;
  char *slash, *name;
  int blanks;
  char nbuf[12];

  treesort ();
  f = s_fopen (TREETMPNAME, "swt");
  if (f)
    {
      if (fprintf (f, "; BayCom-Mailbox Bulletin Listfile\n;\n") < 0)
	goto fehler;
      for (i = 0; i < treelen; i++)
	{
	  blanks = 0;
	  name = tree[i].name;
	  while ((slash = strchr (name, '/')) != NULL)
	    {
	      name = slash + 1;
	      blanks++;
	    }
	  nbuf[blanks] = 0;
	  while (blanks)
	    nbuf[--blanks] = ' ';
	  strcat (nbuf, name);
	  strlwr (nbuf);
	  if (fprintf (f, "%-10s%3d %3d\n", nbuf, tree[i].lifetime_max,
		       tree[i].lifetime_min) < 0)
	    goto fehler;
	}
      s_fclose (f);
      xunlink (TREEBAKNAME);
      xrename (TREENAME, TREEBAKNAME);
      xrename (TREETMPNAME, TREENAME);
    }
  else
    trace (serious, "mbtreesave", "fopen errno=%d", errno);
  return;

fehler:
  trace (serious, "mbtreesave", "write errno=%d", errno);
  s_fclose (f);
}

/*---------------------------------------------------------------------------*/

int
rmboard (char *fullname)
//*************************************************************************
//
//  Entfernt ein Board. Der volle Pfadname muss angegeben werden
//  Return:
//    0     ok
//    1     Board nicht gefunden
//    2     Board nicht leer
//
//*************************************************************************
{
  unsigned i = 0;
  char path[40];
  DIR *d;
  struct dirent *di;

  strupr (fullname);
  while (strcmp (tree[i].name, fullname) != 0)
    {
      i++;
      if (i >= treelen)
	return 1;
    }
  sprintf (path, "%s/%s", m.infopath, fullname);
  strlwr (path);
  d = opendir (path);

  while (d && (di = readdir (d)) != NULL)
    {
      if (di->d_name[0] != '.')
	{
	  closedir (d);
	  return 2;
	}
    }
  closedir (d);
                   //  12345678
  strcpy(tree[i].name,"zzzzzzzz");
  tree[i].name[0]=(char) 0xff;
  treesort ();
  treelen--;
  mbtreesave ();
  return 0;
}

/*---------------------------------------------------------------------------*/

int
mkboard (char *mainboard, char *newboard, int sameboard)
//*************************************************************************
//
//  Erzeugt ein neues Board. Angegeben wird
//  - Das Hauptboard (oder / wenn ein Hauptboard eingerichtet wird)
//  - Das Subboard (ohne /)
//
//  Return:
//    0     ok
//    1     Hauptboard nicht gefunden
//    2     Baum voll
//    3     Board schon vorhanden
//    4     Boardname ungltig
//
//*************************************************************************
{
  unsigned i = 0;
  char name[40];

  if (treelen >= MAXTREEENTRIES)
    {
      trace (serious, "mkboard", "too many boards %s", newboard);
      return 2; //tree full
    }
  strupr (mainboard);
  strupr (newboard);
  if (newboard[0] == 0 || newboard[0] == '/' || (strlen (newboard) > 8)
     || (strlen(mainboard) > 20))
     return 4; //board invalid
  if (!sameboard && finddir (newboard,1) != 0)
    return 3; //board exists
  if (mainboard[0] == '/')
    mainboard[0] = 0;
  name[0] = 0;
  if (mainboard[0])
    {
      while (strcmp (tree[i].name, mainboard) != 0)
	{
	  i++;
	  if (i >= treelen)
	    return 1; //mainboard not found
	}
      strncpy (name, mainboard,20);
      name[20]=0;
      strcat (name, "/");
    }
  strcat (name, newboard);
  strcpy (tree[treelen].name, name);
  char *sl=strrchr(tree[treelen].name,'/');
  if (sl)
    tree[treelen].flatoffset=(byte)(sl+1-tree[treelen].name);
  else
    tree[treelen].flatoffset=0;
  tree[treelen].newestmail = 0;
  tree[treelen].lifetime_min = 0;
  tree[treelen].lifetime_max = m.infolife;
  treelen++;
  treesort ();
  mbtreesave ();
  return 0;
}

/*---------------------------------------------------------------------------*/

int
mvboard (char *old, char *sub, char *newboard)
//*************************************************************************
//
//  Benennt ein Board um. Dabei werden alle Files in dem angegebenen
//  Board in das neue Board verschoben.
//  Sowohl bei old als auch bei new sind die kompletten Boardnamen
//  (main und sub) anzugeben, z.B. SOFTWARE/BAYCOM
//
//  Return:
//   0  alles ok
//   1  neues Hauptboard nicht gefunden
//   2  Baum voll
//   4  altes Board nicht gefunden
//
//*************************************************************************
{
  char oldpath[50];
  char newpath[50];
  char oldbname[30];
  char dirpath[50];
  int oldmask, newmask;
  unsigned int i=0;
  DIR *d;
  struct dirent *di;

  strupr (old);
  strupr (sub);

  if (old[0] == '/')
    old[0] = 0;
  else
    strcat (old, "/");

  sprintf (oldbname, "%s%s", old, sub);
  while (strcmp (tree[i].name, oldbname) != 0)
    {
      i++;
      if (i >= treelen)
	return 4;
    }

  //somewhere here is a bug (reported by Jan)
  //???

  i = mkboard (newboard, sub, 1);
  if (i)
    return i;

//  if(newboard[0]=='/')
  //    newboard[0]=0;
  //  else
  //    strcat(newboard,"/");

  if (newboard[0] != 0)
    strcat (newboard, "/");


  sprintf (oldpath, "%s/%s%s/", m.infopath, old, sub);
  sprintf (newpath, "%s/%s%s", m.infopath, newboard, sub);
  xmkdir (newpath);
  strcat (newpath, "/");

  oldmask = strlen (oldpath);
  newmask = strlen (newpath);

  strcpy (dirpath, oldpath);
  dirpath[oldmask - 1] = 0;

  set_watchdog (2);
  strlwr (dirpath);
  d = opendir (dirpath);

  while ((di = readdir (d)) != NULL)
    {
      if (di->d_name[0] != '.')
	{
	  strcpy (oldpath + oldmask, di->d_name);
	  strcpy (newpath + newmask, di->d_name);
	  strlwr (oldpath);
	  strlwr (newpath);
	  if (xrename (oldpath, newpath) != 0)
	    trace (serious, "mvboard", "rename");
	  waitfor (e_ticsfull);
	}
    }
  closedir (d);

  set_watchdog (1);
  rmemptydir (dirpath);
  rmboard (oldbname);
  return 0;
}

/*---------------------------------------------------------------------------*/

void
mbreadtree (char *beginpath)
//*************************************************************************
//
//  Liest einen kompletten Directorybaum ab 'beginpath' ein,
//  speichert ihn im globalen Array tree und sortiert ihn
//  Falls die Datei TREENAME existiert werden die Informationen
//  aus dieser Datei uebernommen.
//
//*************************************************************************
{
  unsigned i, j, k = 0;

  treelen = 0;
  if (!tree)
  { tree = (bulletin_t *) t_malloc (sizeof (bulletin_t) * MAXTREEENTRIES, "*tre");
  }

  if (!mbtreeload ())
    {
      search ("", beginpath);
      do
	{
	  j = treelen;
	  for (i = k; i < j; i++)
	    search (tree[i].name, beginpath);
	  k = j;
	}
      while (treelen > j);
      mbtreesave ();
    }
  treesort ();
  trace (report, "readtree", "%d bulletins", treelen);
  if (*b->logincall)
    putf ("%d boards.\n", treelen);

}
   
/*---------------------------------------------------------------------------*/

int
nameconvert (char *name)
{
  unsigned int i;

  strupr (name);
  if (convnames)
    {
      for (i = 0; i < convlen; i++)
	{
	  if (strncmp (convnames[i].oldname, name, 6) == 0)
	    {
	      strcpy (name, convnames[i].newname);
	      return 1;
	    }
	}
    }
  return 0;
}

/*---------------------------------------------------------------------------*/
void
convedit(char *befbuf)
{ 
  char cmd[5];
  char board1[7];
  char board2[7];
  befbuf=nexttoken(befbuf,cmd,4);
  befbuf=nexttoken(befbuf,board1,6);
  befbuf=nexttoken(befbuf,board2,6);
  strupr(board1);
  strupr(board2);
  strupr(cmd);
  switch (cmd[0])
  { case 'A': // add
      { short found=0;
        if (convlen==MAXCONVNAME)
          putf("Too many convnames.\n");
        else if (board2[0]==0 || finddir(board2,1)<=0)
          putf("New board does not exist.\n");
        else
        { if (finddir(board1,1))
            putf("Warning: Old board exists.\n");       

          for(unsigned int i=0;i<convlen;i++)
          { if (strcmp(convnames[i].oldname,board1)==0)
            { strcpy(convnames[i].newname,board2);
              found++;
            }
          }  
          if (!found)
          { strcpy(convnames[convlen].oldname,board1);
            strcpy(convnames[convlen].newname,board2);
            convlen++;
            found++;
          }
        }
        if (found)
        { saveconvnames();
          mbconvload();
          putf("%-6s -> %s\n",board1,board2);
          
        }
      }
      break;
    case 'D': // delete
      { for(unsigned int i=0;i<convlen;i++)
        { if (strcmp(convnames[i].oldname,board1)==0||
              strcmp(convnames[i].newname,board1)==0)
          { putf("%-6s -> %s\n",convnames[i].oldname,convnames[i].newname);
            convnames[i].oldname[0]=0;
          }
        }
        saveconvnames();
        mbconvload();
        break;
      }
    case 'L':
      {
        for(unsigned int i=0;i<convlen;i++)
        { if (board1[0]==0 || strcmp(convnames[i].oldname,board1)==0||
                              strcmp(convnames[i].newname,board1)==0)
            putf("%-6s -> %s\n",convnames[i].oldname,convnames[i].newname);
        }
        break;
      }
    default:
      putf("Syntax: convedit add <old> <new>\n"
           "        convedit del <board>\n"
           "        convedit list [<board>]\n");
  }
}
/*---------------------------------------------------------------------------*/

void
saveconvnames (void)
{ char buf[50];
  FILE *f = s_fopen (CONVTMPNAME, "swt");
  if (f)
  { fprintf(f,"; BayCom Mailbox Convnames Listfile\n;\n;\n");
    for (unsigned int i=0;i<convlen;i++)
      if (convnames[i].oldname[0])
      { sprintf(buf,"%-6s %s\n",convnames[i].oldname,convnames[i].newname);
        strlwr(buf);
        fputs(buf,f);
      }
    s_fclose(f);
    xunlink(CONVFILENAME);
    xrename(CONVTMPNAME,CONVFILENAME);
  }
  else
    trace(serious,"saveconvnames","fopen err");
}  
/*---------------------------------------------------------------------------*/

void
mbconvload (void)
{
  FILE *f = s_fopen (CONVFILENAME, "srt");
  char s[80];
  char name[]="convload";
  long line=0L;

  convlen = 0;
  if (f)
    {
      if (convnames)
	convnames = (convname_t *)
	  t_realloc (convnames, sizeof (convname_t) * MAXCONVNAME);
      else
	convnames = (convname_t *)
	  t_malloc (sizeof (convname_t) * MAXCONVNAME, "*cnv");

      while (fgets (s, 30, f))
	{ line++;
	  char *s1 = strchr (s, ';');
	  if (s1)
	    s1[0] = 0;
	  s1 = s;
	  strupr (s);
	  s1 = nexttoken (s1, convnames[convlen].oldname, 8);
	  nexttoken (s1, convnames[convlen].newname, 8);

          if (convnames[convlen].oldname[0] &&
               finddir(convnames[convlen].oldname,1) &&
               strlen(convnames[convlen].newname)>1)
            trace(replog,name,"l%ld Warning:old board %s exists",line,
              convnames[convlen].oldname);
          
          convnames[convlen].oldname[6]=0;

          if (convnames[convlen].newname[0] &&
               finddir(convnames[convlen].newname,1)<=0)
          { trace(replog,name,"l%ld:new board %s does not exist",line,
              convnames[convlen].newname);
            convnames[convlen].newname[0]=0;
          }
          else
            convnames[convlen].newname[6]=0;

	  if (convnames[convlen].oldname[0] && convnames[convlen].newname[0])
	    convlen++;
	  if (convlen >= MAXCONVNAME)
	    break;
	}
      s_fclose (f);
      if (convlen)      
      { qsort((void *) convnames,convlen,sizeof(convname_t),
        (int (*)(const void *,const void *))strcmp);
      }
    }
      convnames = (convname_t *) t_realloc (convnames, 
        sizeof (convname_t) * (convlen + 2));

      trace (report, "convload", "%d convnames", convlen);
      if (*b->logincall)
	putf ("%d convnames.\n", convlen);
    saveconvnames(); //even save empty file
}

/*---------------------------------------------------------------------------*/

int
checklifeconv (char *searchstr)
{
  unsigned i;

  if (convlifes)
    {
      for (i = 0; i < clifelen; i++)
	{
	  if (stristr (searchstr, convlifes[i].string))
	    return convlifes[i].life;
	}
    }
  return (-1);
}

/*---------------------------------------------------------------------------*/

void
mbclifeload (void)
{
  FILE *f = s_fopen (CONVLIFENAME, "srt");
  char s[40];

  clifelen = 0;
  if (f)
    {
      if (convlifes)
	convlifes = (convlife_t *)
	  t_realloc (convlifes, sizeof (convlife_t) * MAXCONVLIFE);
      else
	convlifes = (convlife_t *)
	  t_malloc (sizeof (convlife_t) * MAXCONVLIFE, "*cli");

      while (fgets (s, 30, f))
	{
	  char *s1 = strchr (s, ';');
	  if (s[0])
	    s[strlen (s) - 1] = 0;
	  if (s1)
	    s1[0] = 0;
	  s1 = s;
	  convlifes[clifelen].life = nextdez (&s1);
	  if (s1[0])
	    {
	      s1[15] = 0;
	      strcpy (convlifes[clifelen].string, s1);
	      clifelen++;
	    }
	  if (clifelen >= MAXCONVLIFE)
	    break;
	}
      s_fclose (f);

      if (clifelen)
	convlifes = (convlife_t *)
	  t_realloc (convlifes, sizeof (convlife_t) * (clifelen + 1));
      else
	{
	  t_free (convlifes);
	  convlifes = 0;
	}
      trace (report, "clifeload", "%d convlifes", clifelen);
      if (*b->logincall && clifelen > 0)
	putf ("%d convlifes.\n", clifelen);
    }
}

/*---------------------------------------------------------------------------*/
