/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  ------------------------------------------------------
  Zugriff auf persoenliche Einstellungen der Mailboxuser
  ------------------------------------------------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980404 hrx    added guest-support
//19980408 OE3DZW added check for more strings
//19980427 OE3DZW deleted users will not be logged as error but deleted,
//                added check for newcall              
//19980506 OE3DZW added check for fdelay,fhold,rlimit
//19980615 OE3DZW only every 1000th user is put into log
//19991201 DB1RAS added _UIMPORT in uimport()
//19991202 DB1RAS added _EXDLOGIN and _DEFREA in defaultuser()
//19991216 DB1RAS added _DEFFDELAY in defaultuser()

#include "baycom.h"

/*---------------------------------------------------------------------------*/

#ifdef __FLAT__
#define US_CACHESZ 128
#else
#define US_CACHESZ 8
#endif
#define UTMP  "users.$$$"
#define UHTMP "userhash.$$$"

/*---------------------------------------------------------------------------*/

static handle uf=EOF;
static handle uhf=EOF;
static time_t lastreorg=0;
static int is_null=0;
static user_t us_cache[US_CACHESZ];

/*---------------------------------------------------------------------------*/


static void near userfile_newformat(void);


void hashinit(handle uhf,long hashsize)
//*************************************************************************
//
//  Wenn die Hashtabelle nicht existiert, wird sie hier angelegt
//  und mit Null aufgefuellt.
//
//*************************************************************************
{ 
  if(uhf!=EOF)
  { if(lseek(uhf,0,SEEK_END)!=hashsize*2)
    { char *buffer=(char *)t_malloc(16384,"hshi");
      int i;
      memset(buffer,0,16384);
      for(i=0;i<(hashsize/8192);i++)
        _write(uhf,buffer,16384);
      t_free(buffer);
    }
  }
  else
    trace(tr_abbruch,"hashinit","hash not open");
}

/*---------------------------------------------------------------------------*/

static void us_close(void)
//*************************************************************************
//
//  schliesst ggf ein offenes user-file
//
//*************************************************************************
{ if(uf!=EOF)
  { s_close(uf);
    uf=EOF;
  }
  if(uhf!=EOF)
  { s_close(uhf);
    uhf=EOF;
  }
}

/*---------------------------------------------------------------------------*/

void defaultuser(char *usercall,user_t *u)
//*************************************************************************
//
//  Ueberschreibt einen vorhandenen User mit Defaultwerten
//  Eine ggf. vorhandene Hash-Verkettung wird beibehalten.
//
//*************************************************************************
{ int i;
  time_t lastload=u->lastload;
  unsigned long uhsh=u->nextsamehash;
  unsigned long fpos=u->filepos;
  memset(u,0,sizeof(user_t));           // neuen Eintrag vorbesetzen
  u->nextsamehash=uhsh;  
  u->filepos=fpos;
  //                        ^ aber filepos und nextsamehash lassen
  strcpy(u->call,usercall);               // Rufzeichen hineinschreiben
#ifdef _GUEST
  if (strcmp(u->call,m.guestcall)==0)   
   strcpy(u->mybbs,m.boxname);         
#endif

  strcpy(u->prompt,m.prompt);
  strcpy(u->sprache,msg_landsuch(usercall));
  strcpy(u->firstcmd,m.firstcmd);
  u->helplevel=m.helplevel;
  u->zeilen=m.zeilen;
  u->lf=m.deflf;
#ifdef _PACLEN
  u->paclen=m.defpaclen;
#endif
#ifdef _DEFREA		//db1ras
  u->readlock=m.defrea;
#endif
#ifdef _EXDLOGIN	//db1ras
  u->fhold=m.deffhold;
#endif
#ifdef _DEFFDELAY	//db1ras
  u->fdelay=m.deffdelay;
#endif
  #ifdef _INIT_PACLEN
    u->paclen=m.paclen;
  #endif
  for(i=0;i<8;i++)
    u->opt[i]=m.opt[i];
  u->lastload=lastload;                   // damits abgespeichert wird
  us_deletecache();
}

/*---------------------------------------------------------------------------*/

static void newuserfile(void)
//*************************************************************************
//
//  Legt ein neues File mit Userdaten an.
//  Darf nicht aufgerufen werden, wenn nicht sichergestellt ist,
//  dass das alte File ueberschrieben werden soll.
//
//*************************************************************************
{ user_t ux;

  xunlink(USERHASHNAME);
  trace(serious,"us_open","new users");
  uf=s_open(USERNAME,"sw+b");
  if(uf==EOF)
    trace(tr_abbruch,"us_open","can't create");
  memset(&ux,0,sizeof(user_t));
  defaultuser("DUMMY",&ux);
  _write(uf,&ux,sizeof(user_t));
  s_close(uf);
}

/*---------------------------------------------------------------------------*/

static void us_open(void)
//*************************************************************************
//
//  Oeffnet ein vorhandenes Userfile.
//  Beim ersten Aufruf wird es ggf. angelegt, wenn sichergestellt
//  ist, dass es nicht existiert.
//
//*************************************************************************
{ static int first=1;

  if(uf==EOF)
  {
    userfile_newformat();
 
    if(filetime(USERNAME)==0 || filesize(USERNAME)==0)
    { if(!first)
        trace(tr_abbruch,"us_open","users not available");
      xunlink(USERHASHNAME);
      newuserfile();
      uf=s_open(USERNAME,"sr+b");
    }
    else
    { uf=s_open(USERNAME,"sr+b");
      if(uf==EOF)
        trace(tr_abbruch,"us_open","user open errno=%d",errno);
    }
  }
  if(uhf==EOF)
  userfile_newformat();
    uhf=s_open(USERHASHNAME,"sr+b");
  if(uhf==EOF)
  { if(first)
      uhf=s_open(USERHASHNAME,"sw+b");
    else
      trace(tr_abbruch,"us_open","hash fopen errno=%d",errno);
  }
  first=0;
}

/*---------------------------------------------------------------------------*/

static unsigned newuser(user_t *u,handle uf,handle uhf)
//*************************************************************************
//
//  legt einen USER im User-File ab
//
//  Achtung: es muss an der Stelle sichergestellt sein, dass nicht
//           schon ein USER mit gleichem Namen existiert!
//
//*************************************************************************
{ long unsigned hsh=strcrc(u->call);
  long unsigned idx=0;

  lseek(uhf,(long)hsh*4,SEEK_SET);                 // Eintrag im Hashfile
  if(!_read(uhf,&idx,sizeof(long unsigned)))
  { trace(serious,"newuser","read hash %s",u->call);
    return 0;
  }
  u->nextsamehash=idx;                             // verketten

  lseek(uf,0L,SEEK_END);
  u->filepos=(long unsigned)(filelength(uf)/sizeof(user_t));
  lseek(uhf,(long)hsh*4,SEEK_SET);

  if(!_write(uhf,&(u->filepos),sizeof(long unsigned)))  // zurueckschreiben
  { trace(serious,"newuser","write hash %s",u->call);
    return 0;
  }
  if(!_write(uf,u,sizeof(user_t)))                     // User hinausschreiben
  { trace(serious,"newuser","write data %s",u->call);
    return 0;
  }

  return 1;
}

/*---------------------------------------------------------------------------*/

static unsigned long udrin(char *usercall,handle uf,handle uhf)
//*************************************************************************
//
//  Prueft, ob ein USER mit dem angegebenen Call schon
//  vorhanden ist.
//
//*************************************************************************
{ long unsigned hsh=strcrc(usercall);
  long unsigned idx;
  user_t ux;

  lseek(uhf,(long)hsh*4,SEEK_SET);
  _read(uhf,&idx,sizeof(long unsigned));
  if(idx)
  { int num=0;
    while(idx)
    { waitfor(e_ticsfull);
      lseek(uf,(long)idx*sizeof(user_t),SEEK_SET);
      if(!_read(uf,&ux,sizeof(user_t)))
      { trace(serious,"udrin","read %08X %08X %s %d",idx,hsh,usercall,num);
        return 0;
      }
      if((num++)>200)                     // zyklische Verkettung, abbrechen
      { trace(serious,"udrin","loop %08X %08X %s",idx,hsh,usercall);
        return 0;
      }
      if(strcmp(usercall,ux.call)==0)
        return idx;
      idx=ux.nextsamehash;
    }
  }
  return 0;
}

/*---------------------------------------------------------------------------*/

int us_readcache(char *usercall,user_t *u)
//*************************************************************************
//
//  Sucht einen Eintrag im cache und holt ihn
//
//*************************************************************************
{ int i;
  for(i=0;i<US_CACHESZ;i++)
  { if(strcmp(usercall,us_cache[i].call)==0)
    { memcpy(u,&us_cache[i],sizeof(user_t));
      return 1;
    }
  }
  return 0;
}

/*---------------------------------------------------------------------------*/

void us_putcache(user_t *u)
//*************************************************************************
//
//  Legt einen Eintrag im Cache ab oder ggf. aktualisiert ihn
//
//*************************************************************************
{ int i;
  static int put_pos=0;

  for(i=0;i<US_CACHESZ;i++)
  { if(strcmp(u->call,us_cache[i].call)==0)
    { memcpy(&us_cache[i],u,sizeof(user_t));
      return;
    }
  }
  memcpy(&us_cache[put_pos],u,sizeof(user_t));
  put_pos=(put_pos+1)%US_CACHESZ;
}

/*---------------------------------------------------------------------------*/

void us_deletecache(void)
//*************************************************************************
//
//  Loescht alle Cache-Eintraege (nach reorg)
//
//*************************************************************************
{ int i;
  for(i=0;i<US_CACHESZ;i++)
    us_cache[i].call[0]=0;
}

/*---------------------------------------------------------------------------*/

int loaduser(char *usercall,user_t *u,int anlegen)
//*************************************************************************
//
//  Laedt die Benutzerdaten fuer ein bestimmtes Call.
//  Ggf. wird ein neuer Eintrag erzeugt.
//
//*************************************************************************
{ short int retwert=0;

  if(us_readcache(usercall,u))
    retwert=1;
  else
  { long unsigned idx;
    long unsigned hsh=strcrc(usercall);
    us_open();
    lseek(uhf,(long)hsh*4,SEEK_SET);
    _read(uhf,&idx,sizeof(long unsigned));
    if(idx)
    { int num=0;
      while(idx)
      { lseek(uf,(long)idx*sizeof(user_t),SEEK_SET);
        if(!_read(uf,u,sizeof(user_t)))
        { trace(serious,"loaduser","read %08X %08X %s %d",idx,hsh,usercall,num);
          reorg_noetig++;
          goto ende;
        }
        if((num++)>500)
        { trace(serious,"loaduser","loop %08X %08X %s",idx,hsh,usercall);
          reorg_noetig++;
          goto ende;
        }
        if(strcmp(usercall,u->call)==0)
        { retwert=1;
          goto ende;
        }
        idx=u->nextsamehash;
      }
    }
    if(anlegen)
    { defaultuser(usercall,u);
      retwert=newuser(u,uf,uhf);
    }
    ende:
    us_close();
    if(retwert)
    { u->lastload=ad_time();
#ifdef _LONG_HROUTE
      //if the new mybbs field is empty, we have to use the old one
      if (*u->mybbs == 0) strcpy (u->mybbs, u->oldmybbs);
#endif
      us_putcache(u);
    }
    else
      u->filepos=0;
  }
  return retwert;
}

/*---------------------------------------------------------------------------*/

void reorguser(int delempty)
//*************************************************************************
//
//  Generiert die Hashtabelle neu und ueberprueft die
//  Eintraege auf Plausibilitaet. Geloeschte Eintraege werden
//  entfernt.
//  Bei delempty==1 werden veraltete Eintraege ohne
//  eingegebenem Namen hinausgeworfen.
//
//*************************************************************************
{ handle ualt,uneu,uhneu;
  user_t ux;
  long unsigned pos=0,errors=0,ndeleted=0;
  time_t ti=ad_time();
  long flen=0;
  long maxdays=m.userlife*2;

  trace(replog,"reorg","userlist");
  xunlink(USERNAMEBAK);
  xunlink(USERHASHNAMEBAK);
  xunlink(UTMP);
  xunlink(UHTMP);
  ualt=s_open(USERNAME,"lrb");
  flen=filelength(ualt);
  if(dfree(".",0) < (flen/1024L+USERDISKFULL))
  { s_close(ualt);
    trace(serious,"reorguser","disk full");
    return;
  }
  if (maxdays < 180)
    maxdays=180;
  uneu=s_open(UTMP,"lw+b");
  uhneu=s_open(UHTMP,"lw+b");
  if((ualt!=EOF) && (uneu!=EOF) && (uhneu!=EOF))
  { hashinit(uhneu,HASH32);
    _read(ualt,&ux,sizeof(user_t));
    defaultuser("DUMMY",&ux);
    _write(uneu,&ux,sizeof(user_t));
    while(1)
    { waitfor(e_ticsfull);
      if(!_read(ualt,&ux,sizeof(user_t)))
        break;
      ux.nextsamehash=0;
      ux.restplatz[0]=0;
#ifdef _LONG_HROUTE
      //if the new mybbs field is empty, we have to use the old one
      if (*ux.mybbs == 0) strcpy (ux.mybbs, ux.oldmybbs);
#endif
      if(delempty && ux.name[0]==0)
      { if((ux.mybbsok==0 && ux.mybbstime<(ti-(3600L*24*maxdays)))
                          || ux.mybbstime<(ti-(3600L*24*2*maxdays)))
        continue;
      }              
      if (ux.mybbstime > (ti+3600L*24*30)) //remove incorrect mybbs
      { ux.mybbs[0] = 0;
#ifdef _LONG_HROUTE
        *ux.oldmybbs = 0;
#endif
        ux.mybbstime = 0;
        errors++;
      }  
      if(!ux.call[0])
      { ndeleted++;
        continue;
      }      
      if((strlen(ux.call)>CALLEN)||(strlen(ux.ttypw)>8)|| !mbcallok(ux.call))
      { errors++;
        continue;
      }
      if((ux.mybbs[0]) && mbhadrok(ux.mybbs)!=1)
      { ux.mybbs[0]=0;
        ux.mybbsok=0;
        errors++;
      }

      // --------------------------------------------------------------------
      // - Strings auf Einhaltung der maximalen Laenge pruefen
      // - Zahlenwerte auf Einhaltung des Wertebereiches pruefen (nicht alle)
      // - Fehlerhafte Eintraege werden im Syslog vermerkt
      //   (nicht geloescht!)                                DG9FDL, 06.05.97

      if ((strlen(ux.prompt)>PROMPTLEN) ||      // Prompt
          (strlen(ux.name)>NAMELEN) ||          // Name
          (strlen(ux.mybbs)>MYBBSLEN) ||        // MyBBS
          (strlen(ux.prompt)>PROMPTLEN) ||      // Prompt
          (strlen(ux.ttypw)>8) ||               // TTY-PW
          (strlen(ux.uplink)>9) ||              // Uplink
          (strlen(ux.sprache)>3) ||             // Sprache
          (strlen(ux.notvisible)>REJECTLEN) ||  // Reject
          (strlen(ux.firstcmd)>FIRSTCMDLEN) ||  // Command
          (strlen(ux.uplink)>9) ||              // uplink
          (strlen(ux.ufwd)>39) ||               // ufwd
          (strlen(ux.newcall)>CALLEN) ||        // newcall
          (ux.lf>5) ||                          // lf: (0..5)
          (ux.helplevel>5) ||                   // helpleve (0..2)
          (ux.readlock>2) ||                    // readlock (0..2)
          (ux.pwline>1) ||                      // pwline (0..1)
          (ux.echo>1) ||                        // echo (0..1)
          (ux.status>10) ||                     // status (0..2)
          (ux.nopurge>1) ||                     // nopurge (0..1)
          (ux.nameok>1)  ||                     // nameok (0..1)
          (ux.fhold>1)   ||                     // fhold (0..1)
          (ux.fdelay>60) ||                     // fdelay (0..60)
          (ux.rlimit>1))                        // rlimit (0..1)
      { if (errors < 100)
          trace(replog,"reorguser","data error %s",ux.call);
        errors++;
      }

      // --------------------------------------------------------------------

      if(udrin(ux.call,uneu,uhneu))
      { trace(replog,"reorguser","double %s",ux.call);
        errors++;
      }
      else
        newuser(&ux,uneu,uhneu);
      pos++;
      if((pos%5000)==0)
      { char s[20];
	sprintf(s,"%lu:%s",pos,ux.call);
	lastcmd(s);
	trace(report,"reorguser",s);
      }
    }
    trace(replog,"reorguser","%u user, %u errors, %u deleted",
       pos,errors,ndeleted);
    s_close(ualt);
    s_close(uneu);
    s_close(uhneu);
    xrename(USERNAME,USERNAMEBAK);
    xrename(USERHASHNAME,USERHASHNAMEBAK);
    xrename(UTMP,USERNAME);
    xrename(UHTMP,USERHASHNAME);
    us_deletecache();
    lastreorg=ad_time();
  }
}

/*---------------------------------------------------------------------------*/

void uimport(void)
//*************************************************************************
//
//  Importiert Benutzerdaten aus dem Teebox-Format
//
//*************************************************************************
{ struct
  { char speech[4];
    char lastlog[15];
    char mybbs[7];
    char name[16];
    int  status;
    long lastmybbs;
  } udat;
  struct
  { char call[8];
    long pos;
  } uidx;
  FILE *fd=s_fopen("user3.dat","lrb");
  FILE *fi=s_fopen("user3.idx","lrb");
  user_t ux;
  unsigned num=0, errors=0;

  if(fd && fi)
  { while(1)
    { waitfor(e_ticsfull);
      if(testabbruch())
	break;
      if(!fread(&uidx,sizeof(uidx),1,fi))
	break;
      fseek(fd,uidx.pos,SEEK_SET);
      if(!fread(&udat,sizeof(udat),1,fd))
	break;
      strlwr(udat.name);
      udat.name[0]=toupper(udat.name[0]);
      while(uidx.call[strlen(uidx.call)-1]==' ')
	uidx.call[strlen(uidx.call)-1]=0;
      num++;
      waitfor(e_ticsfull);
      if(mbcallok(uidx.call) && mbcallok(udat.mybbs))
      { //putf("%-6s @ %-6s %s\n",uidx.call,udat.mybbs,udat.name);
	if((num%100)==0)
	  putf("%u:%s\n",num,ux.call);
#ifdef _UIMPORT		//db1ras
      if(udat.lastmybbs > (ad_time() - 3600L*24*180))		// juenger als
       {							// 6 Monate?
#endif
	if(loaduser(uidx.call,&ux,1))
	{ if(ux.mybbsok==0)
	    strcpy(ux.mybbs,udat.mybbs);
	  ux.mybbsok=1;
	  ux.mybbstime=udat.lastmybbs;
	  udat.name[13]=0;
	  if(ux.name[0]==0)
	    strcpy(ux.name,udat.name);
	  saveuser(&ux);
	}
#ifdef _UIMPORT		//db1ras
       }
#endif
      }
      else
        errors++;
    }
    s_fclose(fd);
    s_fclose(fi);
  }
  else
    putf("user3.dat or user3.idx not found.\n");
  putf("%u calls imported, %u errors.\n",num,errors);
}

/*---------------------------------------------------------------------------*/

void saveuser(user_t *u)
//*************************************************************************
//
//  USER abspeichern
//
//*************************************************************************
{ if(u->filepos && (u->lastload>lastreorg))
  {
#ifdef _LONG_HROUTE
    //for backward compatibility: save the mybbs also into oldmybbs
    if (strlen (u->mybbs) <= OLDMYBBSLEN) strcpy (u->oldmybbs, u->mybbs);
    else strcpy (u->oldmybbs, atcall (u->mybbs));
#endif
    us_putcache(u);
    us_open();

    lseek(uf,(unsigned long)u->filepos*sizeof(user_t),SEEK_SET);
    _write(uf,u,sizeof(user_t));
    us_close();
  }
}

/*---------------------------------------------------------------------------*/

void userinit(void)
//*************************************************************************
//
//   Zugriff USER-File vorbereiten
//
//*************************************************************************
{ if(sizeof(user_t)!=512 || is_null)
    trace(tr_abbruch,"user_t","size %d",sizeof(user_t));
  us_open();
  if(uf==EOF)
  { trace(tr_abbruch,"userinit","not open");
    return;
  }
  hashinit(uhf,HASH32);
  us_close();
}

/*---------------------------------------------------------------------------*/

void listuser(char *wildcard,int lokal)
//*************************************************************************
//
//  Listet alle bekannten Benutzerrufzeichen
//
//*************************************************************************
{ user_t us;
  int spalte=0;

  handle fh=s_open(USERNAME,"lrb");

  putf("Directory of all %sknown user-calls:\n",lokal?"locally ":"");
  lseek(fh,sizeof(user_t),SEEK_SET);
  while(_read(fh,&us,sizeof(user_t)))
  { if(!wildcard[0] || strstr(us.call,wildcard))
    { if(!lokal || us.lastboxlogin)
      { putf("%-7s",us.call);
        if((++spalte)>10)
        { putf("\n");
          spalte=0;
        }
      }
    }
    waitfor(e_ticsfull);
    if(testabbruch()) break;
  }
  if(spalte)
    putf("\n");
  s_close(fh);
}

/*---------------------------------------------------------------------------*/

static void scanpar(char *buf,char *ziel,char *such)
//*************************************************************************
//
//  Hilfsroutine, sucht nach einem Parameter, der mit einem
//  Suchbegriff beginnt (Suchbegriff ist nicht Bestandteil)
//
//*************************************************************************
{ char *found=strstr(buf,such);
  int l=strlen(such);

  ziel[0]=0;
  if(found)
  { while(l)
    { found[0]=' ';
      found++;l--;
    }
    while(found[0]>' ')
    { if(l<11)
      { ziel[0]=found[0];
        ziel[1]=0;
        ziel++;
      }
      found[0]=' ';
      found++;
    }
  }
}

/*---------------------------------------------------------------------------*/

void usersuch(char *befbuf)
//*************************************************************************
//
//  Durchsucht saemtliche Benutzerdaten nach Suchbegriffen
//
//*************************************************************************
{ FILE     *uf;
  user_t   us;
  unsigned hitcounter=0;
  unsigned allcounter=0;
  bitfeld  options      = b->optplus;
  unsigned onelogin     = !!(options & o_l);
  long unsigned nologin =   (options & o_o)?0:ULONG_MAX;
  int      mybbsunknown =   (options & o_u)?0:255;
  int      mybbsknown   = !!(options & o_n);
  int      mybbsok      = !!(options & o_m);
  int      pwset        = !!(options & o_p);
  int      newcall      = !!(options & o_q);
  int      ttypwset     = !!(options & o_t);
  int      helplevel0   =   (options & o_h)?0:255;
  int      status       = !!(options & o_s);
  int      readlock     = !!(options & o_r);
  int      userfwd      = !!(options & o_w);
  int      nopurge      = !!(options & o_y);
  int      fhold        = !!(options & o_d);
  int      rlimit       = !!(options & o_i);
  char     call[12];
  char     name[12];
  char     mybbs[12];
  char     uplink[12];
  char     sprache[12];
  char     lastls[12];     // last login string
  time_t   lastlogin=0L;

  strupr(befbuf);
  scanpar(befbuf,call,   "C=");
  scanpar(befbuf,name,   "N=");
  scanpar(befbuf,mybbs,  "@=");
  scanpar(befbuf,uplink, "V=");
  scanpar(befbuf,sprache,"S=");
  scanpar(befbuf,lastls, "L=");

  if(lastls[0])
    lastlogin=parse_time(lastls);

  if(call[0]==0)
  { befbuf+=blkill(befbuf);
    blankweg(befbuf);
    if(strlen(befbuf)>11)
      befbuf[11]=0;
    strcpy(call,befbuf);
  }

  uf=s_fopen(USERNAME,"lrb");
  if(uf)
  { setvbuf(uf,NULL,_IOFBF,16384);
    if(!(b->optplus & o_c))
    { putf("Call   Mybbs               Name         "
           "Logins Last via      ");
      if (newcall)
        putf("New call\n");
      else  
        putf("PTHLRSUN Spr\n");
    }
    fseek(uf,sizeof(user_t),SEEK_SET);
    while(fread(&us,sizeof(user_t),1,uf))
    {
#ifdef _LONG_HROUTE
      //if the new mybbs field is empty, we have to use the old one
      if (*us.mybbs == 0) strcpy (us.mybbs, us.oldmybbs);
#endif
       if(us.logins     >= onelogin                    &&
	us.logins      <= nologin                     &&
	us.status      >= status                      &&
	us.readlock    >= readlock                    &&
	us.nopurge     >= nopurge                     &&
	us.fhold       >= fhold                       &&
	us.rlimit      >= rlimit                      &&
	us.mybbs[0]    <= mybbsunknown                &&
	us.mybbs[0]    >= mybbsknown                  &&
	us.mybbsok     >= mybbsok                     &&
	us.password[0] >= pwset                       &&
	us.ttypw[0]    >= ttypwset                    &&
	us.ufwd[0]     >= userfwd                     &&
	us.helplevel   <= helplevel0                  &&
	us.lastboxlogin>= lastlogin                   &&
        us.newcall[0]  >= newcall                     &&
	(!call[0]      || strstr(us.call,call))       &&
	(!name[0]      || stristr(us.name,name))      &&
	(!mybbs[0]     || strstr(us.mybbs,mybbs))     &&
	(!uplink[0]    || stristr(us.uplink,uplink))  &&
	(!sprache[0]   || strstr(us.sprache,sprache)) &&
        us.call[0])
      { if(!(b->optplus & o_c))
	{ putf("%-6s %c%-18.18s %-12.12s%7ld %4s %-8.8s ",
	       us.call,us.mybbsok?'@':'?',us.mybbs,us.name,us.logins,
	       zeitspanne(ad_time()-us.lastboxlogin,zs_seconds),us.uplink);
          if (newcall)
            putf("%s\n",us.newcall);
          else         
            putf("%u%u%u%u%u%u%u%u %s\n",!!us.password[0]+(us.password[0]==1),
               !!us.ttypw[0],us.helplevel, us.lf,us.readlock,us.status,
               !!us.ufwd[0],us.nopurge,us.sprache);
	}
	hitcounter++;
	if(b->optplus & o_f)
	  break;
      }
      if(((allcounter++)%32)==0)
      { waitfor(e_ticsfull);
	if(testabbruch())
	  break;
      }
    }
    s_fclose(uf);
    putf("%u users, %u found.\n",allcounter,hitcounter);
  }
}


/*---------------------------------------------------------------------------*/

static void near userfile_newformat(void)
{
 
  //no scheduling here - watchdog is not active at this state
  //on K6/200: 14s under Linux, 24s under Win95/DOS for 64000 users

  if(filetime(USERHASHNAME_OLD) > filetime(USERHASHNAME))
  { FILE *fi;
    FILE *fo;
    user_t *us=(user_t*)t_malloc(sizeof(user_t),"usnf");

    trace(replog,"user","converting format "USERHASHNAME_OLD"->"USERHASHNAME);
    fi=s_fopen(USERHASHNAME_OLD,"srb");
    fo=s_fopen(UHTMP,"swb");
    if (fo)
      s_fsetopt(fo,1);
    if(fi && fo)
    { struct { short unsigned l;short unsigned h; } b={0,0};
      while(fread(&b.l,2,1,fi))
        fwrite(&b,4,1,fo);
    }
    if(fo)s_fclose(fo);
    if(fi)s_fclose(fi);
    fi=s_fopen(USERNAME,"srb");
    fo=s_fopen(UTMP,"swb");
    if (fo)
       s_fsetopt(fo,1);
    if(fi && fo)
    { while(fread(us,sizeof(user_t),1,fi))
 
    //copy old 16bit hash to 32bit hash, but leave old hash
    //if user wants to return to an older version
  
     { if (us->nextsamehash==0)
        { us->nextsamehash=us->nextsamehash_old;
          us->filepos=us->filepos_old;
          //us->filepos_old=0;
          //us->nextsamehash_old=0;
        }
        fwrite(us,sizeof(user_t),1,fo);
      }
    }
    if(fi)s_fclose(fi);
    if(fo)s_fclose(fo);
   
    xunlink(USERHASHNAME);
    xrename(UHTMP,USERHASHNAME);
    xunlink(USERNAME);
    xrename(UTMP,USERNAME);
    t_free(us);
  }
}
/*---------------------------------------------------------------------------*/

#ifdef _EXDLOGIN	//db1ras
void defaultguest(char *usercall,user_t *u)
//*************************************************************************
//
//  Ueberschreibt einen vorhandenen User mit Gast-Defaultwerten
//  Eine ggf. vorhandene Hash-Verkettung wird beibehalten.
//
//*************************************************************************
{
  char     name[NAMELEN+1];				// A NAME
  strcpy(name, u->name);
  char     mybbs[MYBBSLEN+1];				// A FORWARD
  strcpy(mybbs, u->mybbs);
  char     nameok		= u->nameok;		// 1 = selbst eingegeben
  time_t   lastboxlogin		= u->lastboxlogin;	// letzter Login
  time_t   lastdirnews		= u->lastdirnews;	// letztes DIR NEWS
  time_t   lastquit		= u->lastquit;		// letztes QUIT
  long     mailsent		= u->mailsent;		// gesendete Nachrichten
  long     mailgot		= u->mailgot;		// erhaltene Nachrichten
  long     mailread		= u->mailread;		// gelesene Nachrichten
  long unsigned logins		= u->logins;		// logins
  char     mybbsok		= u->mybbsok;		// 1 = selbst eingegeben
  time_t   mybbstime		= u->mybbstime;		// Datum letztes MYBBS
  long unsigned  daybytes	= u->daybytes;		// gelesene Bytes heute
  short unsigned status		= u->status;		// A STATUS
  short unsigned helplevel      = u->helplevel;		// Helplevel

  defaultuser(usercall,u);

  strcpy(u->name, name);
  strcpy(u->mybbs, mybbs);
  u->nameok		= nameok;
  u->lastboxlogin	= lastboxlogin;
  u->lastdirnews	= lastdirnews;
  u->lastquit		= lastquit;
  u->mailsent		= mailsent;
  u->mailgot		= mailgot;
  u->mailread		= mailread;
  u->logins		= logins;
  u->mybbsok		= mybbsok;
  u->mybbstime		= mybbstime;
  u->daybytes		= daybytes;
  u->status		= status;
  u->helplevel		= helplevel;

  saveuser(u);
}
#endif
/*---------------------------------------------------------------------------*/
