/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  -----------------
  Diverse Utilities
  -----------------


  Copyright (C)       Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/

//19980115 OE3DZW utcoffset -> ad_timezone
//19880118 OE3DZW sec in timestr
//19980202 OE3DZW added support for CB-calls (hrx)
//19980203 OE3DZW added tm_isdst -> was not defined before
//19980307 OE3DZW added 7plus-check, added typecast
//19980311 OE3DZW Oscar satellite calls are accepted eg KO23
//19980404 hrx    support for guest-callsigns
//19980408 hrx    added case 'e' to chartomakro() (for checkcount)
//19980423 OE3DZW added returnmailok
//19980505 OE3DZW checkcount now long (was int)
//19980916 OE3DZW 7line-check strikter, lines must not be longer 72 chars
//19981001 OE3DZW added strupr to filename2time (problem under http)
//19981025 OE3DZW added temperature to macros (wx-station only)
//19990110 OE3DZW y2k in get_headertime, makeheader
//19990615 DH3MB  Added getmd2sum, getmd5sum, mdsum2str
//                Moved file_crcthp from mbcrc.c to this file
//19990628 DH3MB  Renamed getmd2/5sum -> file_md2/5sum
//                Adapted line_7p, filemd2/5sum, file_crcthp for
//                new CRC- and MD2/5-classes
//19990701 DH3MB  Fixed memory leak in file_md2sum, file_md5sum
//                Rewrote file_crcthp; data is read blockwise
//19990722 DB1RAS added support for cb-calls over the whole world
//19990822 DH3MB  Removed file_crcthp, file_md2sum and file_md5sum
//                Use method readfile() in CRC- and MDx-classes instead!

#include "baycom.h"

/*---------------------------------------------------------------------------*/

char *getheaderadress(char *rline)
//*************************************************************************
//
//  Extrahiert aus einer R: -Headerzeile die H-Adresse
//  der Mailbox, die das File bertragen hat.
//
//*************************************************************************
{ static char headercall[HADRESSLEN+1];
  char *ls;
  if((rline[0]=='R') && (rline[1]==':'))
  { ls=strchr(rline,'@');
    if(!ls)
      ls=strchr(rline,'&');    // wegen AMTOR, kein @
    if(ls)
    { if(ls[1]==':')
	ls+=2;
      else
	ls+=1;
    nexttoken(ls,headercall,HADRESSLEN);
    return headercall;
    }
  }
  return NULL;
}

/*---------------------------------------------------------------------------*/

time_t getheadertime(char *rline)
//*************************************************************************
//
//  Extrahiert aus einer R: -Headerzeile Datum und Uhrzeit, zu dem
//  das File bertragen worden ist.
//  Rckgabe im UNIX-Format.
//
//*************************************************************************
{ struct tm tt;
  char datstr[12];
/* Example
012345678901234567890
R:981012/0043z @:OE3DZW.OE1XAB.#OE1.AUT.EU [BayCom-Mailbox] bcm1.40h3
       '/'   ' '
  yymmdd hhmm
  01234567890
*/
  //tt muss geloescht werden, damit das Ergebnis von mktime stimmt! (deti)
  memset(&tt,0,sizeof(struct tm));
  if((rline[8]=='/') && (rline[14]==' '))
  { strncpy(datstr,rline+2,11);
    //minutes
    datstr[11]=0;
    tt.tm_min=atoi(datstr+9);
    //hours
    datstr[9]=0;
    tt.tm_hour=atoi(datstr+7);
    //day of month
    datstr[6]=0;
    tt.tm_mday=atoi(datstr+4);
    //month
    datstr[4]=0;
    tt.tm_mon=atoi(datstr+2)-1;
    //year
    datstr[2]=0;
    tt.tm_year=atoi(datstr);
    if (tt.tm_year<70)
      tt.tm_year+=100;  //y2k
	
    if(tt.tm_mday && tt.tm_mon<12 && tt.tm_year)
      return ad_mktime(&tt);
  }
  return 0L;
}

/*---------------------------------------------------------------------------*/

time_t yymmdd2ansi(char *yymmdd)
//*************************************************************************
//
//  converts WP-time to Ansitime
//
//*************************************************************************
{ struct tm tt;
  char datstr[7];
/* Example
  981012 
  012345
*/

  //tt muss geloescht werden, damit das Ergebnis von mktime stimmt! (deti)
  memset(&tt,0,sizeof(struct tm));
  strncpy(datstr,yymmdd,6);
  datstr[6]=0;
  //minutes
  tt.tm_min=0;
  //hours
  tt.tm_hour=0;
  //day of month
  tt.tm_mday=atoi(datstr+4);
  //month
  datstr[4]=0;
  tt.tm_mon=atoi(datstr+2)-1;
  //year
  datstr[2]=0;
  tt.tm_year=atoi(datstr);
  if (tt.tm_year<70)
    tt.tm_year+=100;  //y2k

  if(tt.tm_mday && tt.tm_mon<12 && tt.tm_year)
    return ad_mktime(&tt);

  return 0L;

}
/*---------------------------------------------------------------------------*/

long nonlin(long bytes)
//*************************************************************************
//
//  Nichtlineare Darstellung von Byteangaben - Komprimierung 
//
//*************************************************************************
#define NONLINMAX 1048576L
{
  if (bytes > NONLINMAX)
    bytes=NONLINMAX+bytes/1024;
  return bytes;  
}

/*---------------------------------------------------------------------------*/

long inonlin(long bytes)
//*************************************************************************
//
//  Nichtlineare Darstellung von Byteangaben - Entkomprimierung
//
//*************************************************************************
{
  if (bytes > NONLINMAX)
    bytes=(bytes-NONLINMAX)*1024;
  return bytes;  
}

/*---------------------------------------------------------------------------*/

char *dezasc(long dez,int stellen)
//*************************************************************************
//
//  Wandelt eine Zahl in einen druckbaren String um
//
//*************************************************************************
{ static byte s[10];

  if(stellen>9)
  { trace(fatal,"dezasc","too long %d",stellen);
    stellen=9;
  }
  s[stellen]=0;
  while(stellen)
  { stellen--;
    s[stellen]=(dez&127)+'!';
    dez>>=7;
  }
  if(dez)
    trace(serious,"dezasc","too short: %d/%d",dez,stellen);
  return (char *)s;
}

/*---------------------------------------------------------------------------*/

long ascdez(char *sc,int stellen)
//*************************************************************************
//
//  Wandelt einen druckbaren String in eine Zahl um
//
//*************************************************************************
{ long retwert=0;
  byte *s=(byte *)sc;

  if(stellen>9)
  { trace(fatal,"ascdez","too long %d:%s",stellen,s);
    stellen=9;
  }
  while(s[0] && stellen)
  { if(s[0]<'!')
    { trace(serious,"ascdez","range %s",s);
      return 0;
    }
    retwert<<=7;
    retwert|=(s[0]-'!');
    stellen--;
    s++;
  }
  return retwert;
}

/*---------------------------------------------------------------------------*/

static int near nibdez(char nibble)
//*************************************************************************
//
//   Macht aus einem Zeichen im Zahlensystem zur Basis 36
//   eine Dezimalzahl (fr Filenamen).
//
//*************************************************************************
{ if((nibble>='0') && (nibble<='9'))
    return nibble-'0';
  else if((nibble>='A') && (nibble<='Z'))
    return nibble-('A'-10);
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

char deznib(int dez)
//*************************************************************************
//
//  Erzeugt eine einstellige Zahl zur Basis 36 (fr Filenamen).
//
//*************************************************************************
{ if(dez<10)
    return dez+'0';
  else
    return dez+('A'-10);
}

/*---------------------------------------------------------------------------*/

char *makeheader(int full)
//*************************************************************************
//
//  Erzeugt einen forward-Header
//
//*************************************************************************
{ static char h[80];
  int len;
  time_t unixtime=ad_time()+(ad_timezone());

  struct tm *tt=ad_comtime(unixtime);
  int year2; //y2k
  if (tt->tm_year > 99)
    year2 = tt->tm_year-100;
  else
    year2 = tt->tm_year;    
  sprintf(h,"R:%02d%02d%02d/%02d%02dz @:",
          year2,tt->tm_mon+1,tt->tm_mday,tt->tm_hour,tt->tm_min);

  //strftime(h,sizeof(h),"R:%y%m%d/%H%Mz @:",localtime(&unixtime));
  strcat(h,m.boxadress);
  if(full)
  { len=strlen(h);
    m.boxheader[63-len]=0;
    sprintf(h+len," [%s] "BCMLOGO VNUMMER,m.boxheader);
  }
  return h;
}

/*---------------------------------------------------------------------------*/

char *time2filename(time_t unixtime)
//*************************************************************************
//
//  Erzeugt einen neuen Filenamen, abhaengig von der uebergebenen Zeit
//  wird die Funktion in einer Minute 2mal aufgerufen, so wird die laufende
//  Nummer der Nachricht erhoeht
//  der Filename wird dabei von der LOKALZEIT !! berechnet
//
//*************************************************************************
{ static time_t lasttime=0;
  static byte name[9];
  struct tm *tt;
  static unsigned nextfile=0;

  if(!unixtime)
  {  unixtime=ad_time();
    while((nextfile==35) && ((unixtime>>2)==lasttime))
    { wdelay(117);
      unixtime=ad_time();
    }
    if((unixtime>>2) != lasttime)
    { nextfile=0;
      lasttime=(unixtime>>2);
    }
    else
      nextfile++;
  }

  tt=ad_comtime(unixtime);
  name[0]=deznib(tt->tm_year-90);
  name[1]=deznib(tt->tm_mon+1);
  name[2]=deznib(tt->tm_mday);
  name[3]=deznib(tt->tm_hour);
  name[4]=deznib(tt->tm_min>>1);
  name[5]=deznib((tt->tm_min&1)*15+(tt->tm_sec>>2));
  name[6]=deznib(nextfile);
  name[7]=0;
  return (char *)name;
}

/*---------------------------------------------------------------------------*/

time_t filename2time(char *name)
//*************************************************************************
//
//  Extrahiert aus einem Filenamen die Zeit im UNIX-Format
//  und interpretiert die laufende Nummer als Sekunde
//
//*************************************************************************
{ struct tm tt;
  byte s[9];

  memset(&tt,0,sizeof(struct tm));

  if((strlen(name)>8) || (name[0]==' '))
  { trace(fatal,"f2time","%s too long",name);
    name[8]=0;
  }
  strcpy((char *)s,name);
  strupr((char *)s); 
  //trace(report,"f2t","\"%s\"",s);
  tt.tm_sec=nibdez(s[6]);    // laufende Nummer als Sekunde interpretieren
  tt.tm_min=nibdez(s[5])/15;
  tt.tm_min+=nibdez(s[4])<<1;
  tt.tm_hour=nibdez(s[3]);
  tt.tm_mday=nibdez(s[2]);
  tt.tm_mon=nibdez(s[1])-1;
  tt.tm_year=nibdez(s[0])+90;
  tt.tm_isdst=-1; //OE3DZW
  return ad_mktime(&tt);
}

/*---------------------------------------------------------------------------*/

static near int mbamateurcallok(char *call)
//*************************************************************************
//
//  Prft einen String, ob ein gltiges AFU-Call drinsteht
//  wird zur Unterscheidung Info/User verwendet
//  Aber nicht zur Unterscheidung Bulletin/Personal/Ack..
//
//  1..is call 0..is no call
//
//*************************************************************************
{ int i,j=0,len=strlen(call);

  if(strstr(call,"3DA")==call)
    return 1;
  if(len>6)
    return 0;
  for(i=0;i<len;i++)
    if(isdigit(call[i])) j++;
  if((j==1) && (isdigit(call[0])))
    return 0;
  if((j==2) && (isdigit(call[0])) && (isdigit(call[1])))
    return 0;
  // Oscar satellite eg KO23
  if ((len==4) && (call[1]=='O') && isalpha(call[0]) &&
       isdigit(call[2]) && isdigit(call[3]) )
    return 1;  
  return(isalpha(call[3]) &&
	 ((len<5) || isalpha(call[4])) &&
	 ((len<6) || isalpha(call[5])) &&
	 isalnum(call[0]) &&
	 isalnum(call[1]) &&
	 isalnum(call[2]) &&
	 (j>0) && (j<3));
}

/*---------------------------------------------------------------------------*/

static near int mbcbcallok(char *call)
#ifndef CB_OPT		//db1ras
//*************************************************************************
//
//  Checks if a call is a valid cb callsign (only supports German/Swiss
//  callsigns) 
//
//*************************************************************************
{ int len=strlen(call);

  if(len>6)
    return 0;

  return                                         
  (                                              
    (                                            
      (toupper(call[0]) == 'D') &&               
      (toupper(call[1]) == 'A') &&               
      (toupper(call[2]) <= 'Z') &&               
      (isalpha(call[2])) &&                      
      ((call[3]) >= '2') &&                      
      ((call[3]) <= '9') &&                      
      (len == 6)    
    ) ||                                         
    (                                            
      (toupper(call[0]) == 'D') &&               
      (toupper(call[1]) == 'B') &&               
      (toupper(call[2]) >= 'B') &&               
      (toupper(call[2]) <= 'Q') &&               
      (isalpha(call[2])) &&                      
      ((call[3]) >= '2') &&                      
      ((call[3]) <= '9') &&                      
      (len == 6)    
    ) ||                                         
    (                                            
      (toupper(call[0]) == 'D') &&               
      (toupper(call[1]) == 'B') &&               
      (toupper(call[2]) >= 'S') &&               
      (isalpha(call[2])) &&                      
      ((call[3]) >= '2') &&                      
      ((call[3]) <= '9') &&                      
      (len == 6)   
    ) ||                                         
    (                                            
      (toupper(call[0]) == 'D') &&               
      (toupper(call[1]) >= 'C') &&               
      (toupper(call[1]) <= 'D') &&               
      (isalpha(call[2])) &&                      
      ((call[3]) >= '2') &&                      
      ((call[3]) <= '9') &&                      
      (len == 6)   
    ) ||                                         
    (                                            
      (toupper(call[0]) == 'D') &&               
      (toupper(call[1]) == 'E') &&               
      (toupper(call[2]) >= 'B') &&               
      (toupper(call[2]) <= 'Q') &&               
      (isalpha(call[2])) &&                      
      ((call[3]) >= '2') &&                      
      ((call[3]) <= '9') &&                      
      (len == 6)    
    ) ||                                         
    (                                            
      (toupper(call[0]) == 'D') &&               
      (toupper(call[1]) == 'E') &&               
      (toupper(call[2]) >= 'S') &&               
      (isalpha(call[2])) &&                      
      ((call[3]) >= '2') &&                      
      ((call[3]) <= '9') &&                      
      (len == 6)     
    ) ||                                         
    (                                            
      (toupper(call[0]) == 'D') &&               
      (toupper(call[1]) >= 'F') &&               
      (toupper(call[1]) <= 'R') &&               
      (isalpha(call[2])) &&                      
      ((call[3]) >= '2') &&                      
      ((call[3]) <= '9') &&                      
      (len == 6)    
    ) ||                                         
    (                                            
      (toupper(call[0]) == 'H') &&               
      (toupper(call[1]) == 'E') &&               
      (isalpha(call[2])) &&                      
      (isdigit(call[3])) &&                      
      (isdigit(call[4])) &&                      
      (isdigit(call[5])) &&                      
      (len == 6)     
    )                                            
  );                                             
}
#else		//db1ras
//*************************************************************************
//
// Checks if a call is a valid cb callsign
//
//*************************************************************************
{ int len=strlen(call);

  if(len>6)
    return 0;

  return
    (
      (isalpha(call[0])) &&
      (isalpha(call[1])) &&
      (isalpha(call[2])) &&
      (isdigit(call[3])) &&
      (isdigit(call[4])) &&
      (isdigit(call[5])) &&
      (len == 6)
    );
}
#endif		//db1ras

/*---------------------------------------------------------------------------*/

#ifdef _LCF
// check for forwarding               // JJ
int fwdcallok(char *call,int fwdcallformat)
{
 if (fwdcallformat==1)                     //only CB-calls
   return mbcbcallok(call);
 if (fwdcallformat==2)                     //both
   return (mbcbcallok(call)+mbamateurcallok(call));
 return mbamateurcallok(call);         //default: only amateur-calls
}
#endif

/*---------------------------------------------------------------------------*/

int mbcallok(char *call)
//*************************************************************************
//
//  Checks if a string is a valid callsign
//
//*************************************************************************
{
#ifdef _GUEST
 if (strcmp(call,m.guestcall)==0     
    && !strcmp(m.guestcall,"OFF")==0) 
   return 1;                         
#endif

 if (m.callformat==1)		     //only CB-calls
   return mbcbcallok(call);
 if (m.callformat==2)		     //both
   return (mbcbcallok(call)+mbamateurcallok(call));
 return mbamateurcallok(call);	 //default: only amateur-calls
}

/*---------------------------------------------------------------------------*/

int mbhadrok(char *hadr)
//*************************************************************************
//
//  Prft einen String, ob er eine gltige Box-Adresse darstellt
//  Rckgabe:
//    0  ungueltig (<trash->)
//    1  Mailbox   (<call>.<h-adress>)
//    2  Verteiler (<verteiler>)
//
//*************************************************************************
{ char tmp[HADRESSLEN+1];

  if(strlen(hadr)>HADRESSLEN)
    return 0; // too long 
  if(strlen(hadr)<2)
    return 0; // to short 
  strcpy(tmp,hadr);
  strupr(tmp);
  if(strchr(tmp,'.')) // remove h-address
    strchr(tmp,'.')[0]=0;
  if(strstr(tmp,m.boxname) && strchr(tmp,'-')) 
    strchr(tmp,'-')[0]=0; // remove ssid of own boxaddress
  if(strlen(tmp)>6)
    return 0; //too long to be a callsign
  if(mbcallok(tmp))
    return 1; //is a valid callsign
  else
    return 2; //is no valid callsign but a valid boxaddress
}

/*---------------------------------------------------------------------------*/

char mbmailt(char *ziel, char *hadr)
//*************************************************************************
//
//  Stellt die Mailtype fest, falls sie nicht angegeben wurde
//  Rckgabe:
//    'P' personal 
//    'B' bulletin 
//
//*************************************************************************
{ char tmp[HADRESSLEN+1];

  if (mbcallok(ziel))
    return 'P';
  strcpy(tmp,hadr);
  strupr(tmp);
  if(strchr(tmp,'.')) // remove h-address
    strchr(tmp,'.')[0]=0;
  if(strstr(tmp,m.boxname) && strchr(tmp,'-'))
    strchr(tmp,'-')[0]=0; // remove ssid of own boxaddress
  if(strcmp(m.boxname,tmp)!=0 && mbcallok(tmp))
    return 'P';
  return 'B';
}

/*---------------------------------------------------------------------------*/

int seek_lastentry(handle fh,int anzahl)
//*************************************************************************
//
//  Positioniert vom Ende einer Liste um 'anzahl' ungelschte
//  (bzw bei -v auch gelschte) Eintrge in einem LIST-File zurck
//  Rckgabe 0: Kein passender Eintrag enthalten
//
//*************************************************************************
{ unsigned len=(unsigned)(filelength(fh)>>7);
  char buf[16];
  int found=0;

  while(len)
  { len--;
    lseek(fh,128L*len,SEEK_SET);
    _read(fh,buf,16);
    if((buf[14]=='>') || (b->opt&o_v))
    { found++;
      if(found==anzahl)
      { lseek(fh,128L*len,SEEK_SET);
	return 1;
      }
    }
  }
  lseek(fh,0,SEEK_SET);
  return found;
}

/*---------------------------------------------------------------------------*/

time_t seek_fname(handle fh,char *fname,char *ok)
//*************************************************************************
//
//  Sucht in einem List-File nach dem bergebenem Filenamen
//  Da die Filenamen in der Liste sortiert sind, erfolgt die
//  Suche binr, d.h. es sind nur ld(listenlnge)+1 Filezugriffe notwendig
//
//  Wird der Name genau gefunden, so wird fr den nchsten Lesezugriff
//  auf diesen Positioniert und ok[0]==1 zurckgegeben.
//  Wird der Name nicht gefunden, so wird im File hinter die Stelle
//  positioniert, an der der Filename bei richtiger Einsortierung stehen
//  wrde.
//
//*************************************************************************
{ time_t lasttime=0L;
  int vgl;
  long lastseek;
  long index=1,step,seekindex;

  if(fh==EOF)
    return 0L;                           // kein File offen, Fehler --->
  lseek(fh,-128L,SEEK_END);              // zum letzten Eintrag

  lastseek=(unsigned)(lseek(fh,0L,SEEK_CUR)>>7);
  while((index-1)<lastseek)              // Filelnge binr aufrunden
    index<<=1;
  step=index/2;                          // Anfangswerte festlegen
  if(step)
    index=step-1;
  else
    index=0;

  _read(fh,b->line,128);                 // steht auf letzem Eintrag im File
  b->line[7]=0;
  lasttime=filename2time(b->line);       // neuesten Eintrag bestimmen

  if(strncmp(b->line,fname,7)==0)
  { lseek(fh,-128L,SEEK_CUR);            // Eintrag genau gefunden, eins
    *ok=1;                               // zurck, damit es genau passt
    return lasttime;
  }

  if(strncmp(b->line,fname,7)>=0)
  { do
    { if(index>lastseek)                 // nicht hinter das Ende lesen
	seekindex=lastseek;
      else
	seekindex=index;
      lseek(fh,(long)seekindex<<7,SEEK_SET);
      _read(fh,b->line,128);
      vgl=strncmp(b->line,fname,7);
      if(vgl<0)
      { step>>=1;
	index+=step;
      }
      else if(vgl>0)
      { step>>=1;
	index-=step;
      }
      else
      { lseek(fh,-128L,SEEK_CUR);         // Eintrag genau gefunden, eins
	*ok=1;                           // zurck, damit es genau passt
	return lasttime;
      }
    } while(step);
    if(strncmp(b->line,fname,7)>0)
      lseek(fh,-128L,SEEK_CUR);
  }
  *ok=0;                     // Eintrag nicht gefunden, aber File steht
  return lasttime;           // nun an der richtigen Position
}

/*---------------------------------------------------------------------------*/

void scanoptions(char *buf)
{ b->optminus=b->optplus=0;

  buf+=blkill(buf);
  while(buf[0]=='-')
  { do
    { if(isalpha(buf[1]))
      { long bit=(1L<<(toupper(buf[1])-'A'));

	if(buf[2]=='-')
	{ b->optminus|=bit;
	  buf[0]=buf[1]=buf[2]=' ';
	  buf+=1;
	}
	else if(buf[2]=='+')
	{ b->optplus|=bit;
	  buf[0]=buf[1]=buf[2]=' ';
	  buf+=1;
	}
	else
	{ b->optplus|=bit;
	  buf[0]=buf[1]=' ';
	}
      }
      buf+=1;
    } while(isalpha(buf[1]));
    buf+=1;
    buf+=blkill(buf);
  }
  b->optminus=(~b->optminus);
}

/*---------------------------------------------------------------------------*/

void formoptions(void)
{ if(((b->optgroup&1)==0) && b->usermail)
    b->optgroup++;

  b->opt=((u->opt[b->optgroup] & b->optminus) | b->optplus);
  if(!b->continous)
    b->continous=!!(b->opt & o_c);
}

/*---------------------------------------------------------------------------*/

void findbereich(char *buf)
//*************************************************************************
//
//  Extrahiert aus einem String eine Bereichsangabe in der Form
//
//   z.B.   1-   -5   3-5
//
//  wird eine gueltige Bereichsangabe erkannt, so wird diese aus dem String
//  durch Blanks ersetzt und der Rckgabewert beinhaltet den Bereich.
//  Bei keiner oder ungltiger Angabe wird der maximale Bereich
//  zurueckgeliefert.
//
//*************************************************************************
{ unsigned beg,end;
  char *first;

  b->ende=MAXBEREICH;               // maximal darstellbare Zahl
  b->beginn=1;
  b->bereich=0;

  while(buf[0])                     // ganzen String durchsuchen
  { first=buf;
    beg=0;                          // default: von hinten listen ("l -5")
    end=b->ende;
    while(buf[0]==' ')              // Blanks entfernen
      buf++;
    if((buf[0]>'0') && (buf[0]<='9'))    // nach erster Zahl suchen (1..9)
    { beg=atoi(buf);
      while(isdigit(buf[0]))buf++;       // Zahl berspringen
    }

    if(buf[0]=='-')
    { buf++;
      if((buf[0]>'0') && (buf[0]<='9'))  // evtl zweite Zahl
      { end=atoi(buf);
	while(isdigit(buf[0]))buf++;
      }
    }
    else
      end=beg;

    // Bereich nur bernehmen wenn
    // - ein Blank oder Zeilenende folgt
    // - berhaupt ein Bereich angegeben wurde
    // - die Bereichsgrenzen plausibel sind
    //
    if(((buf[0]==' ') || (!buf[0])) && (first!=buf) && (beg<=end))
    { b->beginn=beg;
      b->ende=end;
      b->bereich=1;
      if(buf[0])
      { while(first!=buf)
	{ first[0]=' ';          // Bereichsangabe berschreiben
	  first++;
	}
      }
      else
	first[0]=0;
      break;
    }
    else
    { buf++;
      while((buf[0]!=' ') && buf[0])
	buf++;
    }
  }
}

/*---------------------------------------------------------------------------*/

char *chartomakro(char c,char *ms)
{ ms[0]=0;

  switch(c)
  { case 'a':
      sprintf(ms,"%d",cpuload());
      break;

    case 'b':
      strcpy(ms,b->prompt);
      break;

    case 'c':
      strcpy(ms,b->logincall);
      break;

    case 'd':
      strcpy(ms,datestr(ad_time(),2));
      break;

#ifdef HRX_OPT
    case 'e':                          
      sprintf(ms,"%ld",mbcheckcount()); 
      break;                           
#endif

    case 'f':
      sprintf(ms,"%d",freeblocks_tnc()/3);
      break;

    case 'h':
      sprintf(ms,"%d",u->helplevel);
      break;

    case 'i':
      strcpy(ms,timestr(b->logintime,1));
      break;


#ifdef _WXSTN
    case '1': //temperature
      if (m.wx.data_valid)
        sprintf(ms, "%3.1fC",m.wx.outdoort);
      else
        sprintf(ms,"-");
      break;

    case '2': //humidity
      if (m.wx.data_valid)
        sprintf(ms, "%3.1f%%",m.wx.humidity);
      else
        sprintf(ms,"-");
      break;

    case '3': //air pressure qnh
      if (m.wx.data_valid)
        sprintf(ms, "%3.1fhPa",m.wx.air_pres_qnh);
      else
        sprintf(ms,"-");
      break;

    case '4': //wind speed
      if (m.wx.data_valid)
        sprintf(ms, "%3.1fm/s",m.wx.windspeed);
      else
        sprintf(ms,"-");
      break;
#endif

    case 'l':
      if(u->lastboxlogin)
        sprintf(ms,"%s %s",datestr(u->lastboxlogin,2),timestr(u->lastboxlogin,1));
      else
	sprintf(ms,"Never");
      break;

    case 'm':
#if NODE
      strcpy(ms,b->node?n.call:m.boxname);
#else
      strcpy(ms,m.boxname);
#endif
      break;

    case 'n':
      strcpy(ms,u->name);
      break;

    case 'o':
      sprintf(ms,"%d",listusers(0));
      break;

    case 'p':
      sprintf(ms,"%s",zeitspanne(t->cputics,zs_cpuexact));
      break;

#if NODE
    case 'q':
      sprintf(ms,"%d",get_connects(ALL_LINKS));
      break;
#endif

    case 'r':
      strcpy(ms,"\n");
      break;
    
    case 's':
      strcpy(ms,memfree(0));
      break;
      
    case 't':
      strcpy(ms,timestr(ad_time(),1));
      break;
    
    case 'u':
      strcpy(ms,zeitspanne(ad_time()-einschaltzeit,zs_runtime));
      break;

    case 'v':
      sprintf(ms,"%s",VNUMMER);
      break;

    case 'w':
      sprintf(ms,"%ld Bytes",b->rxbytes+b->txbytes);
      break;

    case '%':
      strcpy(ms,"%");
      break;

    default:
      sprintf(ms,"%%%c",c);
  }
  return ms;
}

/*---------------------------------------------------------------------------*/

char *expandmakro(char *s,char *os)
{ char ms[20];
  int i=0;

  while(*s)
  { if(*s=='%')
    { s++;
      chartomakro(*s,ms);
      strcpy(os+i,ms);
      i+=strlen(ms);
    }
    else
      os[i++]= *s;
    s++;
  }
  os[i]=0;
  return os;
}

/*---------------------------------------------------------------------------*/

void leerzeile(void)
{ int i;

  for(i=0;i<u->lf;i++)
    putv('\n');
}

/*---------------------------------------------------------------------------*/

char *atcall(char *at)
{ int i=0;
  static char atc[12];

  strncpy(atc,at,10);
  atc[10]=0;
  while(atc[i] && atc[i]!='.') i++;
  // while(isalnum(atc[i]) || atc[i]=='-') i++;
  atc[i]=0;
  return atc;
}

/*---------------------------------------------------------------------------*/

int xmkdir (char *path)
//*************************************************************************
//
//  Creates a path, even if several directories have to be created
//
//*************************************************************************
{ char pathcopy[FNAMELEN + 1];
  char *pptr;
  int retwert = 0;

  if (strlen (path) > 39)
  { trace (fatal, "xmkdir", "path %s too long", path);
    path[39] = 0;
  }
  pptr = pathcopy; // Move the pointer to the first char of the pathname

  do
  { strcpy (pathcopy, path);
    strlwr (pathcopy);

    pptr = strchr (pptr, '/'); // Move the pointer to the next slash

    if (pptr)      // If we have found a slash,
    { pptr[0] = 0; // than we cut the string at this position
      pptr++;
    }

    if (filetime (pathcopy))
      if (isdir (pathcopy))
      { if (pptr == NULL)
          return 0;
      }
      else
      { xunlink (pathcopy); // delete an existing file
        trace (serious, "xmkdir", "file %s erased", pathcopy);
      }

    retwert = mkdir (pathcopy); // we return the last code we got from mkdir
  }
  while (pptr); // redo as long as the path contains slashes

  return retwert;
}

/*---------------------------------------------------------------------------*/

static unsigned char decode_7p[256]=
{
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 
0xFF, 0xFF, 0xFF, 0x0 , 0x1 , 0x2 , 0x3 , 0x4 , 0x5 , 0x6 , 
0x7 , 0x8 , 0xFF, 0x9 , 0xA , 0xB , 0xC , 0xD , 0xE , 0xF , 
0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 
0x1A, 0x1B, 0x1C, 0x1D, 0x1E, 0x1F, 0x20, 0x21, 0x22, 0x23, 
0x24, 0x25, 0x26, 0x27, 0x28, 0x29, 0x2A, 0x2B, 0x2C, 0x2D, 
0x2E, 0x2F, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 
0x38, 0x39, 0x3A, 0x3B, 0x3C, 0x3D, 0x3E, 0x3F, 0x40, 0x41, 
0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4A, 0x4B, 
0x4C, 0x4D, 0x4E, 0x4F, 0x50, 0x51, 0x52, 0x53, 0x54, 0x55, 
0x56, 0x57, 0x58, 0x59, 0x5A, 0x5B, 0x5C, 0xFF, 0x5D, 0x5E, 
0x5F, 0x60, 0x61, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 
0x69, 0x6A, 0x6B, 0x6C, 0x6D, 0xFF, 0x6E, 0xFF, 0x6F, 0x70, 
0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78, 0x79, 0x7A, 
0x7B, 0x7C, 0x7D, 0x7E, 0x7F, 0x80, 0x81, 0x82, 0x83, 0x84, 
0x85, 0x86, 0x87, 0x88, 0x89, 0x8A, 0x8B, 0x8C, 0x8D, 0x8E, 
0x8F, 0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 
0x99, 0x9A, 0x9B, 0x9C, 0x9D, 0x9E, 0x9F, 0xA0, 0xA1, 0xA2, 
0xA3, 0xA4, 0xA5, 0xA6, 0xA7, 0xA8, 0xA9, 0xAA, 0xAB, 0xAC, 
0xAD, 0xAE, 0xAF, 0xB0, 0xB1, 0xB2, 0xB3, 0xB4, 0xB5, 0xB6, 
0xB7, 0xB8, 0xB9, 0xBA, 0xBB, 0xBC, 0xBD, 0xBE, 0xBF, 0xC0, 
0xC1, 0xC2, 0xC3, 0xC4, 0xC5, 0xC6, 0xC7, 0xC8, 0xC9, 0xCA, 
0xCB, 0xCC, 0xCD, 0xCE, 0xCF, 0xD0, 0xD1, 0xD2, 0xD3, 0xD4, 
0xD5, 0xD6, 0xD7, 0xFF, 0xFF, 0xFF};

/*---------------------------------------------------------------------------*/

int line_7p(char * p)
//*************************************************************************
//
// Check the CRC of a line 7plus line p, returns 1 if ok, else 0
//
// This part of code has been taken from 7plus by Axel, DG1BBQ 
// (modificiations by oe3dzw)
//
//*************************************************************************

/*
  ------------------------------------------------------
  DG1BBQ > OE3DZW   16.02.98 18:12 9 Lines 186 Bytes #999 @OE3XSR.#OE3.AUT.EU
  Subj: RE:Verwendung 7plus Source-Code fuer BCM?
  From: DG1BBQ @ DB0VER.#NDS.DEU.EU (Axel)
  To:   OE3DZW @ OE3XSR.#OE3.AUT.EU
  Hallo Dietmar,
  hab' ich nix gegen, solange im Quelltext sowie im Manual daruf hingewiesen
  wird.
  73s, Axel.
  ------------------------------------------------------
*/

{
/* Calculate CRC */
  crcthp csequence;
  unsigned short int crc;
  unsigned long cs;
  int i;
  int slen=strlen(p);

  if (slen < 64 || slen > 71) // line must be at least 64 chars long 
     return (0);              // usually 69 chars, +2 extra cr or lf

  if (( (unsigned char) p[62])==(unsigned char) 0xb0 &&
      ( (unsigned char) p[63])==(unsigned char) 0xb1 &&
      ( (unsigned char) p[64])==(unsigned char) 0xb2)     //header-line eg. filename
     return (2);         

  for (i=0; i<64; i++)
  //crc_calc (csequence, p[i]);
  csequence.update(p[i]);
  csequence.result &= 0x3fff; /* strip calculated CRC to 14 bits */

  /* Get crc from code line */

   cs = 0xb640L * decode_7p[(unsigned char)p[66]] +
        0xd8L   * decode_7p[(unsigned char)p[65]] +
                  decode_7p[(unsigned char)p[64]];

   //linenumber = (int) (cs >> 14);    /* upper 9 bits are the line number */
   crc = (unsigned short int) (cs & 0x3fffL); /* lower 14 bits are the CRC */

  return (int) (csequence.result == crc);
}

/*---------------------------------------------------------------------------*/

int returnmailok(void)
//*************************************************************************
//
//  Checks if a return mail should be sent or not 
//  returns 1 when mail should be send
//
//*************************************************************************
{ if (strncmp (b->betreff, "UNKNOWN", 7) != 0 &&
      strncmp (b->betreff, "HOLD"   , 4) != 0 && 
      strncmp (b->betreff, "CP "    , 3) != 0 &&      
      strncmp (b->betreff, "New cal", 7) != 0 &&
      mbcallok(b->herkunft)                   &&
      strcmp  (b->herkunft,b->ziel)      != 0 &&
      b->usermail                             && 
      b->mailtype=='P'                         )
    return 1;
  else
    return 0;
}

/*---------------------------------------------------------------------------*/

#ifdef HRX_OPT
int autosysop(void)  // JJ autosysop
{ int a=0;
  int i=0;
  int issysop=0;
  char *fname="asysop.bcm";
  char line[255];
  char call[255];
  char uplink[255];
  char downlink[255];
  FILE *f;

  f=s_fopen(fname,"lrt");
  if(f)
  { while((a=fgetc(f))!=EOF)
    { line[i]=a;
      i++;
      if(a==CR || a==LF)
      { line[i]=0;
	i=sscanf(line,"%s%s%s%s",call,uplink,downlink);
	if(strcmp(b->logincall,call)==0
	   && (strcmp(b->uplink,m.boxname)==0 && strcmp(uplink,"none")==0 || strcmp(b->uplink,uplink)==0)
	   && (strlen(t->downlink)==0 && strcmp(downlink,"none")==0 || strcmp(t->downlink,downlink)==0)
	   ) issysop=1;
	i=0;
      }
    }
    s_fclose(f);
  }

  if(issysop)
    trace(report,"autosysop","sysoplink detected");

  return issysop;
}
#endif

/*---------------------------------------------------------------------------*/
