/***************************************************************


  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Mailboxsystem


  ----------------------------------------
  White-Pages (including TheBox E/M-Mails)
  ----------------------------------------


  Copyright (C)       Florian Radlherr et al
                      Taubenbergstr. 32
                      D-83627 Warngau

                      Dietmar Zlabinger
                      Reinberg-Heidenreichstein 31
                      A-3861 Eggern

  Alle Rechte vorbehalten / All Rights Reserved GNU GPL

 ***************************************************************/

//19990412 OE3DZW wp-update only if filesize > 24lines approx.
//19990621 OE3DZW old trigger-files will no longer be removed
//19990726 OE3DZW trigger file was not deleted in the internet 1.41a dist
//19991111 OE3DZW changed mycall to [1] (fwdmycall)

#include "baycom.h"

#define WPSUBJECT "WP Update"
#define BPROTMAX 10		// max. protocol version accepted

typedef enum w_file_t
{
  fbb_file,
  w_file,
  unknown_file
}
w_file_t;

/*---------------------------------------------------------------------------*/

void
add_emfile (char *zeile, char *frombox, char *file_flag, char *boardname,
	    char *at)
//*************************************************************************
//
//  Legt die angegebene Verwaltungszeile in den M-Forwardfiles ab
//  die zu den Nachbarboxen in (boxenliste) gehoeren.
//
//  zeile: line to be saved
//  frombox: bbs message in line comes from (to be excluded)
//  file_flag: defines file: m -> old em-style, w -> new wp-style
//
//*************************************************************************
{
  char fwdfname[20];
  char boxcall[8];
  char *boxenliste = b->destboxlist;
  FILE *fwdfile;
  static char name[] = "add_emfile";

  //trace (report, name, "from: %s '%s'", frombox, zeile);

  strcpy (b->at, at);
  weiterleiten (0, boardname);
  if (strchr (boxenliste, '.'))
    strchr (boxenliste, '.')[0] = 0;

  while (boxenliste = nexttoken (boxenliste, boxcall, 6), boxcall[0] >= '0')
  {
    int box = isforwardpartner (boxcall);
    if (box != NOTFOUND && strcmp (frombox, boxcall) != 0)
    {
      strcpy (fwdfname, FWDPATH "/");
      strcat (fwdfname, file_flag);
      strcat (fwdfname, "_");
      strcat (fwdfname, boxcall);
      strcat (fwdfname, ".bcm");
      strlwr (fwdfname);
      fwdfile = s_fopen (fwdfname, "sat");	// fwdtrigger

      if (!fwdfile)		// wenn es nicht gegangen ist, ist evtl.
      {
	mkdir (FWDPATH);	// das Directory noch gar nicht angelegt

	fwdfile = s_fopen (fwdfname, "sat");
      }
      if (fwdfile)
      {
	fprintf (fwdfile, "%s\n", zeile);
	s_fclose (fwdfile);
      }
      else
	trace (serious, name, "fopen %s errno=%d", fwdfname, errno);
      waitfor (e_reschedule);
    }
  }
}

/*---------------------------------------------------------------------------*/

int
get_wcs (char *line)
//*************************************************************************
//
//  calculates simple line-checksum used in W-files
//
//*************************************************************************
{
  int wcs = 0;
  char *c = line;
  while (*c)
  {
    wcs += *c;
    c++;
  }
  return (wcs & 0xff);
}

/*---------------------------------------------------------------------------*/

void
addwp_m (wpdata_t *wp)
//*************************************************************************
// 
//  Add mybbs-info (<bbs>) for <call> to triggerfiles 
// 
//*************************************************************************
// used globals: b->logincall, b->at (modified), b->destboxlist(modified)

{
  char *bef = b->line;
  int wcs = 0;
  //first call "old" thebox-function (backwards compatibility) 
  snprintf (bef,sizeof(b->line), "M @ THEBOX < %s $%s %s %ld", wp->call, wp->bid, wp->bbs, wp->mybbstime);
  add_emfile (bef, b->logincall, "M", "M", "$EM");

  //new w-format
  snprintf (bef + 3,sizeof(b->line)-3, "M %s %s %s %ld %s %ld \"%s\" %s \"%s\"",
	   wp->call, wp->bbs, wp->bid, wp->mybbstime,wp->origin, 
           wp->hops,wp->name,wp->zip, wp->qth);
  // ?.. dummy for zip-code
  wcs = get_wcs (bef + 3);
  //put checksum at begin of line 
  snprintf (bef,sizeof(b->line), "%02X", wcs);
  bef[2] = ' ';			//replace 0 by space

  add_emfile (bef, b->logincall, "W", "M", "$WP");
}

/*---------------------------------------------------------------------------*/

void
addwp_b (wpb_t *wpb)
//*************************************************************************
// 
//  Add bbs-info to triggerfiles 
// 
//*************************************************************************
// used globals: b->logincall, b->at (modified), b->destboxlist(modified)

{
  char *bef = b->line;
  int wcs = 0;

  //new w-format
  snprintf (bef + 3,sizeof(b->line)-3, "B %s %ld %s %s %s %s %ld %ld",
    wpb->hadr,wpb->bversion,wpb->bstatus,wpb->sysopcall,wpb->protocol,
    wpb->hwaddress,wpb->timestamp,(wpb->hops)+1);
  wcs = get_wcs (bef + 3);
  //put checksum at begin of line 
  snprintf (bef,sizeof(b->line), "%02X", wcs);
  bef[2] = ' ';			//replace 0 by space

  add_emfile (bef, b->logincall, "W", "B", "$WP");
}

/*---------------------------------------------------------------------------*/

void
addwp_e (char *at, char *originbbs, char *bid, char *bid_erase,
	 char *erase_user, long hops, char *reason, char *logincall)
//*************************************************************************
//
//  Puts a triggerline into w_<boxcall>.bcm (Used for remote erase in
//  W-Format)
//
//*************************************************************************
{
  char *bef = b->line;

  int wcs = 0;

  snprintf (bef + 3,sizeof(b->line), "E %s %s %s %s %s %ld \"%s\"",
	   at, originbbs, bid, bid_erase, erase_user, ++hops, reason);
  wcs = get_wcs (bef + 3);
  //put checksum at begin of line 
  snprintf (bef,sizeof(b->line), "%02X", wcs);
  bef[2] = ' ';			//replace 0 by space

  add_emfile (bef, b->logincall, "W", "E", "$WP");
}

/*---------------------------------------------------------------------------*/

void
wpgenheader (FILE * f)
//*************************************************************************
//
//  Generates a W header
//
//*************************************************************************
{
  char tmp[120];
  int wcs;

  // version
  snprintf (tmp + 3,sizeof(tmp)-3, "V 10");
  wcs = get_wcs (tmp + 3);
  snprintf (tmp,sizeof(tmp), "%02X", wcs);
  tmp[2] = ' ';
  fprintf (f, "%s\n", tmp);

  // bbs-info 
  long bver=BVERSION;
  snprintf (tmp + 3,sizeof(tmp)-3, "B %s %ld " BSTATUS " %s AX25 %s %ld 1",
	   m.boxadress, bver ,m.sysopcall, m.mycall[1],
           ad_time()+ad_timezone()/3600);
  wcs = get_wcs (tmp + 3);
  snprintf (tmp,sizeof(tmp), "%02X", wcs);
  tmp[2] = ' ';
  fprintf (f, "%s\n", tmp);
}

/*---------------------------------------------------------------------------*/

void
gen_wpmail (char *call)
//*************************************************************************
//
//  Will generate a wp-mail out of data stored in the wp-file
//  This mail will be forwarded like any "standard" personal mail
//
//  Format of new mail: WP or W
//
//*************************************************************************
{
  char fwdfname[20];
  char fwdoldname[20];
  FILE *fwdfile;
  long lines=0;
  static char name[] = "gen_wpmail";

  // do not try to generate files during reorg, it
  // can easily done be later...

  if (sema_test("purgereorg"))
    return;

  int fbbformat = !(b->opt & o_w);	// W or WP format
  
  // create boards if not existing
  mkboard ("/", "WP", 0); //TODO problem with NT why?
  mkboard ("/", "W", 0);

  //open new trigger file
  strcpy (fwdfname, FWDPATH "/W_");
  strcat (fwdfname, call);
  strcat (fwdfname, ".bcm");
  strlwr (fwdfname);

  //simple hack to get rid off _very_ big files
  //due to broken links or sw-bugs
  if (filesize(fwdfname) > 100000L)
    xunlink(fwdfname);

  fwdpara_t *ff=fpara();
  time_t ti=ad_time()+ad_timezone()/3600;

  if ((filesize(fwdfname) > 50*80 || //at least approx 50 lines
       ti-ff->lastwp > 12*3600) &&   //after startup/every 12h 
      (fwdfile = s_fopen (fwdfname, "srt")) != 0)
  { ff->lastwp=ti;
    //remove old trigger file
    strcpy (fwdoldname, FWDPATH "/M_");
    strcat (fwdoldname, call);
    strcat (fwdoldname, ".bcm");
    strlwr (fwdoldname);
    xunlink(fwdoldname);

    char tmpname[30];
    char *buf = b->line;
    snprintf (tmpname,sizeof(tmpname), TEMPPATH "/%s", time2filename (0));
    FILE *tmpfile = s_fopen (tmpname, "swt");
    if (!tmpfile)
    {
      trace (serious, name, "fopen %s", tmpname);
      s_fclose (fwdfile);
      return;
    }
    if (fbbformat)
      fprintf (tmpfile, "SP WP@%s", call);
    else
      fprintf (tmpfile, "SP W@%s #1", call);
    fprintf (tmpfile, "\n" WPSUBJECT "\n");

    wpdata_t *wp=(wpdata_t *) t_malloc(sizeof(wpdata_t),"wpg");

    trace (report, name, "%s type %s", call, fbbformat?"fbb":"w");

    if (!fbbformat)
    {
      wpgenheader (tmpfile);
    }
    // copy data
    while (!feof (fwdfile))
    {
      if (fgets (buf, 250, fwdfile) != NULL)
      {
	buf[250] = 0;
        //trace(report,name,"%s",buf);
	if (strlen (buf) < 20)	// skip empty lines
	  continue;

	if (fbbformat)
	{
	  if (buf[3] == 'M' && wpparse_m (buf+5, wp))
	  { char *spc;
            //replace spaces by dashes
            while ((spc=strchr(wp->name,' '))!=0)
              *spc='-';
            expand_hadr(wp->bbs,2); //FBB needs full address
            if (strchr(wp->bbs,'.')) //check for h-address
	    { snprintf (buf,sizeof(b->line), "On %s %s/U @ %s zip %s %s %s\n",
		     datestr (wp->mybbstime,4),
                     wp->call,wp->bbs,
                     wp->zip,wp->name,wp->qth);
              lines++;
	      fputs (buf, tmpfile);
            }
	  }
	}
	else
	{ lines++;
	  fputs (buf, tmpfile);
	}
      }
    waitfor (e_reschedule);
    }
    s_fclose (fwdfile);
    fprintf (tmpfile, "\nnnnn\nimpdel\n");
    s_fclose (tmpfile);
    xunlink (fwdfname);
    if (lines)
      fork (P_BACK | P_MAIL, 0, mbimport, tmpname);
    else
      xunlink(tmpname);
    t_free(wp);
  }
}

/*---------------------------------------------------------------------------*/

int
wpparse_fbb (char *buf,wpdata_t *wp)
//*************************************************************************
//
// Parses a single WP-line (FBB/W0RLI)
// returns 1 on success, else 0
//
//*************************************************************************
{
#ifdef _BCMNET_FWD
  //CB-BCMNET: we can't support this because we do not have a bid
  return NO;
#else
  char *p = buf;
  char yymmdd[7];
  char tmp[21];

  p = nexttoken (buf,tmp,20);
  if (strcmp(tmp,"On")!=0)
    return 0;

  p = nexttoken (p, yymmdd, 6);
  wp->mybbstime = yymmdd2ansi (yymmdd);

  p = nexttoken (p, tmp, 20);
  if (!strstr (tmp, "/U"))
    return 0;
  subst (tmp, '/', 0);
   if (strlen (tmp) > 6 || !mbcallok (tmp))
    return 0;
  strcpy (wp->call, tmp);

  if (p[0]!='@' || strlen(p) < 4)
    return 0;
  else 
    p+=2;
  
  p = nexttoken (p, wp->bbs, HADRESSLEN);
  if (mbhadrok (wp->bbs) != 1)
    return 0;

  p = nexttoken (p,tmp,20);
  if (strcmp(tmp,"zip")==0)
  {
    p = nexttoken(p,wp->zip,ZIPLEN);
    if (wp->zip[0]==0)
      strcpy(wp->zip,"?");
    p = nexttoken(p,wp->name,NAMELEN);
    if (wp->name[0]==0)
      strcpy(wp->name,"?");
    p = nexttoken(p,wp->qth,QTHLEN);
    if (wp->qth[0]==0)
      strcpy(wp->qth,"?");

  }
  else 
  { strcpy(wp->zip,"?");
    strcpy(wp->name,"?");
    strcpy(wp->qth,"?");
  }
  strcpy(wp->logincall,"?");
  strcpy(wp->origin,"WP");
  wp->hops=1;
  strcpy(wp->bid,newbid(0)); //jede mail eine neue bid?? 
  return 1;
#endif
}

/*---------------------------------------------------------------------------*/

int
wpparse_v (char *buf, long &protnum)
//*************************************************************************
//
//  Parses version line
//  returns 1 on success
//
//*************************************************************************
{
  protnum=atol(buf);
  if (!protnum)
    return 0;
  return 1;
}
/*---------------------------------------------------------------------------*/

int
wpparse_b (char *buf, wpb_t *wpb)
//*************************************************************************
//
//  Parses bbs line
//  returns 1 on success
//
//*************************************************************************
{
  char *p;
  char tmp[21];

  p = nexttoken (buf, wpb->hadr, HADRESSLEN);
  if (mbhadrok (wpb->hadr) != 1)
    return 0;
  p = nexttoken (p, tmp, 20);
  wpb->bversion = atol (tmp);
  if (!wpb->bversion)
    return 0;
  p = nexttoken (p, wpb->bstatus, BSTATUSLEN);
  p = nexttoken (p, wpb->sysopcall, CALLEN);
  p = nexttoken (p, wpb->protocol, BPROTOLEN);
  p = nexttoken (p, wpb->hwaddress, BHWADDRESSLEN);
  p = nexttoken (p, tmp, 20);
  wpb->timestamp = atol (tmp);
  if (!wpb->timestamp)
    return 0;
  p = nexttoken (p, tmp, 20);
  wpb->hops = atol (tmp);
  if (!wpb->hops)
    return 0;

  return 1;
}

/*---------------------------------------------------------------------------*/

int
wpparse_m (char *buf,wpdata_t *wp)
//*************************************************************************
//
//  Parses mybbs line
//  returns 1 on success
//
//*************************************************************************
{

//sample m-line (including csum and flag)
//36 M OE3IGW OE3BBS E29OE3DZW017 918987756 OE3DZW 1

  char tmp[21];
  char *p;

  p = nexttoken (buf, wp->call, CALLEN);
  if (!mbcallok (wp->call))
    return 0;

  p = nexttoken (p, wp->bbs, HADRESSLEN);

  if (mbhadrok (wp->bbs) != 1)
    return 0;
  p = nexttoken (p, wp->bid, BIDLEN);
  if (strlen (wp->bid) < 5)
    return 0;
  p = nexttoken (p, tmp, 20);
  wp->mybbstime=atol(tmp);
  if (!wp->mybbstime) 
    return 0;
  p = nexttoken (p, wp->origin, CALLEN);
  p = nexttoken (p, tmp, 20);
  wp->hops = atol (tmp);
  if (!wp->hops)
    return 0;
  p = nexttoken (p,wp->name,NAMELEN);
  if (wp->name[0]=='\"') //ignore names with bad quotes
    strcpy(wp->name,"?");
  p = nexttoken (p,wp->zip,ZIPLEN);
  if (wp->zip[0]==0)
      strcpy(wp->zip,"?");
  p = nexttoken (p,wp->qth,QTHLEN);
  if (wp->qth[0]==0)
      strcpy(wp->qth,"?");
  strcpy(wp->logincall,"?");
#ifdef _BCMNET_FWD
  if (is_bcmnet_bid(wp->bid, 1)==1) return NO; //ignore MYBBS correction entrys
#endif
  return 1;
}

/*---------------------------------------------------------------------------*/

int
wpparse_e (char *buf, char *at, char *bid, char *originbbs, char *bid_erase,
	   char *origin, long &hops, char *comment)
//*************************************************************************
//
//  Parses erase line
//  returns 1 on success
//
//*************************************************************************
{
  char tmp[20];
  char *p;

  p = nexttoken (buf, at, HADRESSLEN);
  p = nexttoken (p, originbbs, CALLEN);
  p = nexttoken (p, bid, BIDLEN);
  if (strlen (bid) < 5)
    return 0;
  p = nexttoken (p, bid_erase, BIDLEN);
  if (strlen (bid_erase) < 5)   //BUG FOUND
    return 0;
  p = nexttoken (p, origin, CALLEN);
  p = nexttoken (p, tmp, 20);
  hops = atol (tmp);
  if (hops == 0)
    return 0;
  nexttoken(p,comment,99);

  return 1;
}

/*---------------------------------------------------------------------------*/

void
scan_wp (void)
//*************************************************************************
// 
//  Check if incoming mail is valid 
// 
//*************************************************************************
// Used globals: b->herkunft, b->ziel,b->betreff,b->at,b->mailpath
// m.boxname,b->line (modified)
{

  static char name[] = "scan_wp";
  w_file_t ftype = unknown_file;
  int protok = 1;
  long line=0;
  FILE *wpfile;
 

  // check if this mail is an wp-mail
  if (  strcmp(b->uplink,"Import")!=0 &&  
        !(b->forwarding == fwd_standard) &&
	b->mailtype == 'P' &&
	strcmp (b->herkunft, b->frombox) == 0 &&
	(strcmp (atcall (b->at), m.boxname) == 0) || (!*b->at))
  {
    if (strcmp (b->ziel, "WP") == 0 && !(b->opt & o_w))
      ftype = fbb_file;
    if (strcmp (b->ziel, "W") == 0 && (b->opt & o_w))
      ftype = w_file;
    //subject of mail is not checked

    if (ftype == unknown_file)
      return;

    markerased ('W', 0); 

    char *buf=(char *)t_malloc(256,"wp");
    wpdata_t *wp=(wpdata_t*)t_malloc(sizeof(wpdata_t),"wps");

    wpfile = s_fopen (b->mailpath, "srt");
    trace (report, name, "scan %s", b->mailpath);
    if (wpfile)
    { 
      while (!feof (wpfile) && protok)
      { 
	if (fgets (buf, 255, wpfile) == NULL)
	  continue;
        line++;
        if (line < 7||buf[2]!=' ') //skip mail header 
          continue;
	buf[250] = 0;
        subst(buf,'\n',0);
	if (strlen (buf) < 6) //skip empty lines
	  continue;
        //printf("-%s-\n",buf); 
	if (ftype == fbb_file)
	{
	  if (wpparse_fbb (buf,wp))
	  { if (wp->name[0] && wp->name[0]!='?')
               set_name(wp->call,wp->name);
	    set_mybbs (wp->call, wp->bbs, wp->mybbstime-24*3600,
              wp->bid,wp->origin,wp->hops,wp->name);
	  }
	}
	else
	{ 
	  int wcs = 0;
	  sscanf (buf, "%02X", &wcs);
	  char *pbuf = buf + 3;
	  if (wcs != get_wcs (pbuf))
	  {
	    trace (report, name, "cs  err");
	    continue;
	  }
	  if (pbuf[0] == 'V')
	  {
	    long protnum = 0;
	    if (wpparse_v (buf+5, protnum) &&
		protnum <= BPROTMAX)
	    {
	      //trace (report, name, "v %ld", protnum);
	    }
	    else
	      protok = 0;

	  }
	  if (pbuf[0] == 'B')
	  { wpb_t *wpb=(wpb_t *) t_malloc(sizeof(wpb_t),"wpb");

	    if (wpparse_b (buf+5,wpb));
            //store hadr-information in local data base
            //check if information is new and update is needed
            if (wpupdate_hadr(wpb))
	    { addwp_b(wpb);
            }
            t_free(wpb);
	  }
	  if (pbuf[0] == 'M')
	  {
	    if (wpparse_m (buf+5,wp))
            {
              if (wp->name[0] && wp->name[0]!='?')
                set_name(wp->call,wp->name);
	      set_mybbs (wp->call,wp->bbs,wp->mybbstime,
                            wp->bid,wp->origin,wp->hops,wp->name);
            }
 	  }
	  if (pbuf[0] == 'E')
	  {
	    char at[HADRESSLEN + 1];
	    char bid[BIDLEN + 1];
	    char originbbs[CALLEN + 1];
	    char bid_erase[BIDLEN + 1];
	    char origin[CALLEN + 1];
	    char *comment = (char *)t_malloc(100,"wpe");;
	    long hops;

	    if (wpparse_e (buf+5, at, bid, originbbs, bid_erase, origin, 
                hops, comment))
	    { remeraser (at, bid, originbbs, bid_erase, b->logincall, origin,
			 hops, comment);
            }
            t_free(comment);
	  }
	}
      waitfor (e_reschedule);
      }
      s_fclose (wpfile);
    }
    t_free(wp);
    t_free(buf);
  }
}

/*---------------------------------------------------------------------------*/

void
remeraser (char *at, char *bid, char *herkunft, char *bid_erase,
	   char *logincall, char *erase_user, long hops, char *reason)
//*************************************************************************
//
//  Remote erase of mails
//
//*************************************************************************
{
  char *bidname;
  static char name[] = "remerase";
  char s[80];

  //trace (report, name, "erase $%s<%s (from $%s)", bid_erase, at, bid);

  blankweg (bid_erase);
  bid_erase[12] = 0;		// limit max. length


  if (!bidvorhanden(bid) && !tmpbidvorhanden(bid))
  { bidrein(bid,"E/M");  
    snprintf (s,sizeof(s), "E @ %s < %s $%s %s", at, herkunft, bid, bid_erase);
    add_emfile (s, logincall,"M","E","$EM");
    addwp_e (at, herkunft, bid, bid_erase, erase_user, hops, reason, logincall);
  }


  bidname = bidvorhanden (bid_erase);
  if (bidname && bidname[0] && (strcmp (bidname, "E/M") != 0))
  {
    strcpy (s, bidname);
    if (setfullpath (s))
    {
      markerased ('L', 0);
      trace (report, name, "%s ($%s)", s,bid_erase);
    }
  }
  else
    trace (report, name, "not found %s", s);
}

/*---------------------------------------------------------------------------*/
