/***************************************************************

  BayCom(R)           Packet-Radio fuer IBM PC

  BayCom-Terminal
  BayCom-Node
  BayCom-Mailbox

  Routinen fuer Benutzeroberflaeche


  -----------------------------------------------
  Definitionen und Prototypen fuer Taskverwaltung
  -----------------------------------------------


  Copyright (c) 12/91 Florian Radlherr
                      Taubenbergstr. 32
                      83627 Warngau

  Alle Rechte vorbehalten / All Rights Reserved

 ***************************************************************/
//19980311 DG9MHZ changes for Linux Kernel AX25
//19980615 hrx    reimplemented downlink (added to task_t)

#ifndef _TASKING_H
#define _TASKING_H

#define NOTASK    (-1)

// Flags fr laufenden Prozess (wird in t->processkind hinterlegt)
// Die Flags knnen beliebig kombiniert werden
//
#define P_MAIL 1      // hat einen Mailbox-Kontrollblock
#define P_WIND 2      // hat einen Editor-Kontrollblock
#define P_KEYB 4      // benutzt Tastatureingaben
#define P_BACK 8      // luft im Hintergrund (keinerlei Ein- und Ausgabe)
#define P_CLON 16     // DG4MMI: Mailbox-Kontrollblock wird in die neue
                      // Task uebernommen
#define P_AX25K 0x100

typedef unsigned long bitfeld;

typedef enum event_t
{ e_nothing,
  e_ticsfull,
  e_reschedule,
  e_delay,
  e_kbhit
} event_t;

typedef enum io_t
{ io_dummy,
  io_window,
  io_editwin,
  io_tnc,
  io_tty,
  io_file,
  io_socket,
  io_memory,
  io_comp // DG4MMI: Online-COMP
} io_t;

#define MONBUFLEN 2048

typedef struct monbuf_t
{ int      tid;
  bitfeld  opt;
  unsigned in;
  unsigned out;
  int congested;
  char buf[MONBUFLEN];
} monbuf_t;
/*
typedef struct sock_t
{
  char rxbuf[RXLEN];
  char txbuf[TXLEN];
  char mycall[16];
  int handle;
  int rxc;
  int txc;
  int rxlen;
  int wasput;
  int wasda;
  char dasda;
  time_t lastflush;
}
sock_t;
*/
typedef void (* atkill_t)(void);
typedef void (* atkill2_t)(void *);
#define MAX_ATKILL 4

typedef struct task_t
{ event_t  event;
  int      taskid;
  int      port;
  int 	   lock;  		/* >0 avoids rescheduling */
  int      socket;
  int      childpid;
  io_t     input;
  io_t     output;
  monbuf_t *monitor;
  unsigned lastdelay;
  unsigned begintic;
  long     cputics;
  jmp_buf  j;
  unsigned processkind;
  time_t   created;
  time_t   last_input;
#ifdef _UTIMEOUT	//db1ras
  time_t   last_output;
  int      timeout;
#else
  time_t   kill_time;
#endif
  unsigned keybuffer[32];
  int      keybufin;
  int      keybufout;
  void     *e;
  void     *b;
  atkill2_t atkill[MAX_ATKILL];
  void     *atkill_p[MAX_ATKILL];
  int      atkillcnt;
  char     name[20];
  char     lastcmd[60];           // letztes eingegebenes Kommando
  char     stack[TASKSTACK+4];
//#ifdef HRX_OPT
  char     downlink[10];          // letztes Digirufzeichen
//#endif
} task_t;

#define _s_report      0
#define _s_replog      1
#define _s_serious     2
#define _s_fatal       3
#define _s_tr_abbruch  4

#ifdef __FLAT__
#define report      _s_report,__LINE__,__FILE__
#define replog      _s_replog,__LINE__,__FILE__
#define serious     _s_serious,__LINE__,__FILE__
#define fatal       _s_fatal,__LINE__,__FILE__
#define tr_abbruch  _s_tr_abbruch,__LINE__,__FILE__
#else
#define report      _s_report,0,""
#define replog      _s_replog,0,""
#define serious     _s_serious,0,""
#define fatal       _s_fatal,0,""
#define tr_abbruch  _s_tr_abbruch,0,""
#endif

extern int newkeywin;
extern int keytaskid;
extern int deskkey;
extern int init_off;
extern task_t *t;       // Zeiger auf aktuellen TCB
extern long systic;

void call_l2(int off);

//****************************
//       mbtask.cpp
//****************************
int  	 fork(int typ,int port,void(*stfunc)(char *),char *stpar);
void     atkill(atkill_t func);
void     atkill(atkill2_t func,void *p);
void     rm_atkill(atkill_t func);
void     suicide(void);
int  	 waitfor(event_t event);
unsigned waitkey(void);
unsigned waitalt(void);
int  	 wdelay(unsigned zeit);
void 	 maintask(void);
void     initstack(void);
void     setkeyhere(void);
int      iskeytask(void);
int      keywindow(void);
int      gettaskid(void);
char *   gettaskname(int tid);
void     listprocess(long opt);
int      kill(int tid);
int      killport(int port);
int      taskport0(int port);
int      listusers(int putit);
int      eingelogt(char *logincall,int job);
#ifdef _TTY
int      taskexists(int tid); //JJ
#endif
int      taskkbhit(void);
int      mbtalk(char *fromcall,char *tocall,char *message);
int      monitor(int tid,bitfeld opt);
int      setkeywindow(int fenster);
void     lastcmd(char *cmd);
void     post_mortem_dump(char *);
void     task_idle(int active);
void     timeout(int minutes);
void     test_killtime(void);
void     t_lock(void);
void 	 t_unlock(int force);

//****************************
//       mbtrace.cpp
//****************************
void	 trace(int mode,int line,char *file,char *programm,char *string, ...);
void     tracewindow(char *name);
void     cmdlog(char *befehl);
void     tracelog(char *message,int mode);
void     pwlog(char *mycall,char *digi,char *msg);

//****************************
//       mballoc.cpp
//****************************
void *t_malloc(size_t size,char *name);
void *t_realloc(void *blk,size_t newsize);
void t_free(void *blk);
void t_alloclist(void);
void t_allocinit(void);
void t_allfree(int task);
void t_checkfree(void);
int t_settaskid(void *blk,int tid);
#endif
